package acover;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import atosj.Messages;

/** 
 * Generiert eine XML-Datenbasis mit Informationen zu allen Paketen eines
 * Java-Programms, die einer berdeckungsanalyse unterzogen werden sollen.
 * @author Nicos Tegos
 */
public class DatabaseGenerator 
{
	/** Programmrckgabewert: Anzahl der Argumente falsch */
	public static final int RET_ERROR_ARG_NUM = 1;
	/** Programmrckgabewert: ungltiger Dateiname oder falsches Dateiformat */
	public static final int RET_ERROR_INVALID_FILE = 2;
	/** Programmrckgabewert: sonstige Fehler beim erstellen der Datenbank */
	public static final int RET_PROCESSING_ERROR = 3;
	
	private Vector jars = new Vector(); // Vector<ZipFile> alle Bibliotheken, die analysiert werden sollen
	private Vector dirs = new Vector(); // Vector<File> alle Class-Dateien, die analysiert werden sollen
	
	private String[] packageNames; // Namen der Pakete zu denen die berdeckungsanalyse durchgefhrt werden soll
	
	/**
	 * Initialisiert einen neuen Generator zur berdeckungsanalyse.
	 * @param packageNames Die Namen der Pakete, die analysiert werden sollen.
	 * Alle zu analysierenden Pakete mssen im Classpath stehen. Es reicht
	 * nur einen Teil des Paketnamens anzugeben, es werden dann automatisch
	 * alle Pakete die mit der angegebenen Zeichenkette beginnen in die 
	 * Analyse einbezogen.
	 * @throws IOException I/O Fehler bei Zugriff auf ein jar des Classpath.
	 */
	public DatabaseGenerator(String[] packageNames) throws IOException {
		this.packageNames = packageNames;
		
		String[] paths = System.getProperty("java.class.path").split(File.pathSeparator); //$NON-NLS-1$
		for (int i = 0; i < paths.length; i++) 
		{
			if( paths[i].toLowerCase().endsWith(".jar") ) //$NON-NLS-1$
				jars.add( new ZipFile(paths[i]) );
			else
				dirs.add( new File(paths[i]) );
		}
	}
	
	/**
	 * Erstellt eine Datenbasis zu den Paketen, die analysiert werden sollen.
	 * Die XML-Datenbasis ist hierarchisch geordnet nach Klassen und enthaltenen
	 * Methoden. Zu jeder Methode wird die Anzahl ihrer Aufrufe vermerkt.
	 * Aus der Datenbasis knnen spter die Statisken zur berdeckungsanalyse generiert
	 * werden, z.B. mit dem <code>HTMLConverter</code>.
	 * @param filePath Der vollstndige Pfad fr die XML-Datenbasis.
	 * @throws Exception Beliebiger Fehler.
	 */
	public void createXMLDatabase(String filePath) throws Exception 
	{
		try 
		{
			Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
			Element root = doc.createElement("coverage"); //$NON-NLS-1$
			doc.appendChild(root);
			
			root.setAttribute( "jar", "JarName" ); //$NON-NLS-1$ //$NON-NLS-2$
			
			for (int i = 0; i < jars.size(); i++)
				createNodesForJar(doc, (ZipFile) jars.get(i) );
			
			for (int i = 0; i < dirs.size(); i++)
				createNodesForDir( doc, (File) dirs.get(i), (File) dirs.get(i) );
			
			XMLUtilities.writeDocToXMLFile(doc, filePath);
			
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}
	
	/** Gibt an, ob <code>clazz</code> zu einem Paket gehrt, 
	 * dass analysiert werden soll. */
	private boolean isValidPackage(Class clazz) {
		for (int i = 0; i < packageNames.length; i++) {
			if( clazz.getName().startsWith(packageNames[i]) )
				return true;
		}
		
		return false;
	}
	
	/** 
	 * Erstellt fr alle Klassen innerhalb eines jars ein XML-Element 
	 * und fgt es <code>doc</code> hinzu. */
	private void createNodesForJar(Document doc, ZipFile jar) 
	{
		Enumeration e = jar.entries();
		while( e.hasMoreElements() )
		{
			String entryName = ((ZipEntry) e.nextElement()).getName();
			if( entryName.endsWith(".class") ) //$NON-NLS-1$
			{ 
				try 
				{
					Class clazz = Class.forName( entryName.substring(0, entryName.lastIndexOf('.')).replaceAll("/", "."), false, this.getClass().getClassLoader()  ); //$NON-NLS-1$ //$NON-NLS-2$
					appendClassNode(doc, clazz);
				} catch (Throwable e1) {}
			}
		}
	}
	
	/** 
	 * Erstellt rekursiv fr alle Klassen innerhalb Vereichnisses 
	 * <code>rootDir</code> und seinen Unterverzeichnissen ein XML-Element 
	 * und fgt es <code>doc</code> hinzu. */
	private void createNodesForDir(Document doc, File rootDir, File dir) 
	{
		// alle im Verzeichnis enthaltenen class Dateien finden
		File[] classFiles = dir.listFiles( new FileFilter() {
			public boolean accept(File file) {
				if(file.isDirectory() || !file.getName().endsWith(".class") ) //$NON-NLS-1$
					return false;
				return true;
			}
		});
		
		for(int i = 0; i < classFiles.length; ++i)
		{
			try 
			{
				String name = classFiles[i].getAbsolutePath();
				name = name.substring( rootDir.getAbsolutePath().length()+1, name.length() );
				name = name.substring( 0, name.lastIndexOf('.') );
				name = name.replace(File.separatorChar, '.');
				Class clazz = Class.forName( name, false, this.getClass().getClassLoader()  );
				appendClassNode(doc, clazz);
			} catch (Throwable e1) {}
		}
		
		File[] subDirs = dir.listFiles( new FileFilter() {
			public boolean accept(File file) {
				return file.isDirectory();
			}	
		}) ;
		
		// rekursiver Aufruf an alle Unterverzeichnisse
		for (int i = 0; i < subDirs.length; i++)
			createNodesForDir(doc, rootDir, subDirs[i]);
	}
	
	/** 
	 * Erstellt fr die angegebene Klasse ein XML-Element 
	 * und fgt es <code>doc</code> hinzu. Das Element enthlt
	 * Knoten zu allen ausfhrbaren Methoden der Klasse <code>clazz</code>. */
	private void appendClassNode(Document doc, Class clazz) 
	{
		if( !isValidPackage(clazz) )
			return;
		
		int modifiers = clazz.getModifiers();
		if( ! Modifier.isInterface(modifiers) ) // keine Interfaces
		{
			Element classNode = doc.createElement("class"); //$NON-NLS-1$
			classNode.setAttribute("name", clazz.getName()); //$NON-NLS-1$
										
			Method[] methods = clazz.getDeclaredMethods();
			for (int i = 0; i < methods.length; i++) 
			{
				Method method = methods[i];
				
				if( !method.getName().startsWith("access$") //$NON-NLS-1$
					&&!Modifier.isAbstract(method.getModifiers())
					&&!Modifier.isNative(method.getModifiers())
					&& method.getDeclaringClass().equals(clazz) )
				{
					classNode.appendChild( createMethodNode(doc, method) );
				}
			}
			
			doc.getDocumentElement().appendChild(classNode);
		}
	}
	
	/** 
	 * Erstellt die angegebene Methode ein XML-Element 
	 * und fgt es <code>doc</code> hinzu. Das Element enthlt
	 * die Informationen zum Namen, den Modifiern, dem Rckgabewert
	 * und den Parametern der Methode. */
	private Element createMethodNode(Document doc, Method method) 
	{
		Element methodNode = doc.createElement("method"); //$NON-NLS-1$
		methodNode.setAttribute("runs", Integer.toString(0) ); //$NON-NLS-1$
		
		// MODIFIERS
		Element modifiersNode = doc.createElement("modifiers"); //$NON-NLS-1$
		int modifiers = method.getModifiers();
		if( Modifier.isPublic(modifiers) )
		{
			Element modifierNode = doc.createElement("modifier"); //$NON-NLS-1$
			modifierNode.setAttribute("type", "public"); //$NON-NLS-1$ //$NON-NLS-2$
			modifiersNode.appendChild(modifierNode);
		}
		if( Modifier.isProtected(modifiers) )
		{
			Element modifierNode = doc.createElement("modifier"); //$NON-NLS-1$
			modifierNode.setAttribute("type", "protected"); //$NON-NLS-1$ //$NON-NLS-2$
			modifiersNode.appendChild(modifierNode);
		}
		if( Modifier.isPrivate(modifiers) )
		{
			Element modifierNode = doc.createElement("modifier"); //$NON-NLS-1$
			modifierNode.setAttribute("type", "private"); //$NON-NLS-1$ //$NON-NLS-2$
			modifiersNode.appendChild(modifierNode);
		}
		if( Modifier.isFinal(modifiers) )
		{
			Element modifierNode = doc.createElement("modifier"); //$NON-NLS-1$
			modifierNode.setAttribute("type", "final"); //$NON-NLS-1$ //$NON-NLS-2$
			modifiersNode.appendChild(modifierNode);
		}
		if( Modifier.isStatic(modifiers) )
		{
			Element modifierNode = doc.createElement("modifier"); //$NON-NLS-1$
			modifierNode.setAttribute("type", "static"); //$NON-NLS-1$ //$NON-NLS-2$
			modifiersNode.appendChild(modifierNode);
		}
		if( Modifier.isSynchronized(modifiers) )
		{
			Element modifierNode = doc.createElement("modifier"); //$NON-NLS-1$
			modifierNode.setAttribute("type", "synchronized"); //$NON-NLS-1$ //$NON-NLS-2$
			modifiersNode.appendChild(modifierNode);
		}
		
		methodNode.appendChild(modifiersNode);
		
		// RETURNTYPE
		Element returnTypeNode = doc.createElement("returnType"); //$NON-NLS-1$
		returnTypeNode.setAttribute( "type", getTypeName(method.getReturnType()) ); //$NON-NLS-1$
		methodNode.appendChild(returnTypeNode);
		
		// NAME
		Element nameNode = doc.createElement("name"); //$NON-NLS-1$
		nameNode.setAttribute( "value", method.getName() ); //$NON-NLS-1$
		methodNode.appendChild(nameNode);
		
		// PARAMETERS
		Element parametersNode = doc.createElement("parameters"); //$NON-NLS-1$
		Class[] parameters = method.getParameterTypes();
		for (int i = 0; i < parameters.length; i++) 
		{
			Element parameterNode = doc.createElement("param"); //$NON-NLS-1$
			parameterNode.setAttribute( "type", getTypeName(parameters[i]) ); //$NON-NLS-1$
			parametersNode.appendChild(parameterNode);
		}
		
		methodNode.appendChild(parametersNode);
		
		// EXCEPTIONS
		Element exceptionsNode = doc.createElement("exceptions"); //$NON-NLS-1$
		Class[] exceptions = method.getExceptionTypes();
		for (int i = 0; i < exceptions.length; i++) 
		{
			Element exceptionNode = doc.createElement("ex"); //$NON-NLS-1$
			exceptionNode.setAttribute( "type", getTypeName(exceptions[i]) ); //$NON-NLS-1$
			exceptionsNode.appendChild(exceptionNode);
		}
		
		methodNode.appendChild(exceptionsNode);
		
		return methodNode;
	}
	
	/**
	 * Aus der Java API geklaut java.lang.reflect.Field.
	 * @param type Der Datentyp.
	 * @return Der Name des Typs so wie er im Quelltext erscheint.
	 */
	public static String getTypeName(Class type) 
	{
		if (type.isArray()) 
		{
			try {
				Class cl = type;
				int dimensions = 0;
				while (cl.isArray()) 
				{
					dimensions++;
					cl = cl.getComponentType();
				}
				StringBuffer sb = new StringBuffer();
				sb.append(cl.getName());
				for (int i = 0; i < dimensions; i++)
					sb.append("[]"); //$NON-NLS-1$
				
				return sb.toString();
			} catch (Throwable e) { /*FALLTHRU*/ }
		}
		return type.getName();
	} 
	
	/**
	 * Startet den XML-Generator.
	 * @param args Programmargumente
	 * <ul>
	 * <li>args[0] - Der Pfad, der zu erstellenden XML-Datenbasis.</li> 
	 * <li>args[1..n] - Die Namen der Pakete, die analyisiert werden sollen (mssen
	 * nicht vollstndig sein, z.B. &quot;java.l&quot; sucht alle Pakete, die mit &quot;java.l&quot; anfangen).</li>
	 * </ul>
	 */
	public static void main(String[] args) 
	{
		if(args.length < 2)
		{
			showUsage();
			System.exit(RET_ERROR_ARG_NUM);
		}
		
		try 
		{
			String[] classPath = new String[args.length-1];
			System.arraycopy(args, 1, classPath, 0, classPath.length);
			DatabaseGenerator gen = new DatabaseGenerator(classPath);
			gen.createXMLDatabase(args[0]);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(RET_PROCESSING_ERROR);
		}
	}
	
	private static void showUsage() {
		System.out.println(Messages.getString("DatabaseGenerator.0")); //$NON-NLS-1$
		System.out.println(Messages.getString("DatabaseGenerator.45")); //$NON-NLS-1$
		System.out.println(Messages.getString("DatabaseGenerator.46")); //$NON-NLS-1$
		System.out.println(Messages.getString("DatabaseGenerator.47")); //$NON-NLS-1$
	}

}
