package acover;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import atosj.Messages;

/**
 * Berechnet die Statistiken zur berdeckungsanalyse in einer HTML Datei.
 * @author Nicos Tegos
 */
public class HTMLConverter 
{
	/** Programmrckgabewert: Anzahl der Argumente falsch */
	public static final int RET_ERROR_ARG_NUM = 1;
	/** Programmrckgabewert: ungltiger Dateiname oder falsches Dateiformat der XML-Datenbasis */
	public static final int RET_ERROR_INVALID_FILE = 2; // 
	
	private Document doc; // das eingelesene XML Dokument.
	private String jarPath;
	private String xmlPath; // vollstndiger Pfad der XML-Datei, die visualisiert werden soll.
	private Vector packageStatistics = new Vector(); // Vector<PackageCoverageStatistic>
	
	/**
	 * Initialisiert den Visualisierer der berdeckungsanalyse.
	 * @param filePath Vollstndiger Pfad der XML-Datei, mit den
	 * Informationen zur berdeckungsanalyse.
	 * @throws Exception wenn Datei nicht gefunden oder Format nicht
	 * korrekt.
	 */
	public HTMLConverter(String filePath) throws Exception 
	{
		doc = XMLUtilities.readXMLDoc(filePath);
		xmlPath = filePath;
		
		Element element = doc.getDocumentElement();
		jarPath = element.getAttribute("jar"); //$NON-NLS-1$
		
		createStatistics();
	}
	
	private void createStatistics() 
	{
		Element element = doc.getDocumentElement();
				
		NodeList nl = element.getChildNodes();
		for(int i = 0; i < nl.getLength(); ++i)
		{
			Node classNode = nl.item(i);
			if( classNode.getNodeName().equals("class") ) //$NON-NLS-1$
			{
				String className = ((Element)classNode).getAttribute("name"); //$NON-NLS-1$
				ClassCoverageStatistic classStat = new ClassCoverageStatistic(className);
				
				NodeList methodsNl = classNode.getChildNodes();
				for(int j = 0; j < methodsNl.getLength(); ++j)
				{
					Element methodNode = (Element) methodsNl.item(j);
					NodeList methodChildrenNl = methodNode.getChildNodes();
					
					String[] modifiers;
					String returnType;
					String methodName;
					String[] parameters;
					String[] exceptions;
					
					Element modifiersNode = (Element) methodChildrenNl.item(0);
					NodeList modifiersNl = modifiersNode.getChildNodes();
					modifiers = new String[modifiersNl.getLength()];
					for (int k = 0; k < modifiers.length; k++)
						modifiers[k] = ((Element)modifiersNl.item(k)).getAttribute("type"); //$NON-NLS-1$
					
					Element returnTypeNode = (Element) methodChildrenNl.item(1);
					returnType = returnTypeNode.getAttribute("type"); //$NON-NLS-1$
					
					Element methodNameNode = (Element) methodChildrenNl.item(2);
					methodName = methodNameNode.getAttribute("value"); //$NON-NLS-1$
					
					Element parametersNode = (Element) methodChildrenNl.item(3);
					NodeList parametersNl = parametersNode.getChildNodes();
					parameters = new String[parametersNl.getLength()];
					for (int k = 0; k < parameters.length; k++)
						parameters[k] = ((Element)parametersNl.item(k)).getAttribute("type"); //$NON-NLS-1$
					
					Element exceptionsNode = (Element) methodChildrenNl.item(4);
					NodeList exceptionsNl = exceptionsNode.getChildNodes();
					exceptions = new String[exceptionsNl.getLength()];
					for (int k = 0; k < exceptions.length; k++)
						exceptions[k] = ((Element)exceptionsNl.item(k)).getAttribute("type"); //$NON-NLS-1$
					
					MethodCoverageStatistic methodStat = new MethodCoverageStatistic(
							className,
							modifiers,
							returnType,
							methodName,
							parameters,
							exceptions,
							Integer.parseInt( methodNode.getAttribute("runs") ) //$NON-NLS-1$
						);
					
					classStat.addMethodCoverageStatistic(methodStat);
				}
				
				addToPackageStatistics(classStat);
			}
			
		}
	}
	
	private String getPackageName(String className) 
	{
		int lastDotIndex = className.lastIndexOf('.');
		if(lastDotIndex == -1) // default package
			return ""; //$NON-NLS-1$
		
		return className.substring(0, lastDotIndex);
	}
	
	private void addToPackageStatistics(ClassCoverageStatistic classStat) 
	{
		String packageName = getPackageName(classStat.getClassName());
		PackageCoverageStatistic packageStat = null;
		
		for (int i = 0; i < packageStatistics.size(); i++) 
		{
			if( ((PackageCoverageStatistic)packageStatistics.get(i)).getPackageName().equals(packageName) )
			{
				packageStat = (PackageCoverageStatistic) packageStatistics.get(i);
				break;
			}
		}
		
		// nicht gefunden also neu erstellen und hinzufgen
		if(packageStat == null)
		{
			packageStat = new PackageCoverageStatistic(packageName);
			packageStatistics.add(packageStat);
		}
		
		packageStat.addClassCoverageStatistic(classStat);
	}
	
	/**
	 * Erstellt die HTML Datei.
	 * @throws IOException I/O Fehler beim Erstellen/Schreiben der Datei.
	 */
	public void createHTMLOutput() throws IOException 
	{
		int lastDotIndex = xmlPath.lastIndexOf('.');
		String htmlPath;
		if(lastDotIndex == -1)
			htmlPath = xmlPath + ".html"; //$NON-NLS-1$
		else
			htmlPath = xmlPath.substring(0, lastDotIndex) + ".html"; //$NON-NLS-1$
		
		FileWriter fw = new FileWriter( new File(htmlPath) );
		
		StringBuffer out = new StringBuffer(); 
		out.append("<html>"); //$NON-NLS-1$
		out.append(		"<header>"); //$NON-NLS-1$
		out.append(		"<title>Coverage for " + new File(jarPath).getName() + "</title>"); //$NON-NLS-1$ //$NON-NLS-2$
		out.append(		"</header>"); //$NON-NLS-1$
		out.append(		"<body>"); //$NON-NLS-1$
		out.append(			"<center><h2>Coverage Analysis of " + new File(jarPath).getName() + "</h2></center>"); //$NON-NLS-1$ //$NON-NLS-2$
		out.append(			createProjectSummary());
		out.append(		"</body>"); //$NON-NLS-1$
		out.append("</html>"); //$NON-NLS-1$
		
		fw.write( out.toString() );
		fw.close();
	}
	
	private StringBuffer createProjectSummary() 
	{
		StringBuffer ret = new StringBuffer();
		ret.append("<center>"); //$NON-NLS-1$
		ret.append("<table BORDER=\"1\" WIDTH=\"80%\" CELLPADDING=\"3\" CELLSPACING=\"0\">"); //$NON-NLS-1$
		ret.append(		"<tr BGCOLOR=\"#CCCCFF\">"); //$NON-NLS-1$
		ret.append(			"<td COLSPAN=\"4\"><h2>Package Summary</h2></td>"); //$NON-NLS-1$
		ret.append(		"</tr>"); //$NON-NLS-1$
		ret.append(		"<tr>"); //$NON-NLS-1$
		ret.append(			"<th>Package</th>"); //$NON-NLS-1$
		ret.append(			"<th>Percentage Covered</th>"); //$NON-NLS-1$
		ret.append(			"<th>Covered/Total</th>"); //$NON-NLS-1$
		ret.append(		"</tr>"); //$NON-NLS-1$
		
		if(packageStatistics.size() == 0)
		{
			ret.append("<tr><td colspan=\"4\">No packages with relevant (none-Interface) classes found!</td>"); //$NON-NLS-1$
		}
		else
		{
			Collections.sort(packageStatistics, new Comparator() {
				public int compare(Object arg0, Object arg1) {
					return ((PackageCoverageStatistic)arg0).getPackageName().compareTo( ((PackageCoverageStatistic)arg1).getPackageName() );
				}
			});
			for (int i = 0; i < packageStatistics.size(); i++)
				ret.append( createPackageRow( (PackageCoverageStatistic) packageStatistics.get(i) ) );
		}
		
		ret.append("</table>"); //$NON-NLS-1$
		ret.append("</center>"); //$NON-NLS-1$
		
		if(packageStatistics.size() > 0)
		{
			for (int i = 0; i < packageStatistics.size(); i++)
				ret.append( createPackageSummary( (PackageCoverageStatistic) packageStatistics.get(i) ) );
		}
		
		return ret;
	}
	
	private StringBuffer createPackageRow(PackageCoverageStatistic packageStat) 
	{
		int percentage =  (int) (packageStat.getPercentageCoveredClasses() * 100);
		StringBuffer ret = new StringBuffer();
		
		ret.append("<tr>"); //$NON-NLS-1$
		ret.append(		"<td><a href=\"#" + packageStat.getPackageName() + "\">" + packageStat.getPackageName() + "</a></td>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		ret.append(		"<td>"); //$NON-NLS-1$
		ret.append(			"<table border=\"0\" CELLPADDING=\"0\" CELLSPACING=\"0\">"); //$NON-NLS-1$
		ret.append(				"<tr>"); //$NON-NLS-1$
		ret.append(					"<td height=\"100%\">"); //$NON-NLS-1$
		ret.append(						"<table width=\"200\" height=\"100%\" border=\"0\" CELLPADDING=\"0\" CELLSPACING=\"0\">"); //$NON-NLS-1$
		ret.append(							"<tr>"); //$NON-NLS-1$
		ret.append(								"<td width=\"" + percentage + "%\" bgcolor=\"#00ff00\"></td>"); //$NON-NLS-1$ //$NON-NLS-2$
		ret.append(								"<td width=\"" + (100 - percentage) +  "%\" bgcolor=\"#ff0000\"></td>"); //$NON-NLS-1$ //$NON-NLS-2$
		ret.append(							"</tr>"); //$NON-NLS-1$
		ret.append(						"</table>"); //$NON-NLS-1$
		ret.append(					"</td>"); //$NON-NLS-1$
		ret.append(					"<td>" + percentage + "%</td>"); //$NON-NLS-1$ //$NON-NLS-2$
		ret.append(				"</tr>"); //$NON-NLS-1$
		ret.append(			"</table>"); //$NON-NLS-1$
		ret.append(		"</td>"); //$NON-NLS-1$
		ret.append(		"<td align=\"center\">" + packageStat.getCoveredClasses() + "/" + packageStat.getTotalClasses() + "</td>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		ret.append("</tr>"); //$NON-NLS-1$
		
		return ret;
	}
	
	private StringBuffer createPackageSummary(PackageCoverageStatistic packStat) 
	{
		StringBuffer ret = new StringBuffer();
		ret.append("<br>"); //$NON-NLS-1$
		ret.append("<center>"); //$NON-NLS-1$
		ret.append("<table BORDER=\"1\" WIDTH=\"80%\" CELLPADDING=\"3\" CELLSPACING=\"0\">"); //$NON-NLS-1$
		ret.append(		"<tr BGCOLOR=\"#CCCCFF\">");  //$NON-NLS-1$
		ret.append(		"<td COLSPAN=\"4\"><h2>Class summary for package <a name=\"" + packStat.getPackageName() + "\">" + packStat.getPackageName() + "</a></h2></td>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		ret.append(		"</tr>"); //$NON-NLS-1$
		ret.append(		"<tr>"); //$NON-NLS-1$
		ret.append(		"<th>Class</th>"); //$NON-NLS-1$
		ret.append(		"<th>Percentage Covered</th>"); //$NON-NLS-1$
		ret.append(		"<th>Covered/Total</th>"); //$NON-NLS-1$
		ret.append(		"</tr>"); //$NON-NLS-1$
			
			Vector classStats = packStat.getClassStatistics();
			Collections.sort(classStats, new Comparator() {
				public int compare(Object arg0, Object arg1) {
					return ((ClassCoverageStatistic)arg0).getClassName().compareTo( ((ClassCoverageStatistic)arg1).getClassName() );
				}
			});
			
			if(classStats.size() == 0)
				ret.append( "<tr><td colspan=\"4\">No relavant (none-Interface) classes found! This package is regarded as fully covered.</td></tr>"); //$NON-NLS-1$
			
			for (int i = 0; i < classStats.size(); i++)
				ret.append( createClassRow( (ClassCoverageStatistic) classStats.get(i) ) );
			
			ret.append("</table>"); //$NON-NLS-1$
			ret.append("</center>"); //$NON-NLS-1$
			
			if(classStats.size() > 0)
			{
				for (int i = 0; i < classStats.size(); i++)
					ret.append( createClassSummary( (ClassCoverageStatistic) classStats.get(i) ) );
			}
			
		return ret;
	}
	
	private StringBuffer createClassRow(ClassCoverageStatistic classStat) 
	{
		int percentage =  (int) (classStat.getPercentageCovered() * 100);
		
		StringBuffer ret = new StringBuffer(); 		
		
		ret.append("<tr>"); //$NON-NLS-1$
		ret.append("<td><a href=\"#" + classStat.getClassName() + "\">" + classStat.getClassName() + "</a></td>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		ret.append("<td>"); //$NON-NLS-1$
		ret.append(	"<table border=\"0\" CELLPADDING=\"0\" CELLSPACING=\"0\">"); //$NON-NLS-1$
		ret.append(	"<tr>"); //$NON-NLS-1$
		ret.append(		"<td height=\"100%\">"); //$NON-NLS-1$
		ret.append(			"<table width=\"200\" height=\"100%\" border=\"0\" CELLPADDING=\"0\" CELLSPACING=\"0\">"); //$NON-NLS-1$
		ret.append(			"<tr>"); //$NON-NLS-1$
		ret.append(			"<td width=\"" + percentage + "%\" bgcolor=\"#00ff00\"></td>"); //$NON-NLS-1$ //$NON-NLS-2$
		ret.append(			"<td width=\"" + (100 - percentage) +  "%\" bgcolor=\"#ff0000\"></td>"); //$NON-NLS-1$ //$NON-NLS-2$
		ret.append(			"</tr>"); //$NON-NLS-1$
		ret.append(			"</table>"); //$NON-NLS-1$
		ret.append(		"</td>"); //$NON-NLS-1$
		ret.append(		"<td>" + percentage + "%</td>"); //$NON-NLS-1$ //$NON-NLS-2$
		ret.append(	"</tr>"); //$NON-NLS-1$
		ret.append(	"</table>"); //$NON-NLS-1$
		ret.append("</td>"); //$NON-NLS-1$
		ret.append("<td align=\"center\">" + classStat.getCovered() + "/" + classStat.getTotal() + "</td>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		ret.append("</tr>"); //$NON-NLS-1$
		
		return ret;
	}
	
	private StringBuffer createClassSummary(ClassCoverageStatistic classStat) 
	{
		StringBuffer ret = new StringBuffer();
		
		ret.append("<br>"); //$NON-NLS-1$
		ret.append("<center>"); //$NON-NLS-1$
		ret.append("<table BORDER=\"1\" WIDTH=\"80%\" CELLPADDING=\"3\" CELLSPACING=\"0\">"); //$NON-NLS-1$
		ret.append(	"<tr BGCOLOR=\"#CCCCFF\">");  //$NON-NLS-1$
		ret.append(		"<td COLSPAN=\"2\"><h2>Method summary for class <a name=\"" + classStat.getClassName() + "\">" + classStat.getClassName() + "</a></h2></td>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		ret.append(	"</tr>"); //$NON-NLS-1$
		ret.append(	"<tr>"); //$NON-NLS-1$
		ret.append(		"<th>Mehod</th>"); //$NON-NLS-1$
		ret.append(		"<th>Runs</th>"); //$NON-NLS-1$
		ret.append(	"</tr>"); //$NON-NLS-1$
			
		Vector methodStats = classStat.getMethodStatistics();
		Collections.sort(methodStats, new Comparator() {
			
			public int compare(Object arg0, Object arg1) {
				MethodCoverageStatistic stat0 = (MethodCoverageStatistic)arg0;
				MethodCoverageStatistic stat1 = (MethodCoverageStatistic)arg1;
				
				int ret = stat0.getMethodName().compareTo( stat1.getMethodName() );
				if( ret == 0 )
					ret = new Integer(stat0.getParameters().length).compareTo( new Integer(stat1.getParameters().length) );
				
				return ret;
			}
			
		});
		
		if(methodStats.size() == 0)
			ret.append("<tr><td colspan=\"2\">No executeable methods found! This class is regarded as fully covered.</td>"); //$NON-NLS-1$
		
		for (int i = 0; i < methodStats.size(); i++)
			ret.append( createMethodRow( (MethodCoverageStatistic) methodStats.get(i) ) );
		
		ret.append("</table>"); //$NON-NLS-1$
		ret.append("</center>"); //$NON-NLS-1$
		
		return ret;
	}
	
	private StringBuffer createMethodRow(MethodCoverageStatistic methodStat) 
	{
		StringBuffer ret = new StringBuffer();
		ret.append("<tr>"); //$NON-NLS-1$
		ret.append("<td width=\"90%\">"); //$NON-NLS-1$
		
		String[] modifiers = methodStat.getModifiers();
		for(int i = 0; i < modifiers.length; ++i)
		{
			ret.append(modifiers[i]);
			if(i < modifiers.length -1)
				ret.append(' ');
		}
		
		ret.append(" "); //$NON-NLS-1$
		ret.append( methodStat.getReturnType() );
		ret.append( " <b>" + methodStat.getMethodName() + "</b>" ); //$NON-NLS-1$ //$NON-NLS-2$
		
		ret.append("("); //$NON-NLS-1$
		String[] parameters = methodStat.getParameters();
		for(int i = 0; i < parameters.length; ++i)
		{
			ret.append(parameters[i]);
			if(i < parameters.length -1)
				ret.append(", "); //$NON-NLS-1$
		}
		ret.append(")"); //$NON-NLS-1$
		
		String[] exceptions = methodStat.getExceptions();
		if( exceptions.length > 0 )
			ret.append(" throws "); //$NON-NLS-1$
		for(int i = 0; i < exceptions.length; ++i)
		{
			ret.append(exceptions[i]);
			if(i < exceptions.length -1)
				ret.append(", "); //$NON-NLS-1$
		}
		
		ret.append("</td>"); //$NON-NLS-1$
		ret.append("<td width=\"10%\" bgcolor=\"" + (methodStat.covered() ? "#00ff00" : "ff0000") + "\">" + methodStat.getRuns() + "</td>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		ret.append("</tr>"); //$NON-NLS-1$
		
		return ret;
	}
	
	/**
	 * Generiert eine HTML-Datei, die die Ergebnisse aus der XML-Datenbank
	 * visualisiert. Die HTML-Datei wird im Verzeichnis der XML-Datenbank erstellt
	 * und trgt denselben Namen, endet aber auf &quot;.html&quot;.
	 * @param args args[0] der vollstndige Pfad fr die XML-Datenbank.
	 */
	public static void main(String[] args) 
	{
		if(args.length != 1)
		{
			showUsage();
			System.exit(RET_ERROR_ARG_NUM);
		}
		
		try 
		{
			HTMLConverter converter  = new HTMLConverter(args[0]);
			converter.createHTMLOutput();
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(Messages.getString("HTMLConverter.135") + args[0] + Messages.getString("HTMLConverter.136")); //$NON-NLS-1$ //$NON-NLS-2$
			System.exit(RET_ERROR_INVALID_FILE);
		}
	}

	private static void showUsage() {
		System.out.println(Messages.getString("HTMLConverter.137")); //$NON-NLS-1$
		System.out.println(Messages.getString("HTMLConverter.138")); //$NON-NLS-1$
		System.out.println(Messages.getString("HTMLConverter.139")); //$NON-NLS-1$
	}
}
