package acover;
import java.util.Vector;

/**
 * Statistik zur Berechnung der Mazahl der Klassenberdeckung.
 * @author Nicos Tegos
 */
public class PackageCoverageStatistic
{
	private Vector classStatistics = new Vector(); // Vector<ClassCoverageStatistic> Statistiken zu allen Klassen des Pakets
	private String packageName;
	
	private int coveredClasses = 0;
	private int uncoveredClasses = 0;
	
	private int coveredMethods = 0;
	private int uncoveredMethods = 0;
	
	/**
	 * Erstellt eine neue Statistik zur Klassenberdeckung.
	 * @param packageName Der vollqualifizierte Java-Name des Pakets.
	 */
	public PackageCoverageStatistic(String packageName) {
		this.packageName = packageName;
	}
	
	/**
	 * Fgt die Statistik zu einer einzelnen Klasse des Pakets hinzu.
	 * @param stat Die Statistik einer Klasse.
	 */
	public void addClassCoverageStatistic(ClassCoverageStatistic stat) {
		if( stat.covered() )
			++coveredClasses;
		else
			++uncoveredClasses;
		
		coveredMethods += stat.getCovered();
		uncoveredMethods += stat.getUncovered();
		
		classStatistics.add(stat);
	}
	
	/** Liefert den Namen des analysierten Pakets. */
	public String getPackageName() {
		return packageName;
	}
	
	/** Liefert die Statistiken zu allen Klassen des analysierten Pakets. 
	 * @return Vector(ClassCoverageStatistic) */
	public Vector getClassStatistics() {
		return classStatistics;
	}
	
	/**
	 * Gibt an, ob das Paket vollstndig berdeckt ist. D.h. alle Klassen
	 * des Pakets berdeckt wurden.
	 * @return <code>true</code>, wenn berdeckt.
	 */
	public boolean covered() {
		return uncoveredClasses == 0; // pakete ohne klassen gelten als covered
	}
	
	/** Liefert die Anzahl der berdeckten Klassen. */
	public int getCoveredClasses() {
		return coveredClasses;
	}
	
	/** Liefert die Anzahl der nicht berdeckten Klassen. */
	public int getUncoveredClasses() {
		return uncoveredClasses;
	}
	
	/** Liefert die Anzahl aller analysierten Klassen des Pakets. */
	public int getTotalClasses() {
		return uncoveredClasses + coveredClasses;
	}
	
	/** Liefert den Anteil berdeckter Klassen. */
	public float getPercentageCoveredClasses() {
		if( getTotalClasses() == 0 )
			return 1.0f;
		
		return (float)coveredClasses / getTotalClasses();
	}
	
	/** Liefert die Anzahl der berdeckten/ausgefhrten Methoden. */
	public int getCoveredMethods() {
		return coveredMethods;
	}
	
	/** Liefert die Anzahl der nicht berdeckten/ausgefhrten Methoden. */
	public int getUncoveredMethods() {
		return uncoveredMethods;
	}
	
	/** Liefert die Anzahl aller analysierten Methoden. */
	public int getTotalMethods() {
		return uncoveredMethods + coveredMethods;
	}
	
	/** Liefert den Anteil berdeckter/ausgefhrter Methoden. */
	public float getPercentageCoveredMethods() {
		if( getTotalMethods() == 0 )
			return 1.0f;
		
		return (float)coveredMethods / getTotalMethods();
	}
}
