package acover;

import java.io.IOException;

import javassist.ClassPool;
import javassist.Loader;


import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import atosj.Messages;

/**
 * Aktualisiert die XML-Datenbank zur berdeckungsanalyse zur Laufzeit
 * des Testobjekts. D.h. erhht die Zhler der Methodenaufrufe. 
 * @author Nicos Tegos
 */
public class RuntimeCoverageGenerator 
{
	/** Programmrckgabewert: Anzahl der Argumente falsch */
	private static final int RET_ERROR_ARG_NUM = 1;
	/** Programmrckgabewert: ungltiger Dateiname oder falsches Dateiformat der XML-Date.*/
	private static final int RET_ERROR_INVALID_FILE = 2;
	/** Programmrckgabewert: main-Klasse nicht gefunden */
	private static final int RET_ERROR_MAIN_CLASS = 3;
	/** Programmrckgabewert: Interner Fehler */
	private static final int RET_ERROR_INTERNAL = 4;
	
	private static RuntimeCoverageGenerator instance;
	
	private String xmlPath;
	private Document doc; // das XML Dokument, welches alle Funktionen mit ihren bisherigen durchlufen enthlt
	private boolean modified = false;
	
	/**
	 * Liest die XML-Datenbasis ein. Initialisiert einen ShutdownHook, der
	 * die Datenbasis beim Beenden des Prozesses abspeichert.
	 * @param xmlPath Pfad der XML-Datenbasis, die aktualisiert werden soll.
	 */
	public RuntimeCoverageGenerator(String xmlPath) throws Exception 
	{
		this.xmlPath = xmlPath;
		doc = XMLUtilities.readXMLDoc(xmlPath);
		
		// beim beenden alle nderungen in die Datenbank schreiben
		Runtime.getRuntime().addShutdownHook( new Thread() {
			public void run() {
				write();
			}
		});
		
		instance = this;
	}
	
	/**
	 * Gibt an, ob die Klasse in der XML-Datenbasis enthalten ist. D.h.
	 * die Klasse wird einer berdeckungsanalyse unterzogen.
	 * @param className Name der Klasse.
	 * @return <code>true</code>, wenn die Klasse analysiert werden soll.
	 */
	public boolean contains(String className) 
	{
		NodeList nl = doc.getDocumentElement().getChildNodes();
		for(int i = 0; i < nl.getLength(); ++i)
		{
			Node classNode = nl.item(i);
			if( classNode.getNodeName().equals("class") ) //$NON-NLS-1$
			{
				if( className.equals( ((Element)classNode).getAttribute("name") ) ) //$NON-NLS-1$
					return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Gibt an, ob die Methode in der XML-Datenbasis enthalten ist. D.h.
	 * die Methode wird einer berdeckungsanalyse unterzogen.
	 * @param className Name der Klasse der Methode.
	 * @param methodName Name der Methode
	 * @param parameters Klassennamen aller Parameter der Methode in der 
	 * Reihenfolge ihrer Definition, wenn keine dann leeres Array.
	 * @return <code>true</code>, wenn die Klasse analysiert werden soll.
	 */
	public boolean contains(String className, String methodName, String[] parameters) {
		return getMethodNode(className, methodName, parameters) != null;
	}
	
	private Element getMethodNode(String className, String methodName, String[] parameters) 
	{
		NodeList nl = doc.getDocumentElement().getChildNodes();
		for(int i = 0; i < nl.getLength(); ++i)
		{
			Node classNode = nl.item(i);
			if( classNode.getNodeName().equals("class") ) //$NON-NLS-1$
			{
				if( className.equals( ((Element)classNode).getAttribute("name") ) ) //$NON-NLS-1$
				{
					NodeList methodsNl = classNode.getChildNodes();
					for(int j = 0; j < methodsNl.getLength(); ++j)
					{
						Element methodNode = (Element) methodsNl.item(j);
						
						Element methodNameNode = (Element)methodNode.getChildNodes().item(2);
						if( methodName.equals( methodNameNode.getAttribute("value") ) ) //$NON-NLS-1$
						{
							NodeList parameterNl = methodNode.getChildNodes().item(3).getChildNodes();
							if( parameterNl.getLength() == parameters.length )
							{
								boolean valid = true;
								for (int k = 0; k < parameters.length; k++) 
								{
									Element parameterNode = (Element) parameterNl.item(k);
									if( !parameters[k].equals( parameterNode.getAttribute("type") ) ) //$NON-NLS-1$
									{
										valid = false;
										break;
									}
								}
								
								if(valid)
									return methodNode;
							}
						}
					}
				}
			}
		}
		
		return null;
	}
	
	/**
	 * Aktualisiert die XML Datenbank und vermerkt einen zustzlichen Aufruf der angegeben Methode.
	 * Die Datenbasis wird noch nicht abgespeichert, sondern erst beim Beenden des Prozesses.
	 * @param className Der Name der Klasse der Methode.
	 * @param methodName Name der Methode.
	 * @param parameters Klassennamen aller Parameter der Methode in der 
	 * Reihenfolge ihrer Definition, wenn keine dann leeres Array.
	 */
	public synchronized void addRun(String className, String methodName, String[] parameters) 
	{
		Element methodNode = getMethodNode(className, methodName, parameters);
		if(methodNode != null)
		{
			modified = true;
			int runs = Integer.parseInt( methodNode.getAttribute("runs") ); //$NON-NLS-1$
			methodNode.setAttribute( "runs", Integer.toString(runs+1) ); //$NON-NLS-1$
		}
	}
	
	/**
	 * Alle nderungen wieder in die XML-Datei schreiben.
	 */
	public void write() 
	{
		if(modified)
			try {
				XMLUtilities.writeDocToXMLFile(doc, xmlPath);
			} catch (IOException e) {
				System.out.println(Messages.getString("RuntimeCoverageGenerator.8") +  xmlPath); //$NON-NLS-1$
			}
	}
	
	public static RuntimeCoverageGenerator getInstance() {
    	return instance;
    }
	
	/**
	 * Startet das Testobjekt und aktualisiert zur Laufzeit die XML-Datenbank
	 * zur berdeckungsanalyse.
	 * @param args Programmargumente
	 * <ul>
	 * <li>args[0] - Name der main-Klasse, die ausgefhrt werden soll.</li>
	 * <li>args[1] - Vollstndiger Pfad der XML-Datenbasis fr die berdeckungsanalyse.</li>
	 * </ul>
	 */
	public static void main(String[] args) 
	{
		if(args.length != 2)
		{
			showUsage();
			System.exit(RET_ERROR_ARG_NUM);
		}
		
		try 
		{
			new RuntimeCoverageGenerator(args[1]);
			
		} catch (Exception e) {
			System.out.println(Messages.getString("RuntimeCoverageGenerator.9") + args[1] + Messages.getString("RuntimeCoverageGenerator.10")); //$NON-NLS-1$ //$NON-NLS-2$
			System.exit(RET_ERROR_INVALID_FILE);
		}
		
		Loader loader = new Loader();
		loader.delegateLoadingOf( RuntimeCoverageGenerator.class.getName() );
		
		try {
			loader.addTranslator(ClassPool.getDefault(), new RuntimeCoverageTranslator());
		} catch (Exception e) {
			System.out.println(Messages.getString("RuntimeCoverageGenerator.11") + e.getMessage() ); //$NON-NLS-1$
			System.exit(RET_ERROR_INTERNAL);
		}
		
		String[] realArgs = new String[args.length-2];
		System.arraycopy(args, 2, realArgs, 0, realArgs.length);
		
		try {
			loader.run(args[0], realArgs);
		} catch (Throwable e) {
			e.printStackTrace();
			System.out.println(Messages.getString("RuntimeCoverageGenerator.12") + args[0] + Messages.getString("RuntimeCoverageGenerator.13")); //$NON-NLS-1$ //$NON-NLS-2$
			System.exit(RET_ERROR_MAIN_CLASS);
		}
	}
	
	private static void showUsage() {
		System.out.println(Messages.getString("RuntimeCoverageGenerator.14")); //$NON-NLS-1$
		System.out.println(Messages.getString("RuntimeCoverageGenerator.15")); //$NON-NLS-1$
		System.out.println(Messages.getString("RuntimeCoverageGenerator.16")); //$NON-NLS-1$
		System.out.println(Messages.getString("RuntimeCoverageGenerator.17")); //$NON-NLS-1$
	}

}
