/*
 * Created on 17.11.2005
 */
package atosj.app;

import java.io.File;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Vector;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.editors.Changeable;
import atosj.exception.AtosJException;
import atosj.listener.ProjectListener;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;
import atosj.model.urf.URFFile;
import atosj.util.CSVLine;
import atosj.util.CSVParser;

/**
 * Ein ATOSj Projekt verwaltet alle Ressourcen, wie Testsequenzen, Testpakete und die URF-Datei.
 * @author Nicos Tegos
 */
public class Project 
{
	public static final boolean AWT = true;
	public static final boolean SWT = false;
	private String apfPath;
	private TestObject testee;
	private URFFile urf;
	private Vector scripts; // Vector<HTSScript>, alle skripte aus dem SEQ Verzeichnis
	private Vector packages; // Vector<HTSSequence>, alle testpakete aus dem SEQ Verzeichnis
	private Vector openedResources = new Vector(); // Vector<String> die beim Beenden des Projekts gerade bearbeiteten Resourcen.
	private String activeResource; // die die beim Beenden des Projekts gerade aktive Resource (angewhlter Editor).
		
	private Vector listeners = new Vector();
	
	/**
	 * ffnet ein Projekt aus einer Projektdatei.
	 * @param apfPath Vollstndiger Pfad der Projektdatei (Endung apf).
	 * @throws AtosJException Bei Fehlern in der Projekt-Datei.
	 */
	public Project(String apfPath) throws AtosJException 
	{
		this.apfPath = apfPath;
		
		load();
	}
	
	/**
	 * Erstellt ein neues Projekt mit den angegebenen Parametern. Legt dazu eine Projektdatei
	 * im Verzeichnis mit dem Pfad projectDir an. Legt alle Unterverzeichnisse des Projektes an.
	 * @param projectName Der Name des Projektes, muss ein gltiger Dateiname sein.
	 * @param projectDir Das Projektverzeichnis. Der Name des letzten Unterverzeichnisses entspricht
	 * dem Projektnamen.
	 * @param mainClass Vollstndiger Name Main-Klasse des Testobjektes.
	 * @param mainClassDir Ausfhrungsverzeichnis des Testobjektes.
	 * @param mainWindow Name des Hauptfensters des Testobjektes.
	 * @param classPath Vector(String) Der Klassenpfad fr das Testobjekt.
	 * @param projectType Gltig sind die Konstanten <code>AWT</code>, fr Swing Projekte
	 * und <code>SWT</code> fr SWT Projekte.
	 * @throws AtosJException wenn das Erstellen fehlschlgt.
	 */
	public Project(String projectName, 
			String projectDir, 
			String mainClass, 
			String mainClassDir,
			String mainWindow,
			Vector classPath,
			boolean projectType) throws AtosJException
	{
	
		// mainClassDir = stripLastSeperator(mainClassDir); das geht glaube ich bei unix schief
		testee = new TestObject
		(		  projectName,
			      mainClassDir,
				  mainClass,
				  mainWindow,
				  projectDir,
				  classPath,
				  projectType,
				  false,
				  new Vector()
		);
		
		// projektverzeichnis erstellen
		File f = new File(projectDir);
		if( f.exists() )
		{
			throw new AtosJException(Messages.getString("Project.0") //$NON-NLS-1$
					+ "\"" + projectDir + Messages.getString("Project.2")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		if( !f.mkdir() )
		{
			throw new AtosJException(Messages.getString("Project.3") //$NON-NLS-1$
					+ "\"" + projectDir + Messages.getString("Project.5"));			 //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		// Projektdatei erzeugen
		apfPath = projectDir;
		if( !apfPath.endsWith(File.separator) )
			apfPath += File.separator;
		apfPath += projectName + ".apf"; //$NON-NLS-1$
		
		f = new File(apfPath);
		try{
			f.createNewFile();
		}
		catch(IOException e){
			throw new AtosJException(Messages.getString("Project.7") //$NON-NLS-1$
					+ "\"" + apfPath + Messages.getString("Project.9")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		File subDir = new File(projectDir + File.separator + "COV"); //$NON-NLS-1$
		subDir.mkdir();
		
		subDir = new File(projectDir + File.separator + "EXT"); //$NON-NLS-1$
		subDir.mkdir();
		
		subDir = new File(projectDir + File.separator + "BIN"); //$NON-NLS-1$
		subDir.mkdir();
		
		subDir = new File(projectDir + File.separator + "ENV"); //$NON-NLS-1$
		subDir.mkdir();
		
		subDir = new File(projectDir + File.separator + "LOG"); //$NON-NLS-1$
		subDir.mkdir();
		
		subDir = new File(projectDir + File.separator + "SEQ"); //$NON-NLS-1$
		subDir.mkdir();
		
		subDir = new File(projectDir + File.separator + "REF"); //$NON-NLS-1$
		subDir.mkdir();
		
		subDir = new File(projectDir + File.separator + "URF"); //$NON-NLS-1$
		subDir.mkdir();
		
		// URF Datei erzeugen
		String urfPath = testee.getProjectDir() + File.separator + "URF" + File.separator + testee.getProjectName() + ".urf"; //$NON-NLS-1$ //$NON-NLS-2$
		f = new File(urfPath);
		try{
			f.createNewFile();
		}
		catch(IOException e){
			throw new AtosJException(Messages.getString("Project.20") //$NON-NLS-1$
					+ "\"" + urfPath + Messages.getString("Project.22")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		save();
		init();
	}
	
	/**
	 * Speichert die Projektdatei und legt vorher eine Backup-Datei an.
	 * Der Name der Backup Datei ist: Projektverzeichnis +  <code>File.separator</code> + Projektname + .bak
	 * @throws AtosJException Bei IO Fehler.
	 */
	public void save() throws AtosJException
	{
		// backup Datei anlegen
		String backupPath = testee.getProjectDir() + File.separator + testee.getProjectName() + ".bak"; //$NON-NLS-1$
		copyFile(apfPath, backupPath);
		
		String saveMe = ""; //$NON-NLS-1$
		Vector elements = new Vector(); // Vector<String>
		
		elements.add(Messages.getString("Project.25"));  //$NON-NLS-1$
		elements.add("\"" + testee.getProjectName() + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
		elements.removeAllElements();

		elements.add(Messages.getString("Project.29"));  //$NON-NLS-1$
		elements.add("\"" + ( testee.isAWTProject() ? "AWT" : "SWT" ) + "\""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
		elements.removeAllElements();

		elements.add(Messages.getString("Project.35"));  //$NON-NLS-1$
		elements.add("\"" + testee.getMainClassDir() + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
		elements.removeAllElements();
		
		elements.add(Messages.getString("Project.39"));  //$NON-NLS-1$
		elements.add("\"" + testee.getMainClass() + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
		elements.removeAllElements();
		
		elements.add(Messages.getString("Project.43"));  //$NON-NLS-1$
		elements.add("\"" + testee.getMainWindow() + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
		elements.removeAllElements();
		
		elements.add(Messages.getString("Project.47")); //$NON-NLS-1$
		Vector cp = testee.getClassPathList();
		for(int i = 0; i < cp.size(); ++i)
			elements.add("\"" + cp.get(i) + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
		elements.removeAllElements();
		
		elements.add(Messages.getString("Project.51")); //$NON-NLS-1$
		Vector coverPath = testee.getCoveragePath();
		for(int i = 0; i < coverPath.size(); ++i)
			elements.add("\"" + coverPath.get(i) + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
		elements.removeAllElements();
		
		elements.add(Messages.getString("Project.55")); //$NON-NLS-1$
		elements.add( testee.isCoverage() ? Messages.getString("Project.56") : Messages.getString("Project.57") ); //$NON-NLS-1$ //$NON-NLS-2$
		saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
		elements.removeAllElements();
		
		if( !openedResources.isEmpty() )
		{
			elements.add(Messages.getString("Project.59")); //$NON-NLS-1$
			for(int i = 0; i < openedResources.size(); ++i)
				elements.add( "\"" + openedResources.get(i) + "\"" ); //$NON-NLS-1$ //$NON-NLS-2$
			
			saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
			elements.removeAllElements();
		}
		
		if( activeResource != null )
		{
			elements.add(Messages.getString("Project.63")); //$NON-NLS-1$
			elements.add( "\"" + activeResource + "\"" ); //$NON-NLS-1$ //$NON-NLS-2$
			
			saveMe += new CSVLine(elements).toString() + "\n"; //$NON-NLS-1$
			elements.removeAllElements();
		}

		// in datei schreiben
		try 
		{
			FileWriter writer = new FileWriter(apfPath);
			writer.write(saveMe);
			writer.close();
		} catch (IOException e) {
			throw new AtosJException(Messages.getString("Project.67")); //$NON-NLS-1$
		}
	}
	
	/**
	 * Ldt die Projektdatei.
	 * @throws AtosJException bei einem Parse-Fehler oder wenn Projektparameter nicht gesetzt wurden,
	 * oder wenn die Projektdatei nicht gefunden wurde.
	 */
	private void load() throws AtosJException
	{
		CSVParser parser = null;
		try 
		{
			parser = new CSVParser(apfPath);
		} catch (IOException e) {
			throw new AtosJException(Messages.getString("Project.68") +  apfPath +  //$NON-NLS-1$
					Messages.getString("Project.69")); //$NON-NLS-1$
		}
		
		String projectName = ""; //$NON-NLS-1$
		boolean projectType = AWT;
		String mainClass = ""; //$NON-NLS-1$
		String mainClassDir = ""; //$NON-NLS-1$
		Vector classPath = new Vector();
		Vector coveragePath = new Vector();
		boolean coverage = false;
		String mainWindow = ""; //$NON-NLS-1$
		
		CSVLine line;
		while( (line = parser.getNextLine()) != null )
		{
			String firstElement = line.getElement(0);
			if( firstElement.equals(Messages.getString("Project.74")) ) //$NON-NLS-1$
			{
				projectName = line.getElementUnquoted(1);
			}
			else
			if( firstElement.equals(Messages.getString("Project.75")) ) //$NON-NLS-1$
			{
				String projectTypeStr = line.getElementUnquoted(1);
				if (projectTypeStr.equals("AWT")) //$NON-NLS-1$
					projectType = AWT;
				else if (projectTypeStr.equals("SWT")) //$NON-NLS-1$
					projectType = SWT;
				else
					throw new AtosJException(Messages.getString("Project.78")+projectType+"\""); //$NON-NLS-1$ //$NON-NLS-2$
			}
			else
			if( firstElement.equals(Messages.getString("Project.80")) ) //$NON-NLS-1$
			{
				mainClassDir = line.getElementUnquoted(1);
			}
			else
			if( firstElement.equals(Messages.getString("Project.81")) ) //$NON-NLS-1$
			{
				mainClass = line.getElementUnquoted(1);
			}
			else
			if( firstElement.equals(Messages.getString("Project.82")) ) //$NON-NLS-1$
			{
				mainWindow = line.getElementUnquoted(1);
			}
			else
			if( firstElement.equals(Messages.getString("Project.83")) ) //$NON-NLS-1$
			{
				for(int i = 1; i < line.getNumOfElements(); ++i)
					classPath.add( line.getElementUnquoted(i) );
			}
			else
			if( firstElement.equals(Messages.getString("Project.84")) ) //$NON-NLS-1$
			{
				for(int i = 1; i < line.getNumOfElements(); ++i)
					openedResources.add( line.getElementUnquoted(i) );
			}
			else
			if( firstElement.equals(Messages.getString("Project.85")) ) //$NON-NLS-1$
			{
				activeResource = line.getElementUnquoted(1);
			}
			else
			if( firstElement.equals(Messages.getString("Project.86")) ) //$NON-NLS-1$
			{
				coverage = line.getElement(1).equals(Messages.getString("Project.87")); //$NON-NLS-1$
			}
			else
			if( firstElement.equals(Messages.getString("Project.88")) ) //$NON-NLS-1$
			{
				for(int i = 1; i < line.getNumOfElements(); ++i)
					coveragePath.add( line.getElementUnquoted(i) );
			}
		}
		
		if( projectName.equals("") ) //$NON-NLS-1$
		{
			throw new AtosJException(Messages.getString("Project.90") + apfPath + "\"" //$NON-NLS-1$ //$NON-NLS-2$
					+ Messages.getString("Project.92")); //$NON-NLS-1$
		}
		if( mainClass.equals("") ) //$NON-NLS-1$
		{
			throw new AtosJException(Messages.getString("Project.94") + apfPath + "\"" //$NON-NLS-1$ //$NON-NLS-2$
					+ Messages.getString("Project.96")); //$NON-NLS-1$
		}
		if( mainClassDir.equals("") ) //$NON-NLS-1$
		{
			throw new AtosJException(Messages.getString("Project.98") + apfPath + "\"" //$NON-NLS-1$ //$NON-NLS-2$
					+ Messages.getString("Project.100")); //$NON-NLS-1$
		}
		if( classPath.isEmpty() )
		{
			throw new AtosJException(Messages.getString("Project.101") + apfPath + "\"" //$NON-NLS-1$ //$NON-NLS-2$
					+ Messages.getString("Project.103")); //$NON-NLS-1$
		}
		if( mainWindow.equals("") ) //$NON-NLS-1$
		{
			throw new AtosJException(Messages.getString("Project.105") + apfPath + "\"" //$NON-NLS-1$ //$NON-NLS-2$
				+ Messages.getString("Project.107")); //$NON-NLS-1$
		}
		
		testee = new TestObject
		(		 projectName,
			      mainClassDir,
				  mainClass,
				  mainWindow,
				  new File(apfPath).getParent(),
				  classPath,
				  projectType,
				  coverage,
				  coveragePath
		);
		
		init();
	}

	/**
	 * Initialisiert die Testsequenzen, Testpakete und die URF-Datei.
	 * @throws AtosJException Bei nicht geladener URF-Datei.
	 */
	private void init() throws AtosJException 
	{
		scripts = new Vector();
		packages = new Vector();
		
		// urf datei laden, die URF Datei muss als erste geladen werden
		urf = new URFFile( testee.getProjectDir() + File.separator + "URF" + File.separator + testee.getProjectName() + ".urf" ); //$NON-NLS-1$ //$NON-NLS-2$
		
		// scripte laden
		File scriptDir = new File(testee.getProjectDir() + File.separatorChar + "SEQ"); //$NON-NLS-1$
		File[] scriptPaths = scriptDir.listFiles( new FilenameFilter() {
			public boolean accept(File dir, String name) 
			{
				if( dir.getAbsolutePath().equals(testee.getProjectDir() + File.separatorChar + "SEQ") //$NON-NLS-1$
						&& name.endsWith(".hts") ) //$NON-NLS-1$
					return true;
				
				return false;
			}
		});
		
		for(int i = 0; i < scriptPaths.length; ++i)
		{
			try
			{
				HTSScript script = new HTSScript(scriptPaths[i].getAbsolutePath(), this);
				scripts.add(script);
			} catch (AtosJException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
		
		// pakete laden
		File[] packagePaths = scriptDir.listFiles( new FilenameFilter() {
			public boolean accept(File dir, String name) 
			{
				if( dir.getAbsolutePath().equals(testee.getProjectDir() + File.separatorChar + "SEQ") //$NON-NLS-1$
						&& name.endsWith(".pak") ) //$NON-NLS-1$
					return true;
				
				return false;
			}
		});
		
		for(int i = 0; i < packagePaths.length; ++i)
		{
			try 
			{
				HTSPackage pack = new HTSPackage( packagePaths[i].getAbsolutePath(), this );
				packages.add(pack);
			} catch (AtosJException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
	}
	
	/**
	 * Erstellt eine neue Testsequenz und fgt sie dem Projekt hinzu.
	 * @param name der Name des neuen Testsequenz.
	 * @return die neu erstellte Testsequenz.
	 * @throws AtosJException Fehler beim Erstellen.
	 */
	public HTSScript createNewScript(String name) throws AtosJException 
	{
		File f = new File(testee.getProjectDir() + File.separator + "SEQ" + File.separator + name + ".hts"); //$NON-NLS-1$ //$NON-NLS-2$
		if( f.exists() )
			throw new AtosJException(Messages.getString("Project.117") + name + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		
		try {
			f.createNewFile();
		}
		catch(IOException e) {
			throw new AtosJException(Messages.getString("Project.119") + f.getAbsolutePath() + Messages.getString("Project.120") + //$NON-NLS-1$ //$NON-NLS-2$
					Messages.getString("Project.121")); //$NON-NLS-1$
		}
		
		HTSScript newScript = new HTSScript( f.getAbsolutePath(), this );
		scripts.add(newScript);
		fireResourceEvent(newScript, true);
		
		return newScript;
	}
	
	/**
	 * Erstellt eine Kopie der angegebenen Testsequenz. Dazu wird die Datei der Originaltestsequenz
	 * kopiert und unter dem Namen: <i>Kopie X von Originaldateiname</i> abgelegt. Danach wird
	 * die Kopie geladen und der Liste der Testsequenzen hinzugefgt. Alle <code>ProjectListener</code> werden
	 * ber die neu erstellte Testsequenz informiert.
	 * @param script Die Originaltestsequenz.
	 * @return Die Kopie von script.
	 * @throws AtosJException
	 * @see ProjectListener
	 */
	public HTSScript copyScript(HTSScript script) throws AtosJException 
	{
		String newName = Messages.getString("Project.122") + script.getName() + ".hts"; //$NON-NLS-1$ //$NON-NLS-2$
		File pathCopy = new File( testee.getProjectDir() + File.separator + "SEQ" + File.separator + newName ); //$NON-NLS-1$
		int numCopy = 2;
		// Einen noch nicht vergebenen Namen generieren
		while( pathCopy.exists() )
		{
			newName = Messages.getString("Project.125") + numCopy + Messages.getString("Project.126") + script.getName() + ".hts";  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			pathCopy = new File(pathCopy.getParent() + File.separator + newName);
			++numCopy;
		}
		
		try {
			pathCopy.createNewFile();
		}
		catch(IOException e) {
			throw new AtosJException(Messages.getString("Project.128") + pathCopy.getAbsolutePath() + Messages.getString("Project.129") + //$NON-NLS-1$ //$NON-NLS-2$
					Messages.getString("Project.130")); //$NON-NLS-1$
		}
		
		copyFile( script.getPath(), pathCopy.getAbsolutePath() );
				
		HTSScript scriptCopy =  new HTSScript( pathCopy.getAbsolutePath(), this );
		scripts.add(scriptCopy);
		fireResourceEvent(scriptCopy, true);
		
		return scriptCopy;
	}
	
	/**
	 * berprft ob die angegebe Testsequenz schon im Projekt existiert.
	 * @param scriptName Der Name der Testsequenz.
	 * @return <code>true</code> wenn schon enthalten, <code>false</code> sonst.
	 * @see HTSScript#getName()
	 */
	public boolean containsScript(String scriptName) 
	{
		for(int i = 0; i < scripts.size(); ++i) 
		{
			if( ((HTSScript)scripts.get(i)).getName().equalsIgnoreCase(scriptName) )
				return true;
		}
		
		return false;
	}
	
	/**
	 * Lscht die Testsequenz aus dem Projekt. Die Script-Datei wird auf der Festplatte gelscht.
	 * Ebenso werden alle Referenzen auf die Testsequenz aus allen Testpaketen gelscht. Alle
	 * <code>ProjectListener</code> werden ber das Lschen benachrichtigt.
	 * <em>ACHTUNG:</em> Die Pakete werden hiernach nicht gespeichert.
	 * @param script Die zu lschende Testsequenz.
	 * @return true wenn erfolgreich.
	 * @see ProjectListener
	 */
	public boolean deleteScript(HTSScript script) 
	{
		boolean deleted = script.delete();
		if(deleted)
		{
			// aus den Paketen lschen
			for(int i = 0; i < packages.size(); ++i)
			{
				HTSPackage pack = (HTSPackage) packages.get(i);
				if( pack.containsScript(script) )
					pack.removeScriptTotally(script);
			}
			
			scripts.remove(script);
			fireResourceEvent(script, false);
		}
		
		return deleted;
	}
	
	/**
	 * Ersetzt in allen Testsequenzen die alte durch die neue ID.
	 * <em>ACHTUNG:</em> Die Testsequenzen werden hiernach nicht gespeichert.
	 * @param oldId Die alte ID.
	 * @param newId Die neue ID.
	 * @see HTSScript#replaceId(ComponentId, ComponentId)
	 */
	public void replaceId(final ComponentId oldId, final ComponentId newId) 
	{
		for(int i = 0; i < scripts.size(); ++i)
		{
			HTSScript script = (HTSScript)scripts.get(i);		
			script.replaceId(oldId, newId);
		}
	}
	
	/**
	 * Erstellt ein neues Testpaket und fgt es dem Projekt hinzu.
	 * @param name Der Name des neuen Testpakets.
	 * @return Das neu erstellte Testpakets.
	 * @throws AtosJException Fehler beim Erstellen
	 */
	public HTSPackage createNewPackage(String name) throws AtosJException 
	{
		File f = new File(testee.getProjectDir() + File.separator + "SEQ" + File.separator + name + ".pak"); //$NON-NLS-1$ //$NON-NLS-2$
		if( f.exists() )
			throw new AtosJException(Messages.getString("Project.133") + name + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		
		try {
			f.createNewFile();
		}
		catch(IOException e) {
			throw new AtosJException(Messages.getString("Project.135") + f.getAbsolutePath() + Messages.getString("Project.136") + //$NON-NLS-1$ //$NON-NLS-2$
					Messages.getString("Project.137")); //$NON-NLS-1$
		}
		
		HTSPackage pack =  new HTSPackage( f.getAbsolutePath(), this );
		packages.add(pack);
		fireResourceEvent(pack, true);
		
		return pack;
	}
	
	/**
	 * Erstellt eine Kopie des angegebenen Testpakets. Dazu wird die Datei des Originaltestpakets
	 * kopiert und unter dem Namen: <i>Kopie X von Originaldateiname</i> abgelegt. Danach wird
	 * die Kopie geladen und der Liste der Testspakete hinzugefgt. 
	 * Alle <code>ProjectListener</code> werden ber das neu erstellte Testpaket informiert.
	 * @param pack Die Originaltestsequenz.
	 * @return Die Kopie von pack.
	 * @throws AtosJException
	 * @see ProjectListener
	 */
	public HTSPackage copyPackage(HTSPackage pack) throws AtosJException 
	{
		String newName = Messages.getString("Project.138") + pack.getName() + ".pak"; //$NON-NLS-1$ //$NON-NLS-2$
		File pathCopy = new File( testee.getProjectDir() + File.separator + "SEQ" + File.separator + newName ); //$NON-NLS-1$
		int numCopy = 2;
		// Einen noch nicht vergebenen Namen generieren
		while( pathCopy.exists() )
		{
			newName = Messages.getString("Project.141") + numCopy + Messages.getString("Project.142") + pack.getName() + ".pak";  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			pathCopy = new File(pathCopy.getParent() + File.separator + newName);
			++numCopy;
		}
		
		try {
			pathCopy.createNewFile();
		}
		catch(IOException e) {
			throw new AtosJException(Messages.getString("Project.144") + pathCopy.getAbsolutePath() + Messages.getString("Project.145") + //$NON-NLS-1$ //$NON-NLS-2$
					Messages.getString("Project.146")); //$NON-NLS-1$
		}
		
		copyFile( pack.getPath(), pathCopy.getAbsolutePath() );
				
		HTSPackage packCopy =  new HTSPackage( pathCopy.getAbsolutePath(), this );
		packages.add(packCopy);
		fireResourceEvent(packCopy, true);
		
		return packCopy;
	}
	
	/**
	 * berprft ob das angegeben Testpaket schon im Projekt existiert.
	 * @param packageName Der Name des Testpakets.
	 * @return <code>true</code> wenn schon enthalten, <code>false</code> sonst.
	 * @see HTSPackage#getName()
	 */
	public boolean containsPackage(String packageName) 
	{
		for(int i = 0; i < packages.size(); ++i) 
		{
			if( ((HTSPackage)packages.get(i)).getName().equalsIgnoreCase(packageName) )
				return true;
		}
		
		return false;
	}
	
	/**
	 * Lscht das Testpaket aus dem Projekt. Die Paket-Datei wird auf der Festplatte gelscht.
	 * Alle <code>ProjectListener</code> werden ber das Lschen benachrichtigt.
	 * @param pack Die zu lschende Testsequenz.
	 * @return true wenn erfolgreich.
	 * @see ProjectListener
	 */
	public boolean deletePackage(HTSPackage pack) 
	{
		boolean deleted = pack.delete();
		if(deleted)
		{
			packages.remove(pack);
			fireResourceEvent(pack, false);
		}
		
		return deleted;
	}
	
	/**
	 * Gibt die zuletzt editierten Ressourcen zurck. Beim Schlieen
	 * eines Projektes werden alle geffneten Editoren vermerkt.
	 * @return Vector(String) Die Dateinamen aller, zuletzt bearbeiteten Ressourcen.
	 */
	public Vector getEditedResources() 
	{
		Vector resources = new Vector( openedResources.size() );
		
		for(int i = 0; i < openedResources.size(); ++i)
		{
			Object resource = getResource( (String) openedResources.get(i) );
			if(resource != null)
				resources.add(resource);
		}

		return resources;
	}
	
	/**
	 * Gibt die zuletzt editierte Ressource zurck.
	 * @return Die Ressource, deren Editor beim letzten Schlieen
	 * gerade aktiv war oder <code>null</code> falls keine.
	 */
	public Object getActiveResource() {
		if(activeResource == null)
			return null;
		
		return getResource(activeResource);
	}
	
	/**
	 * Gibt zu einem Dateinamen das entsprechende Ressourcen Objekt zurck.
	 * @param fileName Der Dateiname der Ressource, nicht der gesamte Pfad.
	 * @return Die Resource mit dem angegebenen Dateinamen oder 
	 * <code>null</code>, falls nicht existent.
	 */
	private Object getResource(String fileName) 
	{
		String fileNameNoExt = fileName.substring( 0, fileName.lastIndexOf('.') );
		if( fileName.endsWith(".hts") ) //$NON-NLS-1$
		{
			for(int i = 0; i < scripts.size(); ++i) 
			{
				if( ((HTSScript)scripts.get(i)).getName().equals(fileNameNoExt) )
					return scripts.get(i);
			}
		}
		else
		if( fileName.endsWith(".pak") ) //$NON-NLS-1$
		{
			for(int i = 0; i < packages.size(); ++i) 
			{
				if( ((HTSPackage)packages.get(i)).getName().equals(fileNameNoExt) )
					return packages.get(i);
			}
		}
		else
		if( fileName.endsWith(".urf") ) //$NON-NLS-1$
		{
			if( urf.getName().equals(fileNameNoExt) )
				return urf;
		}
		
		return null;
	}
	
	/**
	 * Setzt die zuletzt editierten Ressourcen. Vor dem
	 * Schlieen eines Projekts werden die Ressourcen vermerkt,
	 * deren Editoren gerade geffnet waren.
	 * @param resources Die gerade editierten Ressourcen oder
	 * <code>null</code>, falls keine.
	 * @see #getEditedResources()
	 */
	public void setEditedResources(Object[] resources) 
	{
		openedResources.clear();
		
		if(resources == null)
			return;
		
		for(int i = 0; i < resources.length; ++i)
		{
			Object res = resources[i];
			if( res instanceof HTSScript )
			{
				openedResources.add( ((HTSScript)res).getName() + ".hts" ); //$NON-NLS-1$
			}
			else
			if( res instanceof HTSPackage )
			{
				openedResources.add( ((HTSPackage)res).getName() + ".pak" ); //$NON-NLS-1$
			}
			else
			if( res instanceof URFFile )
			{
				openedResources.add( ((URFFile)res).getName() + ".urf" ); //$NON-NLS-1$
			}
		}
	}
	
	/**
	 * Vermerkt die Ressource deren Editor, gerade aktiv war.
	 * Vor dem Schlieen des Projektes wird die Ressource vermerkt,
	 * damit beim nchsten ffnen der Editor wieder aktiviert wird.
	 * @param res Die Ressource deren Editor gerade aktiv ist oder 
	 * <code>null</code>, falls kein Editor aktiv.
	 * @see #getActiveResource()
	 */
	public void setActiveResource(Object res) 
	{
		if( res instanceof HTSScript )
		{
			activeResource = ((HTSScript)res).getName() + ".hts"; //$NON-NLS-1$
		}
		else
		if( res instanceof HTSPackage )
		{
			activeResource = ((HTSPackage)res).getName() + ".pak"; //$NON-NLS-1$
		}
		else
		if( res instanceof URFFile )
		{
			activeResource = ((URFFile)res).getName() + ".urf"; //$NON-NLS-1$
		}
	}
	
	/**
	 * Gibt den Pfad der Projektdatei zurck.
	 * @return Der absoluten Pfad der apf-Datei.
	 */
	public String getPath() {
		return apfPath;
	}
	
	/**
	 * Gibt den Projektnamen zurck.
	 * @return Der Projektname.
	 */
	public String getName() {
		return testee.getProjectName();
	}
	
	/**
	 * Gibt alle Testsequenzen des Projektes zurck.
	 * @return Vector(HTSScript) Eine Kopie des enthaltenen Testsequenz-Vectors.
	 */
	public Vector getScripts() {
		return new Vector(scripts);
	}
	
	/**
	 * Gibt alle Testpakete des Projektes zurck.
	 * @return Vector(HTSPackage) Eine Kopie des Testpaket-Vectors.
	 */
	public Vector getPackages() {
		return new Vector(packages);
	}
	
	/**
	 * Gibt das Testobjekt zurck.
	 */
	public TestObject getTestObject() {
		return testee;
	}
	
	/**
	 * Gibt die URF-Datei zurck.
	 */
	public URFFile getURFFile() {
		return urf;
	}
	
	/**
	 * Gibt alle genderten Ressourcen zurck, d.h. alle genderten Testsequenzen, Pakete und URFFile.
	 * @return Vector(Changeable) Die genderten Ressourcen oder einen leeren Vector, 
	 * wenn keine genderten Ressourcen vorhanden.
	 */
	public Vector getModifiedResources() 
	{
		Vector modifiedElements = new Vector();
		// modifizierte scripte
		for(int i = 0; i < scripts.size(); ++i)
		{
			if( ((HTSScript)scripts.get(i)).isModified() )
				modifiedElements.add( scripts.get(i) );
		}
		// modifizierte pakete
		for(int i = 0; i < packages.size(); ++i)
		{
			if( ((HTSPackage)packages.get(i)).isModified() )
				modifiedElements.add( packages.get(i) );
		}
		// modifizierte urf datei
		if( urf.isModified() )
			modifiedElements.add(urf);
		
		return modifiedElements;
	}
	
	/**
	 * Kopiert eine Datei.
	 * @param sourcePath Vollstndiger Pfad der Quelldatei.
	 * @param targetPath Vollstndiger Pfad der Zieldatei.
	 * @return <code>true</code>, wenn erfolgreich,  <code>false</code> sonst.
	 */
	public static boolean copyFile(String sourcePath, String targetPath)
	{
		try 
		{
			RandomAccessFile source = new RandomAccessFile(sourcePath,"r"); //$NON-NLS-1$
			RandomAccessFile target = new RandomAccessFile(targetPath, "rw"); //$NON-NLS-1$
			while (target.length() < source.length())
				target.write( source.read() );
			
			source.close();
			target.close();
			return true;
		} catch(IOException e) { return false; }
	}
	
	/**
	 * Entfernt <code>File.seperator</code> am Ende von Verzeichnisnamen, falls vorhanden.
	 * @param dir Verzeichnispfad
	 * @return Verzeichnispfad ohne <code>File.seperator</code> am Ende.
	 * @see File#separator
	 */
	public static String stripLastSeperator(String dir) {
		if( dir.endsWith(File.separator) )
			return dir.substring(0, dir.length()-1);
		
		return dir;
	}

	/**
	 * Gibt den Projekttyp an.
	 * @return <code>true</code>, wenn das Projekt ein AWT Projekt ist,
	 * <code>false</code> sonst.
	 */
	public boolean isAwtProject() {
		return testee.isAWTProject();
	}
	
	/**
	 * Benachrichtigt alle <code>ProjectListener</code> ber das Erstellen
	 * bzw. Lschen von Ressourcen.
	 * @param c Die Ressource.
	 * @param created <code>true</code>, wenn c neu erstellt wurde, 
	 * <code>false</code>, wenn c gelscht wurde.
	 * @see ProjectListener
	 */
	private void fireResourceEvent(Changeable c, boolean created) {
		for(int i = 0; i < listeners.size(); ++i) {
			if(created)
				((ProjectListener)listeners.get(i)).resourceCreated(c);
			else
				((ProjectListener)listeners.get(i)).resourceDeleted(c);
		}
	}
	
	/**
	 * Fgt einen neuen <code>ProjectListener</code> hinzu.
	 * @param listener Der neue Listener.
	 * @see ProjectListener
	 */
	public void addProjectListener(ProjectListener listener) {
		if( !listeners.contains(listener) )
			listeners.add(listener);
	}
	
	/**
	 * Entfernt den angegebenen <code>ProjectListener</code>.
	 * @param listener Der zu entfernende Listener.
	 * @see ProjectListener
	 */
	public void removeProjectListener(ProjectListener listener) {
		listeners.remove(listener);
	}
}

