/*
 * Created on 16.09.2005
 */
package atosj.app;

import java.io.File;
import java.io.Serializable;
import java.util.Vector;

import atosj.component.CustomComponentResolver;
import atosj.component.swing.SwingComponentResolver;
import atosj.component.swt.SWTComponentResolver;
import atosj.exception.AtosJException;
import atosj.exception.MainWindowNotFoundException;
import atosj.listener.AtosJTopLevelWindowListener;
import atosj.listener.FinishedListener;
import atosj.model.recorder.IHTSCommandReceiver;
import atosj.util.AWTWindowMonitor;
import atosj.util.GUIRunner;
import atosj.util.SWTWindowMonitor;

/**
 * Diese Klasse reprsentiert das Testobjekt mit allen seinen
 * Informationen, die aus einem ATOSj Project File (apf)
 * ausgelesen werden knnen.
 * @author Nicos Tegos
 */
public class TestObject implements Serializable
{
	private String projectName;
	private String projectDir;
	private String mainClassDir;
	private String mainClass;
	private String mainWindow;
	private Vector classPath;
	private boolean coverage; // berdeckungsanalyse beim Start des Testobjekts durchfhren.
	private Vector coveragePath; // Vector<String> Classpath der Dateien zu denen eine berdeckungsananlyse durchgefhrt werden soll.
	private boolean projectType;
	
	private CustomComponentResolver resolver;
	
	/**
	 * Initialisiert das Testobjekt und den entsprechenden <code>CustomComponentResolver</code>.
	 * @param projectName Der Name des AtosJ Projekts
	 * @param mainClassDir Das Verzeichnis in dem die Main Klasse liegt und ausgefhrt wird.
	 * @param mainClass Der Name der Main-Klasse.
	 * @param mainWindow Der Titel des Hauptfensters.
	 * @param projectDir Verzeichnis des AtosJ Projekts.
	 * @param classPath Vector(String) Klassenpfad des Testobjekts.
	 * @param projectType Typ des Projekts gltige Werte sind <code>Project.AWT</code>
	 * und <code>Project.SWT</code>.
	 * @param coverage Gibt an ob eine berdeckungsanalyse, whrend der Testdurchlufe durchgefhrt
	 * werden soll. <code>true</code> fr ja, <code>false</code> fr nein.
	 * @see CustomComponentResolver
	 */
	public TestObject(String projectName,
			String mainClassDir,
			String mainClass,
			String mainWindow,
			String projectDir,
			Vector classPath,
			boolean projectType,
			boolean coverage,
			Vector coveragePath)
	{
		super();
		
		this.projectName = projectName;
		this.mainClassDir = mainClassDir;
		this.mainClass = mainClass;
		this.mainWindow = mainWindow;
		this.projectDir = projectDir;
		this.classPath = classPath;
		this.projectType = projectType;
		this.coverage = coverage;
		this.coveragePath = coveragePath;
		
		// Die Info Datei fr Custom Components liegt, wenn vorhanden 
		// im EXT Unterverzeichnis des Projektverzeichnisses
		String customPath = projectDir;
		if( !customPath.endsWith(File.separator) )
			customPath += File.separator;
		customPath += "EXT" + File.separator + "custom.lst";
		
		if (projectType==Project.AWT)
			resolver = new SwingComponentResolver(customPath);
		else
			resolver = new SWTComponentResolver(customPath);
	}
	
	/**
	 * Startet das Testobjekt und wartet maximal <code>timeout</code> Millisekunden 
	 * auf das ffnen des Hauptfensters. Erstellt die <code>WindowMonitor</code>-Instanz.
	 * @param timeout Maximale Zeit fr das ffnen des Hauptfensters.
	 * @param args Die Programmargumente, <code>null</code> fr keine Argumente.
	 * @param windowListener Der Listener fr die berwachung von Fensterereignissen oder <code>null</code>, 
	 * wenn dieser nicht bentigt wird.
	 * @param receiver Der Empfnger fr die im Record Modus aufgezeichneten Kommandos oder 
	 * <code>null</code>, wenn das Testobjekt nicht im Record Modus gestartet werden soll.
	 * @throws MainWindowNotFoundException wenn sich das Hauptfenster 
	 * in der Frist <code>timeout</code> nicht geffnent hat.
	 * @see atosj.util.WindowMonitor
	 */
	public void start(long timeout, String args[], AtosJTopLevelWindowListener windowListener, FinishedListener finishedListener, IHTSCommandReceiver receiver) throws AtosJException
	{
		if (projectType==Project.AWT)
			AWTWindowMonitor.createInstance( (SwingComponentResolver)resolver, receiver );
		else
			SWTWindowMonitor.createInstance( (SWTComponentResolver)resolver, receiver );
		
		GUIRunner runner = new GUIRunner(mainClass, mainWindow);
		// default argumente ertmal leeres array
		if( !runner.openMainWindow(timeout, args == null ? new String[0] : args, windowListener, finishedListener) )
			throw new MainWindowNotFoundException(mainWindow);
	}
	
	/**
	 * Delegiert an die zweite <code>start</code>-Methode und bergibt <code>null</code> bei allen
	 * nicht vorhandenen Parametern.
	 */
	public void start(long timeout, String[] args) throws AtosJException
	{
		start(timeout, args, null, null, null);
	}
	
/*	private void activateClassPath() {
		if(classPath == null)
			return;
		
		for(int i = 0; i < classPath.size(); ++i)
		{
			try {
				ClassPathAdder.addPath( (String) classPath.get(i) + "\\" ); // der Classpathadder braucht ein backslash am ende
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
*/	
	/**
	 * @return Name der Main-Klasse des Testobjekts.
	 */
	public String getMainClass() {
		return mainClass;
	}
	
	/**
	 * @return Titel des Hauptfensters des Testobjekts.
	 */
	public String getMainWindow() {
		return mainWindow;
	}
	
	/**
	 * Setzt den Titel des Hauptfensters des Testobjekts.
	 * @param mainWindow Der Fenstertitel.
	 */
	public void setMainWindow(String mainWindow) {
		this.mainWindow = mainWindow;
	}
	
	/**
	 * @return Name des ATOSj Projekts.
	 */
	public String getProjectName() {
		return projectName;
	}
	
	/**
	 * @return Vollstndiger Pfad des ATOSj Projektverzeichnisses.
	 */
	public String getProjectDir() {
		return projectDir;
	}
	
	/**
	 * @return Das Ausfhrungsverzeichnis des Testobjekts.
	 */
	public String getMainClassDir() {
		return mainClassDir;
	}
	
	/**
	 * Setzt das Verzeichnis in dem das Testobjekt ausgefhrt werden soll.
	 * @param mainClassDir Das Ausfhrungsverzeichnis.
	 */
	public void setMainClassDir(String mainClassDir) {
		this.mainClassDir = mainClassDir;
	}
	
	/**
	 * @return Den Klassenpfad des Testobjekts, alle Eintrge sind durch
	 * <code>File.pathSeperator</code> voneinander getrennt.
	 */
	public String getClassPath() 
	{
		String ret = "";
		for(int i = 0; i < classPath.size(); ++i)
		{
			ret += classPath.get(i);
			if( i != classPath.size()-1 )
				ret += File.pathSeparator;
		}
		
		return ret;
	}
	
	/**
	 * @return Vector(String) Eine Liste Elemente des Klassenpfades fr das Testobjekt.
	 */
	public Vector getClassPathList() {
		return new Vector(classPath);
	}
	
	/**
	 * Setzt den Klassenpfad. Der bergebene Vector wird direkt bernommen.
	 * @param classPath Vector(String) Liste der Elemente des Klassenpfades 
	 * fr das Testobjekt. 
	 */
	public void setClassPathList(Vector classPath) {
		this.classPath = classPath;
	}
	
	/**
	 * Setzt die Main-Klasse des Testobjekts.
	 * @param mainClass Der vollqualifizierte Name der Main-Klasse.
	 */
	public void setMainClass(String mainClass) {
		this.mainClass = mainClass;
	}
	
	/**
	 * Gibt den Projekttyp, bzw. die Art des Testobjekts an.
	 * @return <code>Project.AWT</code> fr ein Swing Projekt
	 * oder <code>Project.SWT</code> fr ein SWT Projekt.
	 */
	public boolean isAWTProject() {
		return projectType == Project.AWT;
	}
	
	/**
	 * @return Den aktuellen <code>CustomComponentResolver</code>.
	 */
	public CustomComponentResolver getResolver() {
		return resolver;
	}
	
	/**
	 * Gibt den Pfad der Datenbank fr die berdeckungsanalyse an. Die Datenbank liegt im
	 * Verzeichnis COV des Projektverzeichnisses unter dem Namen "coverage.xml".
	 * @return Der vollstndige Pfad der XML-Datenbank.
	 */
	public String getCoverageDatabasePath() {
		return projectDir + File.separator + "COV" + File.separator + "coverage.xml";
	}

	/**
	 * Gibt an, ob bei einem Testdurchlauf die Daten fr die berdeckungsanalyse
	 * aktualisiert werden sollen.
	 * @return <code>true</code>, wenn eine berdeckungsanalyse durchgefhrt werden soll,
	 * <code>false</code> sonst.
	 */
	public boolean isCoverage() {
		return coverage;
	}

	/**
	 * Legt fest, ob bei einem Testdurchlauf die Daten fr die berdeckungsanalyse
	 * aktualisiert werden sollen.
	 * @param coverage <code>true</code>, wenn eine berdeckungsanalyse durchgefhrt werden soll,
	 * <code>false</code> sonst.
	 */
	public void setCoverage(boolean coverage) {
		this.coverage = coverage;
	}

	/**
	 * Gibt eine Liste aller Pakete des Testobjektes an, die in der berdeckungsanalyse
	 * bercksichtigt werden sollen.
	 * @return Vector(String) Liste der Anfnge der Paketnamen.
	 * @see #setCoveragePath(Vector)
	 */
	public Vector getCoveragePath() {
		return coveragePath;
	}

	/**
	 * Setzt die Liste aller Pakete des Testobjektes, die in der berdeckungsanalyse
	 * bercksichtigt werden sollen. Die Namen der Pakete mssen nicht vollstndig sein,
	 * es wird nach allen Paketen gesucht, deren Namen mit den angegebenen Zeichenketten
	 * beginnen.
	 * @param coveragePath Liste der Anfnge der Paketnamen.
	 */
	public void setCoveragePath(Vector coveragePath) {
		this.coveragePath = coveragePath;
	}
}
