package atosj.app;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import atosj.exception.ParseException;
import atosj.util.CSVLine;
import atosj.util.CSVParser;

/**
 * Diese Klasse speichert und ldt alle Einstellungen, die der Nutzer vornimmt.
 * So zum Beispiel die gre des Hauptfensters und der Run-Dialoge.
 */
public class UserPreferences 
{
	private static final String FILE_NAME = "user.prf";
	private static String FILE_PATH = "";
	
	/** Schlssel fr: Position und Gre des Dialogs zum Ausfhren von Testpaketen. Wert ist vom Typ <code>Rectangle</code>. */
	public static final String PACKAGE_DLG_RECT = "PACKAGE_DLG_RECT";
	/** Schlssel fr: Zustand des Dialogs zum Ausfhren von Testpaketen. Wert ist <code>MAXIMIZED</code> oder <code>NORMALIZED</code>. */
	public static final String PACKAGE_DLG_STATE = "PACKAGE_DLG_STATE";
	/** Schlssel fr: Position und Gre des Dialogs zum Ausfhren von Testsequenzen. Wert ist vom Typ <code>Rectangle</code>. */
	public static final String SCRIPT_DLG_RECT = "SCRIPT_DLG_RECT";
	/** Schlssel fr: Zustand des Dialogs zum Ausfhren von Testsequenzen. Wert ist <code>MAXIMIZED</code> oder <code>NORMALIZED</code>. */
	public static final String SCRIPT_DLG_STATE = "SCRIPT_DLG_STATE";
	/** Schlssel fr: Position und Gre des Hauptfensters. Wert ist vom Typ <code>Rectangle</code>. */
	public static final String MAIN_WND_RECT = "MAIN_WND_RECT";
	/** Schlssel fr: Zustand des Hauptfensters. Wert ist <code>MAXIMIZED</code> oder <code>NORMALIZED</code>. */
	public static final String MAIN_WND_STATE = "MAIN_WND_STATE";
	/** Wert: Gibt an, dass ein Fenster maximiert ist. */
	public static final String STATE_WND_MAXIMIZED = "MAXIMIZED";
	/** Wert: Gibt an, dass ein Fenster normale Gre hat. */
	public static final String STATE_WND_NORMALIZED = "NORMALIZED";
	/** Schlssel fr: Pfad zu einer ausfhrbaren Datei, die pdf liest z.B. acroread. Wert ist vom Typ <code>String</code>. */
	public static final String PDF_READER_PATH = "PDF_READER_PATH";
	/** Schlssel fr: Pfad zu einer ausfhrbaren Datei, die html Dokumente liest z.B. firefox. Wert ist vom Typ <code>String</code>. */
	public static final String HTML_READER_PATH = "HTML_READER_PATH";
	/**Schlssel fr: Die Spracheinstellung, z.B. 'de'. Wert ist vom Typ <code>String</code>.*/
	public static final String LANGUAGE = "LANGUAGE";
	
	private Rectangle mainWindowRect;
	private String mainWindowState;
	private Rectangle runPackageRect;
	private String runPackageState;
	private Rectangle runScriptRect;
	private String runScriptState;
	
	private String pdfReaderPath;
	private String htmlReaderPath;
	
	private String language_short;
	
	private static UserPreferences instance;
	
	/**
	 * Ldt die gespeicherten Einstellungen aus der Datei "user.prf"
	 * im aktuellen Arbeitsverzeichnis. Wenn die Datei nicht existiert
	 * werden Vorgabewerte verwendet.
	 */
	private UserPreferences() 
	{
		super();
		FILE_PATH = "." + File.separatorChar + FILE_NAME;
		initDefaults();
		readFile();
	}
	
	/**
	 * Gibt die Singleton-Instanz zurck, erstellt diese evtl. zuvor.
	 */
	public static UserPreferences getInstance() {
		if(instance == null)
			instance = new UserPreferences();
		
		return instance;
	}
	
	/**
	 * Setzt den Pfad zur Preference Datei neu und lsst erneut einlesen, der Werte.
	 * @param path
	 */
	public static void setFilePath(String path){
		FILE_PATH = path;
		instance.readFile();
		}
	
	/**
	 * Vorgabewerte fr die Fenstergren initialisieren, wichtig falls die Datei
	 * "user.prf" nicht gelesen werden kann.
	 */
	private void initDefaults() 
	{
		Rectangle displayRect = Display.getDefault().getBounds();
		mainWindowRect = computePreferedRectangle(displayRect, new Point(800, 600));
		mainWindowState = STATE_WND_NORMALIZED;
		runPackageRect = computePreferedRectangle(displayRect, new Point(640, 640));
		runPackageState = STATE_WND_NORMALIZED;
		runScriptRect = computePreferedRectangle(displayRect, new Point(640, 640));
		runScriptState = STATE_WND_NORMALIZED;
	}
	
	/**
	 * Liest die Nutzereinstellung aus der Datei "user.prf" aus.
	 */
	private void readFile() 
	{
		try 
		{
			Rectangle displayRect = Display.getDefault().getBounds();
			CSVParser parser = new CSVParser(FILE_PATH);
			//return; // bugFix eigentlich muesste die Exception unten gecatcht werden
			CSVLine line;
			while( (line = parser.getNextLine()) != null )
			{
				String valueName = line.getElement(0);
				if( valueName.equals(PACKAGE_DLG_RECT) )
				{
					runPackageRect = computePreferedRectangle(displayRect, new Rectangle( getInt(line, 1), getInt(line, 2), getInt(line, 3), getInt(line, 4) ));
				}
				else
				if( valueName.equals(PACKAGE_DLG_STATE) )
				{
					runPackageState = line.getElement(1);
				}
				else
				if( valueName.equals(SCRIPT_DLG_RECT) )
				{
					runScriptRect = computePreferedRectangle(displayRect, new Rectangle( getInt(line, 1), getInt(line, 2), getInt(line, 3), getInt(line, 4) ) );
				}
				else
				if( valueName.equals(SCRIPT_DLG_STATE) )
				{
					runScriptState = line.getElement(1);
				}
				else
				if( valueName.equals(MAIN_WND_RECT) )
				{
					mainWindowRect = computePreferedRectangle(displayRect, new Rectangle( getInt(line, 1), getInt(line, 2), getInt(line, 3), getInt(line, 4) ) );
				}
				else
				if( valueName.equals(MAIN_WND_STATE) )
				{
					mainWindowState = line.getElement(1);
				}
				else
				if( valueName.equals(PDF_READER_PATH) )
				{
					pdfReaderPath = line.getElementUnquoted(1);
				}
				else
				if( valueName.equals(HTML_READER_PATH) )
				{
					htmlReaderPath = line.getElementUnquoted(1);
				}
				else
				if( valueName.equals(LANGUAGE))
				{
					language_short = line.getElementUnquoted(1);
				}
			}
		} catch (IOException e) {
			// egal wir haben ja immer noch die Defaultwerte
		}
		catch(ParseException e2) {
			// egal wir haben ja immer noch die Defaultwerte
		}
		catch(NumberFormatException e3) {
			// egal wir haben ja immer noch die Defaultwerte
		}
	}
	
	/**
	 * Wandelt den Wert aus der kommaseparierten List in eine ganze Zahl um.
	 * @param line Die kommaseparierte Liste.
	 * @param index Die Position des Wertes, der gelesen werden soll.
	 * @return Wert des Eintrages an Position index als ganze Zahl. 
	 * @throws ParseException falls der index ungltig.
	 * @throws NumberFormatException falls der Wert an Position index keine natrliche Zahl ist.
	 */
	private int getInt(CSVLine line, int index) throws ParseException {
		return Integer.parseInt( line.getElement(index) );
	}
	
	/**
	 * Ermittelt die zentrierte Position eines Bildschirmausschnittes mit einer gewnschten Gre. 
	 * Der Bildschirmausschnitt soll die Gre wishedSize haben. Diese wird nur als Hinweis 
	 * genutzt und nur eingehalten, wenn dies auch mglich ist.
	 * @param displayRect Gre des gesamten Bildschirms.
	 * @param wishedSize Gewnschte Gre, x ist die Breite und y die Hhe.
	 * @return Position des ermittelten Bildschirmausschnittes im Zentrum des Bildschirms.
	 */
	private Rectangle computePreferedRectangle(Rectangle displayRect, Point wishedSize) {
		Rectangle prefered = new Rectangle(0, 0, wishedSize.x, wishedSize.y);
				
		if(prefered.width > displayRect.width)
			prefered.width = displayRect.width;
		if(prefered.height > displayRect.height)
			prefered.height = displayRect.height;
		
		// zentrieren
		prefered.x = (displayRect.width - prefered.width)/2;
		prefered.y = (displayRect.height - prefered.height)/2;
		
		return prefered;
	}
	
	/**
	 * Passt Position und Gre des gewnschten Bildschirmausschnittes an die Gesamtgre des
	 * Bildschirms an.
	 * @param displayRect Position und Gesamtgre des Bildschirms.
	 * @param wishedRect Position und Gre des gewnschten Bildschirmausschnittes.
	 * @return Der ermittelte Bildschirmausschnitt.
	 */
	private Rectangle computePreferedRectangle(Rectangle displayRect, Rectangle wishedRect) {
		Rectangle prefered = new Rectangle(wishedRect.x, wishedRect.y, wishedRect.width, wishedRect.height);
				
		if(prefered.width > displayRect.width)
			prefered.width = displayRect.width;
		if(prefered.height > displayRect.height)
			prefered.height = displayRect.height;
		
		if(prefered.x > displayRect.x + displayRect.width)
			prefered.x = displayRect.x;
		
		if(prefered.y > displayRect.y + displayRect.height)
			prefered.y = displayRect.y;
		
		return prefered;
	}
	
	/**
	 * Einstellung des Nutzers werden in einem Schlssel-Wert-Paar
	 * gespeichert. Der Wert kann mit dem Schlssel abgerufen werden.
	 * @param valueName Der Schlssel, siehe definierte Konstanten.
	 * @return Der Wert. Der Typ des Wertes ist fr jeden Schlssel
	 * festgelegt.
	 */
	public Object getPreferedValue(String valueName) 
	{
		if( valueName.equals(PACKAGE_DLG_RECT) )
		{
			return runPackageRect;
		}
		else
		if( valueName.equals(PACKAGE_DLG_STATE) )
		{
			return runPackageState;
		}
		else
		if( valueName.equals(SCRIPT_DLG_RECT) )
		{
			return runScriptRect;
		}
		else
		if( valueName.equals(SCRIPT_DLG_STATE) )
		{
			return runScriptState;
		}
		else
		if( valueName.equals(MAIN_WND_RECT) )
		{
			return mainWindowRect;
		}
		else
		if( valueName.equals(MAIN_WND_STATE) )
		{
			return mainWindowState;
		}
		else
		if( valueName.equals(PDF_READER_PATH) )
		{
			return pdfReaderPath;
		}
		else
		if( valueName.equals(HTML_READER_PATH) )
		{
			return htmlReaderPath;
		}
		else
			if( valueName.equals(LANGUAGE) )
			{
			return language_short;
		}
		else
			return null;
	}
	
	/**
	 * Setzt die gewnschte Nutzereinstellung als Schlssel-Wert-Paar.
	 * @param valueName Der Schlssel, gltige Werte siehe definierte Konstanten.
	 * @param value Der Wert.
	 */
	public void setPreferedValue(String valueName, Object value)
	{
		if( valueName.equals(PACKAGE_DLG_RECT) )
		{
			runPackageRect = (Rectangle)value;
			runPackageState = STATE_WND_NORMALIZED;
		}
		else
		if( valueName.equals(PACKAGE_DLG_STATE) )
		{
			runPackageState = (String)value;
		}
		else
		if( valueName.equals(SCRIPT_DLG_RECT) )
		{
			runScriptRect = (Rectangle)value;
			runScriptState = STATE_WND_NORMALIZED;
		}
		else
		if( valueName.equals(SCRIPT_DLG_STATE) )
		{
			runScriptState = (String)value;
		}
		else
		if( valueName.equals(MAIN_WND_RECT) )
		{
			mainWindowRect = (Rectangle)value;
			mainWindowState = STATE_WND_NORMALIZED;
		}
		else
		if( valueName.equals(MAIN_WND_STATE) )
		{
			mainWindowState = (String)value;
		}
		else
		if( valueName.equals(PDF_READER_PATH) )
		{
			pdfReaderPath = (String)value;
		}
		else
		if( valueName.equals(HTML_READER_PATH) )
		{
			htmlReaderPath = (String)value;
		}
		else
		if ( valueName.equals(LANGUAGE))
		{
			language_short = (String)value;
		}
	}
	
	/**
	 * Speichert alle Nutzervorlieben als Schlssel-Wert-Paare in
	 * kommaseparierten Listen in der Datei "user.prf" im aktuellen 
	 * Arbeitsverzeichnis.
	 */
	public void save() 
	{
		try
		{
			FileWriter writer = new FileWriter("." + File.separator + FILE_NAME);
			
			writer.write( valueToLine(PACKAGE_DLG_RECT, runPackageRect).toString() + "\n" );
			writer.write( valueToLine(PACKAGE_DLG_STATE, runPackageState, false).toString() + "\n" );
			writer.write( valueToLine(SCRIPT_DLG_RECT, runScriptRect).toString() + "\n" );
			writer.write( valueToLine(SCRIPT_DLG_STATE, runScriptState, false).toString() + "\n" );
			writer.write( valueToLine(MAIN_WND_RECT, mainWindowRect).toString() + "\n" );
			writer.write( valueToLine(MAIN_WND_STATE, mainWindowState, false).toString() + "\n" );
			
			if(pdfReaderPath != null)
				writer.write( valueToLine(PDF_READER_PATH, pdfReaderPath, true).toString() + "\n" );
			if(htmlReaderPath != null)
				writer.write( valueToLine(HTML_READER_PATH, htmlReaderPath, true).toString() + "\n" );
			if(language_short != null)
				writer.write( valueToLine(LANGUAGE, language_short, true).toString() + "\n" ); 
			writer.close();
		}
		catch(IOException e) {}
	}
	
	/**
	 * Wandelt ein Schlssel-Wert-Paar mit einem Schlssel vom Typ <code>String</code>
	 * in eine kommaseparierte Liste um.
	 * @param valueName Der Schlssel.
	 * @param str Der Wert.
	 * @param quote wenn <code>true</code>, wird str in Anfhrungszeichen gesetzt.
	 */
	private CSVLine valueToLine(String valueName, String str, boolean quote)
	{
		Vector v = new Vector(1);
		v.add(valueName);
		if(quote)
			v.add( "\"" +  str + "\"" );
		else
			v.add(str);
		
		return new CSVLine(v);
	}
	
	/**
	 * Wandelt ein Schlssel-Wert-Paar mit einem Schlssel vom Typ <code>Rectangle</code>
	 * in eine kommaseparierte Liste um.
	 * @param valueName Der Schlssel.
	 * @param value Der Wert.
	 */
	private CSVLine valueToLine(String valueName, Rectangle value)
	{
		Vector v = new Vector(5);
		v.add(valueName);
		v.add(""+value.x);
		v.add(""+value.y);
		v.add(""+value.width);
		v.add(""+value.height);
		
		return new CSVLine(v);
	}
}
