/*
 * Created on 10.06.2006
 */
package atosj.component;

import java.rmi.RemoteException;
import java.util.Vector;

import atosj.exception.ParseException;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.command.HTSTest;
import atosj.model.hts.command.InvalidCommand;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;
import atosj.util.CSVLine;
import atosj.util.WindowMonitor;

/**
 * Diese Klasse erstellt neue Kommandos fr ein Script anhand
 * von Nutzeraktionen, die ber Listener registriert werden.
 * @author Nicos Tegos
 */
public abstract class CommandCreator implements ICommandCreator 
{
	private MComponent comp;
	private IHTSCommandReceiver receiver;
	private Vector components = new Vector(); // alle Components zu denen listener initialisiert wurden
	private Vector listeners = new Vector(); // beinhaltet die listener zu den components, genauso lang wie components
	
	/**
	 * Initialisiert alle Listener, die Nutzeraktionen aufzeichnen.
	 * @param comp Der Wrapper fr das eigentliche Component, das berwacht werden soll.
	 * @param receiver Der Empfnger fr die, zu den aufgezeichneten Nutzeraktionen erstellten Kommandos.
	 * @see #hookTestListener()
	 */
	public CommandCreator(MComponent comp, IHTSCommandReceiver receiver) {
		super();
		this.comp = comp;
		this.receiver = receiver;
		
		hookTestListener();
	}

	/**
	 * Gibt das enthaltene <code>MComponent</code> zurck.
	 * @see MComponent
	 */
	public MComponent getMComponent() {
		return comp;
	}
	
	/**
	 * Alle Listener auf GUI Objekte, die von diesem <code>CommandCreator</code>
	 * benutzt werden, mssen hier angemeldet werden. <em>ACHTUNG:</em> Es drfen
	 * keine Listener direkt an GUI Objekte angehangen werden, weil 
	 * damit nicht gewhrleistet ist, dass es der erste Listener ist.
	 * @see WindowMonitor#addListener(Object, Object)
	 */
	protected void addListener(Object component, Object listener) 
	{
		components.add(component);
		listeners.add(listener);
		WindowMonitor.getInstance().addListener(component, listener);
	}
	
	/**
	 * Alle Listener die sich auf das gewrappte Component beziehen,
	 * mssen hier angemeldet werden. <em>ACHTUNG:</em> Es drfen
	 * keine Listener direkt an das gewrappte Objekte angehangen werden, weil 
	 * damit nicht gewhrleistet ist, dass es der erste Listener ist.
	 * @see WindowMonitor#addListener(Object, Object)
	 */
	protected void addListener(Object listener) {
		addListener( getMComponent().getComponent(), listener );
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.ICommandCreator#dispose()
	 */
	public void dispose() 
	{
		for(int i = 0; i < components.size(); ++i)
			WindowMonitor.getInstance().removeListener( components.get(i), listeners.get(i) );
		components.clear();
		listeners.clear();
	}
	
	/**
	 * Gibt dem <code>CommandReceiver</code> ein Kommando bekannt.
	 * @param cmd Das aufgezeichnete Kommando.
	 */
	protected void publishCommand(HTSCommand cmd) {
		try {
			receiver.publishHTSCommand(cmd);
		} catch (RemoteException e) {
			// der receiver ist tot, soll uns nicht weiter stren
		}
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.ICommandCreator#publishActionCommand(java.lang.String, java.util.Vector, java.util.Vector)
	 */
	public void publishActionCommand(String actionType, Vector actionParameters, Vector subitemPath) 
	{
		try 
		{
			Vector parameters = new Vector();
			parameters.add(HTSSyntax.CMD_ACTION);
			
			String windowTitle = getMComponent().getComponentId().getWindowName();
			String mainWindowTitle = receiver.getTestObject().getMainWindow();
			boolean isMainWindow = windowTitle.equals(mainWindowTitle);
			if(isMainWindow)
				parameters.add(HTSSyntax.DEF_MAIN);
			else
				parameters.add( '"' + windowTitle + '"' );
			
			String typeName = getMComponent().getComponentId().getClassName(); 
			parameters.add(typeName);
			
			if( typeName.equals(HTSSyntax.TYPE_WINDOW) )
			{
				if(isMainWindow)
					parameters.add(HTSSyntax.DEF_MAIN);
				else
					parameters.add( '"' + windowTitle + '"' );
			}
			else
				parameters.add( '"' + getMComponent().getComponentId().getName() + '"' );
			
			
			parameters.add(actionType);
			
			if( actionParameters != null && actionParameters.size() > 0 )
				parameters.addAll(actionParameters);
			
			if(subitemPath != null && subitemPath.size() > 0)
			{
				parameters.add("SUBITEM");
				parameters.addAll(subitemPath);
			}
			
			publishCommand( createCommand(parameters) );
		}
		catch(RemoteException e) {
			/* nichts machen */
		}
	}
	
	/**
	 * Erstellt ein HTS TEST Kommando.
	 * @param state Der abzutestende Zustand, wie in der <code>HTSSyntax</code> beschrieben, z.B. ENABLESTATE.
	 * @param stateParameters Vector(String) optionale Parameter fr 
	 * einen bestimmten Zustandstest, z.B. der erwartete Text beim Zustand TEXT. 
	 * Auch <code>null</code> oder leere Vectoren erlaubt.
	 * @param subitemPath Vector(String) der Subelementpfad, 
	 * auch <code>null</code> oder leere Vectoren erlaubt.
	 * @see HTSSyntax
	 */
	protected HTSTest createTestCommand(String state, Vector stateParameters, Vector subitemPath) 
	{
		Vector parameters = new Vector();
		parameters.add(HTSSyntax.CMD_TEST);
		
		try 
		{
			String windowTitle = getMComponent().getComponentId().getWindowName();
			String mainWindowTitle = receiver.getTestObject().getMainWindow();
			boolean isMainWindow = windowTitle.equals(mainWindowTitle);
			if(isMainWindow)
				parameters.add(HTSSyntax.DEF_MAIN);
			else
				parameters.add( '"' + windowTitle + '"' );
			
			String typeName = getMComponent().getComponentId().getClassName(); 
			parameters.add(typeName);
			
			if( typeName.equals(HTSSyntax.TYPE_WINDOW) )
			{
				if(isMainWindow)
					parameters.add(HTSSyntax.DEF_MAIN);
				else
					parameters.add( '"' + windowTitle + '"' );
			}
			else
				parameters.add( '"' + getMComponent().getComponentId().getName() + '"' );
			
			parameters.add(state);
			
			if( stateParameters != null && stateParameters.size() > 0 )
				parameters.addAll(stateParameters);
			
			if(subitemPath != null && subitemPath.size() > 0)
			{
				parameters.add("SUBITEM");
				parameters.addAll(subitemPath);
			}
			
			return (HTSTest) createCommand(parameters);
		}
		catch(RemoteException e) {
			return null;
		}
	}
	
	/**
	 * Erstellt ein neues HTS Kommando.
	 * @param parameters Vector(String) Die Parameter des Kommandos.
	 * @return Das neu erstellte Kommando, sind die Parameter ungltig so wird
	 * ein Kommando des Typs <code>InvalidCommand</code> zurckgegeben.
	 */
	protected HTSCommand createCommand(Vector parameters) 
	{
		try {
			return HTSParser.getCommandFromLine( new CSVLine(parameters), getMComponent().getResolver() );
		} catch(ParseException e) {
			try	{
				return new InvalidCommand(e);
			}
			catch(ParseException e2) { return null; } // kann niemals auftreten
		}
	}
	
	/**
	 * Diese Methode muss einen Listener an das gewrappte Component anhngen, 
	 * der den Dialog zum Erstellen von TEST Kommandos zeigt. 
	 * Der Listener muss auf das Drcken von STRG und Rechtsclick reagieren.
	 * @see atosj.dialog.SWTTestCreatorDlg
	 * @see atosj.dialog.AWTTestDialog 
	 */
	protected abstract void hookTestListener();
}
