/*
 * Created on 06.09.2005
 */
package atosj.component;

import java.util.HashMap;
import java.util.Vector;

import atosj.exception.ComponentNotFoundException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;
import atosj.util.Snooze;
import atosj.util.WindowMonitor;

/**
 * Ermittelt zur Laufzeit die Components des Testobjektes anhand
 * ihrer <code>ComponentId</code>. 
 * @author Nicos Tegos
 * @see atosj.component.ComponentId
 */
public abstract class ComponentFinder {

	private ComponentResolver resolver;
	private NamingStrategy naming;
	private HashMap componentToCeatorMap;
	private IHTSCommandReceiver receiver;
	
	/**
	 * Erstellt einen neuen <code>ComponentFinder</code>.
	 * @param resolver Erzeugt die Wrapperklassen fr die Klassen der GUI-Bibliothek.
	 * @param naming Strategie zur Benamung, welche die Namen zu Components ermittelt.
	 * @param receiver Wird das Testobjekt im Record Modus gestartet muss hier der 
	 * Empfnger fr die aufgezeichneten Kommandos stehen, <code>null</code> sonst.
	 */
	public ComponentFinder(ComponentResolver resolver, NamingStrategy naming, IHTSCommandReceiver receiver) 
	{
		this.resolver = resolver;
		this.naming = naming;
		this.receiver = receiver;
		if(receiver != null)
			 componentToCeatorMap = new HashMap();
	}
	
	/**
	 * Liefert die Kindelemente zu einem beliebigen Component.
	 * @param component Das Component, Typ abhngig von der GUI-Bibliothek.
	 * @return Ein Array mit den Kindelementen von <code>component</code> oder
	 * <code>null</code>, falls dieses Component keine Kindelemente hat oder die
	 * Kindelemente nicht in <code>MComponents</code> umgewandelt werden sollen.
	 */
	public abstract Object[] getChildren(Object component);
	
	/**
	 * Entfernt rekursiv alle fr das Capturing initialisierten <code>ICommandCreator</code>.
	 * Die von den <code>ICommandCreator</code> verwendeten Listener werden deaktiviert.
	 * Wird gerufen, beim Schlieen eines Fensters, wenn sich das Testobjekt im Record Modus
	 * befindet.
	 * @param container Ein Component, das andere Components enthalten kann.
	 * @see ICommandCreator#dispose()
	 */
	public synchronized void removeAll(Object container)
	{
		// wenn nicht im aufzeichnungsmodus dann schluss
		if(receiver == null)
			return;
		
		ICommandCreator creator = (ICommandCreator)componentToCeatorMap.get(container);
		if(creator != null)
			creator.dispose();
		
		Object[] children = getChildren(container);
		if(children != null)
		{
			for(int i = 0; i < children.length; ++i)
				removeAll(children[i]);
		}
	}
	
	/**
	 * Sucht das zu <code>id</code> passende Component. Fr das Component
	 * wird ein Wrapper der Klasse <code>MComponent</code> mit Hilfe des
	 * <code>ComponentResolvers</code> ermittelt.
	 * @param id Der eindeutige Identifikator des Components.
	 * @return Das enstprechende <code>MComponent</code>.
	 * @throws ComponentNotFoundException falls nicht gefunden.
	 * @see ComponentResolver
	 */
	public MComponent getMComponent(ComponentId id) throws ComponentNotFoundException 
	{
		Object[] windows = waitForWindowToOpen( id.getWindowName(), 10000 );
		if(windows == null)
			throw new ComponentNotFoundException(id);
		
		MComponent comp = null;
		for(int i = 0; i < windows.length; ++i)	{
			comp = getMComponent(windows[i], id);
			
			if (comp!=null)	break;
		}
		
		if(comp == null)
			throw new ComponentNotFoundException(id);
		
		return comp;
	}
	
	/**
	 * Wartet bis sich ein Fenster mit dem angegebenen Titel geffnet hat.
	 * @param title Der Fenstertitel.
	 * @param timeout Wartezeit in Millisekunden.
	 * @return Die geffneten Fenster mit dem angegeben Titel, <code>null</code> sonst.
	 */
	private Object[] waitForWindowToOpen(String title, int timeout) 
	{
		Object[] windows =  WindowMonitor.getInstance().getWindow(title);
		long startTime = System.currentTimeMillis();
		while( windows == null && System.currentTimeMillis() - startTime <= timeout )
		{
			new Snooze(20);
			windows = WindowMonitor.getInstance().getWindow(title);
		}
		
		return windows;
	}
	
	/**
	 * Sucht rekursive alle Kindelemente von <code>parent</code> durch,
	 * bis ein Component mit der entsprechenden <code>id</code> gefunden wird.
	 * @param parent Das Component das untersucht werden soll.
	 * @param id Der eindeutige Identifikator des gesuchten Components.
	 * @return <code>true</code> wenn gefunden, <code>false</code> sonst.
	 */
	private MComponent getMComponent(Object parent, ComponentId id) 
	{
		// Sonderbehandlung von Fenstern, Titel und Name mssen gleich sein.
		// Es kann aber vorkommen, dass sich der Fenstertitel whrend des Programmlaufs verndert.
		// naming.getName(parent) liefert dann den neuen Fenstertitel whrend id noch den alten
		// Fenstertitel beinhaltet
		if( id.getClassName().equals(HTSSyntax.TYPE_WINDOW) && resolver.getTypeName(parent).equals(HTSSyntax.TYPE_WINDOW) )
			return resolver.getMComponent(parent, id);
		
		if( resolver.getTypeName(parent).equals(id.getClassName()) 
				&& naming.getName(parent).equals(id.getName()) )
			return resolver.getMComponent(parent, id);
		
		Object[] children = getChildren(parent);
		if(children == null)
			return null;
		
		for(int i = 0; i < children.length; ++i)
		{
			MComponent comp = getMComponent(children[i], id);
			if(comp != null)
				return comp;
		}
		
		return null;
	}
	
	/**
	 * Initialisiert den <code>ICommandCreator</code> fr <code>component</code>. 
	 * Die Initialisierung erfolgt nur, falls es noch keinen <code>ICommandCreator</code>
	 * zu <code>component</code> gibt und wenn <code>component</code> ein gltiges Ziel
	 * fr das Aufzeichnen von Nutzeraktionen ist. 
	 * @param component Das Component fr welches das Capturing initialisiert werden soll.
	 * @see EventMonitorWrapper
	 */
	public void initCreator(Object component) 
	{
		if( !componentToCeatorMap.containsKey(component) )
		{
			Object window = getWindow(component);
			String title = WindowMonitor.getInstance().getTitle(window);
			if( title != null && isValidChild(component, window) )
			{
				ComponentId id = new ComponentId(
						naming.getName(component),
						resolver.getTypeName(component),
						title);
				
				ICommandCreator creator = resolver.getMComponent(component, id).initCommandCreator(receiver);
				componentToCeatorMap.put(component, creator);
			}
		}
	}
	
	/**
	 * Gibt an ob Nutzeraktionen fr <code>component</code> aufgezeichnet werden sollen.
	 * Fr bestimmte Components sollen keine Nutzeraktionen aufgezeichnet werden, beispielsweise
	 * fr Zelleneditoren einer Tabelle, die Nutzeraktionen auf den Zelleneditoren werden
	 * der Tabelle zugeordnet und sollen keine eigenstndigen ACTION Kommandos generieren. 
	 * @param component Das Component.
	 * @param container Das Component, welches <code>component</code> potentiell enthlt.
	 * @return <code>true</code>, wenn fr <code>component</code> Nutzeraktionen aufgezeichnet werden
	 * sollen, <code>false</code> sonst.
	 */
	private boolean isValidChild(Object component, Object container) 
	{
		if(component == container)
			return true;
		
		Object[] children = getChildren(container);
		if(children == null)
			return false;
		
		for (int i = 0; i < children.length; i++) 
		{
			if( isValidChild(component, children[i]) )
				return true;
		}
		
		return false;
	}
	
	/**
	 * Ermittelt das Fenster in dem <code>component</code> enthalten ist.
	 */
	public abstract Object getWindow(Object component);
	
	/**
	 * Ermittelt die <code>ComponentId</code>s, zu allen Components innerhalb des eines Fensters.
	 * @param window Das Fenster, Typ ist von der jeweiligen GUI-Bibliothek abhngig.
	 * @return Vector(ComponentId) Alle ComponentIds innerhalb von <code>window</code>.
	 * @see ComponentId
	 */
	public synchronized Vector getComponentIds(Object window) {
		Vector ids = new Vector(100); // der Vector fr die Rckgabewerte
		getComponentIds(window, WindowMonitor.getInstance().getTitle(window), ids);
		
		return ids;
	}
	
	/**
	 * Rekursive Hilfsfunktion fr das Ermitteln der <code>ComponentIds</code>
	 * innerhalb eines Fensters.
	 * @see #getComponentIds(Object)
	 */
	private void getComponentIds(Object parent, String title, Vector ids) 
	{
		ids.add( new ComponentId( 
				naming.getName(parent),
				resolver.getTypeName(parent),
				title)
		);
		
		Object[] children = getChildren(parent);
		if(children != null)
		{
			for (int i = 0; i < children.length; i++) 
				getComponentIds(children[i], title, ids);
		}
	}
	
	/**
	 * Gibt die zugeordnete Strategie zur Benamung von Components zurck.
	 */
	public NamingStrategy getNamingStrategy() {
		return naming;
	}
	
	/**
	 * Gibt den zugeordneten <code>ComponentResolver</code> zurck.
	 */
	public ComponentResolver getResolver() {
		return resolver;
	}
}
