package atosj.component;

import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.util.Vector;

import atosj.exception.ParseException;
import atosj.util.CSVLine;
import atosj.util.CSVParser;

/**
 * Liest nutzereigene Definitionen von Custom-Components aus einer Datei.
 * Diese Definitionen ordnen Components nutzerdefinierte Wrapperklassen, des
 * Typs <code>MComponent</code> zu.
 * zu. Die Wrapperklassen sind im Klassenpfad des Testobjektes. Bietet die
 * Mglichkeit die Wrapperklassen zur Laufzeit zu instanziieren.
 * @author Nicos Tegos
 * @see MComponent
 */
public abstract class CustomComponentResolver extends ComponentResolver
{
	private class CustomInfo implements Serializable {
		public String typeName;
		public String origClassName;
		public String wrapperClassName;
	}
	
	private Vector customInfos = new Vector();
	
	/**
	 * Erstellt einen neuen Resolver zur Zuordnung von nutzerdefinierten
	 * Wrapperklassen.
	 * @param filePath Der vollstndige Pfad zur Plugin Info Datei des Projektes, 
	 * diese muss nicht existieren.
	 */
	public CustomComponentResolver(String filePath) 
	{
		super();
		
		/*
		 * Format:
		 * COMPONENT, COMPONENTTYPE, origClassName, customWrapperClassName
		 */
		try
		{
			CSVParser parser = new CSVParser(filePath);
			CSVLine line = null;
			while( (line = parser.getNextLine() ) != null )
			{
				line.getElementUnquoted(0);
				CustomInfo info = new CustomInfo();
				info.typeName = line.getElement(1).toUpperCase();
				info.origClassName = line.getElement(2);
				info.wrapperClassName = line.getElement(3);
				
				customInfos.add(info);
			}
		}
		catch(IOException e){}
		catch(ParseException e) {}
	}
	
	/**
	 * Konstruiert ein neues Wrapperklassenobjekt zum angegebenen Component.
	 * @param component Die Instanz des Components.
	 * @param id Der eindeutige Bezeichner des Components.
	 * @return Das erstellte Objekt der Wrapperklasse oder <code>null</code>, falls keine
	 * passende Wrapperklasse gefunden wurde, bzw. deren Initialisierung fehlschlug.
	 */
	public MComponent getMComponent(Object component, ComponentId id) {
		for(int i = 0; i < customInfos.size(); ++i) 
		{
			CustomInfo info = (CustomInfo) customInfos.get(i);
			if( info.origClassName.equals(component.getClass().getName()) )
			{
				
				try {
					Class wrapperClass = Class.forName(info.wrapperClassName);
					Constructor c = wrapperClass.getConstructor( new Class[] {component.getClass(), ComponentId.class} );
					return (MComponent) c.newInstance( new Object[] {component, id} );
				} catch(Exception e) {
					return null;
					/*throw new AtosJException( "Die Wrapperklasse " + info.wrapperClassName + " fr die Custom-Component Klasse " 
							+ info.origClassName + " konnte nicht geladen werden: " + e.getMessage() );*/
				}
			}
		}
		
		return null;
	}
	
	/**
	 * Gibt an ob <code>component</code> ein Custom-Component ist.
	 * Es ist ein Custom-Component, wenn in der Plugin Info Datei
	 * ein Eintrag existiert, der diesem Componenttyp eine Custom-Wrapperklasse
	 * zuordnet.
	 * @param component Das Component.
	 * @return <code>true</code> wenn <code>component</code> ein
	 * Custom-Component ist, <code>false</code> sonst.
	 */
	public boolean isCustomComponent(Object component) {
		for(int i = 0; i < customInfos.size(); ++i) 
		{
			CustomInfo info = (CustomInfo) customInfos.get(i);
			if( info.origClassName.equals(component.getClass().getName()) )
				return true;
		}
		
		return false;
	}

	public String getTypeName(Object component) 
	{
		for(int i = 0; i < customInfos.size(); ++i) 
		{
			CustomInfo info = (CustomInfo) customInfos.get(i);
			if( info.origClassName.equals(component.getClass().getName()) )
				return info.typeName;
		}
		
		return "";
	}
	
	/**
	 * Gibt an ob <code>typeName</code> ein Custom-Typ ist.
	 * Er ist ein Custom-Component, wenn in der Plugin Info Datei
	 * ein Eintrag existiert, der diesen HTS Typnamen definiert.
	 * @param typeName Der HTS Typ.
	 * @return <code>true</code> wenn <code>typeName</code> ein
	 * nutzerdefiniert HTS Typ ist, <code>false</code> sonst.
	 */
	public boolean isCustomType(String typeName) {
		for(int i = 0; i < customInfos.size(); ++i) 
		{
			if( ((CustomInfo) customInfos.get(i)).typeName.equals(typeName) )
				return true;
		}
		
		return false;
	}
	
	public String[] getTypeNames() 
	{
		String[] typeNames = new String[customInfos.size()];
		for(int i = 0; i < customInfos.size(); ++i) 
			typeNames[i] = ((CustomInfo) customInfos.get(i)).typeName;
		
		return typeNames;
	}
	
	/**
	 * Die default Implementation liefert immer <code>false</code>.
	 */
	public boolean isCollectionComponent(String type) {
		return false;
	}
}
