//$Id$
package  atosj.component;

import atosj.Messages;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.recorder.IHTSCommandReceiver;
import atosj.util.WindowMonitor;

/**
 * Diese Klasse ist das Bindeglied zwischen ATOSj und den Components
 * der jeweiligen GUI-Bibliothek. Sie ist die Basisklasse aller Wrapperklassen,
 * die die Funktionalitt von HTS Typen auf den tatschlichen Components umsetzen
 * und auch das Aufzeichnen von Nutzeraktionen realisieren. Instanzen der Klasse
 * werden zur Laufzeit des Testobjektes erstellt.
 * @author Nicos Tegos
 */
public abstract class MComponent implements IComponent
{
	public static final int OP_GET_ENABLED = 0;
	public static final int OP_GET_VISIBLE = 1;
	public static final int OP_GET_CHECKED = 2;
	public static final int OP_GET_FOCUS = 3;
	public static final int OP_GET_TEXT = 4;
	public static final int OP_GET_NUM = 5;
	public static final int OP_CLOSE = 6;
	public static final int OP_MAXIMIZE = 7;
	public static final int OP_MINIMIZE = 8;
	public static final int OP_NORMALIZE = 9;
	public static final int OP_CLICK = 10;
	public static final int OP_SELECT = 11;
	public static final int OP_SELECTLAST = 12;
	public static final int OP_DESELECT = 13;
	public static final int OP_UP = 14;
	public static final int OP_DOWN = 15;
	public static final int OP_LEFT = 16;
	public static final int OP_RIGHT = 17;
	public static final int OP_SET_TEXT = 18;
	public static final int OP_SET_CHECKED = 19;
	public static final int OP_SET_FOCUS = 20;
	public static final int OP_ADDSELECT = 21;
	public static final int OP_GET_ITEMCOUNT = 22;
	public static final int OP_GET_COLCOUNT = 23;
	
    private Object _component;
    private ComponentId _id;
           	
    /**
     * Erstellt einen Wrapper fr ein Component der aktuellen GUI-Architektur.
     * @param component Das graph. Objekt.
     * @param id Der eindeutige Bezeichner des components.
     */
    public MComponent(Object component, ComponentId id) {
        _component = component;
        _id = id;
    }

    /**
     * Gibt das gewrappte Component zurck.
     */
    public Object getComponent() {
    	return _component;
    }
    /**
     * Gibt Die ID fr das gewrappte Component zurck.
     */
    public ComponentId getComponentId() {
    	return _id;
    }
    
    /**
     * Convenience Funktion, die den aktuellen <code>ComponentResolver</code> zurckgibt.
     */
    public ComponentResolver getResolver() {
    	return WindowMonitor.getInstance().getComponentFinder().getResolver();
    }
    
    /**
     * Wartet bis alle Events aus der Eventqueue verarbeitet wurden.
     */
    public abstract void sync() throws ComponentException;
    
    /**
     * Wenn wir im Record Modus sind, wird hier der <code>CommandCreator</code> erstellt.
     * @param receiver Der Empfnger aufgezeichneter Aktionen.
     * @return der neu erstellte <code>CommandCreator</code>.
     */
    public abstract ICommandCreator initCommandCreator(IHTSCommandReceiver receiver);

	/**
	 * Zwei <code>MComponents</code> sind gleich, wenn sie das gleiche Component
	 * enthalten.
	 */
    public boolean equals(Object obj) {
        if (obj instanceof MComponent) {
            return _component.equals(((MComponent) obj)._component);
        } else {
            return false;
        }
    }

    public int hashCode() {
        return _component.hashCode();
    }
    
    /**
	 * Simuliert einen Tastendruck. Die Parameter sind abhngig von der verwendeten
	 * GUI-Bibliothek.
	 * @param character Das Zeichen, das eingegeben werden soll.
	 * @param keyCode Der keyCode des Zeichens oder des virtuellen keys,
	 * der gedrckt werden soll.
	 * @param modifiers Die modifier, die gleichzeitig mit dem Zeichen gedrckt werden
	 * sollen.
	 */
	public abstract void pressKey(char character, int keyCode, int modifiers) throws ComponentException;

    /**
     * @throws ComponentException Wenn das Component nicht durch den Nutzer nderbar ist,
     * aufgrund von Unsichtbarkeit, weil es nicht aktiviert ist oder weil es in einem nicht
     * aktivierbaren Fenster liegt.
     * @see #isEnabled()
     * @see #isVisible()
     * @see #isFocussable()
     */
	public void assertEditable() throws ComponentException
	{
	    if( !isVisible() )
	    	throw new ComponentException(Messages.getString("MComponent.0")); //$NON-NLS-1$
	    
	    if( !isEnabled() )
	    	throw new ComponentException(Messages.getString("MComponent.1")); //$NON-NLS-1$
	    
	    if( !isFocussable() )
	    	throw new ComponentException(Messages.getString("MComponent.2")); //$NON-NLS-1$
	}
	
	/**
     * @throws ComponentException Wenn ein Zustand des Components nicht ausgelesen werden kann,
     * weil es unsichtbar ist.
     */
	public void assertReadable() throws ComponentException
	{
		if( !isVisible() )
	    	throw new ComponentException(Messages.getString("MComponent.3")); //$NON-NLS-1$
	}
	
	/**
	 * berprft ob das Element in einem Fenster liegt, das aktiviert werden kann.
	 * Also NICHT in einem NICHT aktivierten Fenster liegt, wenn ein applikations- oder 
	 * systemmodales Fenster geffnet wurde.
	 */
	public abstract boolean isFocussable() throws ComponentException;

    public boolean isEnabled() throws ComponentException {
		throw new InvalidOperationException(this, OP_GET_ENABLED, true);
	}	
	
    public boolean isVisible() throws ComponentException {
		throw new InvalidOperationException(this, OP_GET_VISIBLE, true);
	}
	
	public boolean hasFocus() throws ComponentException {
		throw new InvalidOperationException(this, OP_GET_FOCUS, true);
	}
	
	public void setProperty(String property, String value) throws ComponentException {
		throw new ComponentException(Messages.getString("MComponent.4")); //$NON-NLS-1$
	}
	
	public String getProperty(String property) throws ComponentException {
		throw new ComponentException(Messages.getString("MComponent.5")); //$NON-NLS-1$
	}
}
