/*
 * Created on 10.06.2006
 */
package atosj.component.swing;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import atosj.component.CommandCreator;
import atosj.dialog.AWTTestDialog;
import atosj.exception.ComponentException;
import atosj.model.hts.command.HTSTest;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Ein <code>CommandCreator</code> fr die Swing Architektur.
 * Implementiert die Aufzeichnung von Tastatureingaben und Mausklicks.
 * @author Nicos Tegos
 */
public class SwingCommandCreator extends CommandCreator
{
	public SwingCommandCreator(SwingComponent component, IHTSCommandReceiver receiver) {
		super(component, receiver);
		
		// TODO Default Implementation fr das Erstellen der CLICK Aktion
		addListener( new MouseAdapter() {} );
		addListener( new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				onKeyPressed(e);
			}
		} );
	}
	
	/**
	 * Implementiert den MouseListener fr den Aufruf des Dialogs
	 * zur Erstellung von TEST-Kommandos.
	 * @see CommandCreator#hookTestListener()
	 */
	public final void hookTestListener() {
		addListener( new MouseAdapter() {
			public void mousePressed(MouseEvent e) 
			{
				if(e.getButton() == MouseEvent.BUTTON3 && (e.getModifiers() & KeyEvent.CTRL_MASK) == KeyEvent.CTRL_MASK)
				{
					AWTTestDialog dlg = new AWTTestDialog( e.getComponent(), SwingCommandCreator.this );
					if( dlg.getReturnCode() == AWTTestDialog.OK )
					{
						HTSTest[] selectedElements = dlg.getSelectedElements();
						for (int i = 0; i < selectedElements.length; i++) 
							publishCommand(selectedElements[i]);
					}
				}
			}
		});
	}
	
	/**
	 * Wird bei einem KeyPressed Event gerufen. Diese Implementation generiert das Kommando
	 * ACTION, PRESSKEY ...
	 * Abgeleitete Klassen knnen diese Funktionalitt erweitern oder ignorieren.
	 * @param e Das enstrechende KeyEvent.
	 */
	protected void onKeyPressed(KeyEvent e) 
	{
		if( Character.isLetterOrDigit( (char)e.getKeyCode() ) )
		{
			Vector parameters = new Vector(4);
			parameters.add( "\"" + ((char)e.getKeyCode()) + "\"" );
			if( (e.getModifiers() & KeyEvent.ALT_MASK) != 0 )
				parameters.add(HTSSyntax.ACTION_ALT);
			if( (e.getModifiers() & KeyEvent.CTRL_MASK) != 0 )
				parameters.add(HTSSyntax.ACTION_CTRL);
			if( (e.getModifiers() & KeyEvent.SHIFT_MASK) != 0 )
				parameters.add(HTSSyntax.ACTION_SHIFT);
			
			publishActionCommand(HTSSyntax.ACTION_PRESSKEY, parameters, null);
		}
	}

	protected SwingComponent getAWTComponent() {
		return (SwingComponent) getMComponent();
	}

	/**
	 * Generiert TEST Kommandos fr die Zustnde VISIBLESTATE, ENABLESTATE und FOCUSSTATE.
	 * Abgeleitete Klassen drfen der Liste der generierten Kommandos beliebig viele
	 * weitere TEST Kommandos hinzufgen. Ein TEST Kommando kann mit Hilfe der Funktion
	 * createTestCommand erstellt werden.
	 * @return Vector(HTSTest) Liste der generierten TEST Kommandos.
	 * @see CommandCreator#createTestCommand(String, Vector, Vector)
	 */
	public Vector getTestCommands() 
	{
		Vector commands = new Vector();
		
		try {
			Vector values = new Vector(1);
			boolean visible = getMComponent().isVisible();
			values.add(visible ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE);
			commands.add( createTestCommand(HTSSyntax.READ_VISIBLESTATE, values, null) );
		} catch(ComponentException e){}
		
		try {
			Vector values = new Vector(1);
			boolean enabled = getMComponent().isEnabled();
			values.add(enabled ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE);
			commands.add( createTestCommand(HTSSyntax.READ_ENABLESTATE, values, null) );
		} catch(ComponentException e){}
		
		try {
			Vector values = new Vector(1);
			boolean focus = getMComponent().hasFocus();
			values.add(focus ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE);
			commands.add( createTestCommand(HTSSyntax.READ_FOCUSSTATE, values, null) );
		} catch(ComponentException e){}
		
		return commands;
	}
}
