/*
 * Created on 09.04.2006
 */
package atosj.component.swing;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Point;
import java.awt.Window;

import javax.swing.SwingUtilities;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.MComponent;
import atosj.event.awt.FireableKeyEvent;
import atosj.event.awt.FireableMouseClickEvent;
import atosj.event.awt.FireableSwingEvent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Basisklasse aller Wrapper der Components der Swing Architektur. Zur Aufzeichnung von
 * Kommandos dient der <code>SwingCommandCreator</code>. 
 * @author Nicos Tegos
 * @see atosj.component.swing.SwingCommandCreator
 */
public class SwingComponent extends MComponent 
{
	private static Runnable NULL_RUNNABLE = new Runnable() { public void run() {} };
		
	public SwingComponent(Component component, ComponentId id) {
        super(component, id);
    }

    public Component getAWTComponent() {
    	return (Component) getComponent();
    }
    
    /**
     * Die Funktionsweise gleicht der zweiten <code>pressKey</code> Methode, jedoch wird
     * der Code des eingegebenen Zeichens automatisch ermittelt. Die Ermittlung
     * des Codes funktioniert nur fr Zeichen und nicht fr virtuelle Tasten.
     * @see #pressKey(char, int, int)
     */
    public void pressKey(char character, int modifiers) throws ComponentException 
    {
    	assertEditable();
    	requestFocus();
    	
    	new FireableKeyEvent(this).fire(character, FireableSwingEvent.convertToSwingModifiers(modifiers));  	
    }

    /* (non-Javadoc)
	 * @see atosj.component.IComponent#pressKey(char, int, int)
	 */
	public void pressKey(char character, int keyCode, int modifiers) throws ComponentException
	{
		// TODO diese Funktion vielleicht in assertKeyPressable umbenennen 
		// und nur noch hier verwenden, dafr an allen anderen stellen herauslschen
		// denn sonst ergibt sich fast immer eine doppelte ausfhrung von
		// assertEditable, was durch das suchen des Fenster ziemlich zeitintensiv ist.
		// diese Funktion sollte vorher den Fokus requesten und testen
		// ob das Component auch den Focus hat
		// auch fr SWT berprfen
		assertEditable();
		requestFocus();
		
		new FireableKeyEvent(this).fire(keyCode, character, modifiers);
	}
	
	/* (non-Javadoc)
	 * @see atosj.component.IComponent#click(int, boolean, int)
	 */
	public void click(int numClicks, boolean left, int modifiers) throws ComponentException
	{
		Dimension dimension = getAWTComponent().getSize();
		Point point = new Point(dimension.width / 2, dimension.height / 2);
		// click auf die Mitte
		click(numClicks, left, modifiers, point);
	}
	
	/**
	 * Simuliert einen Mausklick auf eine bestimmte Position innerhalb des Components.
	 * @param numClicks Anzahl der Klicks (2 sind ein Doppelklick).
	 * @param left wenn <code>true</code> Linksklick, sonst Rechtsklick.
	 * @param modifiers Die AtosJ ModifierKey Konstanten verknpft durch
	 * bitweises oder, siehe <code>MODIFIER_*</code>. Oder <code>NO_MODIFIER</code> falls kein 
	 * Modifier gedrckt.
	 * @param p Der Punkt (lokal zum Component) an den geklickt werden soll.
	 * @see atosj.component.IComponent#click(int, boolean, int)
	 */
	protected void click(int numClicks, boolean left, int modifiers, Point p) throws ComponentException 
	{
		// TODO diese Funktion vielleicht in assertClickable umbenennen 
		// und nur noch hier verwenden, dafr an allen anderen stellen herauslschen
		// denn sonst ergibt sich fast immer eine doppelte ausfhrung von
		// assertEditable, was durch das suchen des Fenster ziemlich zeitintensiv ist.
		// auch fr SWT berprfen
		assertEditable();
		
		new FireableMouseClickEvent(this).fire(
				p, 
				numClicks, 
				!left, 
				FireableSwingEvent.convertToSwingModifiers(modifiers)
			);
	}
	    
	public void sync() throws ComponentException
	{
		EventQueue eventQueue = getAWTComponent().getToolkit().getSystemEventQueue();
		int count = 0;
		
		do 
		{
			if (++count > 1000)
				throw new ComponentException(Messages.getString("SwingComponent.0")); //$NON-NLS-1$

			try {
				if (count % 20 == 0)
					Thread.sleep(50);

				SwingUtilities.invokeAndWait(NULL_RUNNABLE);
			} 
			catch (Exception e) {	}
		} 
		while(eventQueue.peekEvent() != null); // so lange noch events in der event queue
    }

    public boolean isEnabled() {
		return getAWTComponent().isEnabled();
	}
	
	public boolean isVisible() {
		return getAWTComponent().isShowing();
	}
	
	/**
	 * Setzt den Focus auf das aktuelle Element, bei Bedarf wird das
	 * Fenster aktiviert.
	 * @throws ComponentException Wenn danach der Fokus nicht auf dem Element liegt.
	 */
	public void requestFocus() throws ComponentException 
	{
		if( hasFocus() )
			return;
		
		if( getAWTComponent().isFocusable() )
			getAWTComponent().requestFocus(); // generiert auch ein entsprechendes FocusEvent
		else
			throw new InvalidOperationException(this, OP_SET_FOCUS, true);
	}
	
	public boolean hasFocus() {
		return getAWTComponent().hasFocus();
	}
	
	public boolean isFocussable() 
	{
		if( hasFocus() )
			return true;
		
		Component root = SwingUtilities.getRoot( getAWTComponent() );
		if( root instanceof Window )
		{
			if( root.hasFocus() )
				return true;
			
			return ((Window)root).isFocusable();
		}
		else // ein Applet, da knnen wir das Fenster nicht feststellen
			return true;
	}

	protected void postEvent(AWTEvent e)	{
		EventQueue eventQueue = getAWTComponent().getToolkit().getSystemEventQueue();
		while (eventQueue.peekEvent() != null)	{}
		eventQueue.postEvent(e);
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new SwingCommandCreator(this, receiver);
	}
}
