package atosj.component.swing;

import java.awt.Frame;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IWindow;
import atosj.event.awt.CloseMainWindowEvent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;
import atosj.util.Snooze;

/**
 * Wrapperklasse fr alle von <code>java.awt.Frame</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SwingFrame extends SwingComponent implements IWindow
{
	/**
	 * Zeichnet das Schlieen, Maximieren, Minimieren und Normalisieren des
	 * Fensters auf.
	 */
	protected class FrameCommandCreator extends SwingCommandCreator
	{
		private boolean maximized;
		public FrameCommandCreator(SwingFrame frame, IHTSCommandReceiver receiver) 
		{
			super(frame, receiver);
			maximized = isMaximized();
			
			addListener( new WindowAdapter() {
				public void windowClosing(WindowEvent arg0) {
					publishActionCommand(HTSSyntax.ACTION_CLOSE, null, null);
				}

				public void windowDeiconified(WindowEvent arg0) {
					publishActionCommand( isMaximized() ? HTSSyntax.ACTION_MAXIMIZE : HTSSyntax.ACTION_NORMALIZE, null, null );
				}

				public void windowIconified(WindowEvent arg0) {
					publishActionCommand(HTSSyntax.ACTION_MINIMIZE, null, null);
				}
			});
			
			addListener(new ComponentAdapter() {
				public void componentResized(ComponentEvent e) {
					if( maximized != isMaximized() )
						publishActionCommand( isMaximized() ? HTSSyntax.ACTION_MAXIMIZE : HTSSyntax.ACTION_NORMALIZE, null, null );
					maximized = isMaximized();
				}
			});
		}

		private Frame getFrame() {
			return ((SwingFrame)getMComponent()).getFrame();
		}
		
		private boolean isMaximized() {
			return (getFrame().getExtendedState() & Frame.MAXIMIZED_BOTH) == Frame.MAXIMIZED_BOTH;
		}
	}
	
	public SwingFrame(Frame button, ComponentId id) {
		super(button, id);
	}
	
	private Frame getFrame() {
		return (Frame) getComponent();
	}
	
	public void close() throws ComponentException 
	{
		assertEditable();
		new CloseMainWindowEvent(this).fire();
	}

	public void maximize() throws ComponentException
	{
		assertEditable();
		
		Frame frame = getFrame();
		if (frame.isResizable() && !(frame.getExtendedState()==Frame.MAXIMIZED_BOTH))	{
			frame.setExtendedState(Frame.MAXIMIZED_BOTH);
			postEvent(new WindowEvent(frame, WindowEvent.WINDOW_STATE_CHANGED));
			new Snooze(500);
		}
		else {
			throw new InvalidOperationException(this, OP_MAXIMIZE, false);
		}
	}

	public void minimize() throws ComponentException
	{
		assertEditable();
		
		Frame frame = getFrame();
		if ( !(frame.getExtendedState()==Frame.ICONIFIED) )	
		{
			//postEvent(new WindowEvent(frame, WindowEvent.WINDOW_GAINED_FOCUS));
			frame.setExtendedState(Frame.ICONIFIED);
			postEvent(new WindowEvent(frame, WindowEvent.WINDOW_ICONIFIED));
			new Snooze(500);
			
		}
		else
			throw new InvalidOperationException(this, OP_MINIMIZE, false);
	}

	public void normalize() throws ComponentException
	{
		assertEditable();
		
		Frame frame = getFrame();
		if( !(frame.getExtendedState()==Frame.NORMAL) )	
		{
			frame.setExtendedState(Frame.NORMAL);
			new WindowEvent(frame, WindowEvent.WINDOW_STATE_CHANGED);
		}
		else
			throw new InvalidOperationException(this, OP_NORMALIZE, false);
	}

	public String getText() throws ComponentException {
		return getFrame().getTitle();
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new FrameCommandCreator(this, receiver);
	}
}
