/*
 * Created on 29.12.2005
 */
package atosj.component.swing;

import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Vector;

import javax.swing.JList;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.ILabel;
import atosj.component.IList;
import atosj.component.IListCommandCreator;
import atosj.component.ListCommandCreatorHelper;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapper fr alle von <code>javax.swing.JList</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SwingList extends SwingComponent implements IList 
{
	/**
	 * Zeichnet nderungen der Selektion in der Liste auf.
	 */
	protected class ListCommandCreator extends SwingCommandCreator implements ListSelectionListener, IListCommandCreator
	{
		private ListCommandCreatorHelper helper;
		
		public ListCommandCreator(SwingList list, IHTSCommandReceiver receiver) {
			super(list, receiver);
			helper = new ListCommandCreatorHelper(this);
			
			addListener(this);
		}
		
		public Vector getTestCommands() 
		{
			Vector commands = super.getTestCommands();
			
			try {
				Vector values = new Vector(1);
				int num = ((IList)getMComponent()).getItemCount();
				values.add("" + num); //$NON-NLS-1$
				commands.add( createTestCommand(HTSSyntax.READ_ITEMCOUNT, values, null) );
			} catch(ComponentException e){}
			
			Object[] selection = getSelection();
			for(int i = 0; i < selection.length; ++i)
			{
				try {
					Vector values = new Vector(1);
					String txt = ((IList)getMComponent()).getText( ((Integer)selection[i]).intValue() );
					values.add( '"' + txt + '"' );
					commands.add( createTestCommand(HTSSyntax.READ_TEXT, values, convertSelectionObjectToPath(selection[i])) );
				} catch(ComponentException e){}
			}
			
			return commands;
		}
		
		public void valueChanged(ListSelectionEvent e) {
			helper.selectionChanged();
		}

		public Object[] getSelection() {
			return ListCommandCreatorHelper.convertToIntegerArray( ((SwingList)getMComponent()).getList().getSelectedIndices() );
		}

		public Vector convertSelectionObjectToPath(Object selectionObject) {
			Vector path = new Vector(1);
			// das selectionObject ist ein listenindex als Integer
			path.add( selectionObject.toString() );
			return path;
		}
	}
	
	public SwingList(JList component, ComponentId id) {
		super(component, id);
	}

	public JList getList() {
		return (JList) getComponent();
	}
	
	public void select(String item) throws ComponentException {
		select( findItem(item) );		
	}

	public void select(int index) throws ComponentException 
	{
		assertEditable();
		assertIndex(index);
		
		revealSubItem(index);
		clickOnItem(index, 1, true, NO_MODIFIER);
	}
	
	public void deselect(int index) throws ComponentException 
	{
		assertEditable();
		assertIndex(index);
		
		if( getList().getSelectionMode() == ListSelectionModel.SINGLE_SELECTION )
			throw new InvalidOperationException(this, OP_DESELECT, false);
		
		revealSubItem(index);
		
		if( getList().isSelectedIndex(index) )
			clickOnItem(index, 1, true, MODIFIER_CTRL);		
	}
	
	public void deselect(String item) throws ComponentException {
		deselect( findItem(item) );
	}

	public void addSelect(int index) throws ComponentException 
	{
		assertEditable();
		assertIndex(index);
		
		if( getList().getSelectionMode() == ListSelectionModel.SINGLE_SELECTION )
			throw new InvalidOperationException(this, OP_ADDSELECT, false);
		
		revealSubItem(index);
		if( !getList().isSelectedIndex(index) )
			clickOnItem(index, 1, true, MODIFIER_CTRL);
	}

	public void addSelect(String item) throws ComponentException {
		addSelect( findItem(item) );
	}

	public int getItemCount() throws ComponentException {
		return getList().getModel().getSize();		
	}

	public String getText(int index) throws ComponentException 
	{
		Component c = getList().getCellRenderer().getListCellRendererComponent(
				getList(),
				getList().getModel().getElementAt(index),
				index,
				getList().isSelectedIndex(index),
				getList().hasFocus()
				);
		
		if(c == null)
			throw new ComponentException(Messages.getString("SwingList.1")); //$NON-NLS-1$
		
		MComponent renderer = getResolver().getMComponent(c, getComponentId());
		if(renderer instanceof ILabel)
			return ((ILabel)renderer).getText();
		else
			throw new ComponentException(Messages.getString("SwingList.2")); //$NON-NLS-1$
	}
	
	/**
	 * Stellt die Gltigkeit des angegebenen Listenindex sicher.
	 * @param index Der Listenindex.
	 * @throws ComponentException wenn der Listenindex ungltig ist.
	 */
	private void assertIndex(int index) throws ComponentException 
	{
		if( index < 0 || index >= getItemCount() )
		{
			int count = getItemCount();
			if( count > 0 )
				throw new ComponentException( Messages.getString("SwingList.3") + index + Messages.getString("SwingList.4") + (count-1)); //$NON-NLS-1$ //$NON-NLS-2$
			else
				throw new ComponentException( Messages.getString("SwingList.5") + index + Messages.getString("SwingList.6")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	/**
	 * Rckt den Listeneintrag an Position <code>index</code>
	 * in den sichtbaren Bereich.
	 * @param index Der Index des Eintrages, 0-basiert.
	 */
	private void revealSubItem(int index) {
		Rectangle rect = getList().getCellBounds(index, index);
		getList().scrollRectToVisible(rect);
	}
	
	/**
	 * Simuliert einen Klick auf die Mitte des angegebenen Listeneintrags.
	 * Der Eintrag muss vorher sichtbar gemacht werden.
	 * @param index Der Index des Eintrages, 0-basiert.
	 * @see #revealSubItem(int)
	 */
	private void clickOnItem(int index, int numOfClicks, boolean left, int modifiers) throws ComponentException 
	{
		Rectangle rect = getList().getCellBounds(index, index);
		click( numOfClicks, left, modifiers, new Point(rect.x + rect.width/2, rect.y + rect.height/2) );
	}
	
	/**
	 * Gibt den Index des ersten Listeneintrages mit dem angegebenen Text zurck.
	 * @param text Der Text des gesuchten Eintrages.
	 * @return den Index.
	 * @throws ComponentException wenn kein Eintrag mit <code>text</code> gefunden wurde.
	 */
	private int findItem(String text) throws ComponentException {
		for(int i = 0; i < getItemCount(); ++i)
			if( getText(i).equals(text) )
				return i;
		
		throw new ComponentException(Messages.getString("SwingList.7") + text + Messages.getString("SwingList.8")); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ListCommandCreator(this, receiver);
	}
}
