package atosj.component.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ComponentResolver;
import atosj.component.ICheckbox;
import atosj.component.ICommandCreator;
import atosj.component.IMenu;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;
import atosj.util.Snooze;

/**
 * Wrapper fr alle von <code>javax.swing.JMenu</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SwingMenu extends SwingComponent implements IMenu 
{
	/**
	 * Zeichnet Ereignisse auf allen Untermenpunkten des Mens auf.
	 */
	private class MenuCommandCreator extends SwingCommandCreator implements ActionListener
	{
		private HashMap itemToPathMap = new HashMap(); // HashMap<JMenuItem, Vector<String>> mapped ein item auf seinen subitemPath
		public MenuCommandCreator(SwingMenu menu, IHTSCommandReceiver receiver) {
			super(menu, receiver);
			
			// wir interessieren uns nur fr das Hauptmen, da wir alle 
			// MenuItems rekursiv durchgehen und ActionListener initialisieren
			if( menu.getMenu().isTopLevelMenu() )
				initMap(menu.getMenu(), new Vector());
		}
		
		/**
		 * Ordent rekursiv jedem Menpunkt seinen Pfad zu.
		 * @param parent Das Obermen.
		 * @param path Vector(String) Die Namen aller bergeordneten Mens.
		 */
		private void initMap(JMenu parent, Vector path)
		{
			for(int i = 0; i < parent.getItemCount(); ++i) 
			{
				JMenuItem item = parent.getItem(i);
				if( item instanceof JMenu )
				{
					// mens selbst interessieren uns nicht, aber deren Kinder
					Vector myPath = new Vector(path);
					myPath.add( '"' + item.getText() + '"');
					initMap( (JMenu)item, myPath);
				}
				else
				{
					Vector myPath = new Vector(path);
					myPath.add( '"' + item.getText() + '"');
					itemToPathMap.put( item, myPath );
					// leider bekommen wir vom SWTEventMonitor keine
					// events zu den items sondern nur zum Men
					// darum machen wir das, was wir eigentlich nicht machen wollten
					// wir hngen uns direkt an das item an
					item.addActionListener(this);
				}
			}
		}

		public void actionPerformed(ActionEvent e) 
		{
			Object item = e.getSource();
			if(item instanceof JCheckBoxMenuItem)
			{
				publishActionCommand(((JCheckBoxMenuItem)item).isSelected() ? HTSSyntax.ACTION_CHECK : HTSSyntax.ACTION_UNCHECK, 
						null, 
						(Vector) itemToPathMap.get(item)
					);			
			}
			else if(item instanceof JRadioButtonMenuItem)
			{
				if( ((JRadioButtonMenuItem)item).isSelected() )
					publishActionCommand( HTSSyntax.ACTION_CHECK, null, (Vector) itemToPathMap.get(item) );
			}
			else
			{
				publishActionCommand( HTSSyntax.ACTION_SELECT, null, (Vector) itemToPathMap.get(item) );
			}
		}
	}
	
	private ComponentResolver resolver;
	public SwingMenu(JMenu component, ComponentId id, ComponentResolver resolver) {
		super(component, id);
		this.resolver = resolver;
	}
	
	private JMenu getMenu() {
		return (JMenu) getAWTComponent();
	}
	
	/**
	 * Sucht das <code>JMenuitem</code> mit dem angegebenen Text, nicht
	 * rekursiv.
	 * @param menu Das Men in dem gesucht werden soll
	 * @param text Der gesuchte Text.
	 * @return Der Menpunkt mit dem angegebenen Text oder <code>null</code>,
	 * wenn der Menpunkt nicht gefunden wurde.
	 */
	private JMenuItem findItem(JMenu menu, String text)
	{
		int numItems = menu.getItemCount();
		for(int i = 0; i < numItems; ++i)
		{
			if( menu.getItem(i).getText().equals(text) )
			{
				return menu.getItem(i);
			}
		}
		
		return null;
	}
	
	private ComponentId createIdForSubItem(String[] path, int index) {
		// ComponentId fr das SubElement erstellen
		String pathStr = ""; //$NON-NLS-1$
		for(int i = 0; i <= index; ++i) 
		{
			pathStr += path[i];
			if(i < index)
				pathStr += "->"; //$NON-NLS-1$
		}
		
		ComponentId subId = new ComponentId(
				getComponentId().getName() + Messages.getString("SwingMenu.2") + pathStr, //$NON-NLS-1$
				getComponentId().getClassName(),
				getComponentId().getWindowName()
		);
		
		return subId;
	}
	
	/**
	 * Findet den Untermenpunkt mit dem angegeben Pfad.
	 * @param path Labels der Meneintrge hierarchisch geordnet,
	 * mit root als erstem Element.
	 * @param show Wenn true wird das Subitem durch clicken der einzelnen bergeordnenten mens sichtbar gemacht.
	 * @throws ComponentException wenn der Menpfad inkorrekt ist.
	 */
	private MComponent getSubitem(String path[], boolean show) throws ComponentException 
	{
		JMenu search = getMenu();
		JMenuItem item = null;
		
		if(show)
			click(1, true, 0);
		
		for(int i = 0; i < path.length; ++i)
		{
			item = findItem(search, path[i]);
			
			if(item == null)
				throw new ComponentException(Messages.getString("SwingMenu.3") + (i+1) + Messages.getString("SwingMenu.4") + path[i] + Messages.getString("SwingMenu.5")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			if(item instanceof JMenu)
			{
				if(show)
				{
					// es dauert immer eine weile bis die popups sich ffnen daher mssen wir warten
					while( !item.isShowing() )
						new Snooze(10);
					resolver.getMComponent( item, createIdForSubItem(path, i) ).click(1, true, NO_MODIFIER);
				}
				search = (JMenu)item;
			}
			else
			if(i != path.length - 1) // zu frh gefunden
			{
				throw new ComponentException(Messages.getString("SwingMenu.6") + (i+1) + Messages.getString("SwingMenu.7") + path[i] + Messages.getString("SwingMenu.8")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
		}
		
		// es dauert immer eine weile bis die popups sich ffnen daher mssen wir warten
		while( show && !item.isShowing() )
			new Snooze(10);
		return resolver.getMComponent( item, createIdForSubItem(path, path.length-1) );
	}

	public void select(String[] path) throws ComponentException {
		getSubitem(path, true).click(1, true, NO_MODIFIER);
	}

	public void setChecked(String[] path, boolean check) throws ComponentException 
	{
		MComponent item = getSubitem(path, false);
		if(item instanceof ICheckbox)
			((ICheckbox)item).setChecked(check);
		else
			throw new InvalidOperationException(item, OP_SET_CHECKED, true);
	}
	
	public boolean isChecked(String[] path) throws ComponentException 
	{
		MComponent item = getSubitem(path, false);
		if(item instanceof ICheckbox)
			return ((ICheckbox)item).isChecked();
		else
			throw new InvalidOperationException(item, OP_SET_CHECKED, true);
	}
	
	public boolean isEnabled(String[] path) throws ComponentException {
		MComponent item = getSubitem(path, false);
		return item.isEnabled();
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new MenuCommandCreator(this, receiver);
	}
}
