/*
 * Created on 27.12.2005
 */
package atosj.component.swing;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.KeyEvent;

import javax.swing.JTable;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICheckbox;
import atosj.component.IEditable;
import atosj.component.ILabel;
import atosj.component.ITableCell;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;

/**
 * Leitet Aktionen auf einer Tabellezelle an ein entsprechendes <code>MComponent</code> weiter.
 * Die Tabellenzelle ist ein virtuelles Konstrukt und hat keine direkte Entsprechung in Swing
 * sondern ist Teil der Tabelle. Diese Klasse sollte nur durch die entsprechende Tabelle erzeugt werden.
 * @author Nicos Tegos
 */
public class SwingTableCell extends SwingComponent implements ITableCell 
{
	private int row;
	private int col;
			
	/**
	 * Erzeugt eine Tabellenzelle.
	 * @param table Die Table in der diese Zelle enthalten ist.
	 * @param row Die Zeile dieser Zelle.
	 * @param col Die Spalte dieser Zelle.
	 */
	public SwingTableCell(SwingTable table, int row, int col) throws ComponentException 
	{
		super(table.getAWTComponent(),
				new ComponentId(
					table.getComponentId().getName() + Messages.getString("SwingTableCell.0") + row + Messages.getString("SwingTableCell.1") + col + ")", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					table.getComponentId().getClassName(),
					table.getComponentId().getWindowName()
				)
			);
		this.row = row;
		this.col = col;
	}

	public JTable getTable() {
		return (JTable) getAWTComponent();
	}
	
	/**
	 * Liefert den Editor der Tabellenzelle.
	 * @return Der Editor.
	 * @throws ComponentException, wenn kein Editor gefunden wurde.
	 */
	private MComponent getEditor() throws ComponentException {
		Component c = getTable().getEditorComponent();
		if(c == null)
			throw new ComponentException(Messages.getString("SwingTableCell.3") + row + Messages.getString("SwingTableCell.4") + col + Messages.getString("SwingTableCell.5") ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		
		return getResolver().getMComponent(c, getComponentId()); 
	}
	
	private MComponent getRenderer() throws ComponentException
	{
		Component c = 
			getTable().getCellRenderer(row, col).getTableCellRendererComponent(
				getTable(), getTable().getModel().getValueAt(row,col), 
				getTable().isCellSelected(row, col), 
				false, 
				row, 
				col);
		
		if(c == null)
			throw new ComponentException(Messages.getString("SwingTableCell.6") + row + Messages.getString("SwingTableCell.7") + col + Messages.getString("SwingTableCell.8") ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		
		return getResolver().getMComponent(c, getComponentId());
	}
	
	public void setText(String text) throws ComponentException
	{
		assertEditable();
		// assertEditable darf nicht berschrieben werden, weil es in der Funktion click verwendet wird
		if( !getTable().isCellEditable(row, col) )
			throw new ComponentException(Messages.getString("SwingTableCell.9") + row + Messages.getString("SwingTableCell.10") + col + Messages.getString("SwingTableCell.11") ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		
		getTable().changeSelection(row, col, false, false);
		getTable().editCellAt(row, col);
		MComponent editor = getEditor();
		if(editor instanceof IEditable)
			((IEditable)editor).setText(text);
		else
			throw new InvalidOperationException(editor, OP_SET_TEXT, true);
		// abschliessen des editierens durch enter
		// achtung auf KEINEN fall table.stopCellEditing benutzen, dies fhrt zu verhalten, was nicht mit der entsprechenden nutzeraktion konform ist
		editor.pressKey(KeyEvent.CHAR_UNDEFINED, KeyEvent.VK_ENTER, 0);
	}
	
	public void setChecked(boolean checked) throws ComponentException
	{
		assertEditable();

		getTable().changeSelection(row, col, false, false);
		getTable().editCellAt(row, col);
		MComponent editor = getEditor();
		if(editor instanceof ICheckbox)
			((ICheckbox)editor).setChecked(checked);
		else
			throw new InvalidOperationException(editor, OP_SET_CHECKED, true);
		// abschliessen des editierens ist hier nicht ntig
		// da der Wert sofort bernommen wird
	}
	
	public String getText() throws ComponentException
	{
		assertReadable();
		MComponent renderer = getRenderer();
		if(renderer instanceof ILabel)
			return ((ILabel)renderer).getText();
		
		throw new InvalidOperationException(renderer, OP_GET_TEXT, true);
	}
	
	public float getNum() throws ComponentException 
	{
		try {
			return Float.parseFloat( getText() );
		} catch(NumberFormatException e) {
			throw new InvalidOperationException(this, OP_GET_NUM, false);
		}
	}
	
	public boolean isChecked() throws ComponentException 
	{
		assertReadable();
		MComponent renderer = getRenderer();
		if(renderer instanceof ICheckbox)
			return ((ICheckbox)renderer).isChecked();
		
		throw new InvalidOperationException(renderer, OP_GET_CHECKED, true);
	}
	
	public void click(int numClicks, boolean left, int modifiers) throws ComponentException {
		// renderer.click(numClicks, left, modifiers); geht nicht, das MouseEvent muss direkt an die Tabelle verschickt werden
		Point p = getTable().getCellRect(row, col, false).getLocation();
		click(numClicks, left, modifiers, p);
	}
}
