/*
 * Created on 27.12.2005
 */
package atosj.component.swing;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeCellEditor;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IListCommandCreator;
import atosj.component.ITree;
import atosj.component.ITreeCell;
import atosj.component.ListCommandCreatorHelper;
import atosj.exception.ComponentException;
import atosj.exception.ComponentNotFoundException;
import atosj.exception.InvalidOperationException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>javax.swing.JTree</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SwingTree extends SwingComponent implements ITree
{
	/**
	 * Zeichnet das Selektieren und Editieren von Eintrgen des Baumes auf.
	 * Generiert TEST Kommandos zu den Zustnden des Baumes (ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class TreeCommandCreator extends SwingCommandCreator  implements TreeSelectionListener, PropertyChangeListener, CellEditorListener, IListCommandCreator
	{
		private ListCommandCreatorHelper helper;
		
		public TreeCommandCreator(SwingTree tree, IHTSCommandReceiver receiver) {
			super(tree, receiver);
			helper = new ListCommandCreatorHelper(this);
			
			addListener(this);
			if( getTree().getCellEditor() != null )
				// Das anhngen als globaler Listener ber AWTEventMonitor geht nicht, da offenbar der SwingEventMonitor nicht funktioniert
				getTree().getCellEditor().addCellEditorListener(this);	
		}
		
		private JTree getTree() {
			return ((SwingTree)getMComponent()).getTree();
		}
		
		public void valueChanged(TreeSelectionEvent e) {
			helper.selectionChanged();
		}
		
		public void propertyChange(PropertyChangeEvent e) 
		{
			if( e.getPropertyName().equals(JTree.CELL_EDITOR_PROPERTY) )
			{
				JTree tree = getTree();
				if( tree.getCellEditor() != null )
					addListener(tree.getCellEditor(), this);
			}
		}

		public void editingCanceled(ChangeEvent e) {
			// interessiert uns nicht
		}
		
		public void editingStopped(ChangeEvent e) {
			TreePath p = getTree().getEditingPath();
			Vector actionParams = new Vector(1);
			// wir nehmen einfach toString des Editorwertes
			actionParams.add( '"' + ((TreeCellEditor)e.getSource()).getCellEditorValue().toString() + '"' );
			publishActionCommand( HTSSyntax.ACTION_EDIT, actionParams, convertToSubitemPath(p) );
		}
		
		/**
		 * @param path Der TreePath.
		 * @return <p>Wandelt path in einen entsprechenden String Vector um
		 * (es wird die toString Methode der einzelnen Elemente des Modells benutzt),
		 * gem HTSSyntax sind dann alle Strings in Anfhrungszeichen.</p>
		 */
		private Vector convertToSubitemPath(TreePath path) {
			int len = path.getPathCount();
			Vector ret = new Vector(len);
			for (int i = 0; i < len; i++)
				ret.add( '"' + path.getPathComponent(i).toString() + '"' );
			
			return ret;
		}
		
		public Object[] getSelection() {
			return getTree().getSelectionPaths() == null ? new TreePath[0] : getTree().getSelectionPaths();
		}
		
		public Vector convertSelectionObjectToPath(Object selectionObject) {
			return convertToSubitemPath( (TreePath)selectionObject );
		}
	}
	
	public SwingTree(JTree component, ComponentId id) {
		super(component, id);
	}

	private JTree getTree() {
		return (JTree) getAWTComponent();
	}
	
	/**
	 * Wandelt den angegebenen Pfad in ein Objekt der Klasse <code>TreePath</code> um.
	 * @param path Der Pfad, Name des Knotens und aller Vaterknoten 
	 * beginnend beim Wurzelknoten.
	 * @return den <code>TreePath</code>.
	 * @throws ComponentNotFoundException, wenn der Pfad nicht gefunden wurde.
	 */
	private TreePath convertToTreePath(String[] path) throws ComponentNotFoundException
	{
		TreeModel model = getTree().getModel();
		Object parent = model.getRoot();
		
		if(parent == null)
			throw new ComponentNotFoundException( getComponentId(), Messages.getString("SwingTree.0") ); //$NON-NLS-1$
		
		Vector modelElements = new Vector(path.length);
		modelElements.add(parent);
		
		if( !path[0].equals(parent.toString()) )
			throw new ComponentNotFoundException( getComponentId(),  Messages.getString("SwingTree.1") + path[0] + Messages.getString("SwingTree.2") ); //$NON-NLS-1$ //$NON-NLS-2$
		
		// model elemente und String elemente sind gleich wenn sie
		// den gleichen string haben
		// bei 1 beginnend weil root schon behandelt wurde
		for(int i = 1; i < path.length; ++i)
		{
			Object nextString = path[i];
			// das nchste element herausfinden
			Object nextParent = null;
			for(int j = 0; j < model.getChildCount(parent); ++j)
			{
				if( nextString.equals(model.getChild(parent, j).toString()) )
				{
					nextParent = model.getChild(parent, j);
					modelElements.add(nextParent);
					break;
				}
			}
			
			// kindelement nicht gefunden
			if(nextParent == null)
				throw new ComponentNotFoundException( getComponentId(), Messages.getString("SwingTree.3") + (i+1) + Messages.getString("SwingTree.4") + path[i] + Messages.getString("SwingTree.5") ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			parent = nextParent;
		}
		
		return new TreePath( modelElements.toArray() );
	}
	
	/**
	 * Expandiert und rckt den angegebenen Knoten in
	 * den sichtbaren Bereich.
	 * @param path Der Pfad des Knotens.
	 */
	private void revealSubItem(TreePath path) {
		getTree().expandPath(path);
		getTree().scrollPathToVisible(path );
	}

	public void select(String[] path) throws ComponentException 
	{
		assertEditable();
		TreePath tPath = convertToTreePath(path);
		if( !getTree().getSelectionModel().isPathSelected(tPath) )
		{
			revealSubItem(tPath);
			ITreeCell cell = new SwingTreeCell(this, tPath);
			cell.click(1, true, NO_MODIFIER);
		}
	}

	public void deselect(String[] path) throws ComponentException 
	{
		assertEditable();
		if( getTree().getSelectionModel().getSelectionMode() == ListSelectionModel.SINGLE_SELECTION )
			throw new InvalidOperationException(this, OP_DESELECT, false);
		
		TreePath tPath = convertToTreePath(path);
		if( getTree().getSelectionModel().isPathSelected(tPath) )
		{
			revealSubItem(tPath);
			ITreeCell cell = new SwingTreeCell(this, tPath);
			cell.click(1, true, MODIFIER_CTRL);
		}
	}

	public void addSelect(String[] path) throws ComponentException {
		assertEditable();
		if( getTree().getSelectionModel().getSelectionMode() == ListSelectionModel.SINGLE_SELECTION )
			throw new InvalidOperationException(this, OP_ADDSELECT, false);
		
		TreePath tPath = convertToTreePath(path);
		if( !getTree().getSelectionModel().isPathSelected(tPath) )
		{
			revealSubItem(tPath);
			ITreeCell cell = new SwingTreeCell(this, tPath);
			cell.click(1, true, MODIFIER_CTRL);
		}
	}

	public ITreeCell getTreeCell(String[] path) throws ComponentException {
		TreePath tPath = convertToTreePath(path);
		revealSubItem(tPath);
		return new SwingTreeCell(this, tPath);
	}
	
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new TreeCommandCreator(this, receiver);
	}
}
