/*
 * Created on 27.12.2005
 */
package atosj.component.swing;

import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;

import javax.swing.JTree;
import javax.swing.tree.TreePath;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.IEditable;
import atosj.component.ITreeCell;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;

/**
 * Leitet Aktionen auf einem Baumknoten an seinen Baum weiter.
 * Der Baumknoten ist ein virtuelles Konstrukt und hat keine direkte Entsprechung in Swing
 * sondern ist Teil eines <code>JTree</code>. Diese Klasse sollte nur durch den entsprechenden
 * Baum erzeugt werden.
 * @author Nicos Tegos
 */
public class SwingTreeCell extends SwingComponent implements ITreeCell
{
	private TreePath path;
	
	/**
	 * Erzeugt einen neuen Knoten.
	 * @param parent Der Baum, der den Knoten enthlt.
	 * @param path Pfad des Knotens.
	 * @throws ComponentException
	 */
	public SwingTreeCell(SwingTree parent, TreePath path) throws ComponentException 
	{
		super(parent.getAWTComponent(),
			new ComponentId(
				parent.getComponentId().getName() + Messages.getString("SwingTreeCell.0") + path.toString() + ")", //$NON-NLS-1$ //$NON-NLS-2$
				parent.getComponentId().getClassName(),
				parent.getComponentId().getWindowName()
			)
		);
		this.path = path;
	}

	private JTree getTree() {
		return (JTree) getAWTComponent();
	}
	
	/**
	 * Liefert den Editor des Knotens, zum Editieren
	 * des Textes.
	 * @return Den Editor.
	 * @throws ComponentException, wenn der Knoten nicht editiert werden kann.
	 */
	private MComponent getEditor() throws ComponentException {
		Component c =  
				getTree().getCellEditor().getTreeCellEditorComponent(
						getTree(),
						path.getLastPathComponent(),
						getTree().isPathSelected(path),
						getTree().isExpanded(path),
						getTree().getModel().isLeaf( path.getLastPathComponent() ),
						getTree().getRowForPath(path)
						);
		
		if(c == null)
			throw new ComponentException(Messages.getString("SwingTreeCell.2")); //$NON-NLS-1$
		
		return getResolver().getMComponent( c, getComponentId() );
	}
	
	private MComponent getRenderer() throws ComponentException {
		Component c =  
			getTree().getCellRenderer().getTreeCellRendererComponent(
					getTree(),
					path.getLastPathComponent(),
					getTree().isPathSelected(path),
					getTree().isExpanded(path),
					getTree().getModel().isLeaf( path.getLastPathComponent() ),
					getTree().getRowForPath(path),
					false
					);
	
		if(c == null)
			throw new ComponentException(Messages.getString("SwingTreeCell.3")); //$NON-NLS-1$
		
		return getResolver().getMComponent( c, getComponentId() );
	}
	
	public void setText(String text) throws ComponentException
	{
		assertEditable();
		// assertEditable darf nicht berschrieben werden, weil es in der Funktion click verwendet wird
		if( !getTree().isPathEditable(path) )
			throw new ComponentException(Messages.getString("SwingTreeCell.4")); //$NON-NLS-1$

		MComponent comp = getEditor();
		getTree().startEditingAtPath(path);
		if(comp instanceof IEditable)
		{
			((IEditable)comp).setText(text);
			
			// abschliessen des editierens durch enter
			// achtung auf KEINEN fall table.stopCellEditing benutzen, dies fhrt zu verhalten, was nicht mit der entsprechenden nutzeraktion konform ist
			comp.pressKey(KeyEvent.CHAR_UNDEFINED, KeyEvent.VK_ENTER, 0);
		}
		else
			throw new InvalidOperationException(this, OP_SET_TEXT, true);
	}
	
	/*public String getText() throws ComponentException
	{
		assertReadable();
		if(getRenderer() instanceof ILabel)
			return  ((ILabel)getRenderer()).getText();
		else
			throw new InvalidOperationException(this, OP_GET_TEXT, true);
	}*/
	
	public void click(int numClicks, boolean left, int modifiers) throws ComponentException
	{
		// getRenderer().click(numClicks, left, modifiers); geht nicht, das event muss direkt an den JTree gesendet werden
		Rectangle rect = getTree().getPathBounds(path);
		click( numClicks, left, modifiers, new Point(rect.x + rect.width/2, rect.y + rect.height/2) );
	}
}
