/*
 * Created on 31.05.2006
 */
package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;

import atosj.component.CommandCreator;
import atosj.dialog.SWTTestCreatorDlg;
import atosj.exception.ComponentException;
import atosj.model.hts.command.HTSTest;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Ein <code>CommandCreator</code> fr die SWT Architektur.
 * Implementiert die Aufzeichnung von Tastatureingaben und Mausklicks.
 * @author Nicos Tegos
 */
public abstract class SWTCommandCreator extends CommandCreator
{
	public SWTCommandCreator(SWTComponent comp, IHTSCommandReceiver receiver) {
		super(comp, receiver);
		
		hookKeyListener(  new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				onKeyPressed(e);
			}
		});
		
		// TODO implementieren
		hookMouseListener( new MouseAdapter() {} );
	}
	
	/**
	 * Wird bei einem KeyPressed Event gerufen. Diese Implementation generiert das Kommando
	 * ACTION, PRESSKEY ...
	 * Abgeleitete Klassen knnen diese Funktionalitt erweitern oder ignorieren.
	 * @param e Das enstrechende <code>KeyEvent</code>.
	 */
	protected void onKeyPressed(KeyEvent e) {
		if( Character.isLetterOrDigit((char)e.keyCode) )
		{
			Vector parameters = new Vector(4);
			parameters.add( "\"" + ((char)e.keyCode) + "\"" );
			if( (e.stateMask & SWT.ALT) != 0 )
				parameters.add(HTSSyntax.ACTION_ALT);
			if( (e.stateMask & SWT.CTRL) != 0 )
				parameters.add(HTSSyntax.ACTION_CTRL);
			if( (e.stateMask & SWT.SHIFT) != 0 )
				parameters.add(HTSSyntax.ACTION_SHIFT);
			
			publishActionCommand(HTSSyntax.ACTION_PRESSKEY, parameters, null);
		}
	}

	/**
	 * Implementiert den <code>MouseListener</code> fr den Aufruf des Dialogs
	 * zur Erstellung von TEST-Kommandos.
	 * @see CommandCreator#hookTestListener()
	 */
	protected void hookTestListener() 
	{
		hookMouseListener( new MouseAdapter() {
			public void mouseDown(MouseEvent e) {
				onTestEvent(e);
			}
		});
	}
	
	/**
	 * Diese Funktion muss implementiert werden, wenn die Funktion <code>onKeyPressed</code> 
	 * funktioniert werden sollen. Die Klasse <code>Widget</code> stellt diese Funktionalitt
	 * leider nicht zur Verfgung.
	 * @see #onKeyPressed(KeyEvent)
	 */
	protected abstract void hookKeyListener(KeyListener listener);
	
	/**
	 * Diese Funktion muss implementiert werden, wenn <code>MouseEvents</code> aufgezeichnet
	 * und der Dialog zur Generierung von TEST Kommandos gezeigt werden soll. 
	 * Die Klasse <code>Widget</code> stellt diese Funktionalitt leider nicht zur Verfgung.
	 * @see #hookTestListener()
	 */
	protected abstract void hookMouseListener(MouseListener listener);
	
	/**
	 * Wird bei jedem <code>MouseEvent</code> gerufen und zeigt den Dialog
	 * zum Erstellen von TEST Kommandos an, wenn die Tastenkombination
	 * Strg + Rechtsklick gedrckt wurde.
	 * @param e Das <code>MouseEvent</code>.
	 * @see SWTTestCreatorDlg
	 * @see #hookTestListener()
	 */
	protected void onTestEvent(MouseEvent e) 
	{
		if(e.button == 3 && (e.stateMask & SWT.CTRL) == SWT.CTRL )
		{
			SWTTestCreatorDlg dlg = new SWTTestCreatorDlg(e.display.getActiveShell(), this);
			if( dlg.open() == SWTTestCreatorDlg.OK )
			{
				HTSTest[] selectedElements = dlg.getSelectedElements();
				for (int i = 0; i < selectedElements.length; i++) 
					publishCommand(selectedElements[i]);
			}	
		}
	}
	

	protected SWTComponent getSWTComponent() {
		return (SWTComponent) getMComponent();
	}
	
	/**
	 * Generiert TEST Kommandos fr die Zustnde VISIBLESTATE, ENABLESTATE und FOCUSSTATE.
	 * Abgeleitete Klassen drfen der Liste der generierten Kommandos beliebig viele
	 * weitere TEST Kommandos hinzufgen. Ein TEST Kommando kann mit Hilfe der Funktion
	 * createTestCommand erstellt werden.
	 * @return Vector(HTSTest) Liste der generierten TEST Kommandos.
	 * @see CommandCreator#createTestCommand(String, Vector, Vector)
	 */
	public Vector getTestCommands() 
	{
		Vector commands = new Vector();
		
		try {
			Vector values = new Vector(1);
			boolean visible = getMComponent().isVisible();
			values.add(visible ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE);
			commands.add( createTestCommand(HTSSyntax.READ_VISIBLESTATE, values, null) );
		} catch(ComponentException e){}
		
		try {
			Vector values = new Vector(1);
			boolean enabled = getMComponent().isEnabled();
			values.add(enabled ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE);
			commands.add( createTestCommand(HTSSyntax.READ_ENABLESTATE, values, null) );
		} catch(ComponentException e){}
		
		try {
			Vector values = new Vector(1);
			boolean focus = getMComponent().hasFocus();
			values.add(focus ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE);
			commands.add( createTestCommand(HTSSyntax.READ_FOCUSSTATE, values, null) );
		} catch(ComponentException e){}
		
		return commands;
	}
}
