/*
 * Created on 09.04.2006
 */
package atosj.component.swt;

import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Widget;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.MComponent;
import atosj.event.swt.FireableKeyEvent;
import atosj.event.swt.FireableMouseEvent;
import atosj.event.swt.FireableSWTEvent;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;
import atosj.model.recorder.IHTSCommandReceiver;
import atosj.util.SWTWindowMonitor;
import atosj.util.Snooze;

/**
 * Basisklasse aller Wrapper der Components der SWT Architektur. Die Aufzeichnung von
 * Kommandos ist nicht implementiert, da die Klasse <code>Widget</code> keinerlei
 * Funktionalitt bereitstellt.
 * @author Nicos Tegos
 */
public class SWTComponent extends MComponent 
{
	/**
	 * Da die Klasse Widget keinerlei Funktionalitt bereitstellt, sie aber nicht
	 * abstract sein soll, machen wir hier eine leere Dummy Implementation. Diese
	 * kann auch in den abgeleiteten Klassen nicht verwendet werden, da sie private ist.
	 */
	private class ComponentCommandCreator extends SWTCommandCreator
	{
		public ComponentCommandCreator(SWTComponent component, IHTSCommandReceiver receiver) {
			super(component, receiver);
		}

		protected void hookTestListener(MouseListener listener) {}
		protected void hookKeyListener(KeyListener listener) {}		
		protected void hookMouseListener(MouseListener listener) {}
	};
	
	public SWTComponent(Widget component, ComponentId id) {
		super(component, id);
	}

	public Widget getSWTComponent() {
		return (Widget) getComponent();
	}
	
	public void assertEditable() throws ComponentException {
		if( isDisposed() )
			throw new ComponentException(Messages.getString("SWTComponent.0")); //$NON-NLS-1$
		super.assertEditable();
	}
	
	public void assertReadable() throws ComponentException {
		if( isDisposed() )
			throw new ComponentException(Messages.getString("SWTComponent.1")); //$NON-NLS-1$
		super.assertReadable();
	}
	
	protected boolean isDisposed() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Boolean(getSWTComponent().isDisposed());
			}
		};

		return ((Boolean)action.go()).booleanValue();
	}
	
	/**
	 * Liefert die Stilkonstanten, die bei der Erstellung des Objektes verwendet wurden.
	 * Diese definieren bei einigen Klassen wichtige Ausprgungen eines Components,
	 * wie beispielsweise read only. Diese Funktion ist threadsafe.
	 * @return Die Stilkonstanten, siehe <code>getStyle</code> Methode der Klasse <code>Widget</code>.
	 */
	protected int getStyle() throws ComponentException {
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return new Integer(getSWTComponent().getStyle());
			}
		};

		return ((Integer)action.go()).intValue();
	}
	
	public boolean isFocussable() throws ComponentException
	{
		if( hasFocus() )
			return true;
		
		SWTThreadSafeAction action = new SWTThreadSafeAction()
		{
			protected Object performAction() throws Throwable 
			{
				// 1. Variante, wir aktivieren vorher das Fenster
				Shell parentShell = SWTWindowMonitor.getShell( getSWTComponent() );
				// setActive bringt das Fenster nicht on top! warum? in der beschreibung steht was anderes
				// TODO funktioniert nicht immer, siehe doppelklick auf list in SWTSampleApp
				parentShell.setActive();
				return getSWTComponent().getDisplay().getActiveShell() == parentShell ? Boolean.TRUE : Boolean.FALSE;
								
				// 2. Variante
				/*Shell shell = SWTWindowMonitor.getShell( getSWTComponent() );
				// TODO herausfinden was Primary Modal bedeutet
				int bits = SWT.PRIMARY_MODAL | SWT.APPLICATION_MODAL | SWT.SYSTEM_MODAL;
				Shell[] shells = getSWTComponent().getDisplay().getShells();
				
				for(int i = 0; i < shells.length; ++i)
				{
					if( (shells[i] != shell) && ((shells[i].getStyle() & bits) > 0) )
						return Boolean.FALSE;
				}
				
				return Boolean.TRUE;*/
			}
		};
		
		return ((Boolean)action.go()).booleanValue();
	}
	
	
	/**
	 * Liefert einen Punkt in der Mitte des <code>Widgets</code>, relativ zum <code>Display</code>.
	 * Dieser Punkt wird benutzt um die Operation Click auszufhren.
	 * In der Klasse <code>Widget</code> gibt es keine <code>getBounds()</code> Funktion, deshalb
	 * wirft diese Methode immer eine Ausnahme.
	 * @return Den Punkt.
	 * @throws ComponentException, default Implementation.
	 * @see #click(int, boolean, int)
	 */
	public Point getPoint() throws ComponentException {
		throw new ComponentException(Messages.getString("SWTComponent.2")); //$NON-NLS-1$
	}
	
	/**
	 * Versucht den Fokus zu erlangen. Basisimplementation wirft nur eine Ausnahme.
	 * @throws ComponentException, wenn der Fokus nicht erlangt werden kann.
	 * @see atosj.component.IComponent#hasFocus()
	 */
	public void requestFocus() throws ComponentException {
		throw new InvalidOperationException(this, OP_SET_FOCUS, false);
	}
	
	/**
	 * Aktiviert die <code>Shell</code>, d.h. macht sie sichtbar falls sie von anderen verdeckt wurde.
	 * @throws ComponentException, falls die <code>Shell</code> nicht aktiviert werden konnte.
	 */
	private void activateShell() throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction()
		{
			protected Object performAction() throws Throwable 
			{
				Shell parentShell = SWTWindowMonitor.getShell( getSWTComponent() );
				parentShell.setActive();
				if( getSWTComponent().getDisplay().getActiveShell() != parentShell )
					throw new ComponentException(Messages.getString("SWTComponent.3")); //$NON-NLS-1$
				
				return null;
			}
		};
		
		action.go();
	}
	
	
	/**
	 * Wartet so lange bis alle events aus der System-Eventqueue verschwunden sind.
	 */
	public void sync() throws ComponentException
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				System.out.println(Messages.getString("SWTComponent.4")); //$NON-NLS-1$
				return new Boolean( getSWTComponent().getDisplay().readAndDispatch() );
			}
		};
		
		while(	((Boolean)action.go()).booleanValue() )
			new Snooze(10);
	}
	

	/* (non-Javadoc)
	 * @see atosj.component.IComponent#click(int, boolean, int)
	 */
	public void click(int numClicks, boolean left, int modifiers) throws ComponentException
	{
		Point p = getPoint();
		click(numClicks, left, modifiers, p.x, p.y);
	}
	
	/**
	 * Simuliert einen Mausklick auf eine bestimmte Position innerhalb des Components.
	 * @param numClicks Anzahl der Klicks (2 sind ein Doppelklick).
	 * @param left wenn <code>true</code> Linksklick, sonst Rechtsklick.
	 * @param modifiers Die AtosJ ModifierKey Konstanten verknpft durch
	 * bitweises oder, siehe <code>MODIFIER_*</code>. Oder <code>NO_MODIFIER</code> falls kein 
	 * Modifier gedrckt.
	 * @param x Die x-Koordinate des Mausklicks relativ zum Display.
	 * @param y Die y-Koordinate des Mausklicks relativ zum Display.
	 * @see atosj.component.IComponent#click(int, boolean, int)
	 */
	public void click(final int numClicks, final boolean left, final int modifiers, final int x, final int y) throws ComponentException 
	{
		assertEditable();
		activateShell();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				new FireableMouseEvent(SWTComponent.this).fire(
						numClicks, 
						left, 
						FireableSWTEvent.convertToSWTModifiers(modifiers), 
						x, y);
				
				return null;
			}
		};
		
		action.go();
	}
	
	 /**
     * Die Funktionsweise gleicht der zweiten <code>pressKey</code> Methode, jedoch wird
     * der Code des eingegebenen Zeichens automatisch ermittelt. Die Ermittlung
     * des Codes funktioniert nur fr Zeichen und nicht fr virtuelle Tasten.
     * @see #pressKey(char, int, int)
     */
	public void pressKey(final char character, final int modifiers) throws ComponentException 
	{
		assertEditable();
		activateShell();
		requestFocus();
				
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				new FireableKeyEvent(SWTComponent.this).fire(
						character, 
						FireableSWTEvent.convertToSWTModifiers(modifiers)
					);				
				
				return null;
			}
		};
		
		action.go();	
	}

	/* (non-Javadoc)
	 * @see atosj.component.IComponent#pressKey(char, int)
	 */
	public void pressKey(final char character, final int keyCode, final int modifiers) throws ComponentException 
	{
		assertEditable();
		activateShell();
		requestFocus();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				new FireableKeyEvent(SWTComponent.this).fire(
						character, 
						keyCode, 
						modifiers 
					);				
				
				return null;
			}
		};
		
		action.go();
	}
	
	/**
	 * Erstellt einen <code>CommandCreator</code>, der keinerlei Funktionalitt besitzt.
	 */
	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new ComponentCommandCreator(this, receiver);
	}
}
