/*
 * Created on 15.04.2006
 */
package atosj.component.swt;

import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.exception.ComponentException;
import atosj.exception.InvalidOperationException;

/**
 * Diese Klasse dient zum Zugriff auf SWT Objekte.
 * Sie fhrt den Code in in der Methode <code>performAction</code>
 * im GUI Thread aus, damit kein <code>InvalidThreadAccess</code> Fehler entsteht. Es wird
 * synchron ausgefhrt, d.h. der GUI Thread blockiert. Wenn Fehler bei der Ausfhrung auftreten,
 * so werden diese in eine ComponentException umgewandelt und weitergeleitet. Zugriffe auf
 * SWT Components whrend des Abspielens einer Testsequenz sollte immer in dieser Klasse
 * gekapselt sein.
 * @author Nicos Tegos
 */
public abstract class SWTThreadSafeAction implements Runnable {

	private Throwable err = null;
	private Object returnValue;
	
	public SWTThreadSafeAction() {
		super();
	}

	/**
	 * Diese Funktion ist nicht dazu gedacht von abgeleitetet Klassen
	 * gerufen oder berschrieben zu werden.
	 */
	public void run() {
		try{
			returnValue = performAction();
		} catch (Throwable e) {
			err = e;
		}
	}
	
	/**
	 * Fhrt den enthaltenen Code im GUI Thread des Testobjekts aus.
	 * @return <code>null</code> oder ein aktionsspezifischer Rckgabewert.
	 */
	protected abstract Object performAction() throws Throwable;

	/**
	 * Fhrt die Aktion aus.
	 * @param comp Das Element an dem die Aktion ausgefhrt wird. Bentigt zur wohlformatierten
	 * Ausgabe eines eventuellen Fehlers.
	 * @param operation Art der Aktion, siehe die Konstanten OP_* in der Klasse <code>MComponent</code>.
	 * @return <code>null</code> oder der Rckgabewert der Aktion. 
	 * @throws ComponentException bei einer Exception whrend der Ausfhrung von <code>performAction</code>.
	 * @see #performAction()
	 */
	public Object go(SWTComponent comp, int operation) throws ComponentException
	{
		err = null;
		returnValue = null;
		Display.getDefault().syncExec(this);
		if(err != null)
		{
			if(err instanceof ComponentException)
				throw (ComponentException)err;
			else
				throw new InvalidOperationException(comp, operation, err);
		}
		
		return returnValue;
	}
	
	public Object go() throws ComponentException
	{
		err = null;
		returnValue = null;
		Display.getDefault().syncExec(this);
		if(err != null)
		{
			if(err instanceof ComponentException)
				throw (ComponentException) err;
			else
				throw new ComponentException( Messages.getString("SWTThreadSafeAction.0") + err.toString() ); //$NON-NLS-1$
		}
		
		return returnValue;
	}
	
	/**
	 * Liefert den Rckgabewert der ausgefhrten Aktion. Dieser ist erst nach der
	 * Ausfhrung von <code>go</code> gltig.
	 * @return <code>null</code> oder der Rckgabewert nach der Ausfhrung der Aktion.
	 */
	public Object getReturnValue() {
		return returnValue;
	}
}
