package atosj.component.swt;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;

import atosj.Messages;
import atosj.component.ComponentId;
import atosj.component.ICommandCreator;
import atosj.component.IListCommandCreator;
import atosj.component.ITree;
import atosj.component.ITreeCell;
import atosj.component.ListCommandCreatorHelper;
import atosj.exception.ComponentException;
import atosj.exception.ComponentNotFoundException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.recorder.IHTSCommandReceiver;

/**
 * Wrapperklasse fr alle von <code>Tree</code> abgeleiteten Klassen.
 * @author Nicos Tegos
 */
public class SWTTree extends SWTControl implements ITree 
{
	/**
	 * Zeichnet das Selektieren und Editieren von Eintrgen des Baumes auf.
	 * Generiert TEST Kommandos zu den Zustnden des Baumes (ENABLESTATE, VISIBLESTATE, FOCUSSTATE).
	 */
	protected class TreeCommandCreator extends ControlCommandCreator implements SelectionListener, IListCommandCreator
	{
		private ListCommandCreatorHelper helper;
		private boolean textModified = false;
		private String origText = null;
		
		public TreeCommandCreator(SWTTree comp, IHTSCommandReceiver receiver) {
			super(comp, receiver);
			helper = new ListCommandCreatorHelper(this);
			addListener(this);
			
			//	capture TextCell editing
			Control[] children = getTree().getChildren();
			FocusListener textCellFocusListener =  new FocusListener() {
				public void focusGained(FocusEvent e) {
					textModified = false;
					// den letzten ungenderten Text des Items speichern
					Text textCell = (Text) e.widget;
					TreeItem item = getItem(textCell);
					origText = item.getText();
				}

				public void focusLost(FocusEvent e) 
				{
					if(textModified)
					{
						Text textCell = (Text) e.widget;
						Vector actionParams = new Vector(1);
						actionParams.add( '"'  + textCell.getText() + '"' );
						
						Vector subitemPath = new Vector();
						TreeItem item = getItem(textCell);
						if(item != null)
							subitemPath.add(0, '"' + origText + '"' );
						
						// alle items  bis zum root item dem subelementpfad hinzufgen
						item = item.getParentItem();
						while(item != null)
						{
							subitemPath.add(0, '"' + item.getText() + '"' );
							item = item.getParentItem();
						}
						
						if( !subitemPath.isEmpty() )
							publishActionCommand(HTSSyntax.ACTION_EDIT, actionParams, subitemPath);
					}
					
					textModified = false;
					origText = null;
				}
				
				private TreeItem getItem(Text t) {
					// das treeitem anhand der koordinaten des textfeldes herausfinden
					Tree tree = getTree();
					Rectangle textCellBounds = t.getBounds();
					Point pCell = new Point(textCellBounds.x, 
							textCellBounds.y); 
					return tree.getItem(pCell);
				}
			};
			ModifyListener textCellModifyListener = new ModifyListener() {

				public void modifyText(ModifyEvent e) {
					textModified = true;
				}
				
			};
			
			KeyListener textCellKeyListener = new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					// editing durch escape drcken abgebrochen
					if(e.keyCode == SWT.ESC)
						textModified = false;
				}
			};
			
			for(int i = 0; i < children.length; ++i)
			{
				if(children[i] instanceof Text)
				{
					addListener(children[i], textCellFocusListener);
					addListener(children[i], textCellModifyListener);
					addListener(children[i], textCellKeyListener);
				}
			}
		}

		private Tree getTree() {
			return ((SWTTree)getMComponent()).getTree();
		}
		
		public void widgetSelected(SelectionEvent arg0) {
			helper.selectionChanged();
		}

		public void widgetDefaultSelected(SelectionEvent arg0) {		
		}
		
		public Object[] getSelection() {
			return getTree().getSelection();
		}

		public Vector convertSelectionObjectToPath(Object selectionObject) 
		{
			Vector path = new Vector();
			
			TreeItem item = (TreeItem)selectionObject;
			path.add( 0, '"' + item.getText() + '"');
			while( item.getParentItem() != null )
			{
				item = item.getParentItem();
				path.add( 0, '"' + item.getText() + '"');
			}
			
			return path;
		}
	}
	
	public SWTTree(Tree component, ComponentId id) {
		super(component, id);
	}
	
	public Tree getTree() {
		return (Tree) getComponent();
	}
	
	/**
	 * Rckt das <code>item</code> in den sichtbaren Bereich.
	 * <em>ACHTUNG:</em> luft nicht im SWT Thread.
	 */
	private void revealSubitem(TreeItem item) throws ComponentException {
		getTree().showItem(item);
	}
	
	/**
	 * Sucht das Item entsprechend des Pfades (anhand der Beschriftungen)
	 * <em>ACHTUNG:</em> luft nicht im SWT Thread.
	 * @param path Der Suchpfad.
	 * @return Das TreeItem.
	 * @throws ComponentException wenn nicht gefunden.
	 */
	private TreeItem findItem(String[] path) throws ComponentException
	{
		return findItem( getTree(), path, 0 );
	}
	
	/**
	 * Rekursive suche nach einem Knoten des Baumes. Sucht immer in der aktuellen Ebene
	 * des Baumes nach einem Knoten, der die Bezeichnung trgt, die an Position <code>searchIndex</code>
	 * im Suchpfad steht.
	 * @param parent Der <code>Tree</code>, dessen Kinder durchsucht werden sollen
	 * oder das <code>TreeItem</code>, dessen Kinder durchsucht werden sollen.
	 * @param search Der Suchpfad, Beschriftung der Knoten beginnend bei der Wurzel.
	 * @param searchIndex Der aktuelle Index im Suchpfad.
	 * @return den gesuchten Knoten des Baumes.
	 * @throws ComponentException wenn der Knoten nicht gefunden wurde.
	 */
	private TreeItem findItem(Widget parent, String[] search, int searchIndex) throws ComponentException
	{
        String text = search[searchIndex];
		
        TreeItem[] items = getChildren(parent);
        for (int i = 0; i < items.length; i++) {
            TreeItem item = items[i];
            if( item.getText().equals(text) )
            {
            	if( search.length-1 == searchIndex ) // wir haben das letzte gefunden
            		return item;
            	else
            	{
            		// also das hier muss sein, denn bei der Verwendung
            		// von org.eclipse.jface.viewers.TreeViewer
            		// werden die Subitems eines Items erst erstellt
            		// wenn das Item expanded wird. 
            		// Da aber Item#setExpanded(boolean), kein Event generiert
            		// mssen wir selbst eins generieren.
            		item.setExpanded(true);
            		Event event = new Event ();
    				event.item = item;
    				item.getParent().notifyListeners(SWT.Expand, event);
            		return findItem(item, search, ++searchIndex);
            	}
            }
        }
        
        throw new ComponentNotFoundException( getComponentId(), Messages.getString("SWTTree.0") + (searchIndex+1) + Messages.getString("SWTTree.1") + search[searchIndex] + Messages.getString("SWTTree.2") ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    }
	
	/**
	 * Liefert alle Kindelemente des angegebenen Baumes oder Knotens
	 * @param parent Ein <code>Tree</code> oder ein <code>TreeItem</code>.
	 * @return Alle Kinder von <code>parent</code> oder ein leeres Array, falls keine vorhanden.
	 */
	private TreeItem[] getChildren(Widget parent) {
		if(parent instanceof Tree)
			return ((Tree)parent).getItems();
		if(parent instanceof TreeItem)
			return ((TreeItem)parent).getItems();
		return null;
	}
	
	/**
	 * Gibt an ob der Knoten zur aktuellen Auswahl im Baum gehrt.
	 * <em>ACHTUNG:</em> luft nicht im SWTThread.
	 * @param item Der Knoten der berprft werden soll.
	 * @return <code>true</code>, wenn <code>item</code> teil der aktuellen Auswahl,
	 * <code>false</code> sonst.
	 */
	private boolean isSelected(TreeItem item) 
	{
		TreeItem[] items = getTree().getItems();
		for (int i = 0; i < items.length; i++) 
		{
			if(items[i] == item)
				return true;
		}
		
		return false;
	}
	
	/**
	 * berprft, ob der Baum die Auswahl mehrerer Eintrge erlaubt.
	 * @throws ComponentException wenn die Tabelle keine Mehrfachauswahl untersttzt.
	 */
	private void assertMulti() throws ComponentException
	{
		if( (getStyle() & SWT.MULTI) == 0 )
			throw new ComponentException(Messages.getString("SWTTree.3")); //$NON-NLS-1$
	}

	public void select(final String[] path) throws ComponentException 
	{
		assertEditable();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				TreeItem item = findItem(path);
				revealSubitem(item);
				
				
				new SWTTreeItem(SWTTree.this, item, path).select(); 
				return null;
			}
		};
		
		action.go(this, OP_SELECT);
	}

	public void deselect(final String[] path) throws ComponentException 
	{
		assertEditable();
		assertMulti();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				TreeItem item = findItem(path);
				revealSubitem(item);
						
				if( isSelected(item) )
					new SWTTreeItem(SWTTree.this, item, path).click(1, true, MODIFIER_CTRL); 
				return null;
			}
		};
		
		action.go(this, OP_DESELECT);

	}

	public void addSelect(final String[] path) throws ComponentException 
	{
		assertEditable();
		assertMulti();
		
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				TreeItem item = findItem(path);
				revealSubitem(item);
						
				if( !isSelected(item) )
					new SWTTreeItem(SWTTree.this, item, path).click(1, true, MODIFIER_CTRL); 
				return null;
			}
		};
		
		action.go(this, OP_SELECT);
	}

	public ITreeCell getTreeCell(final String[] path) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable {
				return findItem(path);
			}
		};
		
		return new SWTTreeItem(SWTTree.this, (TreeItem) action.go(), path);
	}

	public ICommandCreator initCommandCreator(IHTSCommandReceiver receiver) {
		return new TreeCommandCreator(this, receiver);
	}
}
