package atosj.dialog;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.component.ComponentId;
import atosj.exception.AtosJException;

/**
 * Dialog zum Erstellen und Bearbeiten einer <code>CompentId</code> in der URF-Datei.
 * @see atosj.component.ComponentId
 * @see atosj.editors.urf.URFEditor
 */
public class ComponentIdDlg extends Dialog 
{
	private boolean editing;
	private ComponentId id;
	
	private Text windoTitleTxt;
	private Combo typeCombo;
	private Text nameTxt;
	
	/**
	 * Erstellt einen neuen Dialog zur Bearbeitung einer vorhanden ID.
	 * @param shell Das Vaterfenster.
	 * @param toEdit Die zu bearbeitende ID.
	 */
	public ComponentIdDlg(Shell shell, ComponentId toEdit) {
		super(shell);
		
		if(toEdit != null)
		{
			this.id = toEdit;
			editing = true;
		}
		else
		{
			this.id = null;
			editing = false;
		}
	}
	
	/**
	 * Erstellt einen neuen Dialog zum Erstellen einer neuen ID.
	 * @param shell Das Vaterfenster.
	 */
	public ComponentIdDlg(Shell shell) {
		this(shell, null);
	}
	
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, OK, Messages.getString("ComponentIdDlg.0"), true); //$NON-NLS-1$
		createButton(parent, CANCEL, Messages.getString("ComponentIdDlg.1"), false); //$NON-NLS-1$
	}

	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText( editing ? Messages.getString("ComponentIdDlg.2") : Messages.getString("ComponentIdDlg.3")); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		Label lbl = new Label(dlgArea, SWT.NULL);
		lbl.setText(Messages.getString("ComponentIdDlg.4")); //$NON-NLS-1$
		windoTitleTxt = new Text(dlgArea, SWT.BORDER);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 200;
		windoTitleTxt.setLayoutData(gd);
		
		lbl = new Label(dlgArea, SWT.NULL);
		lbl.setText(Messages.getString("ComponentIdDlg.5")); //$NON-NLS-1$
		nameTxt = new Text(dlgArea, SWT.BORDER);
		nameTxt.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		lbl = new Label(dlgArea, SWT.NULL);
		lbl.setText(Messages.getString("ComponentIdDlg.6")); //$NON-NLS-1$
		typeCombo = new Combo(dlgArea, SWT.BORDER | SWT.READ_ONLY);
		typeCombo.setItems( AtosJApp.getInstance().getInput().getTestObject().getResolver().getTypeNames() );
		typeCombo.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		initContent();
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	/**
	 * Wird eine ID bearbeitet, werden hier die Ursprungswerte
	 * in die entsprechenden Eingabefelder eingetragen.
	 */
	private void initContent() 
	{
		if(editing)
		{
			windoTitleTxt.setText( id.getWindowName() );
			typeCombo.setText( id.getClassName() );
			nameTxt.setText( id.getName() );
		}
	}
	
	/**
	 * Erstellt eine neue ID entsprechend der Nutzereingaben und speichert
	 * diese in einer Variablen.
	 */
	private void createId() throws AtosJException 
	{
		String newTitle = windoTitleTxt.getText();
		if( newTitle.length() == 0 )
		{
			windoTitleTxt.setFocus();
			throw new AtosJException(Messages.getString("ComponentIdDlg.7")); //$NON-NLS-1$
		}
		
		String newType = typeCombo.getText();
		if( newType.length() == 0 )
		{
			typeCombo.setFocus();
			throw new AtosJException(Messages.getString("ComponentIdDlg.8")); //$NON-NLS-1$
		}
		
		String newName = nameTxt.getText();
		if( newName.length() == 0 )
		{
			typeCombo.setFocus();
			throw new AtosJException(Messages.getString("ComponentIdDlg.9")); //$NON-NLS-1$
		}
		
		ComponentId newId = new ComponentId(newName, newType, newTitle);
		if( AtosJApp.getInstance().getInput().getURFFile().containsId(newId) )
			throw new AtosJException(Messages.getString("ComponentIdDlg.10")); //$NON-NLS-1$
		
		id = newId;
	}
	
	protected void okPressed() 
	{
		try 
		{
			createId();
		} 
		catch (AtosJException e) 
		{
			MessageDialog.openError(getShell(), 
					editing ? Messages.getString("ComponentIdDlg.11") : Messages.getString("ComponentIdDlg.12"),  //$NON-NLS-1$ //$NON-NLS-2$
					(editing ?  Messages.getString("ComponentIdDlg.13") : Messages.getString("ComponentIdDlg.14")) //$NON-NLS-1$ //$NON-NLS-2$
					+ e.getMessage() );
			return;
		}
		
		super.okPressed();
	}
	
	/**
	 * Liefert eine neue ID entsprechend der Nutzereingaben.
	 */
	public ComponentId getId() {
		return id;
	}
}
