/*
 * Created on 25.11.2005
 */
package atosj.dialog;

import java.io.IOException;
import java.rmi.RemoteException;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.app.UserPreferences;
import atosj.gui.BalloonWindow;
import atosj.gui.SWTToolkit;
import atosj.gui.TableResizeListener;
import atosj.listener.PackagePlayerReceiverListener;
import atosj.listener.PlayerReceiverListener;
import atosj.model.script.HTSScript;
import atosj.rmi.HTSPlayer;
import atosj.util.ReportPrinter;
import atosj.util.Statistics;

/**
 * Dialog zur Ausfhrung einer Testsequenz.
 * @author Nicos Tegos
 */
public class RunScriptDlg extends Dialog 
{
	private static final int ID_RUN = 0;
	private static final int ID_CANCEL = 2;
	private static final int ID_REPORT = 3;
	
	private HTSScript script;
	private HTSPlayer receiver;
	private String reportPath; // dateiname des zuletzt generierten reports
	
	private boolean haltOnError = false;
	private Button haltOnErrorChk;
	private boolean createReport = false;
	private Button createReportChk;
	private int delay = 100;
	private Spinner delayTxt;
	private Label delayLbl;
	
	private Button reportButton, runButton, escapeButton;
	private StdTextSetter stdoutListener; 
	private StdTextSetter stderrListener;
		
	private TableViewer runScriptTV;
	private CommandStatisticsTableContentProvider runScriptProvider = new CommandStatisticsTableContentProvider(0);
	
	private StyledText stdTxt;
	
	/**
	 * Erstellt einen neuen Dialog zur Ausfhrung einer Testsequenz.
	 * @param parent Das Vaterfenster.
	 * @param script Die Testsequenz, die ausgefhrt werden soll.
	 */
	public RunScriptDlg(Shell parent, HTSScript script) 
	{
		super(parent);
		setShellStyle(SWT.TITLE | SWT.MIN | SWT.MAX | SWT.RESIZE | SWT.APPLICATION_MODAL);
		this.script = script;
		
		setupReceiver();
	}
	
	/**
	 * Initialisiert den Listener, der auf den Start und das Ende der Skriptausfhrung reagieren soll,
	 * sowie ber die Ausfhrung von Kommandos unterrichtet wird. Dieser aktualisiert
	 * die graphischen Anzeigen des Dialogs, insbesondere die Ausfhrungsstatistiken.
	 * @see PlayerReceiverListener
	 */
	private void setupReceiver() {
		try {
			receiver = new HTSPlayer(AtosJApp.getInstance().getInput().getTestObject(), script);
		}
		catch(RemoteException e) {
			e.printStackTrace();
			return;
		}
		
		receiver.addListener( new PlayerReceiverListener() {
			public void runningStateChanged(final boolean running) {
				haltOnErrorChk.setEnabled(running ? false : true);
				createReportChk.setEnabled(running ? false : true);
				delayTxt.setEnabled(running ? false : true);
				delayLbl.setEnabled(running ? false : true);
				
				if(running)
				{
					runButton.setText(Messages.getString("RunScriptDlg.0")); //$NON-NLS-1$
					reportButton.setEnabled(false);
					stderrListener.clear();
					stdoutListener.clear();
					stdTxt.setText(""); //$NON-NLS-1$
					runScriptProvider.reset();
				}
				else // beendet
				{
					runButton.setText(Messages.getString("RunScriptDlg.2")); //$NON-NLS-1$
					// markierung der aktuellen zeile entfernen
					runScriptProvider.removeHighlights();
					
					// meldungen ausgeben und evtl. report erstellen
					if( !receiver.isStoppedByUser() ) // nicht durch Nutzer beendet
					{
						if( receiver.hasError() )
						{
							new ErrorDlg(
									Display.getDefault().getActiveShell(),
									Messages.getString("RunScriptDlg.3"), //$NON-NLS-1$
									Messages.getString("RunScriptDlg.4") + script.getName() + Messages.getString("RunScriptDlg.5"), //$NON-NLS-1$ //$NON-NLS-2$
									receiver.getErrors(0)/*TODO: noch in deutsch!*/ ).open(); 
						}
						else
						{
							MessageDialog.openInformation(null, Messages.getString("RunScriptDlg.6"), //$NON-NLS-1$
									Messages.getString("RunScriptDlg.7") + script.getName() + Messages.getString("RunScriptDlg.8")); //$NON-NLS-1$ //$NON-NLS-2$
						}
						
						if(createReport)
						{
							reportPath = ReportPrinter.getInstance().createScriptPdf(
									script, 
									receiver.getCommandStatistics(0), 
									stdoutListener.getText(), stderrListener.getText()
							);
							
							if(reportPath != null)
							{
								reportButton.setEnabled(true);
								reportButton.setToolTipText(reportPath);
							}
						}
					}
				}
			}

			public void commandStatisticsReceived(Statistics statistics) {
				runScriptProvider.setCurrentStatistics(statistics);
			}

			public void commandStarted(int index) {
				runScriptProvider.setRunning(index);
			}
			
		});
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		reportButton = createButton(parent, ID_REPORT, Messages.getString("RunScriptDlg.9"), false); //$NON-NLS-1$
		reportButton.setEnabled(false);
		runButton =    createButton(parent, ID_RUN, Messages.getString("RunScriptDlg.10"), true); //$NON-NLS-1$
		escapeButton = createButton(parent, ID_CANCEL, Messages.getString("RunScriptDlg.11"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_RUN: runPressed();
			break;
			case ID_CANCEL: cancelPressed();
			break;
			case ID_REPORT: reportPressed();
			break;
		}
	}
	
	/**
	 * Startet den PDF Reader und zeigt den Bericht zur Testausfhrung an.
	 */
	protected void reportPressed() 
	{
		String pdfReader = (String) UserPreferences.getInstance().getPreferedValue(UserPreferences.PDF_READER_PATH);
		if(pdfReader != null)
		{
			try {
				Runtime.getRuntime().exec(pdfReader + " \"" + reportPath + "\"" ); //$NON-NLS-1$ //$NON-NLS-2$
			} catch(IOException e) {
				MessageDialog.openError(null, Messages.getString("RunScriptDlg.14"), Messages.getString("RunScriptDlg.15")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
		else
		{
			MessageDialog.openError(null, Messages.getString("RunScriptDlg.16"), Messages.getString("RunScriptDlg.17")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	/**
	 * Startet das Abspielen der Testsequenz.
	 * @see HTSPlayer#start(Shell, int, boolean, StreamListener, StreamListener)
	 */
	protected void runPressed() 
	{
		if( receiver.isRunning() )
		{
			receiver.userStop();
		}
		else
		{
			Runnable run = new Runnable()
			{
				public void run() {
					receiver.start(
							getShell(),
							delay, 
							haltOnError, 
							stdoutListener, 
							stderrListener
					);
				} 
			};
			
			new Thread(run).start();
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#close()
	 */
	public boolean close() 
	{
		if( receiver.isRunning() )
		{
			MessageDialog questionDlg = new MessageDialog(
					Display.getDefault().getActiveShell(), 
					Messages.getString("RunScriptDlg.18"),  //$NON-NLS-1$
					null, 
					Messages.getString("RunScriptDlg.19"),  //$NON-NLS-1$
					MessageDialog.QUESTION, 
					new String[] {Messages.getString("RunScriptDlg.20"), Messages.getString("RunScriptDlg.21")},  //$NON-NLS-1$ //$NON-NLS-2$
					1);
			
			if(questionDlg.open() == 1) // nicht beenden
				return false;
		}
		
		receiver.dispose();
		
		if( getShell().getMaximized() )
			UserPreferences.getInstance().setPreferedValue(UserPreferences.SCRIPT_DLG_STATE, UserPreferences.STATE_WND_MAXIMIZED);
		else
			UserPreferences.getInstance().setPreferedValue(UserPreferences.SCRIPT_DLG_RECT, getShell().getBounds() );
		
		setReturnCode(ID_CANCEL);
		return super.close();
	}
	
	protected void cancelPressed() 
	{
		close();
	}

	/**
	 * Initialisiert den Inhalt der graph. Elemente.
	 */
	private void initContent() {
		runScriptTV.setInput(receiver);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#open()
	 */
	public int open() {
		if(receiver == null)
		{
			MessageDialog.openError( getShell(), Messages.getString("RunScriptDlg.22"), Messages.getString("RunScriptDlg.23") ); //$NON-NLS-1$ //$NON-NLS-2$
			return ID_CANCEL;
		}
		
		return super.open();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		getShell().setText(Messages.getString("RunScriptDlg.24") + script.getName() + Messages.getString("RunScriptDlg.25")); //$NON-NLS-1$ //$NON-NLS-2$
		getShell().setImage( AtosJApp.getInstance().getImage(AtosJApp.IMG_ICON) );
		
		getShell().setBounds( (Rectangle) UserPreferences.getInstance().getPreferedValue(UserPreferences.SCRIPT_DLG_RECT) );
		if( UserPreferences.getInstance().getPreferedValue(UserPreferences.SCRIPT_DLG_STATE).equals(UserPreferences.STATE_WND_MAXIMIZED) )
			getShell().setMaximized(true);
				
		return super.createContents(parent);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		createOptionsArea(dlgArea);
		createTableArea(dlgArea);
		createStdArea(dlgArea);
		
		initContent();
		
		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	private void createOptionsArea(Composite parent) 
	{
		Group optionsArea =  new Group(parent, SWT.NULL);
		optionsArea.setText(Messages.getString("RunScriptDlg.26")); //$NON-NLS-1$
		optionsArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		optionsArea.setLayout( new GridLayout(1, false) );
		
		haltOnErrorChk = SWTToolkit.createButton(optionsArea, SWT.CHECK, Messages.getString("RunScriptDlg.27")); //$NON-NLS-1$
		haltOnErrorChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				haltOnError = haltOnErrorChk.getSelection();
			}
		});
		haltOnErrorChk.setSelection(haltOnError);
		
		createReportChk = SWTToolkit.createButton(optionsArea, SWT.CHECK, Messages.getString("RunScriptDlg.28")); //$NON-NLS-1$
		createReportChk.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				createReport = createReportChk.getSelection();
			}
		});
		createReportChk.setSelection(createReport);
		
		// Spinner fr Verzgerungszeit
		Composite delayArea = new Composite(optionsArea, SWT.NULL);
		GridLayout gl = new GridLayout(2, false);
		gl.marginHeight = gl.marginWidth = 0;
		delayArea.setLayout(gl);
		
		delayTxt = new Spinner(delayArea, SWT.BORDER);
		delayTxt.setMinimum(0);
		delayTxt.setMaximum(1000000);
		delayTxt.setIncrement(100);
		delayTxt.setSelection(delay);
		delayTxt.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				delay = delayTxt.getSelection();
			}
		});
		
		delayLbl = new Label(delayArea, SWT.NULL);
		delayLbl.setText(Messages.getString("RunScriptDlg.29")); //$NON-NLS-1$
	}
	
	private void createTableArea(Composite parent) 
	{
		Composite tableArea = new Composite(parent, SWT.NULL);
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = 0;
		tableArea.setLayout(gl);
		tableArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
				
		runScriptTV = new TableViewer(tableArea, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
		runScriptTV.setColumnProperties(CommandStatisticsTableContentProvider.PROPERTIES);
		runScriptTV.setLabelProvider( new CommandStatisticsTableLabelProvider(runScriptTV) );
		runScriptTV.setContentProvider(runScriptProvider);
		
		Table table = runScriptTV.getTable();
		// fehler onDoubleclick anzeigen
		table.addMouseListener( new MouseAdapter() {
			public void mouseDoubleClick(MouseEvent e) {
				StructuredSelection ssel = (StructuredSelection) runScriptTV.getSelection();
				if( !ssel.isEmpty() )
				{
					CommandStatisticsTableElement element = (CommandStatisticsTableElement) ssel.getFirstElement();
					if( element.hasError() )
					{
						final BalloonWindow exWindow = new BalloonWindow(getShell(), SWT.TITLE | SWT.CLOSE);
						exWindow.setImage(AtosJApp.getInstance().getImage( AtosJApp.IMG_ERROR) );
						exWindow.setTitle(Messages.getString("RunScriptDlg.30")); //$NON-NLS-1$
						exWindow.setText( element.getError() );
						exWindow.setLocation( runScriptTV.getTable().toDisplay(e.x, e.y) );
						exWindow.getShell().addShellListener( new ShellAdapter() {
							public void shellDeactivated(ShellEvent arg0) {
								exWindow.close();
							}
						});
						exWindow.open();
					}
				}
			}
		});
		
		// table.addControlListener( new TableResizeListener(new int[] {10, 42, 16, 16, 16}, 50) );
		new TableResizeListener(table, new int[] {10, 42, 16, 16, 16}, 50);
		table.setLayoutData( new GridData(GridData.FILL_BOTH) );
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunScriptDlg.31")); //$NON-NLS-1$
		
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunScriptDlg.32")); //$NON-NLS-1$
		
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunScriptDlg.33")); //$NON-NLS-1$
		
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunScriptDlg.34")); //$NON-NLS-1$
		
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("RunScriptDlg.35")); //$NON-NLS-1$
	}
	
	private void createStdArea(Composite parent) 
	{
		Composite stdArea = new Composite(parent, SWT.NULL);
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = 0;
		stdArea.setLayout(gl);
		stdArea.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		
		stdTxt = new StyledText(stdArea, SWT.BORDER | SWT.READ_ONLY | SWT.V_SCROLL | SWT.H_SCROLL);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 100;
		stdTxt.setLayoutData(gd);
		
		stdoutListener = new StdTextSetter( stdTxt, stdTxt.getDisplay().getSystemColor(SWT.COLOR_BLUE) );
		stderrListener = new StdTextSetter( stdTxt, stdTxt.getDisplay().getSystemColor(SWT.COLOR_RED) );
	}
}
