/*
 * Created on 04.12.2005
 */
package atosj.dialog;

import java.util.Vector;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.editors.Changeable;
import atosj.editors.ChangeableNameComparator;
import atosj.gui.BaseLabelProvider;
import atosj.gui.SWTToolkit;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;
import atosj.model.urf.URFFile;
import atosj.util.UniversalViewerSorter;

/**
 * Dialog zum Speichern/Verwerfen von nderungen an Ressourcen.
 * @author Nicos Tegos
 */
public class SaveResourcesDlg extends Dialog 
{
	/** Speichern Button gedrckt. */
	public static final int ID_SAVE = 0;
	/** Abbrechen Button gedrckt. */
	public static final int ID_CANCEL = 1;
	
	private CheckboxTableViewer saveTV;
	private Vector toSave; // die elemente die letztlich gespeichert werden sollen
	private Vector toDiscard; // die elemente die letztlich verworfen werden sollen
	private Vector modifiedElements; // die genderten elemente
	
	private String title;
	private String message;
	
	/**
	 * Erstellt einen Dialog zum Speichern/Verwerfen von nderungen an Ressourcen.
	 * @param parent Das Vaterfenster.
	 * @param modifiedElements Vector(Changeable) Die Liste der genderten Ressourcen.
	 * @see Changeable
	 */
	public SaveResourcesDlg(Shell parent, Vector modifiedElements) {
		this(parent, modifiedElements, Messages.getString("SaveResourcesDlg.0"), Messages.getString("SaveResourcesDlg.1")); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	/**
	 * Erstellt einen Dialog zum Speichern/Verwerfen von nderungen an Ressourcen.
	 * @param parent Das Vaterfenster.
	 * @param title Der Titel des Dialogs.
	 * @param message Eine an den Nutzer gerichtete kurze Mitteilung.
	 * @param modifiedElements Vector(Changeable) Die Liste der genderten Ressourcen.
	 * @see Changeable
	 */
	public SaveResourcesDlg(Shell parent, Vector modifiedElements, String title, String message) {
		super(parent);
		this.modifiedElements = modifiedElements;
		this.title = title;
		this.message = message;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) 
	{
		createButton(parent, ID_SAVE, Messages.getString("SaveResourcesDlg.2"), true); //$NON-NLS-1$
		createButton(parent, ID_CANCEL, Messages.getString("SaveResourcesDlg.3"), false); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId) 
	{
		switch(buttonId)
		{
			case ID_SAVE: savePressed();
			break;
			case ID_CANCEL: cancelPressed();
			break;
		}
	}
	
	/**
	 * Speichert alle mit einem Hkchen ausgewhlten Ressourcen in einer Liste
	 * und alle nicht ausgewhlten Ressourcen in einer anderen Liste. Die ausgewhlten
	 * sollen spter gespeichert werden, die anderen verworfen.
	 * @see #getElementsToSave()
	 * @see #getElementsToDiscard()
	 */
	protected void savePressed() 
	{
		// alle Elemente mit Hckchen speichern
		Object[] scriptArray = saveTV.getCheckedElements();
		toSave = new Vector(scriptArray.length);
		for(int i = 0; i < scriptArray.length; ++i)
			toSave.add(scriptArray[i]);
		
		// alle ohne Hckchen verwerfen
		toDiscard = new Vector( modifiedElements.size() );
		for(int i = 0; i < modifiedElements.size(); ++i)
		{
			if( !saveTV.getChecked(modifiedElements.get(i)) )
				toDiscard.add( modifiedElements.get(i) );
		}
		
		setReturnCode(ID_SAVE);
		close();
	}
	
	protected void cancelPressed() 
	{
		setReturnCode(ID_CANCEL);
		close();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		getShell().setText(title);
		return super.createContents(parent);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		Composite dlgArea =  new Composite(parent, SWT.NULL);
		dlgArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = gl.verticalSpacing = 15;
		dlgArea.setLayout(gl);
		
		Label lbl = new Label(dlgArea, SWT.NULL);
		lbl.setText(message);
		
		Table table = new Table(dlgArea, SWT.CHECK|SWT.BORDER);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.widthHint = 350;
		table.setLayoutData(gd);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("SaveResourcesDlg.4")); //$NON-NLS-1$
		col.setWidth(300);
		
		saveTV = new CheckboxTableViewer(table);
		saveTV.setLabelProvider( new BaseLabelProvider() {
			public String getColumnText(Object element, int columnIndex) {
				if(columnIndex == 0)
					return ((Changeable)element).getName();
				
				return ""; //$NON-NLS-1$
			}
			
			public Image getColumnImage(Object element, int columnIndex) {
				if(columnIndex == 0)
				{
					if(element instanceof HTSScript)
						return AtosJApp.getInstance().getImage(AtosJApp.IMG_SCRIPT);
					
					if(element instanceof HTSPackage)
						return AtosJApp.getInstance().getImage(AtosJApp.IMG_PACKAGE);
					
					if(element instanceof URFFile)
						return AtosJApp.getInstance().getImage(AtosJApp.IMG_ID);
				}
				
				return null;
			}
		});
		saveTV.setContentProvider( new ArrayContentProvider() );
		// Sortieren nach Name und Klasse
		saveTV.setSorter( new UniversalViewerSorter(new ChangeableNameComparator(), new Class[] {URFFile.class, HTSScript.class, HTSPackage.class}) );
		
		saveTV.setInput(modifiedElements);
		saveTV.setAllChecked(true);
		
		Composite buttonArea = new Composite(dlgArea, SWT.NULL);
		gl = new GridLayout(2, true);
		gl.marginHeight = gl.marginWidth = 0;
		buttonArea.setLayout(gl);
		
		Button selectAllBtn = SWTToolkit.createButton(buttonArea, SWT.PUSH, Messages.getString("SaveResourcesDlg.6")); //$NON-NLS-1$
		selectAllBtn.setLayoutData( new GridData(GridData.HORIZONTAL_ALIGN_FILL) );
		selectAllBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				saveTV.setAllChecked(true);
			}
			
		});
		Button deselectAllBtn = SWTToolkit.createButton(buttonArea, SWT.PUSH, Messages.getString("SaveResourcesDlg.7")); //$NON-NLS-1$
		deselectAllBtn.setLayoutData( new GridData(GridData.HORIZONTAL_ALIGN_FILL) );
		deselectAllBtn.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				saveTV.setAllChecked(false);
			}
		});

		applyDialogFont(dlgArea);
		return dlgArea;
	}
	
	/**
	 * Liefert alle genderten Ressourcen, die dem Dialog im Konstruktor bergeben wurden.
	 * @return Vector(HTSScript) die genderten Resourcen.
	 */
	public Vector getModifiedElements() {
		return modifiedElements;
	}
	
	/**
	 * Liefert alle Ressourcen, die gespeichert werden sollen.
	 * @return Vector(Changeable) nur gltig, wenn der Dialog mit <code>ID_SAVE</code> beendet wurde.
	 */
	public Vector getElementsToSave() {
		return toSave;
	}
	
	/**
	 * Liefert alle Ressourcen, die verworfen werden sollen.
	 * @return Vector(Changeable) nur gltig, wenn der Dialog mit <code>ID_SAVE</code> beendet wurde.
	 */
	public Vector getElementsToDiscard() {
		return toDiscard;
	}
	
}
