package atosj.dialog;

import java.util.Vector;

import org.eclipse.swt.graphics.RGB;

import atosj.Messages;
import atosj.util.Statistics;

/**
 * Liefert Statistiken eines ausfhrbaren Elements (z.B. Kommando), zur Anzeige in einer Tabelle.
 * @author Nicos Tegos
 **/
public abstract class StatisticsTableElement implements Comparable
{
	public static RGB RGB_IS_RUNNING = new RGB(154, 209, 254);
	public static RGB RGB_NOT_RUN = new RGB(200, 200, 200);
	public static RGB RGB_ERR = new RGB(255, 0, 0);
	public static RGB RGB_PASSED = new RGB(0, 255, 0);
	public static RGB RGB_PART_ERR = new RGB(255, 128, 0); 
	
	/** Status: Fehler in allen Durchlufen. */
	public static final int STAT_ERR = 0;
	/** Status: Fehlerfrei in allen Durchlufen. */
	public static final int STAT_PASSED = 1;
	/** Status: Noch nicht ausgefhrt. */
	public static final int STAT_NOT_RUN = 2;
	/** Status: In Ausfhrung. */
	public static final int STAT_IS_RUNNING = 3;
	/** Status: Fehler in einigen Durchlufen, nicht in allen */
	public static final int STAT_PART_ERR = 4;
		
	private Object element;
	private int elementIndex;
	private Vector statistics = new Vector(); // Vector(Statistics)
	private boolean running = false;
	
	/**
	 * Erstellt eine neue Statistik fr <code>element</code>.
	 * @param element Das ausfhrbare Element.
	 * @param elementIndex Index des Elementes in einer Sammlung gleicher Elementtypen,
	 * z.B. Index eines Kommandos in seinem Skript.
	 */
	public StatisticsTableElement(Object element, int elementIndex) {
		this.element = element;
		this.elementIndex = elementIndex;
	}
	
	/**
	 * Vergleich anhand des Elementindex.
	 */
	public int compareTo(Object other) 
	{
		if(other instanceof StatisticsTableElement)
		{
			Integer thisVal = new Integer( elementIndex );
			Integer otherVal = new Integer( ((StatisticsTableElement) other).elementIndex );
			
			return thisVal.compareTo(otherVal);
		}

		return -1;
	}
	
	public int hashCode() {
		return elementIndex;
	}
	
	/**
	 * Liefert den Elementindex.
	 */
	public int getIndex() {
		return elementIndex;
	}
	
	/**
	 * Liefert das ausfhrbare Element, zu dem Statistiken vermerkt werden.
	 */
	protected Object getElement() {
		return element;
	}
	
	/**
	 * Erweitert die Statistik um einen weitere Ausfhrung.
	 * @param stat Die Statistik des neuen Durchlaufs.
	 */
	public void update(Statistics stat) 
	{
		running = false;
		statistics.add(stat);
	}
	
	/**
	 * Die Ausfhrung des enthaltenen Elementes wurde gestartet.
	 */
	public void setRunning() {
		this.running = true;
	}
	
	/**
	 * Setzt die Statistik zurck.
	 */
	public void reset() {
		running = false;
		statistics.clear();
	}
	
	/** 
	 * Liefert die Nummer (<code>getIndex()+1</code>) des Elements.
	 */
	public String getLineNumber() {
		return "" + (elementIndex+1); //$NON-NLS-1$
	}
	
	/**
	 * Liefert die Durchschnittliche Ausfhrungszeit.
	 */
	public String getAvgDuration() {
		int duration = 0;
		for(int i = 0; i < statistics.size(); ++i)
			duration += ((Statistics)statistics.get(i)).getDuration();
		
		if( getRunsInt() > 0 )
			return "" + duration / statistics.size(); //$NON-NLS-1$
		else
			return "" + 0; //$NON-NLS-1$
	}
	
	/**
	 * Liefert Den Ausfhrungsstatus, siehe STAT_* Konstanten.
	 */
	private int getStatusConst() 
	{
		if( running )
			return STAT_IS_RUNNING;
		
		if( statistics.size() == 0 )
			return STAT_NOT_RUN;
		
		int errors = getNumOfErrors();
		
		if(errors == 0)
			return STAT_PASSED;
		if(errors == statistics.size())
			return STAT_ERR;
		if(errors < statistics.size() )
			return STAT_PART_ERR;
		
		return STAT_NOT_RUN;
	}
	
	/**
	 * Liefert die Anzahl der Fehler in allen Durchlufen.
	 */
	private int getNumOfErrors() 
	{
		int errors = 0;
		for(int i = 0; i < statistics.size(); ++i)
		{
			Statistics stat = (Statistics)statistics.get(i);
			if( stat.hasError() )
				++errors;
		}
		
		return errors;
	}
	
	/**
	 * Liefert Die Anzahl der Ausfhrungen.
	 */
	private int getRunsInt() {
		return statistics.size();
	}
	
	/**
	 * Liefert Den Ausfhrungsstatus in textueller Form.
	 */
	public String getStatus() 
	{
		switch( getStatusConst() )
		{
			case STAT_NOT_RUN: return Messages.getString("StatisticsTableElement.3"); //$NON-NLS-1$
			case STAT_PASSED: return Messages.getString("StatisticsTableElement.4"); //$NON-NLS-1$
			case STAT_ERR: return Messages.getString("StatisticsTableElement.5"); //$NON-NLS-1$
			case STAT_PART_ERR: return Messages.getString("StatisticsTableElement.6"); //$NON-NLS-1$
			case STAT_IS_RUNNING: return Messages.getString("StatisticsTableElement.7"); //$NON-NLS-1$
			default: return ""; //$NON-NLS-1$
		}
	}
	
	/**
	 * Gibt an ob fehlerhafte Ausfhrungen stattgefunden haben.
	 * @return <code>true</code>, wenn fehlerhaften Ausfhrungen.
	 */
	public boolean hasError() {
		int stat = getStatusConst();
		return stat == STAT_ERR || stat == STAT_PART_ERR;
	}
	
	/**
	 * Liefert eine Fehlermeldung.
	 * @return Die Fehlermeldung oder ein leerer <code>String</code>, wenn alle
	 * Ausfhrungen fehlerfrei waren.
	 */
	public String getError() 
	{
		int status = getStatusConst();
		
		if(status == STAT_ERR || status == STAT_PART_ERR)
		{
			if( getRunsInt() == 1 )
			{
				return ((Statistics)statistics.get(0)).getError();
			}
			else
			{
				String error = getNumOfErrors() + Messages.getString("StatisticsTableElement.9") + getRunsInt() + Messages.getString("StatisticsTableElement.10"); //$NON-NLS-1$ //$NON-NLS-2$
				for(int i = 0; i < statistics.size(); ++i)
				{
					Statistics stat = (Statistics)statistics.get(i);
					if( stat.hasError() )
					{
						error += Messages.getString("StatisticsTableElement.11") + (i+1) + ":\n"; //$NON-NLS-1$ //$NON-NLS-2$
						error += stat.getError();
					}
				}
				return error;
			}
		}
		else
			return ""; //$NON-NLS-1$
	}
	
	/**
	 * Liefert die Anzahl der Durchlufe.
	 */
	public String getRuns() {
		return "" + getRunsInt(); //$NON-NLS-1$
	}
	
	/**
	 * Liefert eine Farbe passend zum Ausfhrungsstatus, beispielsweise
	 * rot fr eine fehlerhafte Ausfhrung.
	 */
	public RGB getStatusColor() 
	{
		switch( getStatusConst() )
		{
			case STAT_NOT_RUN: return RGB_NOT_RUN;
			case STAT_PASSED: return RGB_PASSED;
			case STAT_ERR: return RGB_ERR;
			case STAT_PART_ERR: return RGB_PART_ERR;
			case STAT_IS_RUNNING: return RGB_IS_RUNNING;
			default: return new RGB(255, 255, 255);
		}
	}
}

