/*
 * Created on 07.01.2006
 */
package atosj.editors;

import java.util.Iterator;
import java.util.Vector;

import atosj.listener.StateChangeListener;

/**
 * Diese Klasse ist die Basisklasse aller nderbaren Ressourcen in ATOSj.
 * Sie ermglicht die Benachrichtigung von Listenern ber nderungen.
 * @author Nicos Tegos
 */
public abstract class BaseChangeable implements Changeable 
{
	private Vector listeners = new Vector(); // Vector(Object) beliebige Listener
	private boolean modified = false;
	private int validState = VALID;
	private String message = ""; // letzte Fehler oder Warnmeldung

	/**
	 * Benachrichtigt alle <code>StateChangeListener</code> ber nderungen der Ressource.
	 * @param state Konstante, die den genderten Zustand angibt, sieh STAT_* Konstanten
	 * aus der Klasse <code>StateChangeListener</code>..
	 * @param oldValue Der Zustandswert vor der nderung.
	 * @param newValue Der Zustandswert nach der nderung.
	 * @see StateChangeListener
	 */
	protected synchronized void fireStateChange(int state, Object oldValue, Object newValue) {
		Iterator it = listeners.iterator();
		while( it.hasNext() )
		{
			Object listener = it.next();
			if(listener instanceof StateChangeListener)
				((StateChangeListener)listener).stateChanged(this, state, oldValue, newValue);
		}
	}

	/* (non-Javadoc)
	 * @see atosj.editors.Changeable#addStateChangeListener(atosj.listener.StateChangeListener)
	 */
	public synchronized void addStateChangeListener(StateChangeListener listener) {
		addListener(listener);
	}

	/* (non-Javadoc)
	 * @see atosj.editors.Changeable#removeStateChangeListener(atosj.listener.StateChangeListener)
	 */
	public synchronized void removeStateChangeListener(StateChangeListener listener) {
		removeListener(listener);
	}
	
	/**
	 * Registriert einen beliebigen Listener, falls dieser noch
	 * nicht registriert wurde.
	 * @param listener Der neue Listener.
	 */
	protected synchronized void addListener(Object listener) {
		if( !listeners.contains(listener) )
			listeners.add(listener);
	}
	
	/**
	 * Entfernt einen Listener.
	 * @param listener Der zu entfernende Listener.
	 */
	protected synchronized void removeListener(Object listener) {
		listeners.remove(listener);
	}
	
	/**
	 * Liefert alle registrierten Listener.
	 * @return Vector(Object)
	 */
	protected Vector getListeners() {
		return listeners;
	}

	/* (non-Javadoc)
	 * @see atosj.editors.Changeable#isModified()
	 */
	public boolean isModified() {
		return modified;
	}

	/* (non-Javadoc)
	 * @see atosj.editors.Changeable#getValidState()
	 */
	public int getValidState() {
		return validState;
	}

	/**
	 * Liefert die letzte Fehler- oder Warnmeldung.
	 */
	public String getMessage() {
		return message;
	}
	
	/**
	 * Setzt den nderungsstatus Ressource und benachrichtigt alle
	 * Listener, bei einem Wechsel des Zustandswertes.
	 * @param newModified wenn <code>true</code> dann bekommt die Ressource
	 * den Status gendert, sonst ungendert.
	 * @see #isModified()
	 */
	protected void setModified(boolean newModified) 
	{
		if(modified != newModified)
		{
			boolean oldModified = modified;
			modified = newModified;
			fireStateChange(StateChangeListener.STATE_MODIFIED, 
					new Boolean(oldModified),
					new Boolean(newModified));
		}
	}
	
	/**
	 * Kennzeichnet die Ressource als fehlerhaft.
	 * @param message Die Fehlermeldung.
	 * @see #getValidState()
	 */
	protected void setError(String message) 
	{
		this.message = message;
		setValidState(INVALID);
	}
	
	/**
	 * Versieht die Ressource mit einer Warnung.
	 * @param message Die Warnmeldung.
	 * @see #getValidState()
	 */
	protected void setWarning(String message)
	{
		this.message = message;
		setValidState(VALID_WITH_WARNING);
	}
	
	/**
	 * Setzt die Ressource auf einen gltigen Zustand.
	 * Fehler- und Warnmeldungen werden gelscht.
	 * @see #getValidState()
	 */
	protected void setValid()
	{
		this.message = "";
		setValidState(VALID);
	}
	
	/**
	 * Setzt den Zustand der Ressource entsprechend der Konstanten VALID, VALID_WITH_WARNING,
	 * INVALID und benachrichtigt die <code>StateChangeListener</code> ber einen eventuellen
	 * Zustandswechsel.
	 * @param newState Der neue Gltigkeitsstatus.
	 */
	private void setValidState(int newState) 
	{
		if(validState != newState)
		{
			int oldState = validState;
			validState = newState;
			fireStateChange(StateChangeListener.STATE_VALID, new Integer(oldState), new Integer(newState) );
		}
	}
}
