/*
 * Created on 17.11.2005
 */
package atosj.editors;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import atosj.listener.StateChangeListener;
import atosj.listener.UndoManagerListener;

/**
 * Ein graphischer Editor fr das Bearbeiten einer Ressource.
 * Der Editor wird in einem Tab angezeigt.
 * @author Nicos Tegos
 */
public abstract class Editor implements StateChangeListener
{
	private Changeable input;
	private CTabItem tabItem;
	private MenuManager menu;
	private UndoManager undoMngr;
	
	/**
	 * Erstellt einen neuen Editor.
	 * @param parent Der Tabfolder fr das Tab des Editors.
	 * @param undoMngr Sorgt fr das Rckgngigmachen von Aktionen im Editor.
	 */
	public Editor(CTabFolder parent, UndoManager undoMngr) 
	{
		this.undoMngr = undoMngr;
		tabItem = new CTabItem(parent, SWT.CLOSE);
		
		tabItem.addDisposeListener( new DisposeListener() {
			public void widgetDisposed(DisposeEvent arg0) {
				dispose();
			}
		});
		
		tabItem.setData(this);
		
		Composite c = new Composite(parent, SWT.NULL);
		FillLayout fl = new FillLayout();
		fl.marginHeight = fl.marginWidth = 0;
		c.setLayout(fl);
		init();
		createContents(c); // ACHTUNG mus vor set control ausgefhrt werden
		tabItem.setControl(c);
		
		menu = createMenu();
	}
		
	/**
	 * Erstellt einen neuen Editor.
	 * @param parent Der Tabfolder fr das Tab des Editors.
	 */
	public Editor(CTabFolder parent) {
		this(parent, null);
	}
	
	/**
	 * Dient der initialisierung von Variablen vor dem Aufruf
	 * von <code>createContents</code>. Default leere Implementation.
	 * @see #createContents(Composite)
	 */
	protected void init() {}
	/**
	 * Setzt die Beschriftung des Editor Tabs automatisch mit
	 * dem Namen des Inputs.
	 * @see Changeable#getName()
	 */
	protected void setTitle() { tabItem.setText( (input.isModified() ? "*" : "") + getInputName() ); }
	/**
	 * Liefert die aktuelle Beschriftung des Editor Tabs.
	 */
	public String getTitle() { return tabItem.getText(); }
	/**
	 * Setzt das mit <code>getTitleImage</code> ermittelte Bild als Icon fr
	 * das Editor Tab.
	 */
	protected void setImage() { tabItem.setImage( getTitleImage() ); }
	/**
	 * Liefert das Editor Tab.
	 */
	public CTabItem getTabItem() { return tabItem; }
	
	/**
	 * Liefert das Kontextmen fr diesen Editor (wird an das Standardkontextmen
	 * angehangen) oder <code>null</code>, falls kein Zusatzmen. Wird aktiviert, wenn 
	 * mit der rechten Maustaste auf das Tab des Editors gedrckt wird.
	 * @see Editor#createMenu()
	 */	
	public MenuManager getMenu() { return menu; }
	/**
	 * Erstellt das Kontextmen fr diesen Editor. (Einmaliger Aufruf im Konstruktor)
	 * @return Das neu erstellte Kontextmen oder <code>null</code>, 
	 * falls kein Men erwnscht.
	 */
	protected MenuManager createMenu() { return null; }
	
	/**
	 * Setzt die Ressource fr diesen Editor.
	 * @param newInput Die neue Ressource.
	 */
	public void setInput(Changeable newInput) 
	{ 
		if(input != null) {
			((Changeable)input).removeStateChangeListener(this);
		}
		
		input = newInput;
		input.addStateChangeListener(this);
		setTitle();		
		setImage();
		
		refresh(); 
	}
	
	/**
	 * Liefert die aktuell gesetzte Ressource fr den Editor.
	 * @see #setInput(Changeable)
	 */
	public Changeable getInput() { return input; }
	
	/**
	 * Liefert den Namen der aktuellen Ressource.
	 * @see Changeable#getName()
	 */
	public String getInputName() { return input.getName(); };
	
	/**
	 * Soll das Icon fr das Editor Tab ermitteln. Dieses sollte
	 * den Gltigkeitsstatus der Ressource widerspiegeln.
	 * @see Changeable#getValidState()
	 */
	public abstract Image getTitleImage();
	
	/**
	 * Erstellt die graphischen Elemente des Editors.
	 * @param parent Das graph. Vaterelement, mit einem <code>FillLayout</code> 
	 * ohne Rand.
	 */
	public abstract void createContents(Composite parent);
	
	/**
	 * Gibt an ob die Ressource gendert wurde.
	 * Leitet den Aufruf an die Methode <code>isModified</code> aus
	 * der Klasse <code>Changeable</code> weiter.
	 * @see Changeable#isModified()
	 */
	public boolean isModified() {
		return input.isModified();
	}
	
	/**
	 * @param save wenn true dann sollen die derungen bernommen werden,
	 * wenn false dann sollen die nderungen verworfen werden.
	 * @return wenn true wird der Editor geschlossen, wenn false
	 * bleibt er offen.
	 */
	public abstract boolean commit(boolean save);
	
	/**
	 * Ldt den Inhalt aller graph. Elemente neu. 
	 */
	protected abstract void refresh();

	/**
	 * Diese Funktion wird gerufen nachdem das Editor Tab disposed wurde.
	 * Die Basisimplementation entfernt den Editor aus der Liste der Listener
	 * der Ressource.
	 * @see Changeable#removeStateChangeListener(StateChangeListener)
	 */
	protected void dispose() { if(input != null) input.removeStateChangeListener(this); }
	
	/**
	 * Setzt den Focus auf ein <code>Control</code> innerhalb des Editors,
	 * wenn dieser aktiviert wird.
	 */
	public abstract void setFocus();
	
	/**
	 * Wird gerufen bevor der Nutzer zwischen diesem und einem anderen Editor wechselt.
	 * @return <code>true</code> wenn gewechselt werden darf, <code>false</code> sonst.
	 */
	public boolean onSwitch() { return true; }
	
	/**
	 * Setzt die Beschriftung und das Icon des Editor Tabs bei einer nderung der Ressource.
	 */
	public void stateChanged(Changeable changed, int state, Object oldValue, Object newValue) 
	{
		if(state == StateChangeListener.STATE_MODIFIED || state == StateChangeListener.STATE_NAME)
			setTitle();
		
		if(state == StateChangeListener.STATE_VALID)
			setImage();
	}
	
	/**
	 * Registriert einen Listener, der ber Vernderungen des <code>UndoManager</code>s
	 * des Editors benachrichtigt wird.
	 * @param listener Der Listener.
	 * @see #removeUndoManagerListener(UndoManagerListener)
	 */
	public void addUndoManagerListener(UndoManagerListener listener) {
		if(undoMngr != null)
			undoMngr.addUndoManagerListener(listener);
	}
	
	/**
	 * Entfernt den angegebenen Listener.
	 * @param listener Der Listener.
	 * @see #addUndoManagerListener(UndoManagerListener)
	 */
	public void removeUndoManagerListener(UndoManagerListener listener) {
		if(undoMngr != null)
			undoMngr.removeUndoManagerListener(listener);
	}
	
	/**
	 * Liefert den <code>UndoManager</code> dieses Editors-
	 */
	public UndoManager getUndoManager() {
		return undoMngr;
	}
}
