/*
 * Created on 13.04.2006
 */
package atosj.editors;

import java.util.LinkedList;
import java.util.Vector;

import atosj.Messages;
import atosj.listener.UndoManagerListener;

/**
 * Verwaltet das Rckgngigmachen von Nutzeraktionen in einem Editor.
 * @author Nicos Tegos
 */
public abstract class UndoManager {
	private static final int MAX_SIZE = 5;
	private LinkedList undoList;
	private LinkedList redoList;
	
	private Vector listeners = new Vector();
	
	public UndoManager() {
		undoList = new LinkedList();
		redoList = new LinkedList();
	}

	/**
	 * Zeigt an ob Aktionen zum Rckgngigmachen gespeichert sind.
	 * @return <code>true</code> wenn Aktionen zum Rckgngigmachen
	 * gespeichert sind.
	 */
	public boolean hasUndoElements() {
		return !undoList.isEmpty();
	}
	
	/**
	 * Zeigt an ob Aktionen zur Wiederherstellung existieren.
	 * @return <code>true</code> wenn aktuell Aktionen, die vorher rckgngiggemacht 
	 * wurden wiederhergestellt werden knnen.
	 */
	public boolean hasRedoElements() {
		return !redoList.isEmpty();
	}
	
	/**
	 * Fgt der Liste der <code>UndoElement</code>e ein neues Element hinzu.
	 * Es gilt das LIFO Prinzip.
	 */
	protected void addUndoElement(UndoElement element) 
	{
		if(undoList.size() == MAX_SIZE)
			undoList.removeFirst();
		
		undoList.addLast(element);
		redoList.clear();
		fireChange();
	}
	
	/**
	 * Wenn ein Element aus der Undo-Liste genommen wird
	 * kommt es automatisch in die Redo-Liste. Auch fr
	 * die Redo-Liste gilt das LIFO Prinzip.
	 */
	private void addRedoElement(UndoElement element) 
	{
		if(redoList.size() == MAX_SIZE)
			redoList.removeFirst();
		
		redoList.addLast(element);
		fireChange();
	}
	
	/**
	 * Fhrt einen undo Schritt aus, macht eine Aktion rckgngig. 
	 * Danach kann dieses Undo ber <code>redo</code> wiederum rckgngig gemacht werden.
	 */
	public void undo() 
	{
		if( hasUndoElements() )
		{
			UndoElement element = (UndoElement) undoList.removeLast();
			performUndo(element);
			
			addRedoElement(element);
			fireChange();
		}
	}
	
	/**
	 * Fhrt die spezifische Undo-Operation aus.
	 * Beispiel: Lschen eines Tabellenelementes rckgngig machen.
	 */
	protected abstract void performUndo(UndoElement element);
	
	/**
	 * Fhrt einen Redo-Schritt aus. Danach kann ist das entsprechende
	 * Undo wieder verfgbar.
	 */
	public void redo() 
	{
		if( hasRedoElements() )
		{
			UndoElement element = (UndoElement) redoList.removeLast();
			performRedo(element);
			
			if(undoList.size() == MAX_SIZE)
				undoList.removeFirst();
			undoList.addLast(element);

			fireChange();
		}
	}
	
	/**
	 * Fhrt die spezifische Redo-Operation aus.
	 */
	protected abstract void performRedo(UndoElement element);
	
	/**
	 * Leert die Listen der mglichen Undo- und Redo-Aktionen.
	 */
	public void clear() {
		undoList.clear();
		redoList.clear();
		fireChange();
	}
	
	/**
	 * Gibt zu einer bestimmten Operation den Klarnamen an, z.B. "Lschen rckgngig machen".
	 * @param operation Der Operationscode, der ber die Methode <code>getOperation</code>
	 * aus der Klasse <code>UndoElement</code> ermittelt werden kann.
	 * abgefragt wurde.
	 * @return Den Klarnamen der Operation.
	 * @see UndoElement#getOperation()
	 */
	protected abstract String getOperationName(int operation);
	
	/**
	 * Liefert den Klarnamen der aktuellen Aktion in der Undo-Liste.
	 * @return Den Klarnamen oder einen leeren String, falls die Undo-Liste leer ist.
	 * @see #hasUndoElements()
	 */
	public String getCurrentUndoName() {
		if( hasUndoElements() )
			return getOperationName( ((UndoElement)undoList.getLast()).getOperation() )
				+ Messages.getString("UndoManager.0"); //$NON-NLS-1$
		
		return ""; //$NON-NLS-1$
	}
	
	/**
	 * Liefert den Klarnamen der aktuellen Aktion in der Redo-Liste.
	 * @return Den Klarnamen oder einen leeren String, falls die Redo-Liste leer ist.
	 * @see #hasRedoElements()
	 */
	public String getCurrentRedName() {
		if( hasRedoElements() )
			return getOperationName( ((UndoElement)redoList.getLast()).getOperation() )
				+ Messages.getString("UndoManager.2"); //$NON-NLS-1$
		
		return ""; //$NON-NLS-1$
	}
	
	/**
	 * Benachrichtig alle Listener, wenn sich die Undo- oder Redo-Liste
	 * ndert.
	 */
	private synchronized void fireChange() {
		for(int i = 0; i < listeners.size(); ++i)
			((UndoManagerListener)listeners.get(i)).undoManagerChanged(this);
	}
	
	public synchronized void addUndoManagerListener(UndoManagerListener listener) {
		if( !listeners.contains(listener) )
			listeners.add(listener);
	}
	
	public synchronized void removeUndoManagerListener(UndoManagerListener listener) {
		listeners.remove(listener);
	}
}
