/*
 * Created on 28.11.2005
 */
package atosj.editors.pack;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.dialog.AddScriptDlg;
import atosj.dialog.PositionDlg;
import atosj.dialog.PositionWidget;
import atosj.editors.Changeable;
import atosj.editors.Editor;
import atosj.exception.AtosJException;
import atosj.gui.BalloonWindow;
import atosj.gui.TableResizeListener;
import atosj.listener.ClipboardListener;
import atosj.model.pack.HTSPackage;
import atosj.model.pack.PackageElement;
import atosj.model.script.HTSScript;
import atosj.util.AcceleratorHook;
import atosj.util.Clipboard;
import atosj.util.DefaultImageDescriptor;

/**
 * Basisklasse fr alle Aktionen zum Verschieben eines Skriptes in einem Paket.
 */
class MoveAction extends Action implements ISelectionChangedListener
{
	private TableViewer tv;
	private PackageUndoManager undoMngr;
	
	/**
	 * Erstellt eine neue Aktion zur Positionsnderung von Skripten.
	 * @param title Der Titel der Aktion.
	 * @param tooltip Der Tooltip Text.
	 * @param enabledDescr Das Bild fr den Zustand aktiviert.
	 * @param disabledDescr Das Bild fr den Zustand deaktiviert.
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein <code>HTSPackage</code> haben.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	public MoveAction(String title, 
			String tooltip, 
			ImageDescriptor enabledDescr, 
			ImageDescriptor disabledDescr,
			TableViewer tv,
			PackageUndoManager undoMngr)
	{
		super(title);
		setToolTipText(tooltip);
		setImageDescriptor(enabledDescr);
		setDisabledImageDescriptor(disabledDescr);
		
		this.tv = tv;
		this.undoMngr = undoMngr;
		
		tv.addSelectionChangedListener(this);
		setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 );
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent arg0) {
		setEnabled( ((StructuredSelection) tv.getSelection()).size() == 1 );
	}

	/**
	 * Liefert das derzeit in der Tabelle ausgewhlte Skript oder <code>null</code>, falls
	 * keins ausgewhlt. 
	 */
	protected PackageElement getCurrentlySelected() {
		return (PackageElement) ((StructuredSelection)tv.getSelection()).getFirstElement();
	}
	
	protected HTSPackage getPackage() {
		return (HTSPackage)tv.getInput();
	}
	
	/**
	 * Verschiebt das angegebene Element an die neue Position.
	 * @param toMove Das zu verschiebende Element.
	 * @param newPos Die neue Position.
	 * @see HTSPackage#moveElement(PackageElement, int)
	 */
	protected void moveElement(PackageElement toMove, int newPos) {
		int oldPos = toMove.getIndex();
		getPackage().moveElement(toMove, newPos);
		undoMngr.addUndoMove(toMove, oldPos);
	}
}

/**
 * Verschiebt ein Skript um eine Position nach oben im Testpaket.
 */
class MoveUpAction extends MoveAction
{
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein Testpaket haben.
	 * @param undoManager Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	MoveUpAction(TableViewer tv, PackageUndoManager undoManager) {
		super(Messages.getString("PackageEditor.0"), //$NON-NLS-1$
			Messages.getString("PackageEditor.1"), //$NON-NLS-1$
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_UP),
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_UP_DIS),
			tv, undoManager);
	}

	public void run() {
		PackageElement toMove = getCurrentlySelected();
		int index = toMove.getIndex();
		if( index > 0)
			moveElement(toMove, --index);
	}
}

/**
 * Verschiebt ein Skript um eine Position nach unten im Testpaket.
 */
class MoveDownAction extends MoveAction
{
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein Testpaket haben.
	 * @param undoManager Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	MoveDownAction(TableViewer tv, PackageUndoManager undoManager) {
		super(Messages.getString("PackageEditor.2"), //$NON-NLS-1$
		Messages.getString("PackageEditor.3"), //$NON-NLS-1$
		new DefaultImageDescriptor(AtosJApp.IMG_ARROW_DOWN),
		new DefaultImageDescriptor(AtosJApp.IMG_ARROW_DOWN_DIS),
		tv, undoManager);
	}

	public void run() {
		PackageElement toMove = getCurrentlySelected();
		int index = toMove.getIndex();
		if( index < getPackage().getNumberOfScripts()-1 )
			moveElement(toMove, ++index);
	}
}

/**
 * Verschiebt ein Skript an den Anfang des Testpakets.
 */
class MoveTopAction extends MoveAction
{
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein Testpaket haben.
	 * @param undoManager Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	MoveTopAction(TableViewer tv, PackageUndoManager undoManager) {
		super(Messages.getString("PackageEditor.4"), //$NON-NLS-1$
			Messages.getString("PackageEditor.5"), //$NON-NLS-1$
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_TOP),
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_TOP_DIS),
			tv, undoManager);
	}

	public void run() {
		PackageElement toMove = getCurrentlySelected();
		if(toMove.getIndex() > 0)
			moveElement(toMove, 0);
	}
}

/**
 * Verschiebt ein Skript an das Ende des Testpakets.
 */
class MoveBottomAction extends MoveAction
{
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein Testpaket haben.
	 * @param undoManager Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	MoveBottomAction(TableViewer tv, PackageUndoManager undoManager) {
		super(Messages.getString("PackageEditor.6"), //$NON-NLS-1$
			Messages.getString("PackageEditor.7"), //$NON-NLS-1$
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_BOTTOM),
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_BOTTOM_DIS),
			tv, undoManager);
	}

	public void run() {
		PackageElement toMove = getCurrentlySelected();
		if(toMove.getIndex() < getPackage().getNumberOfScripts() -1)
			moveElement(toMove, getPackage().getNumberOfScripts()-1);
	}
}

/**
 * Fgt dem Testpaket neue Skripte hinzu.
 */
class AddScriptAction extends Action
{
	private TableViewer tv;
	private PackageUndoManager undoMngr;

	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein Testpaket haben.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	AddScriptAction(TableViewer tv, PackageUndoManager undoMngr) {
		super(Messages.getString("PackageEditor.8")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'N');
		setToolTipText(Messages.getString("PackageEditor.9")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ADD) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ADD_DIS) );
		
		this.tv = tv;
		this.undoMngr = undoMngr;
	}

	private HTSPackage getPackage() {
		return (HTSPackage) tv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		// aktuellen Index bestimmen
		PackageElement currentElement = (PackageElement) ((StructuredSelection)tv.getSelection()).getFirstElement();
		int currentPos = PositionWidget.NO_CURRENT_POS;
		if(currentElement != null)
			currentPos = currentElement.getIndex();
		
		AddScriptDlg dlg = new AddScriptDlg(
				Display.getCurrent().getActiveShell(), 
				getPackage(), 
				getPackage().getNumberOfScripts(),
				currentPos);
		if( dlg.open() == AddScriptDlg.ID_ADD )
			addScripts(dlg.getScripts(), dlg.getPosition());
	}
	
	private void addScripts(Collection scripts, int startIndex) 
	{
		Iterator it = scripts.iterator();
		Vector addedElements = new Vector(scripts.size());
		int elementIndex = startIndex;
		while( it.hasNext() )
		{
			PackageElement newElement = new PackageElement( (HTSScript)it.next(), elementIndex );
			addedElements.add(newElement);
			
			++elementIndex;
		}
		
		getPackage().addElements(addedElements, startIndex);
		tv.setSelection( new StructuredSelection(addedElements) );
		undoMngr.addUndoAdd(addedElements, startIndex);
	}
}

/**
 * Ruft den Editor fr ein Skript auf.
 */
class EditScriptAction extends Action
{
	private TableViewer tv;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Skripte.
	 */
	EditScriptAction(TableViewer _tv) {
		super(Messages.getString("PackageEditor.10")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'E');
		setToolTipText(Messages.getString("PackageEditor.11")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_EDIT) );
		
		this.tv = _tv;
		
		setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 );
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 );
			}
		});
	}
	
	public void run() {
		// den editor fr das ausgewhlte script ffnen
		PackageElement element = (PackageElement)((StructuredSelection)tv.getSelection()).getFirstElement();
		AtosJApp.getInstance().getMainWindow().openEditor( element.getScript() );
	}
}

/**
 * Lscht ausgewhlte Skripte aus einem Paket.
 */
class RemoveScriptAction extends Action
{
	private TableViewer tv;
	private PackageUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein Testpaket haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	RemoveScriptAction(TableViewer _tv, PackageUndoManager _undoMngr) {
		super(Messages.getString("PackageEditor.12")); //$NON-NLS-1$
		setAccelerator(SWT.DEL);
		setToolTipText(Messages.getString("PackageEditor.13")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE_DIS) );
		
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				setEnabled( !tv.getSelection().isEmpty() );
			}
		});
		
		setEnabled( !tv.getSelection().isEmpty() );
	}

	private HTSPackage getPackage() {
		return (HTSPackage) tv.getInput();
	}
	
	public void run() {
		List elements = ((StructuredSelection)tv.getSelection()).toList();
		getPackage().removeElements(elements);
		undoMngr.addUndoDelete(elements);
	}
}

/**
 * Fgt ausgewhlte Skripte in die ATOSj Zwischenablage ein.
 * @see  atosj.util.Clipboard
 */
class CopyAction extends Action
{
	private TableViewer tv;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein Testpaket haben.
	 */
	CopyAction(TableViewer _tv) 
	{
		super(Messages.getString("PackageEditor.14")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'C');
		this.tv = _tv;
	
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				setEnabled( !tv.getSelection().isEmpty() );		
			}
		});
		
		setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 );
	}

	public void run() {
		Clipboard.getInstance().setElements( ((StructuredSelection)tv.getSelection()).toList(), Clipboard.MODE_COPY );
	}
}

/**
 * Entfernt die ausgewhlten Skripte aus dem Testpaket und
 * fgt sie in die ATOSj Zwischenablage ein.
 * @see atosj.util.Clipboard
 */
class CutAction extends Action
{
	private TableViewer tv;
	private PackageUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein Testpaket haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	CutAction(TableViewer _tv, PackageUndoManager _undoMngr) 
	{
		super(Messages.getString("PackageEditor.15")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'X');
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				setEnabled( !tv.getSelection().isEmpty() );
			}
		});
		
		setEnabled( !tv.getSelection().isEmpty() );
	}
	
	private HTSPackage getPackage() {
		return (HTSPackage) tv.getInput();
	}
	
	public void run() {
		List elements = ((StructuredSelection)tv.getSelection()).toList();
		getPackage().removeElements(elements);
		Clipboard.getInstance().setElements(elements, Clipboard.MODE_CUT);
		undoMngr.addUndoCut(elements);
	}
}

/**
 * Fgt Skripte aus der Zwischenablage in das Paket ein.
 * @see atosj.util.Clipboard
 */
class PasteAction extends Action
{
	private TableViewer tv;
	private PackageUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Skripte
	 * des Testpaketes. Dieser muss als Input ein Testpaket haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	PasteAction(TableViewer _tv, PackageUndoManager _undoMngr) 
	{
		super(Messages.getString("PackageEditor.16")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'V');
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		
		Clipboard.getInstance().addClipboardListener( new ClipboardListener() {
			public void clipboardChanged(Clipboard clip) {
				setEnabled( clip.containsOnlyClass(PackageElement.class) );		
			}
		});
		
		setEnabled( Clipboard.getInstance().containsOnlyClass(PackageElement.class) );
	}

	private HTSPackage getPackage() {
		return (HTSPackage) tv.getInput();
	}
	
	public void run() 
	{
		// aktuellen Index bestimmen
		PackageElement currentElement = (PackageElement) ((StructuredSelection)tv.getSelection()).getFirstElement();
		int currentPos = PositionWidget.NO_CURRENT_POS;
		if(currentElement != null)
			currentPos = currentElement.getIndex();
		
		PositionDlg dlg = new PositionDlg(
				Display.getCurrent().getActiveShell(),
				Messages.getString("PackageEditor.17"), //$NON-NLS-1$
				getPackage().getNumberOfScripts(),
				currentPos);
		if( dlg.open() == AddScriptDlg.ID_ADD )
		{	
			int mode = Clipboard.getInstance().getMode();
			Vector original = Clipboard.getInstance().getElements(); 
			
			if(mode == Clipboard.MODE_COPY)
			{
				Vector copy = new Vector( original.size() );
				for(int i = 0; i < original.size(); ++i)
					copy.add( ((PackageElement)original.get(i)).clone() );
				getPackage().addElements( copy, dlg.getPosition() );
				tv.setSelection( new StructuredSelection(copy) );
				undoMngr.addUndoPaste(copy, dlg.getPosition(), true);
			}
			else
			{
				getPackage().addElements( original, dlg.getPosition() );
				tv.setSelection( new StructuredSelection(original) );
				undoMngr.addUndoPaste(original, dlg.getPosition(), false);
			}
		}
	}
}

/**
 * Editor zum Bearbeiten eines Testpaktes. Zeigt eine Tabelle mit allen enthaltenen Skripten an.
 * @author Nicos Tegos
 * @see atosj.model.pack.HTSPackage
 */
public class PackageEditor extends Editor 
{
	private TableViewer packTable;
	private PackageTableContentProvider packProvider;
	
	private MenuManager contextMenu;
	
	private Action addScriptAction;
	private Action editScriptAction;
	private Action removeScriptAction;
	private Action topAction;	
	private Action upAction;
	private Action downAction;
	private Action bottomAction;
	private Action copyAction;
	private Action cutAction;
	private Action pasteAction;
	
	private AcceleratorHook accHook;
	
	/**
	 * Erstellt einen neuen Editor fr ein Paket.
	 * @param parent Das Tab in dem der Editor erstellt werden soll.
	 */
	public PackageEditor(CTabFolder parent) {
		super(parent, new PackageUndoManager());
	}
	
	protected void init() {
		packProvider = new PackageTableContentProvider();
	}

	/* (non-Javadoc)
	 * @see atosj.editors.Editor#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) 
	{
		Composite area = new Composite(parent, SWT.NULL);
		GridLayout gl = new GridLayout(1, false);
		gl.marginHeight = gl.marginWidth = 5;
		area.setLayout(gl);
		
		createTableArea(area);
	}
	
	/**
	 * Erstellt die Tabelle mit Kontextmen und die Buttons zum Bearbeiten des Pakets.
	 * @param parent Das graph. Vaterelement.
	 */
	private void createTableArea(Composite parent)
	{
		Composite tableArea = new Composite(parent, SWT.NULL);
		tableArea.setLayoutData( new GridData(GridData.FILL_BOTH) );
		GridLayout gl = new GridLayout(2, false);
		gl.marginHeight = gl.marginWidth = 0;
		tableArea.setLayout(gl);
		
		packTable = new TableViewer(tableArea, SWT.BORDER| SWT.MULTI | SWT.FULL_SELECTION);
		packTable.setContentProvider(packProvider);
		packTable.setLabelProvider( new PackageTableLabelProvider() );
		packTable.setColumnProperties(PackageTableContentProvider.PROPERTIES);
				
		Table table = packTable.getTable();
		table.addListener(SWT.MenuDetect, new Listener() {
			public void handleEvent(Event e) {
				packTable.getTable().setMenu( contextMenu.getMenu() );
			}
			
		});
		
		// BalloonWindow mit Fehler/Warnung bei doppelklick auf spalte0
		// ffnen des Scripteditors bei doppelclick auf spalte 1
		table.addMouseListener( new MouseAdapter() {
			public void mouseDoubleClick(MouseEvent e) 
			{
				Point p = new Point(e.x, e.y);
				TableItem item = packTable.getTable().getItem(p);
				if( item == null )
					return;
				
				StructuredSelection ssel = (StructuredSelection) packTable.getSelection();
				if( ssel.isEmpty() )
					return;
				
				PackageElement command = (PackageElement) ssel.getFirstElement();
				
				// klick auf 0. spalte wir zeigen die Fehlermeldung an
				if( item.getBounds(0).contains(p) )
				{	
					int state = command.getScript().getValidState();
					if( state == HTSScript.INVALID || state == HTSScript.VALID_WITH_WARNING )
					{
						final BalloonWindow exWindow = new BalloonWindow(Display.getCurrent(), SWT.TITLE | SWT.CLOSE);
						exWindow.setImage(AtosJApp.getInstance().getImage(state == HTSScript.INVALID ? AtosJApp.IMG_ERROR : AtosJApp.IMG_WARNING) );
						exWindow.setTitle(state == HTSScript.INVALID ? Messages.getString("PackageEditor.18") : Messages.getString("PackageEditor.19")); //$NON-NLS-1$ //$NON-NLS-2$
						exWindow.setText( command.getScript().getMessage() );
						exWindow.setLocation( packTable.getTable().toDisplay(e.x, e.y) );
						exWindow.getShell().addShellListener( new ShellAdapter() {
							public void shellDeactivated(ShellEvent arg0) {
								exWindow.close();
							}
						});
						exWindow.open();
					}
				}
				else // klick auf 1. spalte wir ffnen den editor fr das script
				{
					if( item.getBounds(1).contains(p) )
						editScriptAction.run();
				}
			}
		});
		
		new TableResizeListener(table, new int[] {20, 40, 20, 20}, 40);
		table.setLayoutData( new GridData(GridData.FILL_BOTH) );
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setWidth(200);
		col.setText(Messages.getString("PackageEditor.20")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setWidth(200);
		col.setText(Messages.getString("PackageEditor.21")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setWidth(200);
		col.setText(Messages.getString("PackageEditor.22")); //$NON-NLS-1$
		col = new TableColumn(table, SWT.NULL);
		col.setWidth(200);
		col.setText(Messages.getString("PackageEditor.23")); //$NON-NLS-1$
		
		createActions();
		
		// die Buttonleiste erstellen
		ToolBarManager buttonsMngr = new ToolBarManager(SWT.VERTICAL);
		buttonsMngr.add(addScriptAction);
		buttonsMngr.add(editScriptAction);
		buttonsMngr.add(removeScriptAction);
		buttonsMngr.add( new Separator() );
		buttonsMngr.add(topAction);
		buttonsMngr.add(upAction);
		buttonsMngr.add(downAction);
		buttonsMngr.add(bottomAction);
		buttonsMngr.createControl(tableArea).setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		
		createContextMenu();
		initAccelerators();
	}
	
	/**
	 * Initialisiert alle Aktionen fr das Bearbeiten des Pakets.
	 */
	private void createActions() {
		PackageUndoManager undoMngr = (PackageUndoManager)getUndoManager();
		addScriptAction = new AddScriptAction(packTable, undoMngr);
		editScriptAction = new EditScriptAction(packTable);
		removeScriptAction = new RemoveScriptAction(packTable, undoMngr);
		topAction = new MoveTopAction(packTable, undoMngr);	
		upAction = new MoveUpAction(packTable, undoMngr);
		downAction = new MoveDownAction(packTable, undoMngr);
		bottomAction = new MoveBottomAction(packTable, undoMngr);
		copyAction = new CopyAction(packTable);
		cutAction = new CutAction(packTable, undoMngr);
		pasteAction =  new PasteAction(packTable, undoMngr);
	}
	
	/**
	 * Erstellt das Kontextmen fr die Tabelle.
	 */
	private void createContextMenu() {
		contextMenu = new MenuManager();
		
		contextMenu.add(addScriptAction);
		contextMenu.add(editScriptAction);
		contextMenu.add(removeScriptAction);
		contextMenu.add( new Separator() );
		contextMenu.add(topAction);
		contextMenu.add(upAction);
		contextMenu.add(downAction);
		contextMenu.add(bottomAction);
		contextMenu.add( new Separator() );
		contextMenu.add(copyAction);
		contextMenu.add(cutAction);
		contextMenu.add(pasteAction);
		
		contextMenu.createContextMenu( packTable.getTable() );
	}
	
	/**
	 * Initialisiert die Untersttzung von Tastenkrzeln fr das
	 * Bearbeiten des Pakets.
	 * @see AcceleratorHook
	 */
	private void initAccelerators() 
	{
		accHook = new AcceleratorHook( packTable.getTable() );
		
		accHook.add(addScriptAction);
		accHook.add(editScriptAction);
		accHook.add(removeScriptAction);
		accHook.add(copyAction);
		accHook.add(cutAction);
		accHook.add(pasteAction);
	}
	
	/**
	 * Liefert, das mit diesem Editor assoziierte Paket.
	 */
	private HTSPackage getPackage() {
		return (HTSPackage) getInput();
	}
	
	/* (non-Javadoc)
	 * @see atosj.editors.Editor#commit(boolean)
	 */
	public boolean commit(boolean save) {
		if(save) // speichern
		{
			try {
				getPackage().save();
				return true;
			} catch (AtosJException e) 
			{
				MessageDialog.openError(Display.getCurrent().getActiveShell(),
						Messages.getString("PackageEditor.24"), //$NON-NLS-1$
						e.getMessage());
				return false;
			}
		}
		else // verwerfen
		{
			try {
				getPackage().discard();
				return true;
			}
			catch(AtosJException e)
			{
				MessageDialog.openError(Display.getCurrent().getActiveShell(),
						Messages.getString("PackageEditor.25"), //$NON-NLS-1$
						Messages.getString("PackageEditor.26") //$NON-NLS-1$
						+ e.getMessage()
				);
				return false;
			}
		}
	}

	/* (non-Javadoc)
	 * @see atosj.editors.Editor#setInput(atosj.editors.Changeable)
	 */
	public void setInput(Changeable newInput) {
		super.setInput(newInput);
		((PackageUndoManager)getUndoManager()).setPackage( getPackage() );
	}
	
	/* (non-Javadoc)
	 * @see atosj.editors.Editor#refresh()
	 */
	public void refresh() {
		packTable.setInput( getPackage() );
	}

	/* (non-Javadoc)
	 * @see atosj.editors.Editor#getTitleImage()
	 */
	public Image getTitleImage() 
	{
		switch( getPackage().getValidState() )
		{
			case HTSPackage.VALID: return AtosJApp.getInstance().getImage(AtosJApp.IMG_PACKAGE);
			case HTSPackage.INVALID: return AtosJApp.getInstance().getImage(AtosJApp.IMG_PACKAGE_ERROR);
			case HTSPackage.VALID_WITH_WARNING: return AtosJApp.getInstance().getImage(AtosJApp.IMG_PACKAGE_WARNING);
			default: return null;
		}
	}
	
	public void setFocus() {
		packTable.getTable().setFocus();	
	}
}
