package atosj.editors.script;

import java.rmi.RemoteException;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.dialog.AddScriptDlg;
import atosj.dialog.PositionDlg;
import atosj.dialog.PositionWidget;
import atosj.dialog.addcmd.AddCmdAdvancedDlg;
import atosj.dialog.addcmd.AddCmdDlg;
import atosj.editors.Changeable;
import atosj.editors.Editor;
import atosj.exception.AtosJException;
import atosj.exception.IntegrityException;
import atosj.exception.ParseException;
import atosj.gui.BalloonWindow;
import atosj.gui.TableResizeListener;
import atosj.listener.ClipboardListener;
import atosj.listener.CommandReceiverListener;
import atosj.model.hts.command.CommandIndexComparator;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.command.HTSComment;
import atosj.model.hts.command.HTSComponentCommand;
import atosj.model.hts.command.HTSDisable;
import atosj.model.hts.command.InvalidCommand;
import atosj.model.recorder.HTSCommandReceiver;
import atosj.model.script.HTSScript;
import atosj.model.urf.URFFile;
import atosj.util.AcceleratorHook;
import atosj.util.Clipboard;
import atosj.util.DefaultImageDescriptor;

/**
 * Basisklasse fr alle Aktionen zum Verschieben eines Kommandos in einem Skript.
 */
class MoveAction extends Action implements ISelectionChangedListener
{
	private TableViewer tv;
	private ScriptUndoManager undoMngr;
	
	/**
	 * Erstellt eine neue Aktion zur Positionsnderung von Kommandos.
	 * @param title Der Titel der Aktion.
	 * @param tooltip Der Tooltip Text.
	 * @param enabledDescr Das Bild fr den Zustand aktiviert.
	 * @param disabledDescr Das Bild fr den Zustand deaktiviert.
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	public MoveAction(String title, 
			String tooltip, 
			ImageDescriptor enabledDescr, 
			ImageDescriptor disabledDescr,
			TableViewer tv,
			ScriptUndoManager undoMngr)
	{
		super(title);
		setToolTipText(tooltip);
		setImageDescriptor(enabledDescr);
		setDisabledImageDescriptor(disabledDescr);
		
		this.tv = tv;
		this.undoMngr = undoMngr;
		
		setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 );
		tv.addSelectionChangedListener(this);
	}

	public void selectionChanged(SelectionChangedEvent arg0) {
		setEnabled( ((StructuredSelection) tv.getSelection()).size() == 1 );
	}

	/**
	 * Liefert das derzeit in der Tabelle ausgewhlte Kommando oder <code>null</code>, falls
	 * kein Kommando ausgewhlt. 
	 */
	protected HTSCommand getCurrentlySelected() {
		return (HTSCommand) ((StructuredSelection)tv.getSelection()).getFirstElement();
	}
	
	protected HTSScript getScript() {
		return (HTSScript)tv.getInput();
	}
	
	/**
	 * Verschiebt das angegebene Kommando an die neue Position.
	 * @param toMove Das zu verschiebende Kommando.
	 * @param newPos Die neue Position.
	 * @see HTSScript#moveCommand(HTSCommand, int)
	 */
	protected void moveCommand(HTSCommand toMove, int newPos) {
		int oldPos = toMove.getIndex();
		getScript().moveCommand(toMove, newPos);
		undoMngr.addUndoMove(toMove, oldPos);
	}
}

/**
 * Verschiebt ein Kommando um eine Position nach oben im Skript.
 */
class MoveUpAction extends MoveAction
{
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	MoveUpAction(TableViewer tv, ScriptUndoManager undoMngr) {
		super(Messages.getString("ScriptEditor.0"), //$NON-NLS-1$
			Messages.getString("ScriptEditor.1"), //$NON-NLS-1$
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_UP),
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_UP_DIS),
			tv, undoMngr);
	}

	public void run() {
		HTSCommand toMove = getCurrentlySelected();
		if( toMove.getIndex() > 0)
			moveCommand(toMove, toMove.getIndex()-1);
	}
}

/**
 * Verschiebt ein Kommando um eine Position nach unten im Skript.
 */
class MoveDownAction extends MoveAction
{
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	MoveDownAction(TableViewer tv, ScriptUndoManager undoMngr) {
		super(Messages.getString("ScriptEditor.2"), //$NON-NLS-1$
			Messages.getString("ScriptEditor.3"), //$NON-NLS-1$
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_DOWN),
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_DOWN_DIS),
			tv,undoMngr);
	}

	public void run() {
		HTSCommand toMove = getCurrentlySelected();
		if( toMove.getIndex() < getScript().getNumOfCommands()-1 )
			moveCommand(toMove, toMove.getIndex()+1);
	}
}

/**
 * Verschiebt ein Kommando an den Anfang des Skripts.
 */
class MoveTopAction extends MoveAction implements ISelectionChangedListener
{
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	MoveTopAction(TableViewer tv, ScriptUndoManager undoMngr) {
		super(Messages.getString("ScriptEditor.4"), //$NON-NLS-1$
			Messages.getString("ScriptEditor.5"), //$NON-NLS-1$
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_TOP),
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_TOP_DIS),
			tv, undoMngr);
	}

	public void run() {
		HTSCommand toMove = getCurrentlySelected();
		if(toMove.getIndex() > 0)
			moveCommand(toMove, 0);
	}
}

/**
 * Verschiebt ein Kommando ans Ende des Skripts.
 */
class MoveBottomAction extends MoveAction implements ISelectionChangedListener
{
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	MoveBottomAction(TableViewer tv, ScriptUndoManager undoMngr) {
		super(Messages.getString("ScriptEditor.6"), //$NON-NLS-1$
			Messages.getString("ScriptEditor.7"), //$NON-NLS-1$
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_BOTTOM),
			new DefaultImageDescriptor(AtosJApp.IMG_ARROW_BOTTOM_DIS),		
			tv, undoMngr);
	}

	public void run() {
		HTSCommand toMove = getCurrentlySelected();
		if(toMove.getIndex() < getScript().getNumOfCommands() -1)
			moveCommand(toMove, getScript().getNumOfCommands()-1);
	}
}

/**
 * Fgt dem Skript ein neues Kommando hinzu. Das Kommando wird durch
 * einen graph. orientierten Dialog erstellt.
 */
class AddCommandAction extends Action
{
	private TableViewer tv;
	private ScriptUndoManager undoMngr;
	
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	AddCommandAction(TableViewer tv, ScriptUndoManager undoMngr) {
		super(Messages.getString("ScriptEditor.8")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'N');
		setToolTipText(Messages.getString("ScriptEditor.9")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ADD) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_ADD_DIS) );
		
		this.tv = tv;
		this.undoMngr = undoMngr;
	}

	private HTSScript getScript() {
		return (HTSScript) tv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		// aktuellen Index bestimmen
		HTSCommand currentCmd= (HTSCommand) ((StructuredSelection)tv.getSelection()).getFirstElement();
		int currentPos = PositionWidget.NO_CURRENT_POS;
		if(currentCmd != null)
			currentPos = currentCmd.getIndex();
		
		AddCmdDlg dlg = new AddCmdDlg(
				Display.getCurrent().getActiveShell(), 
				getScript(),
				getScript().getNumOfCommands(),
				currentPos);
		
		if( dlg.open() == AddCmdDlg.ID_ADD )
		{
			getScript().addCommand( dlg.getCommand(), dlg.getPosition() );
			tv.setSelection( new StructuredSelection(dlg.getCommand()) );
			undoMngr.addUndoAdd( dlg.getCommand(), dlg.getPosition() );
		}
	}
}

/**
 * Fgt dem Skript ein neues Kommando hinzu. Das Kommando wird durch
 * einen textuell orientierten Dialog erstellt.
 */
class AddCommandAdvancedAction extends Action
{
	private TableViewer tv;
	private ScriptUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	AddCommandAdvancedAction(TableViewer _tv, ScriptUndoManager _undoMngr) {
		super(Messages.getString("ScriptEditor.10")); //$NON-NLS-1$
		setAccelerator(SWT.ALT | 'N');
		setToolTipText(Messages.getString("ScriptEditor.11")); //$NON-NLS-1$
		
		this.tv = _tv;
		this.undoMngr = _undoMngr;
	}

	private HTSScript getScript() {
		return (HTSScript) tv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		// aktuellen Index bestimmen
		HTSCommand currentCmd= (HTSCommand) ((StructuredSelection)tv.getSelection()).getFirstElement();
		int currentPos = PositionWidget.NO_CURRENT_POS;
		if(currentCmd != null)
			currentPos = currentCmd.getIndex();
		
		AddCmdAdvancedDlg dlg = new AddCmdAdvancedDlg(
				Display.getCurrent().getActiveShell(), 
				getScript(),
				getScript().getNumOfCommands(),
				currentPos);
		
		if( dlg.open() == AddCmdAdvancedDlg.ID_ADD )
		{
			getScript().addCommand( dlg.getCommand(), dlg.getPosition() );
			tv.setSelection( new StructuredSelection(dlg.getCommand()) );
			undoMngr.addUndoAdd( dlg.getCommand(), dlg.getPosition() );
		}
	}
}

/**
 * Bearbeitet ein ausgewhltes Kommando. Das Kommando wird durch
 * einen graph. orientierten Dialog bearbeitet.
 */
class EditCommandAction extends Action
{
	private TableViewer tv;
	private ScriptUndoManager undoMngr;
		
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	EditCommandAction(TableViewer _tv, ScriptUndoManager _undoMngr) {
		super(Messages.getString("ScriptEditor.12")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'E');
		setToolTipText(Messages.getString("ScriptEditor.13")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_EDIT) );
		
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		
		setEnabledState();
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				setEnabledState();
			}
		});
	}

	private void setEnabledState() 
	{
		HTSCommand cmd = (HTSCommand) ((StructuredSelection)tv.getSelection()).getFirstElement();
		if( ((StructuredSelection)tv.getSelection()).size() == 1
				&& cmd != null 
				&& !(cmd instanceof HTSDisable) 
				&& !(cmd instanceof InvalidCommand) )
			setEnabled(true);
		else
			setEnabled(false);
	}
	
	private HTSScript getScript() {
		return (HTSScript) tv.getInput();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		HTSCommand currentCmd = (HTSCommand) ((StructuredSelection)tv.getSelection()).getFirstElement();
		AddCmdDlg dlg = new AddCmdDlg(
				Display.getCurrent().getActiveShell(), 
				getScript(),
				currentCmd);
		
		if( dlg.open() == AddCmdDlg.ID_ADD )
		{
			// das alte lschen und dann das neue an der selben position wieder einfgen
			getScript().removeCommand(currentCmd);
			getScript().addCommand( dlg.getCommand(), currentCmd.getIndex() );
			tv.setSelection( new StructuredSelection(dlg.getCommand()) );
			undoMngr.addUndoEdit( currentCmd, dlg.getCommand() );
		}
	}
}

/**
 * Bearbeitet ein ausgewhltes Kommando. Das Kommando wird durch
 * einen textuell orientierten Dialog bearbeitet.
 */
class EditCommandAdvancedAction extends Action
{
	private TableViewer tv;
	private ScriptUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	EditCommandAdvancedAction(TableViewer _tv, ScriptUndoManager _undoMngr) {
		super(Messages.getString("ScriptEditor.14")); //$NON-NLS-1$
		setAccelerator(SWT.ALT | 'E');
		setToolTipText(Messages.getString("ScriptEditor.15")); //$NON-NLS-1$
		
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		
		setEnabledState();
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				setEnabledState();
			}
		});

	}
	
	private void setEnabledState() 
	{
		if( ((StructuredSelection)tv.getSelection()).size() == 1 )
			setEnabled(true);
		else
			setEnabled(false);
	}

	private HTSScript getScript() {
		return (HTSScript) tv.getInput();
	}
	
	public void run() 
	{
		HTSCommand currentCmd = (HTSCommand) ((StructuredSelection)tv.getSelection()).getFirstElement();
		AddCmdAdvancedDlg dlg = new AddCmdAdvancedDlg(
				Display.getCurrent().getActiveShell(), 
				getScript(),
				currentCmd);
		
		if( dlg.open() == AddCmdDlg.ID_ADD )
		{
			// das alte lschen und dann das neue an der selben position wieder einfgen
			getScript().removeCommand(currentCmd);
			getScript().addCommand( dlg.getCommand(), currentCmd.getIndex() );
			tv.setSelection( new StructuredSelection(dlg.getCommand()) );
			undoMngr.addUndoEdit( currentCmd, dlg.getCommand() );
		}
	}
}

/**
 * Entfernt ausgewhlte Kommandos aus dem Skript.
 */
class RemoveCommandAction extends Action
{
	private TableViewer tv;
	private ScriptUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	RemoveCommandAction(TableViewer _tv, ScriptUndoManager _undoMngr) {
		super(Messages.getString("ScriptEditor.16")); //$NON-NLS-1$
		setAccelerator(SWT.DEL);
		setToolTipText(Messages.getString("ScriptEditor.17")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE) );
		setDisabledImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_REMOVE_DIS) );
		
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		
		setEnabled( !tv.getSelection().isEmpty() );
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				setEnabled( !tv.getSelection().isEmpty() );
			}
		});
	}

	private HTSScript getScript() {
		return (HTSScript) tv.getInput();
	}
	
	public void run() {
		List commands = ((StructuredSelection)tv.getSelection()).toList();
		getScript().removeCommands(commands);
		undoMngr.addUndoDelete(commands);
	}
}

/**
 * Fgt ausgewhlte Kommandos in die ATOSj Zwischenablage ein.
 * @see atosj.util.Clipboard
 */
class CopyAction extends Action
{
	private TableViewer tv;
			
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 */
	CopyAction(TableViewer _tv) {
		super(Messages.getString("ScriptEditor.18")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'C');
		this.tv = _tv;
		
		setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 );
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) {
				setEnabled( !e.getSelection().isEmpty() );		
			}
		});
	}

	public void run() {
		Clipboard.getInstance().setElements( ((StructuredSelection)tv.getSelection()).toList(), Clipboard.MODE_COPY );
	}
}

/**
 * Entfernt die ausgewhlten Kommandos aus dem Skript und
 * fgt sie in die ATOSj Zwischenablage ein.
 * @see atosj.util.Clipboard
 */
class CutAction extends Action
{
	private TableViewer tv;
	private ScriptUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	CutAction(TableViewer _tv, ScriptUndoManager _undoMngr) {
		super(Messages.getString("ScriptEditor.19")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'X');
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		
		setEnabled( !tv.getSelection().isEmpty() );
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent arg0) {
				setEnabled( !tv.getSelection().isEmpty() );
			}
		});
	}

	private HTSScript getScript() {
		return (HTSScript) tv.getInput();
	}
	
	public void run() {
		List commands = ((StructuredSelection)tv.getSelection()).toList();
		getScript().removeCommands(commands);
		
		Clipboard.getInstance().setElements(commands, Clipboard.MODE_CUT);
		undoMngr.addUndoCut(commands);
	}
}

/**
 * Fgt Kommandos aus der Zwischenablage in das Skript ein.
 * @see atosj.util.Clipboard
 */
class PasteAction extends Action
{
	private TableViewer tv;
	private ScriptUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	PasteAction(TableViewer _tv, ScriptUndoManager _undoMngr) {
		super(Messages.getString("ScriptEditor.20")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'V');
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		
		setEnabled( Clipboard.getInstance().containsOnlyClass(HTSCommand.class) );
		Clipboard.getInstance().addClipboardListener( new ClipboardListener() {
			public void clipboardChanged(Clipboard clip) {
				setEnabled( clip.containsOnlyClass(HTSCommand.class) );		
			}
		});
	}

	private HTSScript getScript() {
		return (HTSScript) tv.getInput();
	}
	
	public void run() 
	{
		// aktuellen Index bestimmen
		HTSCommand currentElement = (HTSCommand) ((StructuredSelection)tv.getSelection()).getFirstElement();
		int currentPos = PositionWidget.NO_CURRENT_POS;
		if(currentElement != null)
			currentPos = currentElement.getIndex();
		
		PositionDlg dlg = new PositionDlg(
				Display.getCurrent().getActiveShell(),
				Messages.getString("ScriptEditor.21"), //$NON-NLS-1$
				getScript().getNumOfCommands(),
				currentPos);
		if( dlg.open() == AddScriptDlg.ID_ADD )
		{	
			int mode = Clipboard.getInstance().getMode();
			Vector original = Clipboard.getInstance().getElements(); 
			
			// elemente kopieren und dann hinzufgen
			if( mode == Clipboard.MODE_COPY )
			{
				Vector copy = new Vector( original.size() );
				for(int i = 0; i < original.size(); ++i)
					copy.add( ((HTSCommand)original.get(i)).clone() );
				
				// addElements(copy, dlg.getPosition());
				getScript().addCommands(copy, dlg.getPosition());
				undoMngr.addUndoPaste(copy, dlg.getPosition(), true);
				tv.setSelection( new StructuredSelection(copy) );
			}
			else // nur hinzufgen ohne kopieren
			{
				getScript().addCommands(original, dlg.getPosition());
				undoMngr.addUndoPaste(original, dlg.getPosition(), false);
				tv.setSelection( new StructuredSelection(original) );
			}
		}
	}
}

/**
 * Startet den Recorder zur Aufzeichnung von Nutzeraktionen auf dem Testobjekt.
 * Whrend der Aufzeichnung werden automatisch TEST oder ACTION Kommandos in
 * das Skript eingefgt.
 */
class RecorderAction extends Action implements CommandReceiverListener
{
	private TableViewer tv;
	private HTSCommandReceiver receiver;
	private ScriptUndoManager undoMngr;
	
	/** 
	 * @param tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param receiver Der Empfnger aufgezeichneter Kommandos.
	 * @param undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	RecorderAction(TableViewer tv, HTSCommandReceiver receiver, ScriptUndoManager undoMngr) {
		super(Messages.getString("ScriptEditor.22")); //$NON-NLS-1$
		setToolTipText(Messages.getString("ScriptEditor.23")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_RECORD) );
		
		this.tv = tv;
		this.receiver = receiver;
		this.undoMngr = undoMngr;
		receiver.addListener(this);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	public void run() 
	{
		if( receiver.isRunning() )
			receiver.stop();
		else
		{
			Thread starter = new Thread() {
				public void run() 
				{
					try {
						receiver.start();
						// alle undos aufheben
						undoMngr.clear();
					} 
					catch(final AtosJException e) 
					{
						Display.getDefault().asyncExec( new Runnable() {
							public void run() {
								MessageDialog.openError(null, Messages.getString("ScriptEditor.24"), Messages.getString("ScriptEditor.25") //$NON-NLS-1$ //$NON-NLS-2$
										+ e.getMessage() );
							}
						});
					}
				}
			};
			
			starter.start();
		}
	}

	public void runningStateChanged(boolean running) {
		setText(running ? Messages.getString("ScriptEditor.26") : Messages.getString("ScriptEditor.27") ); //$NON-NLS-1$ //$NON-NLS-2$
		setToolTipText(running ? Messages.getString("ScriptEditor.28") : Messages.getString("ScriptEditor.29")); //$NON-NLS-1$ //$NON-NLS-2$
		setImageDescriptor( running ? new DefaultImageDescriptor(AtosJApp.IMG_STOP) : new DefaultImageDescriptor(AtosJApp.IMG_RECORD) );
	}

	public void commandReceived(HTSCommand cmd) 
	{
		((HTSScript) tv.getInput()).addCommand(cmd);
		// automatisch ComponentId hinzufgen
		if( cmd instanceof HTSComponentCommand )
		{
			URFFile urf = AtosJApp.getInstance().getInput().getURFFile();
			urf.addId( ((HTSComponentCommand)cmd).getComponentId() );
			try {
				urf.save();
			} catch (AtosJException e) {
				e.printStackTrace();
			}
		}
		
		tv.setSelection( new StructuredSelection(cmd), true );
		undoMngr.addUndoAdd( cmd, cmd.getIndex() );
	}
}

/**
 * Blockiert das Aufzeichnen von Kommandos, ohne das Testobjekt zu beenden.
 */
class PauseRecorderAction extends Action
{
	private HTSCommandReceiver receiver;
	
	/** 
	 * @param receiver Der Empfnger aufgezeichneter Kommandos.
	 */
	PauseRecorderAction(HTSCommandReceiver receiver) {
		super(Messages.getString("ScriptEditor.30"), SWT.TOGGLE); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'P');
		setToolTipText(Messages.getString("ScriptEditor.31")); //$NON-NLS-1$
		setImageDescriptor( new DefaultImageDescriptor(AtosJApp.IMG_PAUSE) );
		
		this.receiver = receiver;
	}
	
	public void run() {
		receiver.pausePublish( isChecked() );
	}
}

/**
 * Deaktiviert die ausgewhlten Kommandos, d.h. wandelt sie in Kommandos
 * des Typs <code>HTSDisable</code> um. Diese knnen nicht ausgefhrt werden.
 */
class DisableAction extends Action
{
	private TableViewer tv;
	private Comparator indexComparator;
	private ScriptUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	DisableAction(TableViewer _tv, ScriptUndoManager _undoMngr) {
		super(Messages.getString("ScriptEditor.32")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'D');
				
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		indexComparator = new CommandIndexComparator();
		
		setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 );
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) {
				setEnabled( !e.getSelection().isEmpty() );		
			}
		});
	}
	
	private HTSScript getScript() {
		return (HTSScript) tv.getInput();
	}
	
	private boolean canBeDisabled(HTSCommand cmd) {
		if( cmd instanceof InvalidCommand 
				|| cmd instanceof HTSComment
				|| cmd instanceof HTSDisable)
			return false;
		
		return true;
	}
	
	public void run() {
		List cmds = ((StructuredSelection)tv.getSelection()).toList();
		Collections.sort(cmds, indexComparator);
		Vector disabledCmds = new Vector( cmds.size() );
		for(int i = 0; i < cmds.size(); ++i)
		{
			HTSCommand origCmd = (HTSCommand) cmds.get(i);
			if( canBeDisabled(origCmd) )
			{
				getScript().removeCommand(origCmd);
				try {
					HTSDisable disabled = new HTSDisable(origCmd);
					getScript().addCommand( disabled, origCmd.getIndex() );
					disabledCmds.add(disabled);
				} catch (ParseException e) {
					// sollte nie passieren, dennoch machen wir dann das alte Kommando wieder an die richtige Stelle
					getScript().addCommand( origCmd, origCmd.getIndex() );
				}
			}
		}
		
		if( !disabledCmds.isEmpty() )
		{
			undoMngr.addUndoDisable(disabledCmds);
			tv.setSelection( new StructuredSelection(disabledCmds) );
		}
	}
}

/**
 * Aktiviert alle ausgewhlten Kommandos, die vorher deaktiviert waren.
 * D.h. wandelt sie wieder in ihren Ursprungstyp um.
 */
class EnableAction extends Action
{
	private TableViewer tv;
	private Comparator indexComparator;
	private ScriptUndoManager undoMngr;
	
	/** 
	 * @param _tv Der <code>TableViewer</code> zur Anzeige der Kommandos
	 * des Skriptes. Dieser muss als Input ein <code>HTSScript</code> haben.
	 * @param _undoMngr Der <code>UndoManager</code> zum Rckgngigmachen der Aktion.
	 */
	EnableAction(TableViewer _tv, ScriptUndoManager _undoMngr) {
		super(Messages.getString("ScriptEditor.33")); //$NON-NLS-1$
		setAccelerator(SWT.CTRL | 'A');
		this.tv = _tv;
		this.undoMngr = _undoMngr;
		indexComparator = new CommandIndexComparator();
		
		setEnabled( ((StructuredSelection)tv.getSelection()).size() == 1 );
		tv.addSelectionChangedListener( new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent e) {
				setEnabled( !e.getSelection().isEmpty() );		
			}
		});
	}
	
	private HTSScript getScript() {
		return (HTSScript) tv.getInput();
	}
	
	public void run() 
	{
		List cmds = ((StructuredSelection)tv.getSelection()).toList();
		Collections.sort(cmds, indexComparator);
		Vector disabledCmds = new Vector( cmds.size() );
		for(int i = 0; i < cmds.size(); ++i)
		{
			HTSCommand cmd = (HTSCommand) cmds.get(i);
			if( cmd instanceof HTSDisable )
			{
				getScript().removeCommand(cmd);
				getScript().addCommand( ((HTSDisable)cmd).getOrigCommand(), cmd.getIndex() );
				disabledCmds.add(cmd);
			}
		}	
		
		if( !disabledCmds.isEmpty() )
			undoMngr.addUndoEnable(disabledCmds);
		// TODO selection setzen
	}
}

/**
 * Editor zum Bearbeiten eines Skriptes. Zeigt eine Tabelle mit allen enthaltenen Kommandos an.
 * @author Nicos Tegos
 * @see atosj.model.script.HTSScript
 */
public class ScriptEditor extends Editor 
{
	private TableViewer scriptTV;
	private ScriptTableContentProvider scriptProvider;
	
	private HTSCommandReceiver commandReceiver;
	private MenuManager contextMenu;
	
	private Action addCommandAction;
	private Action addCommandAdvancedAction;
	private Action editCommandAction;
	private Action editCommandAdvancedAction;
	private Action removeCommandAction;
	private Action topAction;	
	private Action upAction;
	private Action downAction;
	private Action bottomAction;
	private Action copyAction;
	private Action cutAction;
	private Action pasteAction;
	private Action recorderAction;
	private Action pauseRecorderAction;
	private Action disableAction;
	private Action enableAction;
	
	private AcceleratorHook accHook;

	/**
	 * Erstellt einen neuen Editor fr ein Skript.
	 * @param parent Das Tab in dem der Editor erstellt werden soll.
	 */
	public ScriptEditor(CTabFolder parent) {
		super( parent, new ScriptUndoManager() );
	}

	/* (non-Javadoc)
	 * @see atosj.editors.Editor#init()
	 */
	protected void init() {
		scriptProvider = new ScriptTableContentProvider();
		try {
			commandReceiver = new HTSCommandReceiver( AtosJApp.getInstance().getInput().getTestObject() );
		} catch (RemoteException e) {
			// kann nich passieren
		}
	}
	
	public void createContents(Composite parent) 
	{
		Composite tableArea = new Composite(parent, SWT.NULL);
		tableArea.setLayout( new GridLayout(2, false) );
		
		scriptTV = new TableViewer(tableArea, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
		scriptTV.setLabelProvider( new ScriptTableLabelProvider(scriptTV) );
		scriptTV.setContentProvider(scriptProvider);
				
		Table table = scriptTV.getTable();
				
		// BalloonWindow mit Fehler/Warnung bei doppelklick
		table.addMouseListener( new MouseAdapter() {
			public void mouseDoubleClick(MouseEvent e) 
			{
				Point p = new Point(e.x, e.y);
				TableItem item = scriptTV.getTable().getItem(p);
				if( item == null )
					return;
				
				StructuredSelection ssel = (StructuredSelection) scriptTV.getSelection();
				if( ssel.isEmpty() )
					return;
				
				HTSCommand command = (HTSCommand) ssel.getFirstElement();
				
				// klick auf 0. spalte wir zeigen die Fehlermeldung an
				if( item.getBounds(0).contains(p) )
				{	
					
					try {
						command.checkIntegrity( AtosJApp.getInstance().getInput(), getScript() );
					}
					catch(IntegrityException ex)
					{
						final BalloonWindow exWindow = new BalloonWindow(Display.getCurrent(), SWT.TITLE | SWT.CLOSE);
						exWindow.setImage(AtosJApp.getInstance().getImage( AtosJApp.IMG_ERROR) );
						exWindow.setTitle(Messages.getString("ScriptEditor.34")); //$NON-NLS-1$
						exWindow.setText( ex.getMessage() );
						exWindow.setLocation( scriptTV.getTable().toDisplay(e.x, e.y) );
						exWindow.getShell().addShellListener( new ShellAdapter() {
							public void shellDeactivated(ShellEvent arg0) {
								exWindow.close();
							}
						});
						exWindow.open();
					}
				}
				else // klick auf 1. spalte wir starten den (advanced) bearbeiten Dialog
				if( item.getBounds(1).contains(p) )
				{
					editCommandAdvancedAction.run();
				}
			}
		});
		
		// listener fr kontext men
		table.addListener(SWT.MenuDetect, new Listener() {
			public void handleEvent(Event e) {
				scriptTV.getTable().setMenu( contextMenu.getMenu() );
			}
			
		});
		
		new TableResizeListener(table, new int[] {10, 90}, new int[] {40, 200});
		table.setLayoutData( new GridData(GridData.FILL_BOTH) );
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		TableColumn col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("ScriptEditor.35")); //$NON-NLS-1$
		
		col = new TableColumn(table, SWT.NULL);
		col.setText(Messages.getString("ScriptEditor.36")); //$NON-NLS-1$
		
		Composite buttonArea = new Composite(tableArea, SWT.NULL);
		buttonArea.setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		GridLayout gl = new GridLayout(1, false);
		gl.marginWidth = gl.marginHeight = 0;
		buttonArea.setLayout(gl);
		
		createActions();
		
		// die Buttonleiste erstellen
		ToolBarManager buttonsMngr = new ToolBarManager(SWT.VERTICAL);
		buttonsMngr.add(addCommandAction);
		buttonsMngr.add(editCommandAction);
		buttonsMngr.add(removeCommandAction);
		buttonsMngr.add( new Separator() );
		buttonsMngr.add(topAction);
		buttonsMngr.add(upAction);
		buttonsMngr.add(downAction);
		buttonsMngr.add(bottomAction);
		buttonsMngr.add( new Separator() );
		buttonsMngr.add(recorderAction);
		buttonsMngr.add(pauseRecorderAction);
		buttonsMngr.createControl(buttonArea).setLayoutData( new GridData(GridData.VERTICAL_ALIGN_BEGINNING) );
		
		createContextMenu();
		initAccelerators();
	}
	
	/**
	 * Initialisiert alle Aktionen fr das Bearbeiten des Skripts.
	 */
	private void createActions() {
		ScriptUndoManager undoManager = (ScriptUndoManager) getUndoManager();
		addCommandAction = new AddCommandAction(scriptTV, undoManager);
		addCommandAdvancedAction = new AddCommandAdvancedAction(scriptTV,undoManager);
		editCommandAction = new EditCommandAction(scriptTV, undoManager);
		editCommandAdvancedAction = new EditCommandAdvancedAction(scriptTV, undoManager);
		removeCommandAction = new RemoveCommandAction(scriptTV, undoManager);
		topAction = new MoveTopAction(scriptTV, undoManager);	
		upAction = new MoveUpAction(scriptTV, undoManager);
		downAction = new MoveDownAction(scriptTV, undoManager);
		bottomAction = new MoveBottomAction(scriptTV, undoManager);
		copyAction = new CopyAction(scriptTV);
		cutAction = new CutAction(scriptTV, undoManager);
		pasteAction =  new PasteAction(scriptTV, undoManager);
		recorderAction = new RecorderAction(scriptTV, commandReceiver, undoManager);
		pauseRecorderAction = new PauseRecorderAction(commandReceiver);
		disableAction = new DisableAction(scriptTV, undoManager);
		enableAction = new EnableAction(scriptTV, undoManager);
	}
	
	/**
	 * Erstellt das Kontextmen fr die Tabelle.
	 */
	private void createContextMenu() {
		contextMenu = new MenuManager();
		
		contextMenu.add(addCommandAction);
		contextMenu.add(addCommandAdvancedAction);
		contextMenu.add(editCommandAction);
		contextMenu.add(editCommandAdvancedAction);
		contextMenu.add(removeCommandAction);
		contextMenu.add( new Separator() );
		contextMenu.add(topAction);
		contextMenu.add(upAction);
		contextMenu.add(downAction);
		contextMenu.add(bottomAction);
		contextMenu.add( new Separator() );
		contextMenu.add(copyAction);
		contextMenu.add(cutAction);
		contextMenu.add(pasteAction);
		contextMenu.add( new Separator() );
		contextMenu.add(enableAction);
		contextMenu.add(disableAction);
				
		contextMenu.createContextMenu( scriptTV.getTable() );
	}
	
	/**
	 * Initialisiert die Untersttzung von Tastenkrzeln fr das
	 * Bearbeiten des Skripts.
	 * @see AcceleratorHook
	 */
	private void initAccelerators() 
	{
		 accHook = new AcceleratorHook( scriptTV.getTable() );
		 accHook.add(addCommandAction);
		 accHook.add(addCommandAdvancedAction);
		 accHook.add(editCommandAction);
		 accHook.add(editCommandAdvancedAction);
		 accHook.add(removeCommandAction);
		 accHook.add(copyAction);
		 accHook.add(cutAction);
		 accHook.add(pasteAction);
		 accHook.add(enableAction);
		 accHook.add(disableAction);
	}

	public boolean commit(boolean save) 
	{
		if(save) // speichern
		{
			try {
				getScript().save();
				return true;
			} catch (AtosJException e) 
			{
				MessageDialog.openError(Display.getCurrent().getActiveShell(),
						Messages.getString("ScriptEditor.37"), //$NON-NLS-1$
						e.getMessage());
				return false;
			}
		}
		else // verwerfen
		{
			try {
				getScript().discard();
				return true;
			}
			catch(AtosJException e)
			{
				MessageDialog.openError(Display.getCurrent().getActiveShell(),
						Messages.getString("ScriptEditor.38"), //$NON-NLS-1$
						Messages.getString("ScriptEditor.39") //$NON-NLS-1$
						+ e.getMessage()
				);
				return false;
			}
		}
	}

	protected void dispose()	{
		super.dispose();
		if (commandReceiver.isRunning()) 
			commandReceiver.stop();
		commandReceiver.dispose();
	}
	
	/**
	 * Liefert, das mit diesem Editor assoziierte Skript.
	 */
	private HTSScript getScript() {
		return (HTSScript) getInput();
	}
	
	public void setInput(Changeable newInput) {
		super.setInput(newInput);
		((ScriptUndoManager)getUndoManager()).setScript( getScript() );
	}
	
	public void refresh() {
		scriptTV.setInput( getScript() ); 
	}

	/* (non-Javadoc)
	 * @see atosj.editors.Editor#getTitleImage()
	 */
	public Image getTitleImage() 
	{
		switch( getScript().getValidState() )
		{
			case HTSScript.VALID: return AtosJApp.getInstance().getImage(AtosJApp.IMG_SCRIPT);
			case HTSScript.INVALID: return AtosJApp.getInstance().getImage(AtosJApp.IMG_SCRIPT_ERROR);
			case HTSScript.VALID_WITH_WARNING: return AtosJApp.getInstance().getImage(AtosJApp.IMG_SCRIPT_WARNING);
			default: return null;
		}
	}

	public void setFocus() {
		scriptTV.getTable().setFocus();	
	}
}
