/*
 * Created on 14.04.2006
 */
package atosj.editors.urf;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;

import atosj.Messages;
import atosj.action.SaveRunner;
import atosj.app.AtosJApp;
import atosj.component.ComponentId;
import atosj.dialog.SaveResourcesDlg;
import atosj.editors.UndoElement;
import atosj.editors.UndoManager;
import atosj.exception.AtosJException;
import atosj.model.urf.URFFile;

/**
 * Verwaltet das Rckgngigmachen und Wiederherstellen von Aktionen in einem <code>URFEditor</code>.
 * @author Nicos Tegos
 * @see atosj.editors.urf.URFEditor
 */
public class URFUndoManager extends UndoManager 
{
	private static final int OP_DELETE = 0;
	private static final int OP_ADD = 1;
	private static final int OP_CHANGE = 2;
	
	private URFFile urf;
	
	public URFUndoManager() {
		super();
	}
	
	/**
	 * Setzt das bearbeitete URF-File.
	 */
	public void setURFFile(URFFile urf) {
		this.urf = urf;
	}

	/* (non-Javadoc)
	 * @see atosj.editors.UndoManager#performUndo(atosj.editors.UndoElement)
	 */
	public void performUndo(UndoElement element) 
	{
		switch( element.getOperation() )
		{
			case OP_DELETE:
				urf.addIds( (Collection) element.getObject() ); 
			break;
			case OP_ADD:
				urf.removeId( (ComponentId)element.getObject() );
			break;
			case OP_CHANGE:
				urf.removeId( (ComponentId) ((Object[])element.getObject())[1] );
				urf.addId( (ComponentId) ((Object[])element.getObject())[0] );
				try {
					urf.save();
					replaceId( (ComponentId) ((Object[])element.getObject())[1], 
							(ComponentId) ((Object[])element.getObject())[0] );
				} catch (AtosJException e) {
					urf.removeId( (ComponentId) ((Object[])element.getObject())[0] );
					urf.addId( (ComponentId) ((Object[])element.getObject())[1] );
					
					MessageDialog.openError( null, Messages.getString("URFUndoManager.0"), e.getMessage() ); //$NON-NLS-1$
				}
			break;
		}
	}

	/* (non-Javadoc)
	 * @see atosj.editors.UndoManager#performRedo(atosj.editors.UndoElement)
	 */
	public void performRedo(UndoElement element) 
	{
		switch( element.getOperation() )
		{
			case OP_DELETE:
				urf.removeIds( (Collection) element.getObject() );
			break;
			case OP_ADD:
				urf.addId( (ComponentId)element.getObject() );
			break;
			case OP_CHANGE:
				urf.removeId( (ComponentId) ((Object[])element.getObject())[0] );
				urf.addId( (ComponentId) ((Object[])element.getObject())[1] );
				try {
					urf.save();
					replaceId( (ComponentId) ((Object[])element.getObject())[0], 
							(ComponentId) ((Object[])element.getObject())[1] );
				} catch (AtosJException e) {
					urf.removeId( (ComponentId) ((Object[])element.getObject())[1] );
					urf.addId( (ComponentId) ((Object[])element.getObject())[0] );
					
					MessageDialog.openError( null, Messages.getString("URFUndoManager.1"), e.getMessage() ); //$NON-NLS-1$
				}
			break;
		}
	}
	
	/**
	 * Speichert das Hinzufgen einer neuen ID, zum spteren Rckgngigmachen.
	 * @param id Die neu hinzugefgte ID.
	 */
	public void addUndoAdd(ComponentId id) {
		addUndoElement( new UndoElement(OP_ADD, id) );
	}
	
	/**
	 * Speichert das Lschen von IDs, zum Rckgngigmachen.
	 * @param ids List(ComponentId) Die aus dem <code>URFFile</code> gelschten IDs.
	 */
	public void addUndoDelete(Collection ids) {
		addUndoElement( new UndoElement(OP_DELETE, ids) );
	}
	
	/**
	 * Speichert das Bearbeiten einer ID, zum spteren Rckgngigmachen.
	 * @param oldId Die ID vor dem Bearbeiten.
	 * @param newId Die ID nach dem Bearbeiten.
	 */
	public void addUndoChange(ComponentId oldId, ComponentId newId) {
		addUndoElement( new UndoElement(OP_CHANGE, new Object[] {oldId, newId}) );
	}

	/* (non-Javadoc)
	 * @see atosj.editors.UndoManager#getOperationName(int)
	 */
	protected String getOperationName(int operation) {
		switch(operation)
		{
			case OP_DELETE:
				return Messages.getString("URFUndoManager.2"); //$NON-NLS-1$
			case OP_ADD:
				return Messages.getString("URFUndoManager.3"); //$NON-NLS-1$
			case OP_CHANGE:
				return Messages.getString("URFUndoManager.4"); //$NON-NLS-1$
			default:
				return""; //$NON-NLS-1$
		}
	}

	/**
	 * Ersetzt die alte durch die neue ID in allen Skripten.
	 * @param oldId Die alte ID (vor dem Bearbeiten).
	 * @param newId Die neue ID (nach dem Bearbeiten).
	 */
	private void replaceId(final ComponentId oldId, final ComponentId newId)
	{
		// vor dem Ersetzen alle genderten Resourcen speichern
		Vector modifiedResources = AtosJApp.getInstance().getInput().getModifiedResources();
		if( !modifiedResources.isEmpty() )
		{
			SaveResourcesDlg dlg = new SaveResourcesDlg(Display.getDefault().getActiveShell(), modifiedResources);
			if( dlg.open() == SaveResourcesDlg.ID_SAVE )
			{
				Vector toSave = dlg.getElementsToSave();
				Vector toDiscard = dlg.getElementsToDiscard();
				
				try
				{
					new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
								false, 
								false,
								new SaveRunner(toDiscard, toSave));
				} catch(Throwable t) {
					
				}
			}
			else // TODO nutzer abbruch muss das ganze Redo beenden -> performRedo mit Rckgabewert einfhren
				return;
		}
		
		// Ersetzung durchfhren
		IRunnableWithProgress replaceRunner = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException ,InterruptedException {
				monitor.beginTask( Messages.getString("URFUndoManager.6"), 1); //$NON-NLS-1$
				AtosJApp.getInstance().getInput().replaceId(oldId, newId);
				monitor.worked(1);
			}
		};
		try
		{
			new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
						false, 
						false,
						replaceRunner);
		} catch(Throwable t) {
			
		}
		
		// Alle resourcen speichern
		modifiedResources = AtosJApp.getInstance().getInput().getModifiedResources();
		if( !modifiedResources.isEmpty() )
		{
			try
			{
				new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run( 
							false, 
							false,
							new SaveRunner(new Vector(0), modifiedResources));
			} catch(Throwable t) {
				
			}
		}
	}
}
