package atosj.event.awt;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import atosj.Messages;
import atosj.component.swing.SwingComponent;
import atosj.exception.ComponentException;

/**
 * Simuliert einen Tastendruck auf einem Component.
 * @author Nicos Tegos
 */
public class FireableKeyEvent extends FireableSwingEvent 
{
	private static final int[] INPUT_MASKS =
		{ InputEvent.ALT_GRAPH_MASK, InputEvent.ALT_MASK, InputEvent.META_MASK, InputEvent.CTRL_MASK, InputEvent.SHIFT_MASK };

	/**
	 * Erstellt ein neues Event fr einen Tastendruck.
	 * @param component Das Component - die Ereignisquelle.
	 */
	public FireableKeyEvent(SwingComponent component) {
		super(component);
	}
	
	/**
	 * Simuliert den Tastendruck eines Zeichens.
	 * @see #fire(int, char, int)
	 */
	public void fire(char ch, int modifiers) throws ComponentException 
	{
		fire(
			getKeyCode(ch), 
			ch, 
			modifiers | getModifiers(ch)	
		);
	}

	/**
	 * Simuliert einen Tastendruck auf dem assozierten Component. Dazu werden entsprechend 
	 * des Parameters <code>keyCode</code> eine Reihe von <code>KeyEvent</code>s generiert 
	 * (KEY_PRESSED, KEY_TYPED, KEY_RELEASED) und der AWT EventQueue hinzugefgt. Ein KeyEvent
	 * vom Typ KEY_TYPED wird nur bei nicht virtuellen Zeichen generiert.
	 * @param keyCode Der code des angegebenen Zeichens, entspricht bei Buchstaben und Ziffern
	 * immer dem Wert in <code>ch</code>.
	 * @param ch Das Zeichen oder <code>KeyEvent.CHAR_UNDEFINED</code>, falls <code>keyCode</code>
	 * eine virtuelle Taste beschreibt.
	 * @param modifiers AWT Modifier fr Steuertasten z.B. <code>KeyEvent.ALT_MASK</code>, die gleichzeitig
	 * mit dem angegebenen Zeichen gedrckt werden sollen. Alle Modifier sind durch bitweises Oder
	 * verknpft.
	 */
	public void fire(int keyCode, char ch, int modifiers) throws ComponentException 
	{
		pressModifiers(modifiers);
		
		long t = System.currentTimeMillis();
		postEvent(new KeyEvent(getSwingComponent(), KeyEvent.KEY_PRESSED, t, modifiers, keyCode, ch));

		if(isTypedChar(keyCode) && ch != KeyEvent.CHAR_UNDEFINED) {
			postEvent(new KeyEvent(getSwingComponent(), KeyEvent.KEY_TYPED, t, modifiers, KeyEvent.VK_UNDEFINED, ch));
		}

		postEvent(new KeyEvent(getSwingComponent(), KeyEvent.KEY_RELEASED, t, modifiers, keyCode, ch));
		
		releaseModifiers(modifiers);
	}

	/**
	 * Liefert zu dem angegebenen Zeichen den entsprechenden AWT Key-Code.
	 * Der Key-Code kann virtuell sein.
	 * @param ch Das Zeichen.
	 * @return Der AWT Key-Code.
	 */
	private int getKeyCode(char ch) 
	{
		if (Character.isLetterOrDigit(ch))
			return Character.isLowerCase(ch) ? Character.toUpperCase(ch) : ch;

		switch (ch) 
		{
			case '\'' :
				return KeyEvent.VK_QUOTE;
			case '"' :
				return KeyEvent.VK_QUOTEDBL;
			case '(' :
				return KeyEvent.VK_LEFT_PARENTHESIS;
			case ')' :
				return KeyEvent.VK_RIGHT_PARENTHESIS;
			case '!' :
				return KeyEvent.VK_EXCLAMATION_MARK;
			case '$' :
				return KeyEvent.VK_5 ;
		}
		
		return ch;
	}
	
	/**
	 * Liefert die Steuertasten, die gedrckt werden mssen, um das angegebene
	 * Zeichen zu erzeugen. Beispielsweise die Shift-Taste fr Grobuchstaben.
	 * @param ch Das Zeichen.
	 * @return Die AWT Modifier-Codes fr die Steuertasten verbunden durch bitweises Oder.
	 */
	private int getModifiers(char ch) 
	{
		if (Character.isUpperCase(ch) )
			return KeyEvent.SHIFT_MASK;
		
		if(ch == '$')
			return KeyEvent.VK_SHIFT;
		
		return 0;
	}

	/**
	 * Sendet KeyPressed Event fr die modifier, d.h. fgt es der AWT EventQueue hinzu.
	 * @param modifiers Die AWT Modifier verknpft durch bitweises Oder.
	 */
	private void pressModifiers(int modifiers) throws ComponentException 
	{
		if(modifiers == 0)
			return;
		
		long t = System.currentTimeMillis();
		int mods = 0;

		for (int i = 0; i < INPUT_MASKS.length; i++) {
			int inputMask = INPUT_MASKS[i];

			if ((modifiers & inputMask) > 0) {
				mods = mods | inputMask;
				int keyCode = getKeyCode(inputMask);
				postEvent(new KeyEvent(getSwingComponent(), KeyEvent.KEY_PRESSED, t, mods, keyCode, KeyEvent.CHAR_UNDEFINED));
			}
		}
	}
	
	/**
	 * Sendet KeyReleased Event fr die modifier, d.h. fgt es der AWT EventQueue hinzu.
	 * @param modifiers Die AWT Modifier verknpft durch bitweises Oder.
	 */
	private void releaseModifiers(int modifiers) throws ComponentException {
		if(modifiers == 0)
			return;
		
		long t = System.currentTimeMillis();
		int mods = modifiers;

		for (int i = 0; i < INPUT_MASKS.length; i++) {
			int inputMask = INPUT_MASKS[i];

			if ((modifiers & inputMask) > 0) {
				mods = mods & (-1 ^ inputMask);
				int keyCode = getKeyCode(inputMask);
				postEvent(new KeyEvent(getSwingComponent(), KeyEvent.KEY_RELEASED, t, mods, keyCode, KeyEvent.CHAR_UNDEFINED));
			}
		}
	}

	/**
	 * Liefert den KeyCode fr Steuertasten.
	 * @param inputCode Der AWT *_MASK Code.
	 */
	private int getKeyCode(int inputCode) {
		switch (inputCode) {
			case InputEvent.ALT_GRAPH_MASK :
				return KeyEvent.VK_ALT_GRAPH;
			case InputEvent.ALT_MASK :
				return KeyEvent.VK_ALT;
			case InputEvent.META_MASK :
				return KeyEvent.VK_META;
			case InputEvent.CTRL_MASK :
				return KeyEvent.VK_CONTROL;
			case InputEvent.SHIFT_MASK :
				return KeyEvent.VK_SHIFT;
			default :
				throw new RuntimeException(Messages.getString("FireableKeyEvent.0")); //$NON-NLS-1$
		}
	}

	/**
	 * Gibt an ob <code>keyCode</code> ein NICHT virtuelles Zeichen ist
	 * @param keyCode Der zu berprfende AWT Key-Code.
	 * @return <code>true</code>, wenn <code>keyCode</code> ein nicht virtuelles 
	 * Zeichen ist.
	 */
	public static final boolean isTypedChar(int keyCode) {
		switch (keyCode) {
			case KeyEvent.VK_ACCEPT :
			case KeyEvent.VK_ADD :
			case KeyEvent.VK_AGAIN :
			case KeyEvent.VK_ALL_CANDIDATES :
			case KeyEvent.VK_ALPHANUMERIC :
			case KeyEvent.VK_ALT :
			case KeyEvent.VK_ALT_GRAPH :
			case KeyEvent.VK_CANCEL :
			case KeyEvent.VK_CAPS_LOCK :
			case KeyEvent.VK_CLEAR :
			case KeyEvent.VK_CODE_INPUT :
			case KeyEvent.VK_COMPOSE :
			case KeyEvent.VK_CONTROL :
			case KeyEvent.VK_CONVERT :
			case KeyEvent.VK_COPY :
			case KeyEvent.VK_CUT :
			case KeyEvent.VK_DELETE :
			case KeyEvent.VK_DOWN :
			case KeyEvent.VK_END :
			case KeyEvent.VK_F1 :
			case KeyEvent.VK_F10 :
			case KeyEvent.VK_F11 :
			case KeyEvent.VK_F12 :
			case KeyEvent.VK_F13 :
			case KeyEvent.VK_F14 :
			case KeyEvent.VK_F15 :
			case KeyEvent.VK_F16 :
			case KeyEvent.VK_F17 :
			case KeyEvent.VK_F18 :
			case KeyEvent.VK_F19 :
			case KeyEvent.VK_F2 :
			case KeyEvent.VK_F20 :
			case KeyEvent.VK_F21 :
			case KeyEvent.VK_F22 :
			case KeyEvent.VK_F23 :
			case KeyEvent.VK_F24 :
			case KeyEvent.VK_F3 :
			case KeyEvent.VK_F4 :
			case KeyEvent.VK_F5 :
			case KeyEvent.VK_F6 :
			case KeyEvent.VK_F7 :
			case KeyEvent.VK_F8 :
			case KeyEvent.VK_F9 :
			case KeyEvent.VK_FINAL :
			case KeyEvent.VK_FIND :
			case KeyEvent.VK_FULL_WIDTH :
			case KeyEvent.VK_HALF_WIDTH :
			case KeyEvent.VK_HELP :
			case KeyEvent.VK_HIRAGANA :
			case KeyEvent.VK_HOME :
			case KeyEvent.VK_INSERT :
			case KeyEvent.VK_JAPANESE_HIRAGANA :
			case KeyEvent.VK_JAPANESE_KATAKANA :
			case KeyEvent.VK_JAPANESE_ROMAN :
			case KeyEvent.VK_KANA :
			case KeyEvent.VK_KANJI :
			case KeyEvent.VK_KATAKANA :
			case KeyEvent.VK_KP_DOWN :
				/*
				 * Commented out since this is present only from jdk 1.3 case KeyEvent.VK_KANA_LOCK:
				 */
			case KeyEvent.VK_KP_LEFT :
			case KeyEvent.VK_KP_RIGHT :
			case KeyEvent.VK_KP_UP :
			case KeyEvent.VK_LEFT :
			case KeyEvent.VK_META :
			case KeyEvent.VK_MODECHANGE :
			case KeyEvent.VK_NONCONVERT :
			case KeyEvent.VK_NUM_LOCK :
			case KeyEvent.VK_PAGE_DOWN :
			case KeyEvent.VK_PAGE_UP :
			case KeyEvent.VK_PASTE :
			case KeyEvent.VK_PAUSE :
			case KeyEvent.VK_PREVIOUS_CANDIDATE :
			case KeyEvent.VK_PRINTSCREEN :
			case KeyEvent.VK_PROPS :
			case KeyEvent.VK_ROMAN_CHARACTERS :
			case KeyEvent.VK_RIGHT :
			case KeyEvent.VK_SCROLL_LOCK :
			case KeyEvent.VK_SHIFT :
			case KeyEvent.VK_STOP :
			case KeyEvent.VK_UNDEFINED :
			case KeyEvent.VK_UNDO :
				/*
				 * Commented out since this is present only from jdk 1.3 case KeyEvent.VK_INPUT_METHOD_ON_OFF:
				 */
			case KeyEvent.VK_UP :
				return false;
		}
		// all others return true
		return true;
	}

}
