package atosj.event.awt;

import java.awt.Point;
import java.awt.event.MouseEvent;

import atosj.component.swing.SwingComponent;
import atosj.exception.ComponentException;
import atosj.util.Snooze;

/**
 * Simuliert einen Mausklick auf einem Component.
 * @author Nicos Tegos
 */
public class FireableMouseClickEvent extends FireableSwingEvent 
{
	
	/**
	 * Erstellt eine neues Mausereignis fr das angegebene Component.
	 * @param component Das Component, fr das ein Mausklick simuliert werden soll.
	 */
	public FireableMouseClickEvent(SwingComponent component) {
		super(component);
	}

	/**
	 * Simuliert einen Mausklick. Fgt dazu eine entsprechende Sequenz von <code>MouseEvent</code>s in
	 * die AWT EventQueue ein.
	 * @param point Die Koordinaten des Mausklicks relativ zum Component.
	 * @param numberOfClicks Anzahl der Mausklicks.
	 * @param popupTrigger <code>true</code> = rechte Maustaste, <code>false</code> = linke Maustaste.
	 * @param modifiers Sequenz von Steuertasten, die gleichzeitig gedrckt werden soll. AWT Modifier
	 * fr Steuertasten durch bitweises Oder verknpft, z.B. <code>KeyEvent.ALT_MASK</code>.
	 */
	public void fire(Point point, int numberOfClicks, boolean popupTrigger, int modifiers) throws ComponentException 
	{
		int x = point.x;
		int y = point.y;

		postEvent(createMouseEvent(MouseEvent.MOUSE_ENTERED, 0, x, y, popupTrigger, modifiers));

		for (int click = 1; click <= numberOfClicks; click++) {
			postEvent(createMouseEvent(MouseEvent.MOUSE_PRESSED, click, x, y, popupTrigger, modifiers));

			new Snooze(5);

			postEvent(createMouseEvent(MouseEvent.MOUSE_RELEASED, click, x, y, popupTrigger, modifiers));
			postEvent(createMouseEvent(MouseEvent.MOUSE_CLICKED, click, x, y, popupTrigger, modifiers));
		}
		
		new Snooze(10);
		postEvent(createMouseEvent(MouseEvent.MOUSE_EXITED, 1, x, y, popupTrigger, modifiers));
	}

	 /**
	 * Erstellt ein einzelnes <code>MouseEvent</code>.
	 * @param id Typ des Events, siehe <code>MouseEvent.MOUSE_*</code> Konstanten.
	 * @param x x-Koordinate des Mausklicks relativ zum Component.
	 * @param y y-Koordinate des Mausklicks relativ zum Component.
	 * @param popupTrigger <code>true</code> = rechte Maustaste, <code>false</code> = linke Maustaste.
	 * @param modifiers Sequenz von Steuertasten, die gleichzeitig gedrckt werden soll. AWT Modifier
	 * fr Steuertasten durch bitweises Oder verknpft, z.B. <code>KeyEvent.ALT_MASK</code>.
	 */
	private MouseEvent createMouseEvent(int id, int click, int x, int y, boolean popupTrigger, int modifiers) 
	{
		if (popupTrigger)
			modifiers |= MouseEvent.BUTTON3_MASK;
		else
			modifiers |= MouseEvent.BUTTON1_MASK;
		
		if (id == MouseEvent.MOUSE_PRESSED || id == MouseEvent.MOUSE_CLICKED || id == MouseEvent.MOUSE_RELEASED)
			return new MouseEvent(getSwingComponent(), id, System.currentTimeMillis(), modifiers, x, y, click, popupTrigger);
		else
			return new MouseEvent(getSwingComponent(), id, System.currentTimeMillis(), 0, x, y, 0, false);
	}
}
