package atosj.event.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;

import atosj.component.swt.SWTComponent;
import atosj.component.swt.SWTThreadSafeAction;
import atosj.exception.ComponentException;

/**
 * Simuliert einen Tastendruck auf einem Component.
 * @author Nicos Tegos
 */
public class FireableKeyEvent extends FireableSWTEvent 
{
	public static int[] MODIFIERS = new int[] { SWT.ALT, SWT.CTRL, SWT.SHIFT };
	
	/**
	 * Erstellt ein neues Event fr einen Tastendruck.
	 * @param component Das Component - die Ereignisquelle.
	 */
	public FireableKeyEvent(SWTComponent component) {
		super(component);
	}
	
	/**
	 * Simuliert den Tastendruck eines Zeichens.
	 * @see #fire(char, int, int)
	 */
	public void fire(char character, int modifiers) throws ComponentException {
		fire( 
			character, 
			getKeyCode(character),
			modifiers | getModifiers(character) 
		);		
	}
	
	/**
	 * Simuliert einen Tastendruck auf dem assozierten Component. Dazu werden entsprechend 
	 * des Parameters <code>keyCode</code> eine Reihe von <code>KeyEvent</code>s generiert.
	 * @param character Das Zeichen oder 0, falls <code>keyCode</code>
	 * eine virtuelle Taste beschreibt.
	 * @param keyCode Der code des angegebenen Zeichens, entspricht bei Buchstaben und Ziffern
	 * immer dem Wert in <code>ch</code>.
	 * @param modifiers SWT Modifier fr Steuertasten z.B. <code>SWT.CTRL</code>, die gleichzeitig
	 * mit dem angegebenen Zeichen gedrckt werden sollen. Alle Modifier sind durch bitweises Oder
	 * verknpft.
	 */
	public void fire(final char character, final int keyCode, final int modifiers) throws ComponentException 
	{
		SWTThreadSafeAction action = new SWTThreadSafeAction() {
			protected Object performAction() throws Throwable 
			{
				pressModifiers(modifiers, true);
				
				Event down = generateEvent(SWT.KeyDown);
				down.character = character;
				down.keyCode = keyCode;
				postEvent(down);
								
				Event up = generateEvent(SWT.KeyUp);
				up.character = character;
				up.keyCode = keyCode;
				postEvent(up);
				
				pressModifiers(modifiers, false);
				
				return null;
			}
		};
		
		action.go();
	}
	
	/**
	 * Liefert die Steuertasten, die gedrckt werden mssen, um das angegebene
	 * Zeichen zu erzeugen. Beispielsweise die Shift-Taste fr Grobuchstaben.
	 * @param ch Das Zeichen.
	 * @return Die SWT Modifier Codes fr die Steuertasten verbunden durch bitweises Oder.
	 */
	private int getModifiers(char ch) 
	{
		// FIXME: leider ist das der falsche ansatz
		// denn wir mssten ja zu jedem zeichen herausbekommen
		// ob dazu Shift gedrckt werden muss, eigentlich msste 
		// Display#post(Event) das richtig regeln, tut es aber nicht
		if( Character.isUpperCase(ch) )
			return SWT.SHIFT;
		
		return 0;
	}
	
	private int getKeyCode(char ch) {
		return ch;
	}
	
	/**
	 * Simuliert das Drcken der angegebenen Steuertasten. Dazu wird
	 * eine Sequenz von entsprechenden <code>KeyEvent</code>s generiert und
	 * der EventQueue hinzugefgt.
	 * @param modifiers Die SWT Modifier Codes fr die Steuertasten verbunden durch bitweises Oder.
	 * @param down <code>true</code> = Tasten drcken, <code>false</code> = Tasten loslassen.
	 */
	public void pressModifiers(int modifiers, boolean down) throws ComponentException 
	{
		if(modifiers == 0)
			return;
		
		for(int i = 0; i < MODIFIERS.length; ++i)
		{
			if( (MODIFIERS[i] & modifiers) > 0 )
			{
				Event event = generateEvent(down ? SWT.KeyDown : SWT.KeyUp);
				event.keyCode = MODIFIERS[i];
				
				postEvent(event);
			}
		}
	}
}
