package atosj.event.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Widget;

import atosj.component.IComponent;
import atosj.component.swt.SWTComponent;
import atosj.event.FireableEvent;
import atosj.exception.ComponentException;

/**
 * Basisklasse zur Simulation von Events fr die SWT Bibliothek.
 * <em>ACHTUNG:</em> Alle Events werden im aktuellen Thread generiert.
 * Der aktuelle Thread <em>MUSS</em> der SWT Thread sein!
 * @author Nicos Tegos
 */
public class FireableSWTEvent extends FireableEvent 
{
	private Display display;
	
	/**
	 * Erstellt ein neues Ereignis.
	 * @param component Das Component - die Ereignisquelle.
	 */
	public FireableSWTEvent(SWTComponent component) {
		super(component);
		display = component.getSWTComponent().getDisplay();
	}
	
	/**
	 * Schreibt ein Event direkt in die System Eventqueue mit Hilfe
	 * der Funktion <code>Display.post(Event)</code>.
	 * @param event Das zu verschickende Event gltig sind derzeit nur
	 * folgende Typen:
	 * <ul>
	 * <li>SWT.MouseDown</li>
	 * <li>SWT.MouseUp</li>
	 * <li>SWT.KeyDown</li>
	 * <li>SWT.KeyUp</li>
	 * </ul>
	 */
	protected void postEvent(Event event) throws ComponentException 
	{
		//sync();
		display.post(event);
		//sync();
	}
	
	/**
	 * Benachrichtigt alle Listener des Components ber
	 * das Auftreten von <code>event</code>, fgt es jedoch
	 * nicht in die EventQueue ein. Es wird die Funktion
	 * <code>Widget.notifyListeners(int, Event)</code> genutzt.
	 */
	protected void notifyListeners(Event event) {
		// sync();
		getSWTComponent().notifyListeners(event.type, event);
		// sync();
	}
	
	/**
	 * Erstellt ein neues SWT Event.
	 * @param type Der Typ des Events.
	 * @return Das neu erstellt Event.
	 */
	protected Event generateEvent(int type) 
	{
		Event event = new Event();
		event.type = type;
		event.time = (int)System.currentTimeMillis();
		event.display = getSWTComponent().getDisplay();
		event.widget = getSWTComponent();
		
		return event;
	}

	protected Widget getSWTComponent() {
		return (Widget) getComponent();
	}
	
	/**
	 * Wandelt Modifer Codes aus der Klasse <code>IComponent</code> in
	 * entsprechende SWT Modifier um.
	 * @param atosjModifierCode Sequenz von ATOSj Modifier Codes verknpft
	 * durch bitweises Oder.
	 * @return Sequenz von SWT Modifier Codes verknpft
	 * durch bitweises Oder.
	 */
	public static int convertToSWTModifiers(int atosjModifierCode) 
	{
		int swtModifierCode = 0;
		if( (atosjModifierCode & IComponent.MODIFIER_ALT) > 0 )
			swtModifierCode |= SWT.ALT;
		
		/*if( (atosjModifierCode & IComponent.MODIFIER_ALT_GR) > 0 )
			swtModifierCode |= SWT.ALT_GR; // TODO den keyCode fr ALT_GR rausfinden
			*/
			
		if( (atosjModifierCode & IComponent.MODIFIER_CTRL) > 0 )
			swtModifierCode |= SWT.CTRL;
			
		if( (atosjModifierCode & IComponent.MODIFIER_SHIFT) > 0 )
			swtModifierCode |= SWT.SHIFT;
		
		return swtModifierCode;
	}
}
