/*
 * Created on 17.11.2005
 */
package atosj.gui;

import java.util.Comparator;
import java.util.Vector;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabFolder2Adapter;
import org.eclipse.swt.custom.CTabFolderEvent;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.app.Project;
import atosj.editors.Changeable;
import atosj.editors.Editor;
import atosj.editors.EditorManager;
import atosj.editors.IEditorFactory;
import atosj.editors.pack.PackageEditor;
import atosj.editors.pack.PackageTreeDir;
import atosj.editors.pack.PackageTreeEntry;
import atosj.editors.script.ScriptEditor;
import atosj.editors.script.ScriptTreeDir;
import atosj.editors.script.ScriptTreeEntry;
import atosj.editors.urf.URFEditor;
import atosj.editors.urf.URFTreeEntry;
import atosj.listener.EditorListener;
import atosj.listener.InputListener;
import atosj.listener.ProjectListener;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;
import atosj.model.urf.URFFile;
import atosj.util.UniversalViewerSorter;

/**
 * Die Projektansicht eines ATOSj Projektes. Diese beinhaltet einen
 * Baum auf der linken Seite, zur Navigation (Erstellen, Lschen usw. von Ressourcen) und
 * einen <code>TabFolder</code> auf der rechten Seite (Multi-Editor fr Ressourcen).
 * @author Nicos Tegos
 * @see atosj.editors.EditorManager
 */
public class MasterDetailsBlock implements InputListener, ProjectListener {

	private StructuredSelection currentSel;
	private SashForm sash;
	private CTabFolder navigatorTf;
	private TreeViewer tv;
	private CTabFolder editorTf;
	private EditorManager editorMngr;
	
	/**
	 * Erstellt eine neue Projektansicht. Das sollte nur
	 * einmal beim Erstellen des Hauptfensters geschehen.
	 * @param parent Das graph. Vaterelement
	 */
	public MasterDetailsBlock(Composite parent) 
	{
		createSash(parent);
		AtosJApp.getInstance().addInputListener(this);
	}
	
	private void createSash(Composite parent)
	{
		FillLayout fl = new FillLayout();
		fl.marginHeight = fl.marginWidth = 0;
		parent.setLayout(fl);
		
		sash = new SashForm(parent, SWT.HORIZONTAL);
		Composite masterArea = new Composite(sash, SWT.NULL);
		Composite detailsArea = new Composite(sash, SWT.NULL);
		sash.setWeights( new int[] {20, 80} );
		createMasterPart(masterArea);
		createDetailsPart(detailsArea);
		createEditorMngr();
	}
	
	/**
	 * Erstellt den Multi-Editor fr Ressourcen auf der rechten Seite.
	 * Muss nach <code>createDetailsPart</code> gerufen werden. 
	 */
	private void createEditorMngr() {
		editorMngr = new EditorManager(editorTf);
		
		editorMngr.addEditor(HTSScript.class, 
				new IEditorFactory () {
					public Editor createEditor(CTabFolder parent) {
						return new ScriptEditor(parent);
					}
			
		});
		
		editorMngr.addEditor(URFFile.class,
				new IEditorFactory() {
					public Editor createEditor(CTabFolder parent) {
						return new URFEditor(parent);
					}
		});
		
		editorMngr.addEditor(HTSPackage.class,
				new IEditorFactory() {
					public Editor createEditor(CTabFolder parent) {
						return new PackageEditor(parent);
					}
		});
	}
	
	/**
	 * Erstellt den Baum zur Navigation auf der linken Seite.
	 * @param parent Das graph. Vaterelement.
	 */
	private void createMasterPart(Composite parent)
	{
		parent.setLayout( new FillLayout() );
		navigatorTf = new CTabFolder(parent, SWT.TOP);
		navigatorTf.setSimple(false);
		navigatorTf.setBorderVisible(true);
		final CTabItem navigatorTab = new CTabItem( navigatorTf, SWT.NULL );
		
		tv = new TreeViewer(navigatorTf, SWT.SINGLE);
		navigatorTab.setControl( tv.getControl() );
		navigatorTab.setText(Messages.getString("MasterDetailsBlock.0")); //$NON-NLS-1$
		navigatorTf.setSelection(navigatorTab);
		AtosJApp.getInstance().addInputListener( new InputListener() {
			public void inputChanged(Object oldObj, Object newObj) {
				if(newObj != null)
					navigatorTab.setText( ((Project)newObj).getName() );
			}
		});
		
				
		tv.setContentProvider( new MasterTreeContentProvider() );
		tv.setLabelProvider( new TreeEntryLabelProvider() );
		// sortieren nach Name
		Comparator comp = new Comparator() {
			public int compare(Object arg0, Object arg1) {
				Changeable entry0 =  (Changeable) ((TreeEntry)arg0).getWrappedObject();
				Changeable entry1 = (Changeable) ((TreeEntry)arg1).getWrappedObject();
				
				return entry0.getName().compareTo( entry1.getName() );
			}
		};
		tv.setSorter( new UniversalViewerSorter(comp, new Class[] {ScriptTreeDir.class, ScriptTreeEntry.class, PackageTreeDir.class, PackageTreeEntry.class, URFTreeEntry.class}) );
		tv.addDoubleClickListener( new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent e) {
				TreeEntry entry = (TreeEntry) ((StructuredSelection)e.getSelection()).getFirstElement();
				editorMngr.addEditor( entry.getWrappedObject() );
			}
			
		});
		
		tv.getTree().addListener(SWT.MenuDetect, new Listener() {
			public void handleEvent(Event e) {
				Tree tree = (Tree) e.widget;
				TreeItem item = tree.getItem( tree.toControl(e.x, e.y) );
				if(item == null)
				{
					tree.setMenu(null);
				}
				else
				{
					if( item.getData() instanceof TreeEntry )	{
						tree.setMenu( ((TreeEntry)item.getData()).getContextMenu(tv).createContextMenu(tree) );
					}
				}
			}
			
		});
	}
	
	/**
	 * Erstellt den <code>CTabFolder</code> fr den 
	 * Multi-Editor auf der rechten Seite.
	 * @param parent Das graphische Vaterelement.
	 */
	private void createDetailsPart(Composite parent)
	{
		parent.setLayout( new FillLayout() );
		editorTf = new CTabFolder(parent, SWT.TOP);
		editorTf.setSimple(false);
		editorTf.setMaximizeVisible(true);
		editorTf.setBorderVisible(true);
		editorTf.addCTabFolder2Listener( 
				new CTabFolder2Adapter() 
		{
			int[] lastWeights = {20, 80};
			public void maximize(CTabFolderEvent e) {
				editorTf.setMaximized(true);
				lastWeights = sash.getWeights();
				sash.setWeights( new int[] {0, 100} );
			}
			
			public void restore(CTabFolderEvent arg0) {
				editorTf.setMaximized(false);
				sash.setWeights(lastWeights);
			}
		});
	}
	
	/**
	 * ffnet den Editor fr die Ressource.
	 * @param editMe Die Ressource, deren Editor geffnet werden soll.
	 * @see EditorManager#addEditor(Object)
	 */
	public void openEditor(Object editMe) {
		editorMngr.addEditor(editMe);
		
		// das entsprechende TreeEntry finden
		TreeEntry entry = findEntry( (TreeEntry)tv.getInput(), editMe );
		if(entry != null)
		{
			tv.reveal(entry);
			tv.setSelection( new StructuredSelection(entry) );
		}
	}
	
	/**
	 * Sucht zu einem Objekt den entsprechenden Eintrag im Baum, dieser
	 * kann dann revealed, selected usw. werden.
	 * @param parent Der Baumeintrag bei dem die Suche starten soll.
	 * @param editMe Das Objekt dessen Baumeintrag gesucht werden soll.
	 * @return Das entsprechende TreeEntry oder <code>null</code>, wenn nicht gefunden.
	 */
	private TreeEntry findEntry(TreeEntry parent, Object editMe)
	{
		if(parent.getWrappedObject() == editMe)
			return parent;
		
		TreeEntry[] children = parent.getChildren();
		for(int i = 0; i < children.length; ++i)
		{
			TreeEntry found = findEntry(children[i], editMe);
			if(found != null)
				return found;
		}
		
		return null;
	}
	
	/**
	 * @see EditorManager#getEditedObjects()
	 */
	public Object[] getEditedObjects() {
		return editorMngr.getEditedObjects();
	}
	
	/**
	 * @see EditorManager#getActiveObject()
	 */
	public Object getActiveObject() {
		return editorMngr.getActiveObject();
	}

	/**
	 * Reagiert auf das ffnen bzw. Schlieen eines Projektes.
	 * Wird ein Projekt geffnet wird der Navigator Baum aktualisiert
	 * und es werden die zuletzt geffneten Editoren wieder geffnet.
	 */
	public void inputChanged(Object oldObj, Object newObj) 
	{
		Project proj = (Project)newObj;
		editorMngr.closeAllEditors();
		if(oldObj != null)
			((Project)oldObj).removeProjectListener(this);
		
		if(proj != null)
		{
			proj.addProjectListener(this);
			tv.setInput(new TreeProject(tv, proj));
			tv.expandAll();
			
			// alle zuletzt in diesem Projekt geffneten Editoren wieder ffnen
			Vector resources = proj.getEditedResources();
			for(int i = 0; i < resources.size(); ++i)
				editorMngr.addEditor( resources.get(i) );
			// den aktiven editor setzen
			Object active = proj.getActiveResource();
			if(active != null)
				editorMngr.addEditor(active);
			
			// das bild im navigator tree entsprechend des Projekttyps setzen
			if( navigatorTf.getItemCount() > 0 )
				navigatorTf.getItem(0).setImage( proj.isAwtProject() 
						? AtosJApp.getInstance().getImage(AtosJApp.IMG_SWING) 
						: AtosJApp.getInstance().getImage(AtosJApp.IMG_ECLIPSE) );
		}
		else
		{
			
			tv.setInput(null);
		}
	}
	
	/**
	 * @see EditorManager#addEditorListener(EditorListener)
	 */
	public void addEditorListener(EditorListener listener) {
		editorMngr.addEditorListener(listener);
	}
	
	/**
	 * @see EditorManager#removeEditorListener(EditorListener)
	 */
	public void removeEditorListener(EditorListener listener) {
		editorMngr.removeEditorListener(listener);
	}

	/**
	 * Wird eine Ressource im Projekt gelscht, wird
	 * ihr Editor geschlossen. Falls dieser zuvor geffnet war.
	 * @see EditorManager#closeEditor(Object) 
	 */
	public void resourceDeleted(Changeable c) {
		// Editor schliessen wenn Resource gelscht
		editorMngr.closeEditor(c);		
	}

	/**
	 * Wird eine Ressource im Projekt neu erstellt, wird
	 * auch gleich ihr Editor geffnet.
	 * @see EditorManager#addEditor(Object)
	 */
	public void resourceCreated(Changeable c) {
		// Editor ffnen wenn Resource erstellt
		editorMngr.addEditor(c);
	}
}
