/*
 * Created on 17.11.2005
 */
package atosj.gui;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.graphics.Image;

import atosj.editors.Changeable;
import atosj.listener.StateChangeListener;

/**
 * Basisklasse fr alle Eintrge im Navigator Baum des
 * <code>MasterDetailsBlocks</code>.
 * @author Nicos Tegos
 * @see atosj.gui.MasterDetailsBlock
 */
public abstract class TreeEntry implements StateChangeListener 
{
	private Object wrappedObj;
	private TreeViewer tv;
	private TreeNodeContainer parent; 
	private static TreeEntry[] EMPTY_ARRAY = new TreeEntry[0];
	
	/**
	 * Ummantelt ein Element, zur Anzeige von Informationen 
	 * in einem <code>TreeViewer</code>. Stellt einen Knoten des Baumes dar.
	 * @param tv Der <code>TreeViewer</code> fr diesen Knoten.
	 * @param toWrap Das ummantelte Element. Implementiert <code>toWrap</code>
	 * das Interface <code>Changeable</code>, dann werden Vernderungen von Eigenschaften,
	 * wie z.B. die Gltigkeit im Baum visualisiert.
	 * @param parent Der Vaterknoten oder, <code>null</code> fr Wurzelelemente.
	 * @see Changeable
	 */
	public TreeEntry(TreeViewer tv, Object toWrap, TreeNodeContainer parent) 
	{ 
		this.tv = tv; 
		this.parent = parent;
		this.wrappedObj = toWrap;
		if(wrappedObj instanceof Changeable)
			((Changeable)wrappedObj).addStateChangeListener(this);
	}
	
	/** 
	 * Liefert die Beschriftung des Knotens.
	 * Die Basisimplementation liefert einen leeren String. 
	 **/
	public String getName() { return ""; }
	
	/** 
	 * Liefert das Bild fr den Knoten. Dieses sollte den Zustand
	 * des ummantelten Elements wiederspiegeln (gltig/ungltig).
	 * Die Basisimplementation liefert <code>null</code>.
	 * @return Das Bild oder <code>null</code>.
	 * */
	public Image getImage() { return null; }
	/** 
	 * Gibt an, ob der Knoten Kinder hat.
	 * Die Basisimplementation liefert <code>false</code>.
	 * @return <code>true</code> wenn Kinder vorhanden.
	 * */
	public boolean hasChildren() { return false; }
	/**
	 * Liefert die Kinder dieses Knotens.
	 * Die Basisimplementation liefert ein leeres Array.
	 * @return Die Kinder oder ein leeres Array.
	 * */
	public TreeEntry[] getChildren() { return EMPTY_ARRAY; }
	/**
	 * Liefert ein Kontextmen mit Aktionen fr diesen Knoten.
	 * Die Basisimplementation liefert ein leeres Men. 
	 * */
	public MenuManager getContextMenu(TreeViewer tv) { return new MenuManager(); }
	/** Liefert das ummantelte Element. */
	public Object getWrappedObject() { return wrappedObj; }
	/** Liefert den Baum dieses Knotens. */
	public TreeViewer getTreeViewer() { return tv; }
	/** Aktualisiert diesen Knoten im Baum, sich der Name des ummantelten Elements ndert.
	 * Das geschieht nur, wenn das ummantelte Element vom Typ <code>Changeable</code> ist. */
	public void stateChanged(Changeable changed, int state, Object oldValue, Object newValue) {
		getTreeViewer().refresh(this, true);
		
		if(state == StateChangeListener.STATE_NAME) // nach umbenennen auch den parent aktualisieren damit die sortierung aktualisiert wird 
		{
			if( getParent() != null )
				getTreeViewer().refresh( getParent(), false );
		}
	}
	/** Liefert den Vaterknoten oder <code>null</code>, wenn es sich um einen Wurzelknoten handelt. */
	public TreeNodeContainer getParent() { return parent; }
	/** Entfernt den Listener auf Vernderungen des ummantelten Elements,
	 * falls dieses eine <code>Changeable</code> ist. 
	 * @see Changeable#removeStateChangeListener(StateChangeListener) */
	public void unbind() 
	{
		if(getWrappedObject() instanceof Changeable)
			((Changeable)getWrappedObject()).removeStateChangeListener(this);
	}
}
