package atosj.model.hts.command;

import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.Vector;

import atosj.Messages;
import atosj.component.ComponentResolver;
import atosj.component.IButton;
import atosj.component.ICheckbox;
import atosj.component.ICombobox;
import atosj.component.IComponent;
import atosj.component.IEditbox;
import atosj.component.IList;
import atosj.component.IMenu;
import atosj.component.IRadio;
import atosj.component.ISpinner;
import atosj.component.ITabFolder;
import atosj.component.ITable;
import atosj.component.ITree;
import atosj.component.IWindow;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS ACTION Kommandos.
 * @author Nicos Tegos
 */
public class HTSAction extends HTSComponentCommand
{
	private class ClickExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			((IComponent)component).click( ((Integer)params.get(1)).intValue(), 
					((Boolean)params.get(0)).booleanValue(),
					((Integer)params.get(2)).intValue());
			return null;
		}
	}
	
	private class PresskeyExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			((IComponent)component).pressKey( ((Character)params.get(0)).charValue(), 
					((Integer)params.get(1)).intValue());
			return null;
		}
	}
	
	private class PropertyExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			((IComponent)component).setProperty( (String) params.get(0), 
					(String) params.get(1) );
			
			return null;
		}
	}
	
	private String actionType; // Die Aktionsart z.B. EDIT
	private Executor executor; // Ausfhrer der Aktion.
	
	/**
	 * Erstellt ein neues ACTION-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 * @param resolver Der Resolver fr die Components, dieser gibt gltige
	 * Custom-Components an.
	 */
	public HTSAction(CSVLine line, ComponentResolver resolver) throws ParseException {
		super(line, resolver);
	}
	
	private void parseCommand(String[] validActions, int[] possibleSubitems, SubitemPathValidator pathValidator, Executor[] testExecutors) throws ParseException
	{
		SubitemPathValidator[] pathValidators = new SubitemPathValidator[validActions.length];
		Arrays.fill(pathValidators, pathValidator);
		
		parseCommand(validActions, possibleSubitems, pathValidators, testExecutors);
	}
	
	/**
	 * berprft die Parameter des ACTION-Kommandos auf Gltigkeit.
	 * @param validActions Gltige Aktionen fr den aktuellen Componenttyp, siehe HTS Schlsselwrter.
	 * @param possibleSubitems Gibt an, ob die angegebenen Aktionen auch fr Subelemente von zusammengesetzen Components
	 * gltig sind, siehe SUBITEM_PATH_* Konstanten.
	 * @param pathValidator Parser fr die Subelementpfade zu den Aktionen. Zu jeder Aktion, die in <code>possibleSubitems</code>
	 * mit SUBITEM_PATH_OBLIGED oder SUBITEM_PATH_VOLUNTARY deklariert ist, muss in diesem Array ein <code>SubitemPathValidator</code>
	 * stehen.
	 * @param actionExecutors Zu jeder Aktion muss eine entsprechende Ausfhrungsroutine bereitgestellt werden.
	 * @throws ParseException Wenn die Parameter des Kommandos ungltig sind.
	 */
	private void parseCommand(String[] validActions, int[] possibleSubitems, SubitemPathValidator pathValidator[], Executor[] actionExecutors) throws ParseException
	{
		actionType = getParam(3, Messages.getString("HTSAction.0")); //$NON-NLS-1$
		
		Vector params = null;
		Vector subitemPath = null;
		
		// berprfen ob der aktionstyp in der menge der gltigen aktionstypen enthalten ist.
		int validIndex = -1;
		for(int i = 0; i < validActions.length; ++i)
		{
			if( validActions[i].equals(actionType) )
			{
				validIndex = i;
				break;
			}
		}
		
		if(validIndex == -1) // keinen der gltigen aktionstypen gefunden
		{
			throw new InvalidParamException(line, 3, actionType,
					validActions);
		}
		
		if( actionType.equals(HTSSyntax.ACTION_CLICK) )
		{
			params = new Vector(3);
			
			String button = getParam(4, Messages.getString("HTSAction.1")); //$NON-NLS-1$
			if( button.equals(HTSSyntax.ACTION_LEFT) )
			{
				params.add(Boolean.TRUE);
			}
			else
			if( button.equals(HTSSyntax.ACTION_RIGHT) )
			{
				params.add(Boolean.FALSE);
			}
			else
				throw new InvalidParamException(line, 4, button, 
						new String[] {HTSSyntax.ACTION_LEFT, HTSSyntax.ACTION_RIGHT});
			
			params.add( getNum(5, 1, Messages.getString("HTSAction.2")) ); //$NON-NLS-1$
			int modifiers = IComponent.NO_MODIFIER;
			
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				int i = 6;
				for(; i < getParamSize() && i < 9; ++i)
				{
					String param = getParam(i, Messages.getString("HTSAction.3")); //$NON-NLS-1$
					int ret = getModifier(param, modifiers);
					if(ret != -1)
						modifiers |= ret;
					else
						throw new InvalidParamException(line, i, param, new String[] {HTSSyntax.ACTION_ALT, HTSSyntax.ACTION_CTRL, HTSSyntax.ACTION_SHIFT} );
				}
				
				assertParamSize(i);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				int i = 6;
				for(; i < getParamSize() && i < 9; ++i)
				{
					String param = getParam(i, Messages.getString("HTSAction.4")); //$NON-NLS-1$
					int ret = getModifier(param, modifiers);
					if(ret != -1)
						modifiers |= ret;
					else
						break;
				}
				
				subitemPath = getSubItemPath(i, pathValidator[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				int i = 6;
				for(; i < getParamSize() && i < 9; ++i)
				{
					String param = getParam(i, Messages.getString("HTSAction.5")); //$NON-NLS-1$
					int ret = getModifier(param, modifiers);
					if(ret != -1)
						modifiers |= ret;
					else
						break;
				}
				
				if( getParamSize() > i )
				{
					subitemPath = getSubItemPath(i, pathValidator[validIndex]);
				}
			}
			
			params.add( new Integer(modifiers) );
		}
		else if( actionType.equals(HTSSyntax.ACTION_PRESSKEY) )
		{
			params = new Vector(2);
			String keyString = getParamUnquoted(4, Messages.getString("HTSAction.6")); //$NON-NLS-1$
			if( keyString.length() != 1 )
				throw new InvalidParamException(line, 4, keyString, Messages.getString("HTSAction.7")); //$NON-NLS-1$
			
			params.add( new Character(keyString.charAt(0)) );
			
			// oh je jetzt noch die modifier;
			int modifiers = IComponent.NO_MODIFIER;
			// TODO code dopplung siehe commando klick, vielleicht in eigene Funktion das parsen der modifier
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				int i = 5;
				for(; i < getParamSize() && i < 9; ++i)
				{
					String param = getParam(i, Messages.getString("HTSAction.8")); //$NON-NLS-1$
					int ret = getModifier(param, modifiers);
					if(ret != -1)
						modifiers |= ret;
					else
						throw new InvalidParamException(line, i, param, new String[] {HTSSyntax.ACTION_ALT, HTSSyntax.ACTION_CTRL, HTSSyntax.ACTION_SHIFT} );
				}
				
				assertParamSize(i);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				int i = 6;
				for(; i < getParamSize() && i < 9; ++i)
				{
					String param = getParam(i, Messages.getString("HTSAction.9")); //$NON-NLS-1$
					int ret = getModifier(param, modifiers);
					if(ret != -1)
						modifiers |= ret;
					else
						break;
				}
				
				subitemPath = getSubItemPath(i, pathValidator[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				int i = 6;
				for(; i < getParamSize() && i < 9; ++i)
				{
					String param = getParam(i, Messages.getString("HTSAction.10")); //$NON-NLS-1$
					int ret = getModifier(param, modifiers);
					if(ret != -1)
						modifiers |= ret;
					else
						break;
				}
				
				if( getParamSize() > i )
				{
					subitemPath = getSubItemPath(i, pathValidator[validIndex]);
				}
			}
			
			params.add( new Integer(modifiers) );
		}
		else if( actionType.equals(HTSSyntax.ACTION_PROPERTY) )
		{
			params = new Vector(2);
			params.add( getParamUnquoted(4, Messages.getString("HTSAction.11")) ); //$NON-NLS-1$
			params.add( getParamUnquoted(5, Messages.getString("HTSAction.12")) ); //$NON-NLS-1$
		}
		else if( actionType.equals(HTSSyntax.ACTION_EDIT) )
		{
			params = new Vector(1);
			params.add( getParamUnquoted(4, Messages.getString("HTSAction.13")) ); //$NON-NLS-1$
			
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				assertParamSize(5);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				subitemPath = getSubItemPath(5, pathValidator[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				if( getParamSize() >  5 )
					subitemPath = getSubItemPath(5, pathValidator[validIndex]);
			}
			
		}
		else if( actionType.equals(HTSSyntax.ACTION_SELECT)  // parameterlose akionen werden auf gleiche weise geparsed
				|| actionType.equals(HTSSyntax.ACTION_ADDSELECT)
				|| actionType.equals(HTSSyntax.ACTION_DESELECT)
				|| actionType.equals(HTSSyntax.ACTION_CHECK)
				|| actionType.equals(HTSSyntax.ACTION_UNCHECK)
				|| actionType.equals(HTSSyntax.ACTION_CLOSE)
				|| actionType.equals(HTSSyntax.ACTION_MAXIMIZE)
				|| actionType.equals(HTSSyntax.ACTION_MINIMIZE)
				|| actionType.equals(HTSSyntax.ACTION_NORMALIZE)
			)
		{
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				assertParamSize(4);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				subitemPath = getSubItemPath(4, pathValidator[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				if( getParamSize() >  4 )
				{
					subitemPath = getSubItemPath(4, pathValidator[validIndex]);
				}
			}	
		}
		
		executor = actionExecutors[validIndex];
		executor.setParams(params);
		executor.setSubitemPath(subitemPath);
	}
	
	/**
	 * Liefert zu einem HTS Modifier Schlsselwort wie ALT einen entsprechenden
	 * ATOSj Modifier Code.
	 * @param param Der Kommando Parameter, evtl. ein Modifier Schlsselwort.
	 * @param currentModifiers Die bisherigen ATOSj Modifier, verknpft durch bitweises Oder.
	 * @return Die bisherigen Modifier verknpft mit dem neuen Modifer durch bitweises Oder,
	 * oder -1, falls <code>param</code> kein Modifier Schlsselwort war.
	 * @throws ParseException wenn der Modifier schon in <code>currentModifiers</code>
	 * enthalten war.
	 */
	private int getModifier(String param, int currentModifiers) throws ParseException 
	{
		if( param.equals(HTSSyntax.ACTION_ALT) )
		{
			if( (currentModifiers & IComponent.MODIFIER_ALT) == 0 )
				currentModifiers |= IComponent.MODIFIER_ALT;
			else
				throw new InvalidParamException(line, index, param, Messages.getString("HTSAction.14")); //$NON-NLS-1$
		}
		else
		if( param.equals(HTSSyntax.ACTION_CTRL) )
		{
			if( (currentModifiers & IComponent.MODIFIER_CTRL) == 0 )
				currentModifiers |= IComponent.MODIFIER_CTRL;
			else
				throw new InvalidParamException(line, index, param, Messages.getString("HTSAction.15")); //$NON-NLS-1$
		}
		else
		if( param.equals(HTSSyntax.ACTION_SHIFT) )
		{
			if( (currentModifiers & IComponent.MODIFIER_SHIFT) == 0 )
				currentModifiers |= IComponent.MODIFIER_SHIFT;
			else
				throw new InvalidParamException(line, index, param, Messages.getString("HTSAction.16")); //$NON-NLS-1$
		}
		else
			return -1; // kein modifier keyword
		
		return currentModifiers;
	}
	
	protected void parseCustomCommand() throws ParseException
	{
		parseCommand( new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_PROPERTY}, 
				new int[] { SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[])null,
				new Executor[] {new ClickExecutor(), new PresskeyExecutor(), new PropertyExecutor() }
			);
	}
	
	protected void parseButtonCommand() throws ParseException
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor selectEx = executor = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IButton)component).select();
				return null;
			}
		};
		
		parseCommand( new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT}, 
				new int[] { SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[])null,
				new Executor[] {clickEx, presskeyEx, selectEx}
			);
	}
	
	protected void parseCheckboxCommand() throws ParseException 
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor selectEx = executor = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IButton)component).select();
				return null;
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ICheckbox)component).setChecked(true);
				return null;
			}
		};
		Executor uncheckEx = executor = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ICheckbox)component).setChecked(false);
				return null;
			}
		};
		
		parseCommand( new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_CHECK, HTSSyntax.ACTION_UNCHECK}, 
				new int[] { SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[])null,
				new Executor[] {clickEx, presskeyEx, selectEx, checkEx, uncheckEx}
			);
	}
	
	protected void parseComponentCommand() throws ParseException
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
				
		parseCommand( new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[])null,
				new Executor[] {clickEx, presskeyEx}
			);
	}
	
	protected void parseComboboxCommand() throws ParseException 
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor selectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				Object pathElement = subitemPath.get(0);
				if(pathElement instanceof Integer)
					((ICombobox)component).select( ((Integer) pathElement).intValue() );
				else
					((ICombobox)component).select( (String) pathElement );
				
				return null;
			}
		};
		Executor editEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ICombobox)component).setText( (String) params.get(0) );
				return null;
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				String param = getParam(paramIndex, Messages.getString("HTSAction.17")); //$NON-NLS-1$
				if( HTSParser.isQuoted(param) )
					path.add( HTSParser.stripQuotes(param) );
				else
					path.add( getNum(paramIndex, 0, Messages.getString("HTSAction.18")) ); //$NON-NLS-1$
				
				assertParamSize(paramIndex+1);
				return path;
			}
		};
				
		parseCommand( 
				new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_EDIT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_FORBIDDEN},
				pathValidator,
				new Executor[] {clickEx, presskeyEx, selectEx, editEx}
			);
	}
	
	protected void parseEditboxCommand() throws ParseException 
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor editEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IEditbox)component).setText( (String) params.get(0) );
				return null;
			}
		};
				
		parseCommand( new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_EDIT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[])null,
				new Executor[] {clickEx, presskeyEx, editEx}
			);
	}
	
	protected void parseSpinnerCommand() throws ParseException 
	{
		Executor editEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ISpinner)component).setText( (String) params.get(0) );
				return null;
			}
		};
				
		parseCommand( new String[] { HTSSyntax.ACTION_EDIT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[])null,
				new Executor[] {editEx}
			);
	}
	
	protected void parseLabelCommand() throws ParseException 
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
				
		parseCommand( new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[])null,
				new Executor[] {clickEx, presskeyEx}
			);
	}
	
	protected void parseListCommand() throws ParseException 
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor selectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				Object pathElement = subitemPath.get(0);
				if(pathElement instanceof Integer)
					((IList)component).select( ((Integer)pathElement).intValue() );
				else
					((IList)component).select( (String) pathElement );
				return null;
			}
		};
		Executor addselectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				Object pathElement = subitemPath.get(0);
				if(pathElement instanceof Integer)
					((IList)component).addSelect( ((Integer) pathElement).intValue() );
				else
					((IList)component).addSelect( (String) pathElement );
				return null;
			}
		};
		Executor deselectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				Object pathElement = subitemPath.get(0);
				if(pathElement instanceof Integer)
					((IList)component).deselect( ((Integer)pathElement).intValue() );
				else
					((IList)component).deselect( (String) pathElement );
				return null;
			}
		};
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				String param = getParam(paramIndex, Messages.getString("HTSAction.19")); //$NON-NLS-1$
				if( HTSParser.isQuoted(param) )
					path.add( HTSParser.stripQuotes(param) );
				else
					path.add( getNum(paramIndex, 0, Messages.getString("HTSAction.20")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( 
				new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_ADDSELECT, HTSSyntax.ACTION_DESELECT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED},
				pathValidator,
				new Executor[] {clickEx, presskeyEx, selectEx, addselectEx, deselectEx}
			);
	}
	
	protected void parseMenuCommand() throws ParseException
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor selectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IMenu)component).select( (String[]) subitemPath.toArray( new String[subitemPath.size()]) );
				return null;
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IMenu)component).setChecked( (String[]) subitemPath.toArray( new String[subitemPath.size()]), true );
				return null;
			}
		};
		Executor uncheckEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IMenu)component).setChecked( (String[]) subitemPath.toArray( new String[subitemPath.size()]), false );
				return null;
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				for(int i = paramIndex; i < getParamSize(); ++i)
					path.add( getParamUnquoted(i, Messages.getString("HTSAction.21")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( 
				new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_CHECK, HTSSyntax.ACTION_UNCHECK}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED},
				pathValidator,
				new Executor[] {clickEx, presskeyEx, selectEx, checkEx, uncheckEx}
			);
	}
	
	protected void parseRadiobuttonCommand() throws ParseException 
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor selectEx = executor = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IButton)component).select();
				return null;
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IRadio)component).check();
				return null;
			}
		};
		
		parseCommand( new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_CHECK}, 
				new int[] { SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[])null,
				new Executor[] {clickEx, presskeyEx, selectEx, checkEx}
			);
	}
	
	protected void parseTabfolderCommand() throws ParseException 
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor selectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				Object pathElement = subitemPath.get(0);
				if(pathElement instanceof Integer)
					((ITabFolder)component).select( ((Integer) pathElement).intValue() );
				else
					((ITabFolder)component).select( (String) pathElement );
				return null;
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				String param = getParam(paramIndex, Messages.getString("HTSAction.22")); //$NON-NLS-1$
				if( HTSParser.isQuoted(param) )
					path.add( HTSParser.stripQuotes(param) );
				else
					path.add( getNum(paramIndex, 0, Messages.getString("HTSAction.23")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( 
				new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED},
				pathValidator,
				new Executor[] {clickEx, presskeyEx, selectEx}
			);
	}
	
	protected void parseTableCommand() throws ParseException 
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor selectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITable)component).select( ((Integer)subitemPath.get(0)).intValue() );
				return null;
			}
		};
		Executor addselectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITable)component).addSelect( ((Integer)subitemPath.get(0)).intValue() );
				return null;
			}
		};
		Executor deselectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITable)component).deselect( ((Integer)subitemPath.get(0)).intValue() );
				return null;
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITable)component).getTableCell( ((Integer)subitemPath.get(0)).intValue(), 
						((Integer)subitemPath.get(1)).intValue() ).setChecked(true);
				return null;
			}
		};
		Executor uncheckEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITable)component).getTableCell( ((Integer)subitemPath.get(0)).intValue(), 
						((Integer)subitemPath.get(1)).intValue() ).setChecked(false);
				return null;
			}
		};
		Executor editEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITable)component).getTableCell( ((Integer)subitemPath.get(0)).intValue(), 
						((Integer)subitemPath.get(1)).intValue() ).setText( (String)params.get(0) );
				return null;
			}
		};
		
		SubitemPathValidator pathValidator1 = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				path.add( getNum(paramIndex, 0, Messages.getString("HTSAction.24")) ); //$NON-NLS-1$
				path.add( getNum(paramIndex+1, 0, Messages.getString("HTSAction.25")) ); //$NON-NLS-1$
				
				assertParamSize(paramIndex + 2);
				return path;
			}
		};
		
		SubitemPathValidator pathValidator2 = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				path.add( getNum(paramIndex, 0, Messages.getString("HTSAction.26")) ); //$NON-NLS-1$
				
				assertParamSize(paramIndex + 1);
				return path;
			}
		};
		
		parseCommand( 
				new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_ADDSELECT, HTSSyntax.ACTION_DESELECT, HTSSyntax.ACTION_CHECK, HTSSyntax.ACTION_UNCHECK, HTSSyntax.ACTION_EDIT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED},
				new SubitemPathValidator[] { null, null, pathValidator2, pathValidator2, pathValidator2, pathValidator1, pathValidator1, pathValidator1},
				new Executor[] {clickEx, presskeyEx, selectEx, addselectEx, deselectEx, checkEx, uncheckEx, editEx}
			);
	}
	
	protected void parseTreeCommand() throws ParseException
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor selectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITree)component).select( (String[]) subitemPath.toArray(new String[subitemPath.size()]) );
				return null;
			}
		};
		Executor addselectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITree)component).addSelect( (String[]) subitemPath.toArray(new String[subitemPath.size()]) );
				return null;
			}
		};
		Executor deselectEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITree)component).deselect( (String[]) subitemPath.toArray(new String[subitemPath.size()]) );
				return null;
			}
		};
		Executor editEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((ITree)component).getTreeCell( (String[]) subitemPath.toArray(new String[subitemPath.size()]) ).setText( (String)params.get(0) );
				return null;
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				for(int i = paramIndex; i < getParamSize(); ++i)
					path.add( getParamUnquoted(i, Messages.getString("HTSAction.27")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( 
				new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_SELECT, HTSSyntax.ACTION_ADDSELECT, HTSSyntax.ACTION_DESELECT, HTSSyntax.ACTION_EDIT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED},
				pathValidator,
				new Executor[] {clickEx, presskeyEx, selectEx, addselectEx, deselectEx, editEx}
			);
	}
	
	protected void parseWindowCommand() throws ParseException 
	{
		Executor clickEx = new ClickExecutor();
		Executor presskeyEx = new PresskeyExecutor();
		Executor closeEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IWindow)component).close();
				return null;
			}
		};
		Executor maximizeEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IWindow)component).maximize();
				return null;
			}
		};
		Executor minimizeEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IWindow)component).minimize();
				return null;
			}
		};
		Executor normalizeEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				((IWindow)component).normalize();
				return null;
			}
		};
		
		parseCommand( 
				new String[] { HTSSyntax.ACTION_CLICK, HTSSyntax.ACTION_PRESSKEY, HTSSyntax.ACTION_CLOSE, HTSSyntax.ACTION_MAXIMIZE, HTSSyntax.ACTION_MINIMIZE, HTSSyntax.ACTION_NORMALIZE}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[])null,
				new Executor[] {clickEx, presskeyEx, closeEx, maximizeEx, minimizeEx, normalizeEx}
			);
	}
	
	public void execute(IHTSPlayer player) throws ExecutionException 
	{
		try 
		{
			MComponent comp = getMComponent( getComponentId(player.getTestObject()) );
			executor.execute(comp);
		}
		catch(ComponentException e) // alle ComponentExceptions in ExecutionException umwandeln
		{
			throw new ExecutionException( this, e.getMessage() );
		}
		catch(RemoteException e) // der Player im AtosJ Prozess ist tot
		{
			// wir machen einfach nix
			// throw new ExecutionException( this, e.getMessage() );
		}
	}
	
	public Object clone() {
		try {
			return new HTSAction( (CSVLine) line.clone(), getResolver() );
		} catch(ParseException e) {
			return null;
		}
	}
}
