/*
 * Created on 07.09.2005
 */
package atosj.model.hts.command;

import java.io.Serializable;
import java.util.Vector;

import atosj.Messages;
import atosj.app.Project;
import atosj.editors.AtosJCloneable;
import atosj.exception.ExecutionException;
import atosj.exception.IntegrityException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.script.HTSScript;
import atosj.model.urf.URFFile;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Basisklasse aller Kommandos der Sprache HTS.
 * Jedes Kommando bernimmt das Parsen Kommandoparameter und
 * die Ausfhrung zur Laufzeit.
 * 
 * @author Nicos Tegos
 */
public abstract class HTSCommand implements Serializable, AtosJCloneable
{
	protected CSVLine line;
	protected int index = -1;
	private String error;
	
	/**
	 * Erstellt ein neues Kommando.
	 * berprft die Korrektheit der Paramterliste dieses Kommandos
	 * und fhrt eventuell mgliches preprocessing der Parameter durch.
	 * @param line Die Kommaseparierte Liste die diesem Kommando entspricht.
	 * @throws ParseException sollte die Paramterliste Fehler enthalten,
	 * wie z.B. Unvollstndigkeit wird diese Exception geworfen.
	 */
	public HTSCommand(CSVLine line) throws ParseException 
	{
		this.line = line;
	}
	
	/**
	 * Fhrt das Kommando aus.
	 * @param player
	 * @throws ExecutionException wenn das Kommando nicht ausgefhrt werden kann,
	 * auch wenn das Kommando ausgefhrt wurde aber der Vergleich von Soll- und Istwert
	 * fehlgeschlagen ist.
	 */
	public abstract void execute(IHTSPlayer player) throws ExecutionException;
	
	/**
	 * Liefert die Position dieses Kommandos in seinem Skript, 0-basiert.
	 * @see #setIndex(int)
	 */
	public int getIndex() {
		return index;
	}
	
	/**
	 * Setzt die Positon neu.
	 * @see #getIndex()
	 */
	public void setIndex(int index) {
		this.index = index;
	}
	
	/**
	 * Liefert einen Parameter aus der Parameterliste.
	 * @param index Der Index des Parameters in der Liste.
	 * @param parameterName Der Name des Parameters, dieser wird nur
	 * zur Anzeige in Fehlernachrichten verwendet.
	 * @return Den Parameterwert als String, so wie er in der Liste
	 * steht.
	 * @throws ParseException wenn an der angegebenen Position kein Paramter existiert.
	 */
	protected String getParam(int index, String parameterName) throws ParseException {
		try {
			return line.getElement(index + 1);
		} catch(ParseException e) {
			throw new ParseException(line, Messages.getString("HTSCommand.0") + (index+1) + " (" + parameterName + Messages.getString("HTSCommand.2")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
	}
	
	/**
	 * Liefert einen Zeichenketten-Parameter.
	 * @param index Die Position in der Parameterliste, 0-basiert.
	 * @param parameterName Der Name des Parameters, dieser wird nur
	 * zur Anzeige in Fehlernachrichten verwendet.
	 * @return String ohne Anfhrungszeichen
	 * @throws ParseException falls an der angegebenen Position kein Element enthalten ist
	 * ODER, wenn der Parameter nicht in Anfhrungszeichen steht.
	 */
	protected String getParamUnquoted(int index, String parameterName) throws ParseException {
		String parameter = getParam(index, parameterName);
		if( HTSParser.isQuoted(parameter) )
			return HTSParser.stripQuotes(parameter);
		else
			throw new ParseException(line, Messages.getString("HTSCommand.3") + (index+1) + " (" + parameterName + Messages.getString("HTSCommand.5")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	/**
	 * Liefert die Anzahl der Parameter des Kommandos. Das Schlsselwort des Kommandos
	 * wird nicht mitgerechnet.
	 */
	protected int getParamSize() {
		return line.getNumOfElements() - 1;
	}
	
	/**
	 * Liefert das HTS-Schlsselwort fr dieses Kommandos.
	 */
	public String getCommandKey()
	{
		try {
			return line.getElement(0);
		} catch (ParseException e) {
			// shall never happen
		}
		
		return ""; //$NON-NLS-1$
	}
	
	/**
	 * Liefert die Liste der Kommandoparameter.
	 */
	public String[] getParameters() 
	{
		Vector ret = new Vector( line.getNumOfElements() );
		// wir starten bei 1, weil 0 das schlsselwort fr das Kommando ist
		for(int i = 1; i < line.getNumOfElements(); ++i)
		{
			try {
				ret.add( line.getElement(i) );
			}catch(ParseException e) { /*kann nicht passieren*/ }
		}
		
		return (String[]) ret.toArray( new String[ret.size()] );
	}
	
	/**
	 * Wandelt einen Parameter in eine ganze Zahl um. Der Parameter darf nicht
	 * in Anfhrungszeichen stehen.
	 * @param index Die Position in der Parameterliste, 0-basiert.
	 * @param parameterName Der Name des Parameters, dieser wird nur
	 * zur Anzeige in Fehlernachrichten verwendet.
	 * @return Eine ganze Zahl.
	 * @throws ParseException wenn keine Zahl.
	 * @see #getNum(int, int, String)
	 */
	protected Integer getNum(int index, String parameterName) throws ParseException {
		try {
			int num = Integer.parseInt( getParam(index, parameterName) );
			return new Integer(num);
		} catch(NumberFormatException e) {
			throw new InvalidParamException(line, index, getParam(index, parameterName), parameterName + Messages.getString("HTSCommand.7")); //$NON-NLS-1$
		}
	}
	
	/**
	 * Wandelt einen Parameter in eine ganze Zahl um. Der Parameter darf nicht
	 * in Anfhrungszeichen sein.
	 * @param index Die Position in der Parameterliste, 0-basiert.
	 * @param minValue Der Mindestwert der Zahl.
	 * @param parameterName Der Name des Parameters, dieser wird nur
	 * zur Anzeige in Fehlernachrichten verwendet.
	 * @return Eine ganze Zahl.
	 * @throws ParseException wenn keine Zahl oder wenn der Wert kleiner als
	 * <code>minValue</code> ist.
	 */
	protected Integer getNum(int index, int minValue, String parameterName) throws ParseException 
	{
		try {
			int num = Integer.parseInt( getParam(index, parameterName) );
			if(num < minValue)
				throw new InvalidParamException(line, index, getParam(index, parameterName), parameterName + Messages.getString("HTSCommand.8") + minValue + "."); //$NON-NLS-1$ //$NON-NLS-2$
			
			return new Integer(num);
		} catch(NumberFormatException e) {
			throw new InvalidParamException(line, index, getParam(index, parameterName), parameterName + Messages.getString("HTSCommand.10") + minValue + "."); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	/**
	 * Wandelt einen Parameter in reelle Zahl um. Der Parameter darf nicht
	 * in Anfhrungszeichen sein. Es wird ein Punkt als Dezimalzeichen erwartet.
	 * @param index Die Position in der Parameterliste, 0-basiert.
	 * @param paramName Der Name des Parameters, dieser wird nur
	 * zur Anzeige in Fehlernachrichten verwendet.
	 * @return Eine reelle Zahl.
	 * @throws ParseException wenn keine reelle Zahl.
	 */
	protected Float getFloat(int index, String paramName) throws ParseException
	{
		try {
			// Wert als Float
			return new Float( Float.parseFloat(getParam(index, paramName)) );
		} catch(NumberFormatException e) {
			throw new InvalidParamException(line, index, getParam(index, paramName), paramName + Messages.getString("HTSCommand.12")); //$NON-NLS-1$
		}
	}
	
	/**
	 * berprft, ob an der angegebenen Position in der Parameter Liste
	 * das Schlsselwort SUBITEM steht.
	 * @param index Die Position in der Parameterliste, an der das Schlsselwort
	 * SUBITEM erwartet wird.
	 * @throws ParseException wenn das Schlsselwort bei <code>index</code>
	 * nicht gefunden wird.
	 */
	protected void assertSubitem(int index) throws ParseException 
	{
		if( index >= getParamSize() || !getParam(index, HTSSyntax.ACTION_SUBITEM).equals(HTSSyntax.ACTION_SUBITEM) )
			throw new InvalidParamException(line, index, getParam(index, HTSSyntax.ACTION_SUBITEM), HTSSyntax.ACTION_SUBITEM);
	}
	
	/**
	 * berprft, ob die Lnge der Parameterliste, dem angegebenen Wert entspricht.
	 * @param expectedParamSize Die erwartete Lnge der Parameterliste.
	 * @throws ParseException wenn die tatschliche und die erwartetet 
	 * Lnge der Paramterliste nicht bereinstimmen.
	 */
	protected void assertParamSize(int expectedParamSize) throws ParseException {
		if( getParamSize() != expectedParamSize )
			throw new ParseException(line, Messages.getString("HTSCommand.13")); //$NON-NLS-1$
	}
	
	/**
	 * Erstellt den Subelementpfad ab dem angegebenen Index.
	 * @param subItemIndex Die Position in der Parameterliste an der das Schlsselwort
	 * SUBITEM erwartet wird.
	 * @param pathValidator Gibt gltige Bedingungen fr den Pfad an.
	 * @return Einen Vector mit den Elementen des Pfades. 
	 * Natrliche Zahlen sind vom Typ Integer und Zeichenketten sind vom Typ String.
	 * @throws ParseException wenn an der Position <code>subItemIndex</code> ein
	 * anderes Wort als SUBITEM gefunden wurde oder wenn nach dem Wort
	 * SUBITEM keine weiteren Parameter gefunden wurden.
	 */
	protected Vector getSubItemPath(int subItemIndex, SubitemPathValidator pathValidator) throws ParseException 
	{
		assertSubitem(subItemIndex);
		// alle Parameter bis zum Ende den Pfad
		int pathLen = getParamSize() - (subItemIndex+1);
		if(pathLen == 0)
			throw new ParseException(line, Messages.getString("HTSCommand.14")); //$NON-NLS-1$
		
		return pathValidator.getSubitemPath(subItemIndex+1);
	}
	
	public String toString() {
		return line.toString();
	}

	/**
	 * berprft, ob das Kommando Fehlerfrei ist.
	 * @param p Das Projekt welches das URFFile enthlt.
	 * @param script Das besitzende <code>HTSScript</code>.
	 * @throws IntegrityException falls fehlerhaft.
	 * @see URFFile
	 * @see #setError(String)
	 */
	public void checkIntegrity(Project p, HTSScript script) throws IntegrityException {
		if(error != null)
			throw new IntegrityException(this, error);
	}
	
	/**
	 * Setzt das Kommando auf fehlerhaft oder fehlerfrei.
	 * @param error Die Fehlernachricht, oder <code>null</null> wenn fehlerfrei.
	 */
	public void setError(String error) {
		this.error = error;
	}
	
	/**
	 * Setzt die zwischengespeicherte Werte eines Kommandos wieder auf den Ausgangszustand zurck
	 * und beendet das Kommando. Defaultimplementation ist leer.
	 */
	public void reset() {
		
	}
	
	/* (non-Javadoc)
	 * @see atosj.editors.AtosJCloneable#clone()
	 */
	public abstract Object clone();
}
