/*
 * Created on 23.10.2005
 */
package atosj.model.hts.command;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.HTSPlayer;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS COMPARE Kommandos.
 * @author Nicos Tegos
 */
public class HTSCompare extends HTSCommand
{
	private String firstVarName; // der erste Vergleichoperand ist immer eine Variable (Istwert)
	private Object secondParam; // der zweite Operand kann eine Variable oder ein Wert sein (Sollwert)
	
	private String varType; // Datentyp der Variablen STR, NUM, BOOL
	private String modeVar; // Typ des 2. Operanden VAR oder VAL
	private String modeComp; // Vergleichmodi siehe HTS Spec z.B. grer als ist "GRT"
	private Float tolerance; // gibt einen Toleranzbereich an der Vergleichswert muss im Intervall [Sollwert-Toleranz, Sollwert+Toleranz] liegen
	
	/**
	 * Erstellt ein neues COMPARE-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSCompare(CSVLine line) throws ParseException
	{
		super(line);
		
		/*vergleich_var_textuell = COMPARE, $Variable1$, STR, VAR, $Variable2$
		vergleich_var_numerisch = COMPARE, $Variable1$, NUM, VAR, $Variable2$
		[,vergleichsmodus]
		vergleich_wert_textuell = COMPARE, $Variable1$, STR, VAL, "Wert"
		vergleich_wert_numerisch = COMPARE, $Variable1$, NUM, VAL, "Wert"
		[,vergleichsmodus]*/
		
		firstVarName = getParamUnquoted(0, Messages.getString("HTSCompare.0")); //$NON-NLS-1$
		varType = getParam(1, Messages.getString("HTSCompare.1")); //$NON-NLS-1$
		if( !(varType.equals(HTSSyntax.COMPARE_NUM) 
			|| varType.equals(HTSSyntax.COMPARE_STR)
			|| varType.equals(HTSSyntax.COMPARE_BOOL))
		)
		{
			throw new InvalidParamException(line, 1, varType,
				new String[]  {HTSSyntax.COMPARE_NUM, HTSSyntax.COMPARE_STR, HTSSyntax.COMPARE_BOOL} );
		}
		
		modeVar = getParam(2, Messages.getString("HTSCompare.2")); //$NON-NLS-1$
		if( modeVar.equals(HTSSyntax.COMPARE_VAR) )
		{
			if( varType.equals(HTSSyntax.COMPARE_STR)
				|| varType.equals(HTSSyntax.COMPARE_BOOL) )
			{
				secondParam = getParamUnquoted(3, Messages.getString("HTSCompare.3")); // Variablenname als String //$NON-NLS-1$
				assertParamSize(4);
			}
			else
			if( varType.equals(HTSSyntax.COMPARE_NUM) )
			{
				secondParam = getParamUnquoted(3, Messages.getString("HTSCompare.4")); // Variablenname als String //$NON-NLS-1$
				
				if( getParamSize() > 4 )
				{
					modeComp = getParam(4, Messages.getString("HTSCompare.5"));					 //$NON-NLS-1$
					if( !( modeComp.equals(HTSSyntax.COMPARE_EQ)
							|| modeComp.equals(HTSSyntax.COMPARE_GRT) 
							|| modeComp.equals(HTSSyntax.COMPARE_GEQ)
							|| modeComp.equals(HTSSyntax.COMPARE_LSS)
							|| modeComp.equals(HTSSyntax.COMPARE_LEQ)
							|| modeComp.equals(HTSSyntax.COMPARE_NEQ)
							|| modeComp.equals(HTSSyntax.COMPARE_TOL)) )
					{
						throw new InvalidParamException(line, 4, modeComp,
								Messages.getString("HTSCompare.6")  //$NON-NLS-1$
								+ HTSSyntax.COMPARE_EQ + ", " + HTSSyntax.COMPARE_GRT + ", "  //$NON-NLS-1$ //$NON-NLS-2$
								+ HTSSyntax.COMPARE_GEQ + ", " + HTSSyntax.COMPARE_LSS + ", " //$NON-NLS-1$ //$NON-NLS-2$
								+ HTSSyntax.COMPARE_LEQ + ", " + HTSSyntax.COMPARE_NEQ  //$NON-NLS-1$
								+ Messages.getString("HTSCompare.12") + HTSSyntax.COMPARE_TOL); //$NON-NLS-1$
					}
					
					if( modeComp.equals(HTSSyntax.COMPARE_TOL) )
					{
						tolerance =getFloat(5, Messages.getString("HTSCompare.13")); //$NON-NLS-1$
						assertParamSize(6);
					}
					else
						assertParamSize(5);
				}
			}
			else
			{
				throw new InvalidParamException(line, 1, varType,
					new String[] {HTSSyntax.COMPARE_STR, HTSSyntax.COMPARE_NUM, HTSSyntax.COMPARE_BOOL} );
			}
		}
		else
		if( modeVar.equals(HTSSyntax.COMPARE_VAL) )
		{
			if( varType.equals(HTSSyntax.COMPARE_NUM) )
			{
				secondParam = getFloat(3, Messages.getString("HTSCompare.14")); //$NON-NLS-1$
				
				if( getParamSize() > 4 )
				{
					modeComp = getParam(4, Messages.getString("HTSCompare.15"));					 //$NON-NLS-1$
					if( !( modeComp.equals(HTSSyntax.COMPARE_EQ)
						|| modeComp.equals(HTSSyntax.COMPARE_GRT) 
						|| modeComp.equals(HTSSyntax.COMPARE_GEQ)
						|| modeComp.equals(HTSSyntax.COMPARE_LSS)
						|| modeComp.equals(HTSSyntax.COMPARE_LEQ)
						|| modeComp.equals(HTSSyntax.COMPARE_NEQ)
						|| modeComp.equals(HTSSyntax.COMPARE_TOL)) )
					{
						throw new InvalidParamException(line, 4, modeComp,
								Messages.getString("HTSCompare.16")  //$NON-NLS-1$
								+ HTSSyntax.COMPARE_EQ + ", " + HTSSyntax.COMPARE_GRT + ", "  //$NON-NLS-1$ //$NON-NLS-2$
								+ HTSSyntax.COMPARE_GEQ + ", " + HTSSyntax.COMPARE_LSS + ", " //$NON-NLS-1$ //$NON-NLS-2$
								+ HTSSyntax.COMPARE_LEQ + ", " + HTSSyntax.COMPARE_NEQ  //$NON-NLS-1$
								+ Messages.getString("HTSCompare.22") + HTSSyntax.COMPARE_TOL); //$NON-NLS-1$
					}
					
					if( modeComp.equals(HTSSyntax.COMPARE_TOL) )
					{
						tolerance =getFloat(5, Messages.getString("HTSCompare.23")); //$NON-NLS-1$
						assertParamSize(6);
					}
					else
						assertParamSize(5);
				}
			}
			else
			if( varType.equals(HTSSyntax.COMPARE_STR) )
			{
				// Wert als String
				secondParam = getParamUnquoted(3, Messages.getString("HTSCompare.24")); //$NON-NLS-1$
				assertParamSize(4);
			}
			else
			if( varType.equals(HTSSyntax.COMPARE_BOOL) )
			{
				// Wert als Boolean
				String str = getParam(3, Messages.getString("HTSCompare.25")); //$NON-NLS-1$
				if( str.equals(HTSSyntax.COMPARE_TRUE) )
				{
					secondParam = Boolean.TRUE;
				}
				else
				if( str.equals(HTSSyntax.COMPARE_FALSE) )
				{
					secondParam = Boolean.FALSE;
				}
				else
				{
					throw new InvalidParamException(line, 3, str,
							new String[] {HTSSyntax.COMPARE_TRUE , HTSSyntax.COMPARE_FALSE});
				}
				
				assertParamSize(4);
			}
			else
			{
				throw new InvalidParamException(line, 1, varType,
					new String[] {HTSSyntax.COMPARE_STR, HTSSyntax.COMPARE_NUM, HTSSyntax.COMPARE_BOOL} );
			}
		}
		else
		{
			throw new InvalidParamException(line, 2, modeVar,
					new String[] {HTSSyntax.COMPARE_VAR, HTSSyntax.COMPARE_VAL} );
		}
	}
		
	/** 
	 * Fhrt den Vergleich durch.
	 */
	public void execute(IHTSPlayer player) throws ExecutionException
	{
		HTSPlayer localPlayer = (HTSPlayer) player;
		
		Object first = localPlayer.getVar(firstVarName);
		if(first == null)
			throw new ExecutionException(this, Messages.getString("HTSCompare.26") + firstVarName); //$NON-NLS-1$
		
		Object second;
		if( modeVar.equals(HTSSyntax.COMPARE_VAR) )
		{
			second = localPlayer.getVar( (String)secondParam );
			if(second == null)
				throw new ExecutionException(this, Messages.getString("HTSCompare.27") + secondParam); //$NON-NLS-1$
		}
		else
		{
			second = secondParam; // ist schon ein Wert
		}
		
		if( !first.getClass().equals(second.getClass()) )
		{
			String firstType = HTSSyntax.COMPARE_STR;
			if( first instanceof Float)
				firstType = HTSSyntax.COMPARE_NUM;
				else
					if( first instanceof Boolean)
						firstType = HTSSyntax.COMPARE_BOOL;
			
			String secondType = HTSSyntax.COMPARE_STR;
			if( second instanceof Float)
				secondType = HTSSyntax.COMPARE_NUM;
				else
					if( second instanceof Boolean)
						secondType = HTSSyntax.COMPARE_BOOL;
			
			throw new ExecutionException(this, Messages.getString("HTSCompare.28") //$NON-NLS-1$
					+ Messages.getString("HTSCompare.29") + firstType  //$NON-NLS-1$
					+ Messages.getString("HTSCompare.30") + secondType); //$NON-NLS-1$
		}
		
		if( varType.equals(HTSSyntax.COMPARE_BOOL) )
		{
			if( !first.equals(second) )
				throw new ExecutionException(this, Messages.getString("HTSCompare.31") //$NON-NLS-1$
						+ Messages.getString("HTSCompare.32") + first +  Messages.getString("HTSCompare.33") + second) ; //$NON-NLS-1$ //$NON-NLS-2$
		}
		else
		if( varType.equals(HTSSyntax.COMPARE_STR) )
		{
			if( !first.equals(second) )
				throw new ExecutionException(this, Messages.getString("HTSCompare.34") //$NON-NLS-1$
						+ Messages.getString("HTSCompare.35") + first +  Messages.getString("HTSCompare.36") + second) ; //$NON-NLS-1$ //$NON-NLS-2$
		}
		else // numerischer Vergleich in seiner ganzen Bandbreite
		{
			int result = ((Float)first).compareTo((Float)second);
			if( modeComp == null || modeComp.equals(HTSSyntax.COMPARE_EQ) ) // implizit auf gleichheit testen
			{
				if(result != 0)
					throw new ExecutionException(this, Messages.getString("HTSCompare.37") //$NON-NLS-1$
							+ Messages.getString("HTSCompare.38") + first +  Messages.getString("HTSCompare.39") + second) ; //$NON-NLS-1$ //$NON-NLS-2$
			}
			else
			if( modeComp.equals(HTSSyntax.COMPARE_GRT) )
			{
				
				if(result != 1)
					throw new ExecutionException(this, Messages.getString("HTSCompare.40") //$NON-NLS-1$
							+ Messages.getString("HTSCompare.41") + first +  Messages.getString("HTSCompare.42") + second ) ; //$NON-NLS-1$ //$NON-NLS-2$
			}
			else
			if( modeComp.equals(HTSSyntax.COMPARE_GEQ) )
			{
				if(result == -1)
					throw new ExecutionException(this, Messages.getString("HTSCompare.43")  //$NON-NLS-1$
							+ Messages.getString("HTSCompare.44") + first.toString() +  Messages.getString("HTSCompare.45") + second.toString() ) ; //$NON-NLS-1$ //$NON-NLS-2$
			}
			else
			if( modeComp.equals(HTSSyntax.COMPARE_LSS) )
			{
				if(result != -1)
					throw new ExecutionException(this, Messages.getString("HTSCompare.46") //$NON-NLS-1$
							+ Messages.getString("HTSCompare.47") + first.toString() +  Messages.getString("HTSCompare.48") + second.toString() ) ; //$NON-NLS-1$ //$NON-NLS-2$
			}
			else
			if( modeComp.equals(HTSSyntax.COMPARE_LEQ) )
			{
				if(result == 1)
					throw new ExecutionException(this, Messages.getString("HTSCompare.49") //$NON-NLS-1$
							+ Messages.getString("HTSCompare.50") + first.toString() +  Messages.getString("HTSCompare.51") + second.toString() ) ; //$NON-NLS-1$ //$NON-NLS-2$
			}
			else
			if( modeComp.equals(HTSSyntax.COMPARE_NEQ) )
			{
				if(result == 0)
					throw new ExecutionException(this, Messages.getString("HTSCompare.52") //$NON-NLS-1$
							+ Messages.getString("HTSCompare.53") + first.toString() +  Messages.getString("HTSCompare.54") + second.toString() ) ; //$NON-NLS-1$ //$NON-NLS-2$
			}
			else
			if( modeComp.equals(HTSSyntax.COMPARE_TOL) )
			{
				Float begin = new Float( ((Float)second).floatValue() - tolerance.floatValue() );
				Float end = new Float( ((Float)second).floatValue() + tolerance.floatValue() );
				Float isValue = (Float) first;
				
				if( isValue.compareTo(begin) == -1 || isValue.compareTo(end) == 1 )
					throw new ExecutionException(this, Messages.getString("HTSCompare.55") //$NON-NLS-1$
							+ Messages.getString("HTSCompare.56") + second.toString() //$NON-NLS-1$
							+ Messages.getString("HTSCompare.57") + tolerance.toString() //$NON-NLS-1$
							+ Messages.getString("HTSCompare.58") + isValue.toString() ) ; //$NON-NLS-1$
			}
		}
	}
	
	public Object clone() {
		try {
			return new HTSCompare( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
