package atosj.model.hts.command;

import java.util.Vector;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.app.Project;
import atosj.app.TestObject;
import atosj.component.ComponentId;
import atosj.component.ComponentResolver;
import atosj.component.CustomComponentResolver;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.IntegrityException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;
import atosj.model.script.HTSScript;
import atosj.util.CSVLine;
import atosj.util.CSVParser;
import atosj.util.WindowMonitor;

/**
 * Ein Kommando, welches auf die Components des Testobjekts zugreift.
 * Die ersten 3 Parameter beinhalten die ComponentId des Kommandos.
 * 
 * @author Nicos Tegos
 * @see atosj.component.ComponentId
 */
public abstract class HTSComponentCommand extends HTSCommand 
{
	 /** Das Kommando kann nur auf einem Unterelement eines zusammengesetzten Components ausgefhrt werden. */
	protected static final int SUBITEM_PATH_OBLIGED = 0;
	/** Das Kommando kann auf einem Unterelement oder auf dem zusammengesetzten Component selbst ausgefhrt werden. */
	protected static final int SUBITEM_PATH_VOLUNTARY = 1;
	/** Das Kommando kann nur auf dem Component selbst ausgefhrt werden. */
	protected static final int SUBITEM_PATH_FORBIDDEN = 2;
	
	private String dummyTitle; // name des fensters (in Anfhrungszeichen) in dem sich das control befindet oder Das Schlsselwort MAIN.
	private String typeName; // Typ des Controls z.B. TABLE
	private String controlName; // name des Controls
	
    private ComponentResolver resolver; // Kennt die gltigen typNamen
	
    /**
	 * Erstellt ein neues Component-Kommando. Ermittelt die ComponentId.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 * @param resolver Der Resolver fr die Components, dieser gibt gltige
	 * Custom-Components an.
	 */
	public HTSComponentCommand(CSVLine line, ComponentResolver resolver) throws ParseException
	{
		super(line);
		this.resolver = resolver;
		
		dummyTitle = getParam(0, Messages.getString("HTSComponentCommand.0")); //$NON-NLS-1$
						
		if( CSVParser.isQuoted(dummyTitle) )
		{
			dummyTitle = HTSParser.stripQuotes(dummyTitle);
		}
		else
		{
			if( !dummyTitle.equals(HTSSyntax.DEF_MAIN) ) 
				throw new InvalidParamException(line, 1, dummyTitle, HTSSyntax.DEF_MAIN + Messages.getString("HTSComponentCommand.1")); //$NON-NLS-1$
		}
		
		typeName = getParam(1, Messages.getString("HTSComponentCommand.2")); //$NON-NLS-1$
		
		if( !resolver.isSupportedType(typeName) )
			throw new InvalidParamException(line, 1, typeName,
					resolver.getTypeNames() );
		
		if( typeName.equals(HTSSyntax.TYPE_WINDOW) ) // bei Fenstern mssen titel und elementname gleich sein
		{
			if( !getParam(2, Messages.getString("HTSComponentCommand.3")).equals(getParam(0, Messages.getString("HTSComponentCommand.4"))) ) //$NON-NLS-1$ //$NON-NLS-2$
				throw new InvalidParamException(line, 2, getParam(2, Messages.getString("HTSComponentCommand.5")), getParam(0, Messages.getString("HTSComponentCommand.6"))); //$NON-NLS-1$ //$NON-NLS-2$
			
			controlName = dummyTitle;			
		}
		else
		{
			controlName = getParamUnquoted(2, Messages.getString("HTSComponentCommand.7")); //$NON-NLS-1$
		}
		
		if( (resolver instanceof CustomComponentResolver) 
				&& ((CustomComponentResolver)resolver).isCustomType(typeName) )
		{
			parseCustomCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_BUTTON) )
		{
			parseButtonCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_CHECKBOX) )
		{
			parseCheckboxCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_COMBOBOX) )
		{
			parseComboboxCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_COMPONENT) )
		{
			parseComponentCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_EDITBOX) )
		{
			parseEditboxCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_SPINNER) )
		{
			parseSpinnerCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_LABEL) )
		{
			parseLabelCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_LIST) )
		{
			parseListCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_MENU) )
		{
			parseMenuCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_RADIOBUTTON) )
		{
			parseRadiobuttonCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_TABFOLDER) )
		{
			parseTabfolderCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_TABLE) )
		{
			parseTableCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_TREE) )
		{
			parseTreeCommand();
		}
		else
		if( typeName.equals(HTSSyntax.TYPE_WINDOW) )
		{
			parseWindowCommand();
		}
		else
		{
			throw new InvalidParamException(line, 1, typeName,
					resolver.getTypeNames() );
		}
	}
	
	/** Parsen des Kommandos fr Custom-Components. */
	protected abstract void parseCustomCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ BUTTON. */
	protected abstract void parseButtonCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ CHECKBOX. */
	protected abstract void parseCheckboxCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ COMPONENT. */
	protected abstract void parseComponentCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ COMBOBOX. */
	protected abstract void parseComboboxCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ EDITBOX. */
	protected abstract void parseEditboxCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ SPINNER. */
	protected abstract void parseSpinnerCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ LABEL. */
	protected abstract void parseLabelCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ LIST. */
	protected abstract void parseListCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ MENU. */
	protected abstract void parseMenuCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ RADIOBUTTON. */
	protected abstract void parseRadiobuttonCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ TABFOLDER. */
	protected abstract void parseTabfolderCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ TABLE. */
	protected abstract void parseTableCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ TREE. */
	protected abstract void parseTreeCommand() throws ParseException;
	/** Parsen des Kommandos fr den Typ WINDOW. */
	protected abstract void parseWindowCommand() throws ParseException; 
	
	public ComponentResolver getResolver() {
		return resolver;
	}
	
	/**
	 * Liefert die ID des Components fr dieses Kommando.
	 * @param testee Das Testobjekt wird bentigt, um das evtl. enthaltene
	 * Schlsselwort MAIN durch den tatschlichen Fenstertitel zu ersetzten.
	 * @return Die Id des hier zu behandelnden Components.
	 * @see #getComponentId()
	 */
	protected ComponentId getComponentId(TestObject testee) 
	{
		String realTitle;
		if( dummyTitle.equals(HTSSyntax.DEF_MAIN) )
			realTitle = testee.getMainWindow();
		else
			realTitle = HTSParser.stripQuotes(dummyTitle);
		
		return new ComponentId( typeName.equals(HTSSyntax.TYPE_WINDOW) ? realTitle : controlName, typeName, realTitle );
	}
	
	/**
	 * Liefert die ID des hier zu behandelnden Components, 
	 * das Schlsselwort MAIN wird nicht aufgelst.
	 */
	public ComponentId getComponentId() {
		return new ComponentId(controlName, typeName, dummyTitle);
	}
	
	/**
	 * Erstellt ein neues Kommandos, welches <code>newId</code> statt der bisherigen ID verwendet.
	 * @param newId Die neue ID.
	 * @return Das neue Kommando, this bleibt unverndert.
	 * @throws ParseException Bei einem Fehler beim Erstellen des neuen Kommandos.
	 */
	public HTSComponentCommand replaceId(ComponentId newId) throws ParseException
	{
		Vector newParams = new Vector();
		newParams.add( getCommandKey() ); // kommando name
		if( newId.getWindowName().equals(HTSSyntax.DEF_MAIN) )
			newParams.add( newId.getWindowName() );
		else
			newParams.add( '"' + newId.getWindowName() + '"' );
		
		newParams.add( newId.getClassName() );
		
		if( newId.getName().equals(HTSSyntax.DEF_MAIN) )
			newParams.add( newId.getName() );
		else
			newParams.add( '"' + newId.getName() + '"' );
			
		for(int i = 3; i < getParamSize(); ++i)
			newParams.add( getParam(i, Messages.getString("HTSComponentCommand.8")) ); //$NON-NLS-1$
		
		return (HTSComponentCommand) HTSParser.getCommandFromLine( new CSVLine(newParams), AtosJApp.getInstance().getInput().getTestObject().getResolver() );
	}
	
	public MComponent getMComponent(ComponentId id) throws ComponentException {
		return WindowMonitor.getInstance().getMComponent(id);
	}
	
	/**
	 * berprft, ob das Kommando fehlerfrei ist.
	 * Zustzlich wird die referentielle Integritt mit der URF-Datei berprft, d.h.
	 * ob die enthaltene ComponentId in der URFDatei enthalten ist.
	 */
	public void checkIntegrity(Project p, HTSScript script) throws IntegrityException 
	{
		super.checkIntegrity(p, script);
		
		ComponentId id = getComponentId();
		if( !p.getURFFile().containsId(id) )
			throw new IntegrityException(this, id);
	}
}
