/*
 * Created on 24.10.2005
 */
package atosj.model.hts.command;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.HTSPlayer;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS COPY Kommandos.
 * @author Nicos Tegos
 */
public class HTSCopy extends HTSFileOperation
{
	private boolean force = false;
	private boolean ifSrcExists = false;
	
	private String srcDir;
	private String srcFileName;
	private String targetDir;
	private String targetFileName;
	
	/**
	 * Erstellt ein neues COPY-Kommandos.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSCopy(CSVLine line) throws ParseException
	{
		super(line);
		/*<QUELLVERZEICHNIS>, <QUELLDATEI>, <ZIELVERZEICHNIS>,
          <ZIELDATEI> [,FORCE] [,IFSRCEXISTS]*/
				
		// Quellpfad
		srcDir = getDir(0);
		srcFileName = getParamUnquoted(1, Messages.getString("HTSCopy.0")); //$NON-NLS-1$
		// Zielpfad
		targetDir = getDir(2);
		targetFileName = getParamUnquoted(3, Messages.getString("HTSCopy.1")); //$NON-NLS-1$
				
		if( getParamSize() == 5)
		{
			String param5 = getParam(4, Messages.getString("HTSCopy.2")); //$NON-NLS-1$
			if( param5.equals(HTSSyntax.COPY_FORCE) )
			{
				force = true;
			}
			else
			if( param5.equals(HTSSyntax.COPY_IFSRCEXISTS) )
			{
				ifSrcExists = true;
			}
			else
			{
				throw new InvalidParamException(line, 4, param5, 
						new String[] {HTSSyntax.COPY_FORCE, HTSSyntax.COPY_IFSRCEXISTS});
			}
		}
		else
		if(getParamSize() == 6)
		{
			String param5 = getParam(4, Messages.getString("HTSCopy.3")); //$NON-NLS-1$
			if( param5.equals(HTSSyntax.COPY_FORCE) )
			{
				force = true;
			}
			else
			{
				throw new InvalidParamException(line, 4, param5, 
						HTSSyntax.COPY_FORCE);
			}
			
			String param6 = getParam(5, Messages.getString("HTSCopy.4")); //$NON-NLS-1$
			if( param6.equals(HTSSyntax.COPY_IFSRCEXISTS) )
			{
				ifSrcExists = true;
			}
			else
			{
				throw new InvalidParamException( line, 5, param6,
						HTSSyntax.COPY_IFSRCEXISTS);
			}
		}
		else
			assertParamSize(4);
	}
	
	/**
	 * Fhrt das Kopieren einer Datei aus.
	 */
	public void execute(IHTSPlayer player) throws ExecutionException
	{
		HTSPlayer localPlayer = (HTSPlayer) player;
		
		// Quellverzeichnis
		String srcPath = makePath(srcDir, srcFileName, localPlayer.getTestObject() );
		
		// Zielverzeichnis
		String targetPath = makePath(targetDir, targetFileName,	localPlayer.getTestObject() );
		
		copy(this, srcPath, targetPath, ifSrcExists, force);
	}
	
	/**
	 * Kopiert eine Datei.
	 * @param cmd Das HTS Kommando, welches diese Funktion benutzt.
	 * @param srcPath Der vollstndige Pfad der Quelldatei.
	 * @param targetPath Der vollstndige Zielpfad.
	 * @param ifSrcExists wenn <code>true</code> dann wird kein Fehler gemeldet, 
	 * falls die Quelldatei nicht existiert sonst Fehler.
	 * @param force wenn <code>true</code> dann werden auch Dateien die versteckt oder
	 * schreibgeschtzt sind berschrieben, wenn <code>false</code> werden in 
	 */
	public static void copy(HTSCommand cmd, String srcPath, String targetPath, boolean ifSrcExists, boolean force) throws ExecutionException
	{
		File src = new File(srcPath);
		File target = new File(targetPath);
		
		if( !ifSrcExists  && !src.exists() )
			throw new ExecutionException(cmd, Messages.getString("HTSCopy.5") + srcPath + Messages.getString("HTSCopy.6")); //$NON-NLS-1$ //$NON-NLS-2$

		if( ifSrcExists  && !src.exists() )
			return; // nicht so schlimm wir drfen aufhren
		
		if( target.exists() )
		{
			if( !force && target.isHidden() )
				throw new ExecutionException(cmd, Messages.getString("HTSCopy.7") + srcPath + Messages.getString("HTSCopy.8")); //$NON-NLS-1$ //$NON-NLS-2$
			
			if( !force && !target.canWrite() )
				throw new ExecutionException(cmd, Messages.getString("HTSCopy.9") + targetPath + Messages.getString("HTSCopy.10")); //$NON-NLS-1$ //$NON-NLS-2$
			
			target.delete();
		}
		
		// perform copy
		try
		{
			FileInputStream fis = new FileInputStream(src);
			FileOutputStream fos = new FileOutputStream(target);
			
			int bytesRead = 0;
			byte[] buffer = new byte[1024];
			
			bytesRead = fis.read(buffer);
			while(bytesRead != -1)
			{
				fos.write(buffer, 0, bytesRead);
				bytesRead = fis.read(buffer);
			}
			
			fis.close();
			fos.close();
		}
		catch(IOException e)
		{
			throw new ExecutionException(cmd, Messages.getString("HTSCopy.11") + srcPath + "\"" //$NON-NLS-1$ //$NON-NLS-2$
					+ Messages.getString("HTSCopy.13") + targetPath + Messages.getString("HTSCopy.14")); //$NON-NLS-1$ //$NON-NLS-2$
				
		}
	}
	
	public Object clone() {
		try {
			return new HTSCopy( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
