package atosj.model.hts.command;

import java.util.Vector;

import atosj.component.ComponentResolver;
import atosj.exception.ExecutionException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSParser;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Deaktiviert ein HTS Kommando (DISABLE).
 * 
 * @author Nicos Tegos
 */
public class HTSDisable extends HTSCommand 
{
	private HTSCommand origCmd;
	
	/**
	 * Konstruktor fr das Kommando DISABLE, wenn es aus
	 * der Skriptdatei gelesen wurde.
	 * @param line Beginnt mit DISABLE und dann folgt das
	 * deaktivierte Kommando.
	 */
	public HTSDisable(CSVLine line, ComponentResolver resolver) throws ParseException 
	{
		super(line);
		Vector origElems = new Vector( line.getNumOfElements() );
		// das element an index 0 ist das schlsselwort DISABLE, deshalb starten wir bei 1
		for(int i = 1; i < line.getNumOfElements(); ++i)
			origElems.add( line.getElement(i) );
		
		CSVLine origLine =  new CSVLine(origElems);
		try {
			origCmd = HTSParser.getCommandFromLine(origLine, resolver);
		}
		catch(ParseException e) {
			origCmd = new InvalidCommand(e);
		}
	}
	
	/**
	 * Erstellt ein deaktiviertes Kommando aus eine aktiven Kommando.
	 * @param origCmd Darf KEIN <code>InvalidCommand</code> sein. Sinnlos ist es
	 * die Kommandos HTSComment und HTSDisable zu deaktivieren.
	 * @throws ParseException 
	 * @see InvalidCommand
	 */
	public HTSDisable(HTSCommand origCmd) throws ParseException
	{
		super( createCSVLine(origCmd) );
		this.origCmd = origCmd;
	}
	
	/**
	 * Erstellt die kommaseparierte Liste fr das DISABLE Kommando. Es
	 * wird das Schlsselwort vor <code>origCmd</code> angehangen.
	 * @param origCmd Das zu deaktivierende Kommando.
	 * @return Die kommaseparierte Liste des deaktivierten Kommandos.
	 */
	private static CSVLine createCSVLine(HTSCommand origCmd) throws ParseException
	{
		CSVLine origLine = origCmd.line;
		Vector elements = new Vector( origLine.getNumOfElements()+1 );
		// Das Schlsselwort DISABLE vorne anhngen, dann folgen die Elemente des
		// originalkommandos
		elements.add(HTSSyntax.CMD_DIS);
		for (int i = 0; i < origLine.getNumOfElements(); i++) {
			elements.add( origLine.getElement(i) );
		}
		
		return new CSVLine(elements);
	}
	
	/**
	 * Liefert das deaktivierte Kommando.
	 */
	public HTSCommand getOrigCommand() {
		return origCmd;
	}

	/**
	 * Deaktivierte Kommandos knnen nicht ausgefhrt werden.
	 */
	public void execute(IHTSPlayer player) throws ExecutionException
	{
		// this command is ignored
	}
	
	public Object clone() {
		try {
			return new HTSDisable( (HTSCommand) origCmd.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
