/*
 * Created on 08.11.2005
 */
package atosj.model.hts.command;

import java.io.File;
import java.io.IOException;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.HTSPlayer;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;
import atosj.util.Snooze;

/**
 * Klasse zum Parsen und Ausfhren eines HTS LAUNCH Kommandos.
 * @author Nicos Tegos
 */
public class HTSLaunch extends HTSFileOperation
{
	private static final int WAIT_NOWAIT = 0; // nicht auf Ausfhrungsende warten
	private static final int WAIT_FOREVER = 1; // auf Ausfhrungsende warten
	private static final int WAIT_TIME = 2; // die in time angegebene Zeitspanne in ms warten
	
	private int waitState = WAIT_NOWAIT; // einer der drei WAIT_* Modi
	private int time = 0; // warte Zeit auf Terminierung in ms
	
	private String progName;
	private String arguments;
	private int returnCode;
	private String logfileName;
	
	private String dir;
	
	/**
	 * Erstellt ein neues LAUNCH-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSLaunch(CSVLine line) throws ParseException 
	{
		super(line);
		// <VERZEICHNIS>, <PROGRAMMNAME>, <PARAMETER>, FOREVER, <RETURNCODE>, [,<LOGFILE>]
		
		dir = getDir(0);
		
		progName = getParamUnquoted(1, Messages.getString("HTSLaunch.0")); //$NON-NLS-1$
		arguments = getParamUnquoted(2, Messages.getString("HTSLaunch.1")); //$NON-NLS-1$
		
		String waitMode = getParam(3, Messages.getString("HTSLaunch.2")); //$NON-NLS-1$
		if( waitMode.equals(HTSSyntax.LAUNCH_FOREVER) )
		{
			waitState = WAIT_FOREVER;
			returnCode = getNum(4, Messages.getString("HTSLaunch.3")).intValue(); //$NON-NLS-1$
						
			if( getParamSize() == 6 )
				logfileName = getParamUnquoted(5, Messages.getString("HTSLaunch.4")); //$NON-NLS-1$
			else
				assertParamSize(5);
		}
		else
		if( waitMode.equals(HTSSyntax.LAUNCH_TIME) )
		{
			waitState = WAIT_TIME;
			time = getNum(4, 1, Messages.getString("HTSLaunch.5")).intValue(); //$NON-NLS-1$
			returnCode = getNum(5, Messages.getString("HTSLaunch.6")).intValue(); //$NON-NLS-1$
						
			if( getParamSize() == 7 )
				logfileName = getParamUnquoted(6, Messages.getString("HTSLaunch.7")); //$NON-NLS-1$
			else
				assertParamSize(6);
		}
		else
		if( waitMode.equals(HTSSyntax.LAUNCH_NOWAIT) )
		{
			waitState = WAIT_NOWAIT;
			
			if( getParamSize() == 5 )
				logfileName = getParamUnquoted(4, Messages.getString("HTSLaunch.8")); //$NON-NLS-1$
			else
				assertParamSize(4);
		}
		else
		{
			throw new InvalidParamException(line, 3, waitMode,
					new String[] {HTSSyntax.LAUNCH_FOREVER, HTSSyntax.LAUNCH_NOWAIT, HTSSyntax.LAUNCH_TIME});
		}
	}

	/** Startet eine andere externe Applikation. */
	public void execute(IHTSPlayer player) throws ExecutionException
	{
		HTSPlayer localPlayer = (HTSPlayer) player;
		
		String progPath = makePath( dir, progName, localPlayer.getTestObject() ); 
		File f = new File(progPath);
		String progDir =  f.getParent();
		if(progDir == null)
			progDir = new File(".").getAbsolutePath(); //$NON-NLS-1$
		
		Process process = null;
		try {
			// FIXME: ACHTUNG wird ATOSj aus Eclipse heraus gestartet schlgt die Funktion Runtime.exec immer fehl (IOException error=2) 
			process = Runtime.getRuntime().exec(progPath + " " + arguments, null, new File(progDir) ); //$NON-NLS-1$
		} 
		catch (IOException e) {
			throw new ExecutionException(this, Messages.getString("HTSLaunch.11") + progPath); //$NON-NLS-1$
		}
			
		switch(waitState)
		{
			case WAIT_FOREVER:
			{ 
				try
				{
					process.waitFor();
				}
				catch(InterruptedException e)
				{
					throw new ExecutionException(this, Messages.getString("HTSLaunch.12") + progPath + Messages.getString("HTSLaunch.13")); //$NON-NLS-1$ //$NON-NLS-2$
				}
				
				int ret = process.exitValue();
				copyLog(progDir, localPlayer.getTestObject());
				if(ret != returnCode)
				{
					throw new ExecutionException(this, Messages.getString("HTSLaunch.14") + progPath + Messages.getString("HTSLaunch.15")  //$NON-NLS-1$ //$NON-NLS-2$
							+ Messages.getString("HTSLaunch.16") + returnCode //$NON-NLS-1$
							+ Messages.getString("HTSLaunch.17") + ret); //$NON-NLS-1$
				}
			}
			break;
			
			case WAIT_TIME:
			{
				new Snooze(time);
				try
				{
					int ret = process.exitValue();
					copyLog(progDir, localPlayer.getTestObject());
					if(ret != returnCode)
					{
						throw new ExecutionException(this, Messages.getString("HTSLaunch.18") + progPath + Messages.getString("HTSLaunch.19")  //$NON-NLS-1$ //$NON-NLS-2$
								+ Messages.getString("HTSLaunch.20") + returnCode //$NON-NLS-1$
								+ Messages.getString("HTSLaunch.21") + ret); //$NON-NLS-1$
					}
				}
				catch(IllegalThreadStateException e)
				{
					throw new ExecutionException(this, Messages.getString("HTSLaunch.22") + progPath + Messages.getString("HTSLaunch.23")); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			break;
		}
	}
	
	/**
	 * Kopiert eine evtl. vorhandene Log-Datei aus dem Ausfhrungsverzeichnis
	 * der gestarteten Applikation in das LOG Verzeichnis des ATOSj-Projekts.
	 * @param progDir vollstndiger Pfad des Ausfhrungsverzeichnisses
	 * der gestarteten Applikation.
	 * @param testee Testobjekt zur Auflsung des LOG-Verzeichnispfades.
	 */
	private void copyLog(String progDir, TestObject testee) throws ExecutionException
	{
		if(logfileName == null)
			return;
		
		HTSCopy.copy(this, 
				progDir + File.separatorChar + logfileName, 
				makePath(HTSSyntax.FILEOP_LOG, logfileName, testee), 
				true, 
				true
		);
	}

	public Object clone() {
		try {
			return new HTSLaunch( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
