package atosj.model.hts.command;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS LOOP Kommandos.
 * @author Nicos Tegos
 */
public class HTSLoop extends HTSCollectionCommand
{
	private int repeats; // Anzahl der geforderten Schleifendurchlufe
	private int currentRepeats = 0; // Anzahl der bisher durchgefhrten Schleifendurchlufe

	/**
	 * Erstellt ein neues LOOP-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSLoop(CSVLine line) throws ParseException
	{
		super(line);
		
		repeats = getNum(0, 1, Messages.getString("HTSLoop.0")).intValue(); //$NON-NLS-1$
		assertParamSize(1);
	}
	
	/**
	 * Gibt an, ob die maximale Anzahl an Schleifendurchlufen erreicht wurde.
	 * @return <code>true</code>, wenn die maximale Anzahl erreicht wurde.
	 */
	public boolean isFinished() {
		return currentRepeats == repeats;
	}
	
	/**
	 * Setzt die Anzahl der bisherigen Schleifendurchlufe wieder auf 0.
	 * Damit kann ein neuer Durchlauf gestartet werden.
	 */
	public void reset() {
		currentRepeats = 0;
	}
	
	/**
	 * Aktualisiert die Anzahl der Schleifendurchlufe, d.h. erhht den Zhler.
	 * Fr die Ausfhrung der im LOOP-Block enthaltenen Kommandos ist der HTSPlayer
	 * zustndig.
	 * @see #isFinished()
	 */
	public void execute(IHTSPlayer player) throws ExecutionException {
		++currentRepeats;
	}
	
	/**
	 * Setzt das syntaktischen Ende zu diesem Kommando, d.h. ENDLOOP.
	 * @param end wenn <code>null</code> wird, das
	 * Kommando auf fehlerhaft gesetzt.
	 * @see HTSCommand#setError(String)
	 */
	public void setEnd(HTSEndLoop end) {
		super.setEnd(end);
		if(end == null)
			setError(HTSSyntax.CMD_LOOP + Messages.getString("HTSLoop.1") + HTSSyntax.CMD_ENDLOOP + "."); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public Object clone() {
		try {
			return new HTSLoop( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}