/*
 * Created on 23.10.2005
 */
package atosj.model.hts.command;

import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.Vector;

import atosj.Messages;
import atosj.component.ComponentResolver;
import atosj.component.IButton;
import atosj.component.ICheckbox;
import atosj.component.ICombobox;
import atosj.component.IComponent;
import atosj.component.ILabel;
import atosj.component.IList;
import atosj.component.IMenu;
import atosj.component.IRadio;
import atosj.component.ITabFolder;
import atosj.component.ITable;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS READ Kommandos.
 * @author Nicos Tegos
 */
public class HTSRead extends HTSComponentCommand
{
	private class VisibleExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			return new Boolean(component.isVisible());
		}
	};
	
	private class EnabledExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			return new Boolean(component.isEnabled());
		}
	}
	
	private class FocusExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			return new Boolean(component.hasFocus());
		}
	};
	
	private class PropertyExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			((IComponent)component).getProperty( (String) params.get(0) );
			
			return null;
		}
	}
	
	private String state; // Zustand des Controls der ausgelesen werden soll
	private String variableName; // variable in die das Ergebnis geschrieben werden soll
	
	private Executor executor; // Der Ausfhrer des Kommandos

	/**
	 * Erstellt ein neues READ-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSRead(CSVLine line, ComponentResolver resolver) throws ParseException
	{
		super(line, resolver);
	}
	
	private void parseCommand(String[] validStates, int[] possibleSubitems, SubitemPathValidator pathValidator, Executor[] readExecutors) throws ParseException
	{
		SubitemPathValidator[] pathValidators = new SubitemPathValidator[validStates.length];
		Arrays.fill(pathValidators, pathValidator);
		
		parseCommand(validStates, possibleSubitems, pathValidators, readExecutors);
	}
	
	private void parseCommand(String[] validStates, int[] possibleSubitems, SubitemPathValidator pathValidators[], Executor[] readExecutors) throws ParseException
	{
		state = getParam(3, Messages.getString("HTSRead.0")); //$NON-NLS-1$
		
		Vector params = null;
		Vector subitemPath = null;
		
		// berprfen ob der aktionstyp in der menge der gltigen aktionstypen enthalten ist.
		int validIndex = -1;
		for(int i = 0; i < validStates.length; ++i)
		{
			if( validStates[i].equals(state) )
			{
				validIndex = i;
				break;
			}
		}
		
		if(validIndex == -1) // keinen der gltigen aktionstypen gefunden
		{
			throw new InvalidParamException(line, 3, state,
					validStates);
		}
		
		variableName = getParamUnquoted(4, Messages.getString("HTSRead.1")); //$NON-NLS-1$
		
		if( state.equals(HTSSyntax.READ_PROPERTY) )
		{
			params = new Vector(1);
			params.add( getParam(5, Messages.getString("HTSRead.2")) ); //$NON-NLS-1$
			
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				assertParamSize(6);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				subitemPath = getSubItemPath(6, pathValidators[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				if( getParamSize() > 6 )
					subitemPath = getSubItemPath(6, pathValidators[validIndex]);
			}
		}
		else if( state.equals(HTSSyntax.READ_ENABLESTATE)
				|| state.equals(HTSSyntax.READ_FOCUSSTATE)
				|| state.equals(HTSSyntax.READ_VISIBLESTATE)
				|| state.equals(HTSSyntax.READ_CHECKSTATE)
				|| state.equals(HTSSyntax.READ_TEXT)
				|| state.equals(HTSSyntax.READ_NUM)
				|| state.equals(HTSSyntax.READ_ITEMCOUNT) )
		{
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				assertParamSize(5);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				subitemPath = getSubItemPath(5, pathValidators[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				if( getParamSize() > 5 )
					subitemPath = getSubItemPath(5, pathValidators[validIndex]);
			}
		}
		
				
		executor = readExecutors[validIndex];
		executor.setParams(params);
		executor.setSubitemPath(subitemPath);
	}
	
	protected void parseCustomCommand() throws ParseException 
	{
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_PROPERTY}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { new EnabledExecutor(), new FocusExecutor(), new VisibleExecutor(), new PropertyExecutor() }
			);
	}
	
	protected void parseComponentCommand() throws ParseException
	{
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { new EnabledExecutor(), new FocusExecutor(), new VisibleExecutor()}
			);
	}
	
	protected void parseButtonCommand() throws ParseException 
	{
		parseLabelCommand();
	}
	
	protected void parseCheckboxCommand() throws ParseException 
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return ((IButton)component).getText();
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Float( ((IButton)component).getNum() );
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Boolean(((ICheckbox)component).isChecked());
			}
		};
		
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_CHECKSTATE}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { new EnabledExecutor(), new FocusExecutor(), new VisibleExecutor(), numEx, textEx, checkEx}
			);
	}
	
	protected void parseComboboxCommand() throws ParseException 
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return ((ICombobox)component).getText();
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Float( ((ICombobox)component).getNum() );
			}
		};
		Executor itemcountEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Float(((ICombobox)component).getItemCount());
			}
		};
		
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_ITEMCOUNT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { new EnabledExecutor(), new FocusExecutor(), new VisibleExecutor(), numEx, textEx, itemcountEx }
			);
	}
	
	protected void parseEditboxCommand() throws ParseException
	{
		parseLabelCommand();
	}
	
	protected void parseSpinnerCommand() throws ParseException
	{
		parseLabelCommand();
	}
	
	protected void parseLabelCommand() throws ParseException 
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return ((ILabel)component).getText();
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Float( ((ILabel)component).getNum() );
			}
		};
		
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { new EnabledExecutor(), new FocusExecutor(), new VisibleExecutor(), numEx, textEx }
			);
	}
	
	protected void parseListCommand() throws ParseException 
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return ((IList)component).getText( ((Integer)subitemPath.get(0)).intValue() );
			}
		};
		Executor itemcountEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Float(((IList)component).getItemCount());
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				path.add( getNum(paramIndex, 0, Messages.getString("HTSRead.3")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_TEXT, HTSSyntax.READ_ITEMCOUNT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_FORBIDDEN},
				pathValidator,
				new Executor[] { new EnabledExecutor(), new FocusExecutor(), new VisibleExecutor(), textEx, itemcountEx }
			);
	}
	
	protected void parseMenuCommand() throws ParseException
	{
		Executor enabledEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				if(subitemPath == null)
					return new Boolean( component.isEnabled() );
				else
					return new Boolean( ((IMenu)component).isEnabled( (String[]) subitemPath.toArray(new String[subitemPath.size()]) ) ); 
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Boolean( ((IMenu)component).isChecked( (String[]) subitemPath.toArray(new String[subitemPath.size()]) ) ); 
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				for(int i = paramIndex; i < getParamSize(); ++i)
					path.add( getParamUnquoted(i, Messages.getString("HTSRead.4")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_CHECKSTATE}, 
				new int[] {SUBITEM_PATH_VOLUNTARY, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED},
				pathValidator,
				new Executor[] { enabledEx, new FocusExecutor(), new VisibleExecutor(), checkEx }
			);
	}
	
	protected void parseRadiobuttonCommand() throws ParseException 
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return ((IButton)component).getText();
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Float( ((IButton)component).getNum() );
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Boolean(((IRadio)component).isChecked());
			}
		};
		
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_CHECKSTATE}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { new EnabledExecutor(), new FocusExecutor(), new VisibleExecutor(), numEx, textEx, checkEx}
			);
	}
	
	protected void parseTabfolderCommand() throws ParseException
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				if( subitemPath == null )
					return ((ITabFolder)component).getText();
				else
					return ((ITabFolder)component).getText( ((Integer)subitemPath.get(0)).intValue() ); 
			}
		};
		Executor itemcountEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Float(((ITabFolder)component).getItemCount());
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				path.add( getNum(paramIndex, 0, Messages.getString("HTSRead.5")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_TEXT, HTSSyntax.READ_ITEMCOUNT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_VOLUNTARY, SUBITEM_PATH_FORBIDDEN},
				pathValidator,
				new Executor[] { new EnabledExecutor(), new FocusExecutor(), new VisibleExecutor(), textEx, itemcountEx }
			);
	}
	
	protected void parseTableCommand() throws ParseException 
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return ((ITable)component).getTableCell( ((Integer)subitemPath.get(0)).intValue(),
						((Integer)subitemPath.get(1)).intValue() ).getText();
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
					float val = ((ITable)component).getTableCell( ((Integer)subitemPath.get(0)).intValue(),
							((Integer)subitemPath.get(1)).intValue() ).getNum();
					return new Float(val);
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				boolean val = ((ITable)component).getTableCell( ((Integer)subitemPath.get(0)).intValue(),
						((Integer)subitemPath.get(1)).intValue() ).isChecked();
				return new Boolean(val);
			}
		};
		Executor itemcountEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Float(((ITable)component).getItemCount());
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				path.add( getNum(paramIndex, 0, Messages.getString("HTSRead.6")) ); //$NON-NLS-1$
				path.add( getNum(paramIndex+1, 0, Messages.getString("HTSRead.7")) ); //$NON-NLS-1$
				
				assertParamSize(paramIndex + 2);
				return path;
			}
		};
		
		parseCommand( new String[] {HTSSyntax.READ_ENABLESTATE, HTSSyntax.READ_FOCUSSTATE, HTSSyntax.READ_VISIBLESTATE, HTSSyntax.READ_TEXT, HTSSyntax.READ_NUM, HTSSyntax.READ_CHECKSTATE, HTSSyntax.READ_ITEMCOUNT}, 
				new int[] {SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED, SUBITEM_PATH_FORBIDDEN},
				pathValidator,
				new Executor[] { new EnabledExecutor(), new FocusExecutor(), new VisibleExecutor(), textEx, numEx, checkEx, itemcountEx }
			);
	}
	
	protected void parseTreeCommand() throws ParseException
	{
		parseComponentCommand(); // am Baum kann man nix weiter auslesen
	}
	
	protected void parseWindowCommand() throws ParseException
	{
		parseComponentCommand(); // am Fenster kann man auch nix weiter auslesen
	}
	
	/** Liest den Zustand eines Components aus und speichert ihn in einer Variable. */
	public void execute(IHTSPlayer player) throws ExecutionException
	{
		try // alle ComponentExceptions in ExecutionException umwandeln
		{
			MComponent comp = getMComponent( getComponentId(player.getTestObject()) );
			player.setVar( variableName, executor.execute(comp) );
			
		}
		catch(ComponentException e)
		{
			throw new ExecutionException( this, e.getMessage() );
		}
		catch(RemoteException e) // der Player im AtosJ Prozess ist tot
		{
			// wir machen einfach nix
			// throw new ExecutionException( this, e.getMessage() );
		}
	}
	
	public Object clone() {
		try {
			return new HTSRead( (CSVLine) line.clone(), getResolver() );
		} catch(ParseException e) {
			return null;
		}
	}
}
