package atosj.model.hts.command;

import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.Vector;

import atosj.Messages;
import atosj.component.ComponentResolver;
import atosj.component.ICheckbox;
import atosj.component.ICombobox;
import atosj.component.IComponent;
import atosj.component.ILabel;
import atosj.component.IList;
import atosj.component.IMenu;
import atosj.component.IRadio;
import atosj.component.ITabFolder;
import atosj.component.ITable;
import atosj.component.MComponent;
import atosj.exception.ComponentException;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS TEST Kommandos.
 * @author Nicos Tegos
 */
public class HTSTest extends HTSComponentCommand
{	
	private class VisibleExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			return new Boolean(component.isVisible());
		}
	};
	
	private class EnabledExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			return new Boolean(component.isEnabled());
		}
	}
	
	private class FocusExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			return new Boolean(component.hasFocus());
		}
	};
	
	private class PropertyExecutor extends Executor {
		public Object execute(IComponent component)	throws ComponentException {
			return ((IComponent)component).getProperty( (String) params.get(0) );
		}
	}
	
	private String state; // Zustand des Controls der abgefrat werden soll, z.B. ENABLESTATE
	private Comparable mustValue; // Sollwert bei TEXT oder NUM
	private String compOperator; // optionaler Vergleichsoperator z.B. GRT oder LSS
	private float tolerance; // optionaler Toleranzbereich
	
	private Executor executor;
	
	/**
	 * Erstellt ein neues TEST-Kommando.
	 * @param line Kommaspeparierte Liste des Kommandos.
	 */
	public HTSTest(CSVLine line, ComponentResolver resolver) throws ParseException
	{
		super(line, resolver);
	}
	
	private void parseCommand(String[] validStates, int[] possibleSubitems, SubitemPathValidator pathValidator, Executor[] testExecutors) throws ParseException
	{
		SubitemPathValidator[] pathValidators = new SubitemPathValidator[validStates.length];
		Arrays.fill(pathValidators, pathValidator);
		
		parseCommand(validStates, possibleSubitems, pathValidators, testExecutors);
	}
	
	private void parseCommand(String[] validStates, int[] possibleSubitems, SubitemPathValidator pathValidators[], Executor[] testExecutors) throws ParseException
	{
		state = getParam(3, Messages.getString("HTSTest.0")); //$NON-NLS-1$
		
		Vector params = null;
		Vector subitemPath = null;
		
		// berprfen ob der aktionstyp in der menge der gltigen aktionstypen enthalten ist.
		int validIndex = -1;
		for(int i = 0; i < validStates.length; ++i)
		{
			if( validStates[i].equals(state) )
			{
				validIndex = i;
				break;
			}
		}
		
		if(validIndex == -1) // keinen der gltigen aktionstypen gefunden
		{
			throw new InvalidParamException(line, 3, state,
					validStates);
		}
		
		if( state.equals(HTSSyntax.TEST_PROPERTY) )
		{
			params = new Vector(1);
			params.add( getParamUnquoted(4, Messages.getString("HTSTest.1")) ); //$NON-NLS-1$
			mustValue = getParamUnquoted(5, Messages.getString("HTSTest.2")); //$NON-NLS-1$

			assertParamSize(6);
		}
		else if( state.equals(HTSSyntax.TEST_TEXT) )
		{
			mustValue = getParamUnquoted(4, Messages.getString("HTSTest.3")); //$NON-NLS-1$
			int postTextIndex = 5;
			
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				assertParamSize(postTextIndex);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				subitemPath = getSubItemPath(postTextIndex, pathValidators[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				if( getParamSize() > postTextIndex )
					subitemPath = getSubItemPath(postTextIndex, pathValidators[validIndex]);
			}
		}
		else if( state.equals(HTSSyntax.TEST_NUM) )
		{
			int postNumIndex = parseNumParameters();
			
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				assertParamSize(postNumIndex);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				subitemPath = getSubItemPath(postNumIndex, pathValidators[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				if( getParamSize() > postNumIndex )
					subitemPath = getSubItemPath(postNumIndex, pathValidators[validIndex]);
			}
		}
		else if( state.equals(HTSSyntax.TEST_ITEMCOUNT) )
		{
			int postItemcountIndex = parseItemcountParameters();
			
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				assertParamSize(postItemcountIndex);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				subitemPath = getSubItemPath(postItemcountIndex, pathValidators[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				if( getParamSize() > postItemcountIndex )
					subitemPath = getSubItemPath(postItemcountIndex, pathValidators[validIndex]);
			}
		}
		else if( state.equals(HTSSyntax.TEST_CHECKSTATE)
				|| state.equals(HTSSyntax.TEST_ENABLESTATE)
				|| state.equals(HTSSyntax.TEST_FOCUSSTATE)
				|| state.equals(HTSSyntax.TEST_VISIBLESTATE) )
		{
			String param = getParam(4, Messages.getString("HTSTest.4")); //$NON-NLS-1$
			if( param.equals(HTSSyntax.COMPARE_TRUE) )
			{
				mustValue = Boolean.TRUE;
			}
			else if( param.equals(HTSSyntax.COMPARE_FALSE) )
			{
				mustValue = Boolean.FALSE;
			}
			else
				throw new InvalidParamException(line, 4, param, new String[] {HTSSyntax.COMPARE_TRUE, HTSSyntax.COMPARE_FALSE} );
			
			if( possibleSubitems[validIndex] == SUBITEM_PATH_FORBIDDEN )
			{
				assertParamSize(5);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_OBLIGED )
			{
				subitemPath = getSubItemPath(5, pathValidators[validIndex]);
			}
			else if( possibleSubitems[validIndex] == SUBITEM_PATH_VOLUNTARY )
			{
				if( getParamSize() > 5 )
					subitemPath = getSubItemPath(5, pathValidators[validIndex]);
			}
		}
		
		executor = testExecutors[validIndex];
		executor.setParams(params);
		executor.setSubitemPath(subitemPath);
	}
	
	protected void parseCustomCommand() throws ParseException
	{
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_PROPERTY},
				new int[] {
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, 
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN
				},
				(SubitemPathValidator[]) null,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				new PropertyExecutor() }
				);
	}
	
	protected void parseButtonCommand() throws ParseException
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return ((ILabel)component).getText() ;
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((ILabel)component).getNum() );
			}
		};
			
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE, 
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_TEXT, HTSSyntax.TEST_NUM},
				new int[] {
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN, 
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				textEx, numEx}
				);
	}
	
	protected void parseCheckboxCommand() throws ParseException
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return ((ILabel)component).getText() ;
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((ILabel)component).getNum() );
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Boolean( ((ICheckbox)component).isChecked() );
			}
		};
			
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_TEXT, HTSSyntax.TEST_NUM,
				HTSSyntax.TEST_CHECKSTATE },
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				textEx, numEx,
				checkEx}
				);
	}
	
	protected void parseComboboxCommand() throws ParseException
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return ((ILabel)component).getText() ;
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((ILabel)component).getNum() );
			}
		};
		
		Executor itemcountEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((ICombobox)component).getItemCount() );
			}
		};
			
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_TEXT, HTSSyntax.TEST_NUM,
				HTSSyntax.TEST_ITEMCOUNT},
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				textEx, numEx,
				itemcountEx}
				);
	}
	
	protected void parseComponentCommand() throws ParseException
	{
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE},
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN },
				(SubitemPathValidator[]) null,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor() }
				);
	}
	
	protected void parseEditboxCommand() throws ParseException
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return ((ILabel)component).getText() ;
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((ILabel)component).getNum() );
			}
		};
			
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_TEXT, HTSSyntax.TEST_NUM},
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				textEx, numEx}
				);
	}
	
	protected void parseSpinnerCommand() throws ParseException
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return ((ILabel)component).getText() ;
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((ILabel)component).getNum() );
			}
		};
			
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_TEXT, HTSSyntax.TEST_NUM},
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				textEx, numEx}
				);
	}
	
	protected void parseLabelCommand() throws ParseException 
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return ((ILabel)component).getText() ;
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((ILabel)component).getNum() );
			}
		};
			
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_TEXT, HTSSyntax.TEST_NUM},
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN},
				(SubitemPathValidator[]) null,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				textEx, numEx}
				);
	}
	
	protected void parseListCommand() throws ParseException
	{
		Executor itemcountEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((IList)component).getItemCount() );
			}
		};
		Executor textEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return ((IList)component).getText( ((Integer)subitemPath.get(0)).intValue() ) ;
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				path.add( getNum(paramIndex, 0, Messages.getString("HTSTest.5")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_ITEMCOUNT, HTSSyntax.TEST_TEXT},
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_OBLIGED},
				pathValidator,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				itemcountEx, textEx}
			);
	}
	
	protected void parseMenuCommand() throws ParseException
	{
		Executor enabledEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				if(subitemPath == null)
					return new Boolean( component.isEnabled() );
				else
					return new Boolean( ((IMenu)component).isEnabled( (String[]) subitemPath.toArray(new String[subitemPath.size()]) ) ); 
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Boolean( ((IMenu)component).isChecked( (String[]) subitemPath.toArray(new String[subitemPath.size()]) ) ); 
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				for(int i = paramIndex; i < getParamSize(); ++i)
					path.add( getParamUnquoted(i, Messages.getString("HTSTest.6")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_CHECKSTATE},
				new int[] {
				SUBITEM_PATH_VOLUNTARY,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_OBLIGED },
				pathValidator,
				new Executor[] { 
				enabledEx,
				new FocusExecutor(),
				new VisibleExecutor(),
				checkEx}
				);
	}
	
	protected void parseRadiobuttonCommand() throws ParseException
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return ((ILabel)component).getText() ;
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((ILabel)component).getNum() );
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Boolean( ((IRadio)component).isChecked() );
			}
		};
			
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_TEXT, HTSSyntax.TEST_NUM,
				HTSSyntax.TEST_CHECKSTATE},
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN },
				(SubitemPathValidator[]) null,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				textEx, numEx,
				checkEx}
				);
	}
	
	protected void parseTabfolderCommand() throws ParseException
	{
		Executor itemcountEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return new Float( ((ITabFolder)component).getItemCount() );
			}
		};
		Executor textEx = new Executor() {
			public Object execute(IComponent component) throws ComponentException {
				return ((ITabFolder)component).getText( ((Integer)subitemPath.get(0)).intValue() ) ;
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				path.add( getNum(paramIndex, 0, Messages.getString("HTSTest.7")) ); //$NON-NLS-1$
				
				return path;
			}
		};
		
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_ITEMCOUNT, HTSSyntax.TEST_TEXT},
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN, SUBITEM_PATH_VOLUNTARY},
				pathValidator,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				itemcountEx, textEx}
			);
	}
	
	protected void parseTableCommand() throws ParseException
	{
		Executor textEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return ((ITable)component).getTableCell( ((Integer)subitemPath.get(0)).intValue(),
						((Integer)subitemPath.get(1)).intValue() ).getText();
			}
		};
		Executor numEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
					float val = ((ITable)component).getTableCell( ((Integer)subitemPath.get(0)).intValue(),
							((Integer)subitemPath.get(1)).intValue() ).getNum();
					return new Float(val);
			}
		};
		Executor checkEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				boolean val = ((ITable)component).getTableCell( ((Integer)subitemPath.get(0)).intValue(),
						((Integer)subitemPath.get(1)).intValue() ).isChecked();
				return new Boolean(val);
			}
		};
		Executor itemcountEx = new Executor() {
			public Object execute(IComponent component)	throws ComponentException {
				return new Float(((ITable)component).getItemCount());
			}
		};
		
		SubitemPathValidator pathValidator = new SubitemPathValidator() {
			public Vector getSubitemPath(int paramIndex) throws ParseException {
				Vector path = new Vector(1);
				path.add( getNum(paramIndex, 0, Messages.getString("HTSTest.8")) ); //$NON-NLS-1$
				path.add( getNum(paramIndex+1, 0, Messages.getString("HTSTest.9")) ); //$NON-NLS-1$
				
				assertParamSize(paramIndex + 2);
				return path;
			}
		};
		
		parseCommand( 
				new String[] {
				HTSSyntax.TEST_ENABLESTATE,
				HTSSyntax.TEST_FOCUSSTATE,
				HTSSyntax.TEST_VISIBLESTATE,
				HTSSyntax.TEST_TEXT, HTSSyntax.TEST_NUM,
				HTSSyntax.TEST_CHECKSTATE,
				HTSSyntax.TEST_ITEMCOUNT},
				new int[] {
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_FORBIDDEN,
				SUBITEM_PATH_OBLIGED, SUBITEM_PATH_OBLIGED,
				SUBITEM_PATH_OBLIGED,
				SUBITEM_PATH_FORBIDDEN
				},
				pathValidator,
				new Executor[] { 
				new EnabledExecutor(),
				new FocusExecutor(),
				new VisibleExecutor(),
				textEx, numEx,
				checkEx,
				itemcountEx}
			);
	}
	
	protected void parseTreeCommand() throws ParseException
	{
		parseComponentCommand();
	}
	
	protected void parseWindowCommand() throws ParseException
	{
		parseComponentCommand();
	}
	
	/**
	 * Liest den Sollwert und den Vergleichsmodus eines Sollwertvergleichs mit
	 * einer reellen Zahl (NUM) aus.
	 * @return Der Index nach dem letzten Parameter den Sollwertvergleich.
	 */
	private int parseNumParameters() throws ParseException
	{
		// Vergleichsparameter
		mustValue = getFloat(4, Messages.getString("HTSTest.10")); //$NON-NLS-1$
		
		// wenn es einen Vergleichsparameter gibt, dann existieren auch
		// potentiell Vergleichsmodus und auch toleranz wert
		if( getParamSize() > 5)
		{
			compOperator = getParam(5, Messages.getString("HTSTest.11")); //$NON-NLS-1$
			if( (compOperator.equals(HTSSyntax.COMPARE_EQ) 
					|| compOperator.equals(HTSSyntax.COMPARE_GRT) 
					|| compOperator.equals(HTSSyntax.COMPARE_GEQ)
					|| compOperator.equals(HTSSyntax.COMPARE_LSS)
					|| compOperator.equals(HTSSyntax.COMPARE_LEQ)
					|| compOperator.equals(HTSSyntax.COMPARE_NEQ)
					|| compOperator.equals(HTSSyntax.COMPARE_TOL)) )
			{
				// eventuellen toleranzwert suchen
				if( compOperator.equals(HTSSyntax.COMPARE_TOL) )
				{
					tolerance = getFloat(6, Messages.getString("HTSTest.12")).floatValue(); //$NON-NLS-1$
					return 7;
				}
				else
				{
					return 6;
				}
			}
			else
			{
				throw new InvalidParamException(line, 5, compOperator,
					new String[] {HTSSyntax.COMPARE_EQ, HTSSyntax.COMPARE_GRT 
						, HTSSyntax.COMPARE_GEQ, HTSSyntax.COMPARE_LSS 
						, HTSSyntax.COMPARE_LEQ, HTSSyntax.COMPARE_NEQ
						, HTSSyntax.COMPARE_TOL});
			}
		}
		else // implizites EQ
		{
			return 5;
		}
	}
	
	/**
	 * Liest den Sollwert und den Vergleichsmodus eines Sollwertvergleichs fr
	 * den Zustand ITEMCOUNT aus.
	 * @return Der Index nach dem letzten Parameter des Sollwertvergleichs.
	 */
	private int parseItemcountParameters() throws ParseException
	{
		// Vergleichsparameter
		mustValue = new Float( getNum(4, 0, Messages.getString("HTSTest.13")).intValue() ); // TODO vorerst workaround fr classcastexception bei der ausfhrung in execute wird Float verlangt //$NON-NLS-1$
		
		// wenn es einen Vergleichsparameter gibt, dann existiert auch
		// potentiell der Vergleichsmodus
		if( getParamSize() > 5)
		{
			compOperator = getParam(5, Messages.getString("HTSTest.14")); //$NON-NLS-1$
			if( compOperator.equals(HTSSyntax.COMPARE_EQ) 
					|| compOperator.equals(HTSSyntax.COMPARE_GRT) 
					|| compOperator.equals(HTSSyntax.COMPARE_GEQ)
					|| compOperator.equals(HTSSyntax.COMPARE_LSS)
					|| compOperator.equals(HTSSyntax.COMPARE_LEQ)
					|| compOperator.equals(HTSSyntax.COMPARE_NEQ) )
			{
				return 6;
			}
			else
			{
				throw new InvalidParamException(line, 5, compOperator,
					new String[] {HTSSyntax.COMPARE_EQ, HTSSyntax.COMPARE_GRT 
						, HTSSyntax.COMPARE_GEQ, HTSSyntax.COMPARE_LSS 
						, HTSSyntax.COMPARE_LEQ, HTSSyntax.COMPARE_NEQ});
			}
		}
		else // implizites EQ
		{
			return 5;
		}
	}
	
	/** Fhrt den Vergleich mit Soll- und Istwer durch. */
	public void execute(IHTSPlayer player) throws ExecutionException
	{
		try { // alle ComponentExceptions in ExecutionExceptions umwandeln
		
		MComponent comp = getMComponent( getComponentId(player.getTestObject()) );
		
		// alle Booleschen Vergleich
		if(state.equals(HTSSyntax.TEST_ENABLESTATE)
				|| state.equals(HTSSyntax.TEST_FOCUSSTATE)
				|| state.equals(HTSSyntax.TEST_VISIBLESTATE)
				|| state.equals(HTSSyntax.TEST_CHECKSTATE))
		{
			boolean is = ((Boolean)executor.execute(comp)).booleanValue();
			boolean must = ((Boolean)mustValue).booleanValue();
			if( must != is )
			{
				throw new ExecutionException(this, 
					Messages.getString("HTSTest.15") + state + Messages.getString("HTSTest.16")  //$NON-NLS-1$ //$NON-NLS-2$
					+ (must ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE)
					+ Messages.getString("HTSTest.17")  //$NON-NLS-1$
					+ (is ? HTSSyntax.COMPARE_TRUE : HTSSyntax.COMPARE_FALSE));
			}
		}
		else
		if( state.equals(HTSSyntax.TEST_PROPERTY) )
		{
			String current = (String) executor.execute(comp);
			if( !current.equals(mustValue) )
				throw new ExecutionException(this,
					Messages.getString("HTSTest.18") + HTSSyntax.TEST_PROPERTY + Messages.getString("HTSTest.19") + //$NON-NLS-1$ //$NON-NLS-2$
					Messages.getString("HTSTest.20") + mustValue +  //$NON-NLS-1$
					Messages.getString("HTSTest.21") + current); //$NON-NLS-1$
		}
		else
		if(state.equals(HTSSyntax.TEST_TEXT))
		{
			String current = (String) executor.execute(comp);
			if( !current.equals(mustValue) )
				throw new ExecutionException(this,
					Messages.getString("HTSTest.22") + HTSSyntax.TEST_TEXT + Messages.getString("HTSTest.23") + //$NON-NLS-1$ //$NON-NLS-2$
					Messages.getString("HTSTest.24") + mustValue +  //$NON-NLS-1$
					Messages.getString("HTSTest.25") + current); //$NON-NLS-1$
		}
		else
		if(state.equals(HTSSyntax.TEST_NUM) || state.equals(HTSSyntax.TEST_ITEMCOUNT))
		{
			String property = state;
			Float currentValue = (Float) executor.execute(comp); 
			int result = currentValue.compareTo((Float)mustValue); 
			if( compOperator == null || compOperator.equals(HTSSyntax.COMPARE_EQ) ) // implizit auf gleichheit testen
			{
				if(result != 0)
					throw new ExecutionException(this,
							Messages.getString("HTSTest.26") + property + Messages.getString("HTSTest.27") + //$NON-NLS-1$ //$NON-NLS-2$
							Messages.getString("HTSTest.28") + mustValue +  //$NON-NLS-1$
							Messages.getString("HTSTest.29") + currentValue); //$NON-NLS-1$
			}
			else
			if( compOperator.equals(HTSSyntax.COMPARE_GRT) )
			{
				if(result != 1)
					throw new ExecutionException(this,
							Messages.getString("HTSTest.30") + property + Messages.getString("HTSTest.31") + //$NON-NLS-1$ //$NON-NLS-2$
							Messages.getString("HTSTest.32") + mustValue +  //$NON-NLS-1$
							Messages.getString("HTSTest.33") + currentValue); //$NON-NLS-1$
			}
			else
			if( compOperator.equals(HTSSyntax.COMPARE_GEQ) )
			{
				if(result == -1)
					throw new ExecutionException(this,
							Messages.getString("HTSTest.34") + property + Messages.getString("HTSTest.35") + //$NON-NLS-1$ //$NON-NLS-2$
							Messages.getString("HTSTest.36") + mustValue +  //$NON-NLS-1$
							Messages.getString("HTSTest.37") + currentValue); //$NON-NLS-1$
			}
			else
			if( compOperator.equals(HTSSyntax.COMPARE_LSS) )
			{
				if(result != -1)
					throw new ExecutionException(this,
							Messages.getString("HTSTest.38") + property + Messages.getString("HTSTest.39") + //$NON-NLS-1$ //$NON-NLS-2$
							Messages.getString("HTSTest.40") + mustValue +  //$NON-NLS-1$
							Messages.getString("HTSTest.41") + currentValue); //$NON-NLS-1$
			}
			else
			if( compOperator.equals(HTSSyntax.COMPARE_LEQ) )
			{
				if(result == 1)
					throw new ExecutionException(this,
							Messages.getString("HTSTest.42") + property + Messages.getString("HTSTest.43") + //$NON-NLS-1$ //$NON-NLS-2$
							Messages.getString("HTSTest.44") + mustValue +  //$NON-NLS-1$
							Messages.getString("HTSTest.45") + currentValue); //$NON-NLS-1$
			}
			else
			if( compOperator.equals(HTSSyntax.COMPARE_NEQ) )
			{
				if(result == 0)
					throw new ExecutionException(this,
							Messages.getString("HTSTest.46") + property + Messages.getString("HTSTest.47") + //$NON-NLS-1$ //$NON-NLS-2$
							Messages.getString("HTSTest.48") + mustValue +  //$NON-NLS-1$
							Messages.getString("HTSTest.49") + currentValue); //$NON-NLS-1$
			}
			else
			if( compOperator.equals(HTSSyntax.COMPARE_TOL) )
			{
				Float begin = new Float( ((Float)mustValue).floatValue() - tolerance );
				Float end = new Float( ((Float)mustValue).floatValue() + tolerance );
				
				if( currentValue.compareTo(begin) == -1 || currentValue.compareTo(end) == 1 )
					throw new ExecutionException(this, Messages.getString("HTSTest.50") + property + Messages.getString("HTSTest.51") //$NON-NLS-1$ //$NON-NLS-2$
							+ Messages.getString("HTSTest.52") + begin + ", " + end + "]" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
							+ Messages.getString("HTSTest.55") + currentValue); //$NON-NLS-1$
			}
		}
		} catch(ComponentException e) { // alle ComponentExceptions in ExecutionException umwandeln
			throw new ExecutionException( this, e.getMessage() );
		}
		catch(RemoteException e) // der Player im AtosJ Prozess ist tot
		{
			// wir machen einfach nix
			// throw new ExecutionException( this, e.getMessage() );
		}
	}
	
	public String toCaptureDlgString() 
	{
		try 
		{
			Vector v = new Vector();
			for(int i = 3; i < getParamSize(); ++i)
				v.add( getParam(i, Messages.getString("HTSTest.56")) ); //$NON-NLS-1$
			
			return new CSVLine(v).toString();
		} catch(ParseException e) {
			return toString();
		}
	}
	
	public Object clone() {
		try {
			return new HTSTest( (CSVLine) line.clone(), getResolver() );
		} catch(ParseException e) {
			return null;
		}
	}
}
