package atosj.model.hts.command;

import atosj.Messages;
import atosj.exception.ExecutionException;
import atosj.exception.InvalidParamException;
import atosj.exception.ParseException;
import atosj.model.hts.parser.HTSSyntax;
import atosj.rmi.HTSPlayer;
import atosj.rmi.IHTSPlayer;
import atosj.util.CSVLine;

/**
 * Klasse zum Parsen und Ausfhren eines HTS WHILE Kommandos.
 * Leider kann Java keine Mehrfachvererbung, sonst htten wir auch noch
 * von <code>HTSCompare</code> abgeleitet, denn das Parsing ist das gleiche.
 * @author Nicos Tegos
 */
public class HTSWhile extends HTSCollectionCommand 
{
	private String firstVarName; // der erste Vergleichoperand ist immer eine Variable (Istwert)
	private Object secondParam; // der zweite Operand kann eine Variable oder ein Wert sein (Sollwert)
	
	private String varType; // Datentyp der Variablen STR, NUM, BOOL
	private String modeVar; // Typ des 2. Operanden VAR oder VAL
	private String modeComp; // Vergleichmodi siehe HTS Spec z.B. grer als ist "GRT"
	private Float tolerance; // gibt einen Toleranzbereich an der Vergleichswert muss im Intervall [Sollwert-Toleranz, Sollwert+Toleranz] liegen
	
	private boolean evalResult; // das Ergebnis der Auswertung des letzten Ausdrucks
	
	public HTSWhile(CSVLine line) throws ParseException 
	{
		super(line);

		/*vergleich_var_textuell = COMPARE, $Variable1$, STR, VAR, $Variable2$
		vergleich_var_numerisch = COMPARE, $Variable1$, NUM, VAR, $Variable2$
		[,vergleichsmodus]
		vergleich_wert_textuell = COMPARE, $Variable1$, STR, VAL, "Wert"
		vergleich_wert_numerisch = COMPARE, $Variable1$, NUM, VAL, "Wert"
		[,vergleichsmodus]*/
		
		firstVarName = getParamUnquoted(0, Messages.getString("HTSWhile.0")); //$NON-NLS-1$
		varType = getParam(1, Messages.getString("HTSWhile.1")); //$NON-NLS-1$
		if( !(varType.equals(HTSSyntax.COMPARE_NUM) 
			|| varType.equals(HTSSyntax.COMPARE_STR)
			|| varType.equals(HTSSyntax.COMPARE_BOOL))
		)
		{
			throw new InvalidParamException(line, 1, varType,
				new String[]  {HTSSyntax.COMPARE_NUM, HTSSyntax.COMPARE_STR, HTSSyntax.COMPARE_BOOL} );
		}
		
		modeVar = getParam(2, Messages.getString("HTSWhile.2")); //$NON-NLS-1$
		if( modeVar.equals(HTSSyntax.COMPARE_VAR) )
		{
			if( varType.equals(HTSSyntax.COMPARE_STR)
					|| varType.equals(HTSSyntax.COMPARE_BOOL) )
				{
					secondParam = getParamUnquoted(3, Messages.getString("HTSWhile.3")); // Variablenname als String //$NON-NLS-1$
					assertParamSize(4);
				}
				else
				if( varType.equals(HTSSyntax.COMPARE_NUM) )
				{
					secondParam = getParamUnquoted(3, Messages.getString("HTSWhile.4")); // Variablenname als String //$NON-NLS-1$
					
					if( getParamSize() > 4 )
					{
						modeComp = getParam(4, Messages.getString("HTSWhile.5"));					 //$NON-NLS-1$
						if( !( modeComp.equals(HTSSyntax.COMPARE_EQ)
								|| modeComp.equals(HTSSyntax.COMPARE_GRT) 
								|| modeComp.equals(HTSSyntax.COMPARE_GEQ)
								|| modeComp.equals(HTSSyntax.COMPARE_LSS)
								|| modeComp.equals(HTSSyntax.COMPARE_LEQ)
								|| modeComp.equals(HTSSyntax.COMPARE_NEQ)
								|| modeComp.equals(HTSSyntax.COMPARE_TOL)) )
						{
							throw new InvalidParamException(line, 4, modeComp,
									Messages.getString("HTSWhile.6")  //$NON-NLS-1$
									+ HTSSyntax.COMPARE_EQ + ", " + HTSSyntax.COMPARE_GRT + ", "  //$NON-NLS-1$ //$NON-NLS-2$
									+ HTSSyntax.COMPARE_GEQ + ", " + HTSSyntax.COMPARE_LSS + ", " //$NON-NLS-1$ //$NON-NLS-2$
									+ HTSSyntax.COMPARE_LEQ + ", " + HTSSyntax.COMPARE_NEQ  //$NON-NLS-1$
									+ Messages.getString("HTSWhile.12") + HTSSyntax.COMPARE_TOL); //$NON-NLS-1$
						}
						
						if( modeComp.equals(HTSSyntax.COMPARE_TOL) )
						{
							tolerance =getFloat(5, Messages.getString("HTSWhile.13")); //$NON-NLS-1$
							assertParamSize(6);
						}
						else
							assertParamSize(5);
					}
				}
				else
				{
					throw new InvalidParamException(line, 1, varType,
						new String[] {HTSSyntax.COMPARE_STR, HTSSyntax.COMPARE_NUM, HTSSyntax.COMPARE_BOOL} );
				}
		}
		else
		if( modeVar.equals(HTSSyntax.COMPARE_VAL) )
		{
			if( varType.equals(HTSSyntax.COMPARE_NUM) )
			{
				secondParam = getFloat(3, Messages.getString("HTSWhile.14")); //$NON-NLS-1$
				
				if( getParamSize() > 4 )
				{
					modeComp = getParam(4, Messages.getString("HTSWhile.15"));					 //$NON-NLS-1$
					if( !( modeComp.equals(HTSSyntax.COMPARE_EQ)
						|| modeComp.equals(HTSSyntax.COMPARE_GRT) 
						|| modeComp.equals(HTSSyntax.COMPARE_GEQ)
						|| modeComp.equals(HTSSyntax.COMPARE_LSS)
						|| modeComp.equals(HTSSyntax.COMPARE_LEQ)
						|| modeComp.equals(HTSSyntax.COMPARE_NEQ)
						|| modeComp.equals(HTSSyntax.COMPARE_TOL)) )
					{
						throw new InvalidParamException(line, 4, modeComp,
								Messages.getString("HTSWhile.16")  //$NON-NLS-1$
								+ HTSSyntax.COMPARE_EQ + ", " + HTSSyntax.COMPARE_GRT + ", "  //$NON-NLS-1$ //$NON-NLS-2$
								+ HTSSyntax.COMPARE_GEQ + ", " + HTSSyntax.COMPARE_LSS + ", " //$NON-NLS-1$ //$NON-NLS-2$
								+ HTSSyntax.COMPARE_LEQ + ", " + HTSSyntax.COMPARE_NEQ  //$NON-NLS-1$
								+ Messages.getString("HTSWhile.22") + HTSSyntax.COMPARE_TOL); //$NON-NLS-1$
					}
					
					if( modeComp.equals(HTSSyntax.COMPARE_TOL) )
					{
						tolerance =getFloat(5, Messages.getString("HTSWhile.23")); //$NON-NLS-1$
						assertParamSize(6);
					}
					else
						assertParamSize(5);
				}
			}
			else
			if( varType.equals(HTSSyntax.COMPARE_STR) )
			{
				// Wert als String
				secondParam = getParamUnquoted(3, Messages.getString("HTSWhile.24")); //$NON-NLS-1$
				assertParamSize(4);
			}
			else
			if( varType.equals(HTSSyntax.COMPARE_BOOL) )
			{
				// Wert als Boolean
				String str = getParam(3, Messages.getString("HTSWhile.25")); //$NON-NLS-1$
				if( str.equals(HTSSyntax.COMPARE_TRUE) )
				{
					secondParam = Boolean.TRUE;
				}
				else
				if( str.equals(HTSSyntax.COMPARE_FALSE) )
				{
					secondParam = Boolean.FALSE;
				}
				else
				{
					throw new InvalidParamException(line, 3, str,
							new String[] {HTSSyntax.COMPARE_TRUE , HTSSyntax.COMPARE_FALSE});
				}
				
				assertParamSize(4);
			}
		}
		else
		{
			throw new InvalidParamException(line, 2, modeVar,
					new String[] {HTSSyntax.COMPARE_VAR, HTSSyntax.COMPARE_VAL} );
		}
	}

	/**
	 * Wertet den Booleschen Ausdruck dieses Kommandos aus. 
	 * Fr die Ausfhrung der im WHILE-Block enthaltenen Kommandos ist der
	 *  <code>HTSPlayer</code> zustndig.
	 *  @see #isFinished()
	 */
	public void execute(IHTSPlayer player) throws ExecutionException 
	{
		HTSPlayer localPlayer = (HTSPlayer) player;
		
		Object first = localPlayer.getVar(firstVarName);
		if(first == null)
			throw new ExecutionException(this, Messages.getString("HTSWhile.26") + firstVarName); //$NON-NLS-1$
		
		Object second;
		if( modeVar.equals(HTSSyntax.COMPARE_VAR) )
		{
			second = localPlayer.getVar( (String)secondParam );
			if(second == null)
				throw new ExecutionException(this, Messages.getString("HTSWhile.27") + secondParam); //$NON-NLS-1$
		}
		else
		{
			second = secondParam; // ist schon ein Wert
		}
		
		if( !first.getClass().equals(second.getClass()) )
		{
			String firstType = HTSSyntax.COMPARE_STR;
			if( first instanceof Float)
				firstType = HTSSyntax.COMPARE_NUM;
			else
			if( first instanceof Boolean)
				firstType = HTSSyntax.COMPARE_BOOL;
			
			String secondType = HTSSyntax.COMPARE_STR;
			if( second instanceof Float)
				secondType = HTSSyntax.COMPARE_NUM;
			else
			if( second instanceof Boolean)
				secondType = HTSSyntax.COMPARE_BOOL;
			
			throw new ExecutionException(this, Messages.getString("HTSWhile.28") //$NON-NLS-1$
					+ Messages.getString("HTSWhile.29") + firstType  //$NON-NLS-1$
					+ Messages.getString("HTSWhile.30") + secondType); //$NON-NLS-1$
		}
		
		if( varType.equals(HTSSyntax.COMPARE_BOOL) )
		{
			evalResult = first.equals(second);
		}
		else if( varType.equals(HTSSyntax.COMPARE_STR) )
		{
			evalResult = first.equals(second);
		}
		else // numerischer Vergleich in seiner ganzen Bandbreite
		{
			int result = ((Float)first).compareTo((Float)second);
			if( modeComp == null || modeComp.equals(HTSSyntax.COMPARE_EQ) ) // implizit auf gleichheit testen
			{
				evalResult = result == 0;
			}
			else if( modeComp.equals(HTSSyntax.COMPARE_GRT) )
			{
				evalResult = result == 1;
			}
			else if( modeComp.equals(HTSSyntax.COMPARE_GEQ) )
			{
				evalResult = result >= 0;
			}
			else if( modeComp.equals(HTSSyntax.COMPARE_LSS) )
			{
				evalResult = result == -1;
			}
			else if( modeComp.equals(HTSSyntax.COMPARE_LEQ) )
			{
				evalResult = result <= 0;
			}
			else if( modeComp.equals(HTSSyntax.COMPARE_NEQ) )
			{
				evalResult = result != 0;
			}
			else if( modeComp.equals(HTSSyntax.COMPARE_TOL) )
			{
				Float begin = new Float( ((Float)second).floatValue() - tolerance.floatValue() );
				Float end = new Float( ((Float)second).floatValue() + tolerance.floatValue() );
				Float isValue = (Float) first;
				
				if( isValue.compareTo(begin) == -1 || isValue.compareTo(end) == 1 )
					evalResult = false;
				else
					evalResult = true;
			}
		}
	}

	/**
	 * Gibt das Ergebnis des zuletzt in <code>execute()</code> ausgewerteten Ausdrucks zurck.
	 * @return <code>true</code>, wenn die Bedingung im Schleifenkopf beim letzen Mal zu wahr
	 * evaluierte.
	 */
	public boolean isFinished() {
		return !evalResult;
	}
	
	/**
	 * Setzt das syntaktischen Ende zu diesem Kommando, d.h. ENDWHILE.
	 * @param end wenn <code>null</code> wird, das
	 * Kommando auf fehlerhaft gesetzt.
	 * @see HTSCommand#setError(String)
	 */
	public void setEnd(HTSEndWhile end) {
		super.setEnd(end);
		if(end == null)
			setError(HTSSyntax.CMD_WHILE + Messages.getString("HTSWhile.31") + HTSSyntax.CMD_ENDWHILE + "."); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public Object clone() 
	{
		try {
			return new HTSWhile( (CSVLine) line.clone() );
		} catch(ParseException e) {
			return null;
		}
	}
}
