package atosj.model.hts.parser;

import java.io.IOException;
import java.util.Vector;

import atosj.Messages;
import atosj.component.ComponentResolver;
import atosj.exception.ParseException;
import atosj.model.hts.command.HTSAction;
import atosj.model.hts.command.HTSCleanup;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.command.HTSComment;
import atosj.model.hts.command.HTSCompare;
import atosj.model.hts.command.HTSCopy;
import atosj.model.hts.command.HTSDelete;
import atosj.model.hts.command.HTSDisable;
import atosj.model.hts.command.HTSEndLoop;
import atosj.model.hts.command.HTSEndWhile;
import atosj.model.hts.command.HTSExists;
import atosj.model.hts.command.HTSLaunch;
import atosj.model.hts.command.HTSLoop;
import atosj.model.hts.command.HTSMessage;
import atosj.model.hts.command.HTSQuestion;
import atosj.model.hts.command.HTSRead;
import atosj.model.hts.command.HTSStart;
import atosj.model.hts.command.HTSTest;
import atosj.model.hts.command.HTSWait;
import atosj.model.hts.command.HTSWhile;
import atosj.model.hts.command.HTSWindowExists;
import atosj.model.hts.command.InvalidCommand;
import atosj.util.CSVLine;
import atosj.util.CSVParser;

/**
 * bernimmt das Parsen eines HTS-Skriptes.
 * @author Nicos Tegos
 */
public class HTSParser extends CSVParser {
	ComponentResolver resolver;
	
	/**
	 * Initialisiert den Parser.
	 * @param scriptLocation Der absolute Pfad einer HTS-Datei.
	 * @param resolver Der <code>ComponentResolver</code>, abhngig davon knnen unterschiedliche
	 * Elementtypen gltig sein (Custom-Components).
	 * @throws IOException Wenn die HTS-Datei nicht gelesen werden konnte.
	 */
	public HTSParser(String scriptLocation, ComponentResolver resolver) throws IOException {
		super(scriptLocation);
		this.resolver = resolver;
	}

	/**
	 * Liefert alle in der Skriptdatei enthaltenen Kommandos.
	 * Ungltige Zeilen werden als <code>InvalidCommand</code> zurckgegeben.
	 * @return Vector(HTSCommand) alle Kommandos in der Datei.
	 * @see InvalidCommand
	 */
	public Vector parse() 
	{
		Vector commands = new Vector();
		HTSCommand commandToAdd;
		int index = 0; 
		
		while ( (commandToAdd=getNextHTSCommand()) != null )
		{
			commands.add(commandToAdd);
			commandToAdd.setIndex(index);
			++index;
		}
		
		return commands;
	}

	/** 
	 * Liefert das nchste Kommando in der Datei.
	 * @return Das nchste Kommando oder <code>null</code> bei EOF.
	 * Bei einer Zeile, die nicht geparsed werden konnte wird ein
	 * <code>InvalidCommand</code> zurckgegeben.
	 */
	private HTSCommand getNextHTSCommand()
	{
		try 
		{
			CSVLine line = getNextLine();
			
			
			if(line == null)
				return null; // Dateiende erreicht
			
			return getCommandFromLine(line, resolver);
		
		}
		catch(ParseException e) {
			try	{
				return new InvalidCommand(e);
			}
			catch(ParseException e2) { return null; } // kann niemals auftreten
		}
	}
	
	/**
	 * Funktion fr das Erstellen eines Kommandos aus einer kommaseparierten Liste.
	 * @param line Die kommaseparierte Liste.
	 * @param resolver Der <code>ComponentResolver</code>, abhngig davon knnen unterschiedliche
	 * Elementtypen gltig sein (Custom-Components).
	 * @return Das Kommando.
	 * @throws ParseException wenn das Kommando ungltig ist.
	 */
	public static HTSCommand getCommandFromLine(CSVLine line, ComponentResolver resolver) throws ParseException 
	{
		String cmdName = line.getElement(0);
		if( cmdName.equals(HTSSyntax.CMD_LOOP) )
		{
			return new HTSLoop(line);
		}
		else	
		if( cmdName.equals(HTSSyntax.CMD_ENDLOOP) )
		{
			return new HTSEndLoop(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_ACTION) )
		{
			return new HTSAction(line, resolver);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_READ) )
		{
			return new HTSRead(line, resolver);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_TEST) )
		{
			return new HTSTest(line, resolver);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_COMP) )
		{
			return new HTSCompare(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_COPY) )
		{
			return new HTSCopy(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_DEL) )
		{
			return new HTSDelete(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_QUEST) )
		{
			return new HTSQuestion(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_LAUNCH) )
		{
			return new HTSLaunch(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_WEX) )
		{
			return new HTSWindowExists(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_START) )
		{
			return new HTSStart(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_CLEAN) )
		{
			return new HTSCleanup(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_WAIT) )
		{
			return new HTSWait(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_MESS) )
		{
			return new HTSMessage(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_COMM) )
		{
			return new HTSComment(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_EXISTS) )
		{
			return new HTSExists(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_DIS) )
		{
			return new HTSDisable(line, resolver);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_WHILE) )
		{
			return new HTSWhile(line);
		}
		else
		if( cmdName.equals(HTSSyntax.CMD_ENDWHILE) )
		{
			return new HTSEndWhile(line);
		}
		else 
		{	
			throw new ParseException(line, Messages.getString("HTSParser.0") + cmdName); //$NON-NLS-1$
		}
	}
}
