/*
 * Created on 28.11.2005
 */
package atosj.model.pack;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import atosj.Messages;
import atosj.app.Project;
import atosj.editors.BaseChangeable;
import atosj.editors.Changeable;
import atosj.exception.AtosJException;
import atosj.listener.ContentChangeListener;
import atosj.listener.StateChangeListener;
import atosj.model.script.HTSScript;
import atosj.util.CSVLine;
import atosj.util.CSVParser;

/**
 * Diese Klasse reprsentiert ein ATOSj Testpaket.
 * Ein Testpaket ist eine Sammlung von mehreren Skripten.
 * @author Nicos Tegos
 */
public class HTSPackage extends BaseChangeable implements StateChangeListener
{
	private LinkedList packageElements = new LinkedList(); // LinkedList<PackageElement>
	private String filePath;
	
	private Project project;
	
	/**
	 * Ldt eine Paketdatei. Darf erst geschehen nachdem im Projekt
	 * alle Skriptdateien geladen wurden.
	 * @param pakPath Der absolute Pfad der Paketdatei.
	 * @throws AtosJException bei nicht gefundener Datei oder
	 * bei inkorrekter Syntax der Datei.
	 */
	public HTSPackage(String pakPath, Project project) throws AtosJException 
	{
		this.filePath = pakPath;
		this.project = project;
		
		load();
	}

	/**
	 * Liefert alle im Paket enthaltenen Skripte, gekapselt in einem <code>PackageElement</code>.
	 * @return Vector(PackageElement) Eine Kopie der internen Liste von Skripten.
	 * @see PackageElement
	 */
	public Vector getElements() {
		return new Vector(packageElements);
	}
	
	/**
	 * Liefert alle <code>PackageElements</code>, die das angegebene Skript enthalten.
	 * @param script Das Skript.
	 * @return Vector(PackageElement) alle Elemente die <code>script</code> enthalten.
	 */
	public Vector getElementsForScript(HTSScript script) {
		Vector ret = new Vector();
		
		for(int i = 0; i < getNumberOfScripts(); ++i)
		{
			if( getScript(i).equals(script) )
				ret.add( packageElements.get(i) );
		}
		
		return ret;
	}
	
	/**
	 * Liefert alle enthaltenen Skripte.
	 * @return Vector(HTSScript) Die Skriptliste. 
	 */
	public Vector getScripts() {
		Vector ret = new Vector( packageElements.size() );
		for(int i = 0; i < getNumberOfScripts(); ++i)
			ret.add( getScript(i) );
		
		return ret;
	}
	
	/** Liefert das Skript an Position <code>i</code> */
	private HTSScript getScript(int i) {
		return (HTSScript) ((PackageElement)packageElements.get(i)).getScript();
	}
	
	/** 
	 * Liefert die Anzahl an Elementen im Paket. 
	 * Mehrfach vorkommende Skripte werden auch mehrfach gezhlt.
	 */
	public int getNumberOfScripts() {
		return packageElements.size();
	}
	
	/** Liefert das <code>PackageElement</code> an Position <code>i</code> */
	public PackageElement getElement(int index) {
		return (PackageElement) packageElements.get(index);
	}
	
	/**
	 * Diese Funktion numeriert die <code>PackageElements</code> nach ihrer Position in der Liste.
	 * Muss immer nach strukturellen nderungen der Liste gerufen werden.
	 */
	private void renumber() {
		for(int i = 0; i < packageElements.size(); ++i)
			((PackageElement)packageElements.get(i)).setIndex(i);
		
		fireContentChange();
	}
	
	/**
	 * Entfernt alle Vorkommen dieses Skripts aus dem Paket.
	 * @param script Das zu lschende Skript.
	 */
	public void removeScriptTotally(HTSScript script) 
	{
		for(int i = 0; i < getNumberOfScripts(); ++i)
		{
			if( getScript(i).equals(script) )
				packageElements.remove(i);
		}
		script.removeStateChangeListener(this);
		
		renumber();
		setModified(true);
	}
	
	/**
	 * Entfernt das angegebene Element aus dem Paket.
	 * @param element Das zu lschende <code>PackageElement</code>. 
	 * Dieses wird unabhngig von seinem aktuell gesetzten Index aus
	 * der Liste gelscht.
	 */
	public void removeElement(PackageElement element) 
	{
		packageElements.remove(element);
		
		if( !containsScript( element.getScript() ) ) 
			element.getScript().removeStateChangeListener(this);
		
		renumber();
		setModified(true);
	}
	
	/**
	 * Entfernt alle angegebenen Elemente aus dem Paket.
	 * @param elements Collection(PackageElement) Die zu lschenden <code>PackageElements</code>. 
	 * Diese werden unabhngig von ihrem aktuell gesetzten Index aus der Liste gelscht.
	 */
	public void removeElements(Collection elements) 
	{
		packageElements.removeAll(elements);
		
		Iterator it = elements.iterator();
		while( it.hasNext() )
		{
			PackageElement element = (PackageElement) it.next();
			if( !containsScript( element.getScript() ) ) 
				element.getScript().removeStateChangeListener(this);
		}
		
		renumber();
		setModified(true);
	}
	
	/**
	 * Fgt dem Paket ein neues Element hinzu
	 * @param element Das hinzuzufgende Element. Das Element wird an der Position in Liste eingefgt,
	 * die durch <code>PackageElement.getIndex()</code> geliefert wird. 
	 * Gltiger Wertebereich ist [0, <code>getNumberOfScripts()</code>];
	 */
	public void addElement(PackageElement element) 
	{
		packageElements.add(element.getIndex(), element);
		element.getScript().addStateChangeListener(this);
		renumber();
		setModified(true);
	}
	
	/**
	 * Fgt alle angegebenen Elemente dem Paket hinzu.
	 * @param elements Collection(PackageElement) Die hinzuzufgenden Elemente. 
	 * @param index Einfgeposition in der Liste. Gltiger Wertebereich ist [0, getNumberOfScripts()].
	 * Alle Elemente werden nacheinander beginnend bei <code>index</code> eingefgt.
	 */
	public void addElements(Collection elements, int index) 
	{
		packageElements.addAll(index, elements);
		Iterator it = elements.iterator();
		while( it.hasNext() )
			((PackageElement)it.next()).getScript().addStateChangeListener(this);
		
		renumber();
		setModified(true);
	}
	
	/**
	 * Lscht ein Element aus seiner aktuellen Position und fgt es an der 
	 * Position <code>newIndex</code> ein.
	 * @param element Das zu verschiebende Element.
	 * @param newIndex Die neue Position, gltiger Wertebereich [0, <code>getNumberOfScripts()</code>]
	 * @see PackageElement#getIndex()
	 */
	public void moveElement(PackageElement element, int newIndex)
	{
		packageElements.remove( element.getIndex() );
		packageElements.add(newIndex, element);
		
		renumber();
		setModified(true);
	}
	
	/** Liefert <code>true</code>, wenn dieses Paket das angegebene Skript enthlt. */
	public boolean containsScript(HTSScript script) 
	{
		for(int i = 0; i < getNumberOfScripts(); ++i)
			if( getScript(i).equals(script) )
				return true;
		
		return false;
	}
	
	/** Liefert den Namen dieses Pakets, d.h. den Dateinamen ohne Endung. */
	public String getName() {
		File f = new File(filePath);
		String fileName = f.getName();
		return fileName.substring( 0, fileName.lastIndexOf('.') );
	}
	
	/** Liefert den absolute Pfad dieser Package Datei. */
	public String getPath() {
		return filePath;
	}
	
	/** Zwei Pakete sind gleich, wenn sie den gleichen Namen tragen. */
	public boolean equals(Object obj) 
	{
		if(obj == this)
			return true;
		
		if(obj instanceof HTSPackage && obj != null)
		{
			return getName().equals( ((HTSPackage)obj).getName() );
		}
		
		return false;
	}
	
	/**
	 * Ldt die Paket-Datei.
	 * @throws AtosJException Syntaktischer Fehler in der Paket-Datei
	 * oder Paket-Datei nicht gefunden.
	 */
	private void load() throws AtosJException 
	{
		packageElements.clear();
		
		try
		{
			CSVParser parser = new CSVParser(filePath);
			CSVLine line; 
			while( (line = parser.getNextLine()) != null )
				addScript( line.getElementUnquoted(0) );
		}
		catch(IOException ex)
		{
			throw new AtosJException(Messages.getString("HTSPackage.0") //$NON-NLS-1$
					+ filePath + Messages.getString("HTSPackage.1")); //$NON-NLS-1$
		}
		
		updateValidState();
	}
	
	/**
	 *  Hilfsfunktion fr die Initialisierung des Pakets. Konstruiert fr ein Skript
	 *  das entsprechende <code>PackageElement</code> und fgt es der Liste hinzu.
	 *  @param name Der Skriptname.
	 *  @see HTSScript#getName()
	 */
	private void addScript(String name) 
	{
		Vector allScripts = project.getScripts();
		
		// Script nach namen suchen
		for(int i = 0; i < allScripts.size(); ++i)
		{
			HTSScript s = (HTSScript) allScripts.get(i);
			if( s.getName().equals(name) )
			{
				packageElements.add( new PackageElement(s, packageElements.size()) );
				s.addStateChangeListener(this);
			}
		}
	}
	
	/**
	 * Verwirft die letzten nderungen durch erneutes Laden der Datei.
	 * @throws AtosJException Syntaktischer Fehler in der Paket-Datei
	 * oder Datei kann nicht geffnet werden.
	 */
	public void discard() throws AtosJException
	{
		if( !isModified() )
			return;
		load();
		fireContentChange();
		setModified(false);
	}
	
	/** Speichert die Paket-Datei, sofern sie zuvor gendert wurde.	 */
	public void save() throws AtosJException 
	{
		if( !isModified() )
			return;
		
		try
		{
			FileWriter writer = new FileWriter(filePath);
			// ComponentIds in CSV Zeilen umwandeln und rausschreiben
			for(int i = 0; i < packageElements.size(); ++i) 
			{
				Vector lineElements = new Vector(1);
				lineElements.add( "\"" + getScript(i).getName() + "\"" ); //$NON-NLS-1$ //$NON-NLS-2$
				CSVLine line = new CSVLine(lineElements);
				
				writer.write( line.toString() + "\n" ); //$NON-NLS-1$
			}
			writer.close();
			
			updateValidState();
			setModified(false);
		}
		catch(IOException ex) {
			throw new AtosJException(Messages.getString("HTSPackage.5") +  getName() + Messages.getString("HTSPackage.6")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	/**
	 * Lscht die Paket-Datei.
	 * @return <code>true</code>, wenn das Lschen erfolgreich war,
	 * <code>false</code> sonst.
	 */
	public boolean delete() 
	{
		boolean deleted = true;
		File f = new File( getPath() );
		
		if( f.exists() )
			deleted = f.delete();
		
		if(deleted)
		{
			for(int i = 0; i < getNumberOfScripts(); ++i)
				getScript(i).removeStateChangeListener(this);
		}
		
		return deleted;
	}
	
	/**
	 * Benennt das Paket um. Die Paket-Datei wird dazu umbenannt.
	 * @return <code>true</code>, wenn die Paket-Datei erfolgreich
	 * umbenannt werden konnte.
	 */
	public boolean rename(String newName) {
		File current = new File(filePath); 
		File renamed = new File(current.getParent() + File.separator + newName + ".pak"); //$NON-NLS-1$
		
		boolean ret = current.renameTo(renamed);
		if(ret == true)
		{
			filePath = renamed.getAbsolutePath(); // Achtung der getPath() ndert sich nach dieser Operation nicht
			fireStateChange(StateChangeListener.STATE_NAME, current.getAbsolutePath(), filePath);
		}
		
		return ret;
	}
	
	/**
	 * berprft die Korrektheit aller enthaltenen Skripte und
	 * setzt dem entsprechend den Status des Pakets und benachrichtigt 
	 * bei nderung des alle <code>StateChangeListener</code>.
	 */
	public void updateValidState() 
	{
		// fehler, wenn eines der Scripte Fehlerhaft ist
		for(int i = 0; i < getNumberOfScripts(); ++i) {
			if(getScript(i).getValidState() == INVALID)
			{
				setError(Messages.getString("HTSPackage.8")); //$NON-NLS-1$
				return;
			}
		}
		
		// warnung wenn eines der Scripte Warnungen hat
		for(int i = 0; i < getNumberOfScripts(); ++i) {
			if(getScript(i).getValidState() == VALID_WITH_WARNING)
			{
				setWarning(Messages.getString("HTSPackage.9") + //$NON-NLS-1$
					Messages.getString("HTSPackage.10")); //$NON-NLS-1$
				return;
			}
		}
		
		setValid();
	}
	
	/** Registriert einen Listener, der ber Vernderungen des Paketinhalts informiert wird. */ 
	public synchronized void addContentChangeListener(ContentChangeListener listener) {
		addListener(listener);
	}
	
	/** Entfernt einen Listener, der ber Vernderungen des Paketinhalts informiert wird. */
	public synchronized void removeContentChangeListener(ContentChangeListener listener) {
		removeListener(listener);
	}
	
	/** 
	 * Informiert alle <code>ContentChangeListener</code> ber Vernderungen des
	 * Paketinhalts.
	 */
	protected synchronized void fireContentChange() {
		Iterator it = getListeners().iterator();
		while( it.hasNext() )
		{
			Object listener = it.next();
			if(listener instanceof ContentChangeListener)
				((ContentChangeListener)listener).contentChanged(this);
		}
	}
	
	/**
	 * Hrt auf die Vernderungen der enthaltenen Skripte und verndert dementsprechend
	 * seinen Status und benachrichtigt alle Listener.
	 */
	public void stateChanged(Changeable changed, int state, Object oldValue, Object newValue) {
		// wir hren auf vernderungen der integritt unserer scripte
		if(state == StateChangeListener.STATE_VALID)
		{
			updateValidState();
			fireContentChange();
		}
		else
		if(state == StateChangeListener.STATE_NAME)
		{
			setModified(true);
			fireContentChange();
		}
		else
		if(state == StateChangeListener.STATE_MODIFIED) 
		{
			fireContentChange();
		}
	}
}
