/*
 * Created on 07.09.2005
 */
package atosj.model.script;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import atosj.Messages;
import atosj.app.Project;
import atosj.component.ComponentId;
import atosj.editors.BaseChangeable;
import atosj.editors.Changeable;
import atosj.exception.AtosJException;
import atosj.exception.IntegrityException;
import atosj.exception.ParseException;
import atosj.listener.ContentChangeListener;
import atosj.listener.StateChangeListener;
import atosj.model.hts.command.HTSCleanup;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.command.HTSComponentCommand;
import atosj.model.hts.command.HTSMessage;
import atosj.model.hts.command.HTSQuestion;
import atosj.model.hts.command.HTSStart;
import atosj.model.hts.parser.HTSParser;

/** 
 * Diese Klasse reprsentiert ein Skript in der Sprache HTS.
 * @author Nicos Tegos
 */
public class HTSScript extends BaseChangeable implements Script, StateChangeListener
{
	private Vector commands; // Vector<HTSCommand>
	private String path;
	private Project project;
	
	/**
	 * Ldt ein HTS-Skript aus der Skriptdatei.
	 * @param scriptLocation Der vollstndige Pfad der Skriptdatei.
	 * @param project Das Projekt in dem sich das Skript befindet.
	 * @throws AtosJException Fehler beim Laden des Skripts.
	 */
	public HTSScript(String scriptLocation, Project project) throws AtosJException 
	{
		this.path = scriptLocation;
		this.project = project;
		
		project.getURFFile().addStateChangeListener(this);
		load();
	}
	
	/**
	 * Liefert alle Kommandos des Skripts.
	 * @return Vector(HTSCommand) Eine Kopie des Kommandovectors.
	 */
	public Vector getCommands() {
		Vector ret = new Vector(commands.size());
		ret.addAll(commands);
		return ret;
	}
	
	/**
	 * Liefert das Kommando an der angegebenen Position.
	 * @param index Der Kommandoindex im Skript.
	 */
	public HTSCommand getCommand(int index) {
		return (HTSCommand) commands.elementAt(index);
	}
	
	public int getNumOfCommands() {
		return commands.size();
	}
	
	/**
	 * Liefert den Namen dieses Skripts, d.h. den Dateinamen ohne Endung.
	 */
	public String getName() {
		File f = new File(path);
		String fileName = f.getName();
		return fileName.substring( 0, fileName.lastIndexOf('.') );
	}
	
	/**
	 * Liefert den absolute Pfad der Skriptdatei.
	 */
	public String getPath() {
		return path;
	}
	
	/**
	 * Zwei Skripte sind gleich, wenn sie den gleichen Namen tragen.
	 */
	public boolean equals(Object obj) 
	{
		if(obj == this)
			return true;
		
		if(obj instanceof HTSScript && obj != null)
		{
			return getName().equals( ((HTSScript)obj).getName() );
		}
		
		return false;
	}

	/**
	 * Nimmt eine neue Nummerierung der Kommandos vor und berprft die Struktur des Skripts. 
	 * Benachrichtigt alle Listener darber, dass sich der Inhalt des Skripts 
	 * verndert hat.
	 */
	private void renumber() {
		for(int i = 0; i < commands.size(); ++i)
			getCommand(i).setIndex(i);
		
		restructure();
		fireContentChange();
	}
	
	/**
	 * berprft die Struktur des Skripts.
	 * @see HTSScriptStructureParser#parse()
	 */
	private void restructure() {
		new HTSScriptStructureParser(this).parse();
	}
	
	/**
	 * Entfernt ein Kommando aus dem Skript.
	 * @param cmd Das zu entfernende Kommando.
	 */
	public void removeCommand(HTSCommand cmd) {
		commands.remove(cmd);
		renumber();
		
		setModified(true);
	}
	
	/**
	 * Entfernt alle angegebenen Kommando aus dem Skript.
	 * @param cmds Collection(HTSCommand) Die zu entfernenden Kommandos.
	 */
	public void removeCommands(Collection cmds) {
		commands.removeAll(cmds);
		renumber();
		
		setModified(true);
	}
	
	/**
	 * Fgt ein Kommando am Ende des Skripts an.
	 * @param cmd Das Kommando.
	 */
	public void addCommand(HTSCommand cmd) {
		addCommand(cmd, getNumOfCommands());
	}
	
	/**
	 * Fgt ein Kommando an einer bestimmten Position im Skript ein.
	 * @param cmd Das Kommando.
	 * @param index Die Einfgeposition.
	 */
	public void addCommand(HTSCommand cmd, int index) {
		commands.add(index, cmd);
		renumber();
		
		setModified(true);
	}
	
	/**
	 * Fgt alle angegebenen Kommandos an einer bestimmten Position im Skript ein.
	 * @param cmds Collection(HTSCommand) Die Kommandos.
	 * @param index Die Einfgeposition.
	 */
	public void addCommands(Collection cmds, int index) {
		commands.addAll(index, cmds);
		renumber();
		
		setModified(true);
	}
	
	/**
	 * Verschiebt ein Kommando innerhalb des Skripts.
	 * @param cmd Das zu verschiebende Kommando, muss im Skript enthalten sein.
	 * @param newIndex Die neue Position, gltiger Wertebereich 
	 * [0, <code>getNumOfCommands()</code>]
	 */
	public void moveCommand(HTSCommand cmd, int newIndex)
	{
		commands.remove( cmd.getIndex() );
		commands.add(newIndex, cmd);
		
		renumber();
		setModified(true);
	}
	
	/**
	 * Ersetzt in allen <code>HTSComponentCommands</code> die alte durch die neue ID.
	 * @param oldId Die alte ID.
	 * @param newId Die neue ID.
	 * @see atosj.model.hts.command.HTSComponentCommand
	 * @see ComponentId
	 */
	public void replaceId(ComponentId oldId, ComponentId newId)	
	{
		for(int i = 0; i < commands.size(); ++i)
		{
			if( commands.get(i) instanceof HTSComponentCommand )
			{
				HTSComponentCommand cmd = (HTSComponentCommand) commands.get(i);
				if( cmd.getComponentId().equals(oldId) )
				{
					try {
						HTSComponentCommand newCmd = cmd.replaceId(newId);
						removeCommand(cmd);
						addCommand( newCmd, cmd.getIndex() );
					} catch (ParseException e) {}
				}
			}
		}
	}
	
	/**
	 * berprft die Korrektheit jedes einzelnen Kommandos des Skripts.
	 * Setzt den validState und benachrichtigt bei nderung des
	 * validState alle <code>StateChangeListener</code>.
	 */
	private void updateValidState() 
	{
		// ein script ist ungltig wenn eines seiner Kommandos ungltig ist
		for(int i = 0; i < getNumOfCommands(); ++i)
		{
			try {
				getCommand(i).checkIntegrity(project, this);
			}
			catch(IntegrityException ex) {
				setError(Messages.getString("HTSScript.0")); //$NON-NLS-1$
				return;
			}
		}
		
		// warnung, wenn es kein start kommando gibt
		boolean hasStart = false;
		for(int i = 0; i < getNumOfCommands(); ++i) {
			if( getCommand(i) instanceof HTSStart )
			{
				hasStart = true;
				break;
			}
		}
		
		if(!hasStart)
		{
			setWarning(Messages.getString("HTSScript.1") + //$NON-NLS-1$
					Messages.getString("HTSScript.2")); //$NON-NLS-1$
			return;
		}
		
		setValid();
	}
	
	/**
	 * Gibt an, ob das Skript Kommandos enthlt, die das Eingreifen des Tester
	 * erfordern. Dazu gehrt z.B. MESSAGE.
	 * @return <code>true</code>, wenn interaktiv.
	 */
	public boolean isInteractive() {
		for(int i = 0; i < commands.size();  ++i)
		{
			if( commands.get(i) instanceof HTSQuestion
				|| commands.get(i) instanceof HTSMessage )
				return true;
		}
		
		return false;
	}
	
	/**
	 * Gibt an, ob das Skript ein CLEANUP Kommando enthlt.
	 * @return <code>true</code>, wenn CLENUP enthalten.
	 */
	public boolean hasCleanup() {
		for(int i = 0; i < commands.size();  ++i)
		{
			if( commands.get(i) instanceof HTSCleanup )
				return true;
		}
		
		return false;
	}
	
	/**
	 * Ldt die Kommandos aus der Skriptdatei.
	 * @throws AtosJException Wenn die Skriptdatei nicht geladen werden kann, z.B.
	 * weil sie nicht existiert oder der Zugriff verweigert wurde.
	 */
	private void load() throws AtosJException 
	{
		HTSParser parser = null;
		try 
		{
			parser = new HTSParser(path, project.getTestObject().getResolver());
		} catch (IOException e) {
			throw new AtosJException(Messages.getString("HTSScript.3") //$NON-NLS-1$
					+ path + Messages.getString("HTSScript.4")); //$NON-NLS-1$
		}
		
		commands = parser.parse();
		restructure();
		updateValidState();
	}

	public void save() throws AtosJException 
	{
		if( !isModified() )
			return;
		
		try
		{
			FileWriter writer = new FileWriter(path);
			Iterator it = commands.iterator();
			while( it.hasNext() ) // ComponentIds in CSV Zeilen umwandeln und rausschreiben
			{
				HTSCommand cmd = (HTSCommand) it.next();
				writer.write( cmd.toString() + "\n" ); //$NON-NLS-1$
			}
			writer.close();
			
			updateValidState();
			setModified(false);
		}
		catch(IOException ex) {
			throw new AtosJException(Messages.getString("HTSScript.6") +  getName() + Messages.getString("HTSScript.7")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	public void discard() throws AtosJException 
	{
		if( !isModified() )
			return;
		
		load();
		fireContentChange();
		setModified(false);
	}
	
	/**
	 * Lscht die Skriptdatei.
	 * @return <code>true</code>, wenn die Datei erfolgreich gelscht wurde.
	 */
	public boolean delete() 
	{
		boolean deleted = true;
		File f = new File( getPath() );
		
		if( f.exists() )
			deleted = f.delete();
		
		if(deleted)
			project.getURFFile().removeStateChangeListener(this);
		
		return deleted;
	}
	
	/**
	 * Benennt das Skript und die Skriptdatei um.
	 * @return <code>true</code>, wenn die Datei erfolgreich umbenannt wurde.
	 */
	public boolean rename(String newName) {
		File current = new File(path); 
		File renamed = new File(current.getParent() + File.separator + newName + ".hts"); //$NON-NLS-1$
		
		boolean ret = current.renameTo(renamed);
		if(ret == true)
		{
			String oldName = getName();
			path = renamed.getAbsolutePath(); // Achtung der getPath() ndert sich nach dieser Operation nicht
			fireStateChange( StateChangeListener.STATE_NAME, oldName, getName() );
		}
		
		return ret;
	}
	
	/** Registriert einen Listener, der ber Vernderungen des Skriptinhalts informiert wird. */ 
	public synchronized void addContentChangeListener(ContentChangeListener listener) {
		addListener(listener);
	}
	
	/** Entfernt einen Listener, der ber Vernderungen des Skriptinhalts informiert wird. */ 
	public synchronized void removeContentChangeListener(ContentChangeListener listener) {
		removeListener(listener);
	}
	
	/** 
	 * Informiert alle <code>ContentChangeListener</code> ber Vernderungen des
	 * Skriptinhalts. Beispielsweise, wenn Kommandos hinzugefgt oder gelscht wurden.
	 */
	protected synchronized void fireContentChange() {
		Iterator it = getListeners().iterator();
		while( it.hasNext() )
		{
			Object listener = it.next();
			if(listener instanceof ContentChangeListener)
				((ContentChangeListener)listener).contentChanged(this);
		}
	}

	/** 
	 * Hrt auf das Speichern der URF-Datei. Wurde die URF-Datei gespeichert, wird fr
	 * alle Kommandos des Skripts die berprfung auf referenzielle Integritt vorgenommen.
	 */
	public void stateChanged(Changeable changed, int state, Object oldValue, Object newValue) {
		// wir hren auf vernderungen des URFFiles
		// nur beim Speichern
		if( state == StateChangeListener.STATE_MODIFIED && newValue.equals(Boolean.FALSE) )
		{
			updateValidState();
			fireContentChange();
		}
	}
}
