package atosj.model.script;

import atosj.Messages;
import atosj.model.hts.command.HTSCleanup;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.command.HTSEndLoop;
import atosj.model.hts.command.HTSEndWhile;
import atosj.model.hts.command.HTSLoop;
import atosj.model.hts.command.HTSWhile;

/**
 * berprft ein Skript auf Wohlgeformtheit. Dazu gehrt z.B. die Prfung, dass ein Blockkommando
 * wie LOOP mit einem entsprechenden Endkommando wie ENDLOOP abgeschlossen ist.
 * @author Nicos Tegos
 */
public class HTSScriptStructureParser 
{
	private HTSScript script;
	private int index = -1; // index des aktuellen kommandos
	private int cmdSize;
	private HTSCommand currentCmd;
	
	private int numCleanup = 0; // Anzahl der Cleanup Kommandos in einem Script, max. 1 ist erlaubt. 
	
	/**
	 * Erstellt einen neuen Wohlgeformtheitsvalidator.
	 * @param script Das zu prfende Skript.
	 */
	public HTSScriptStructureParser(HTSScript script) {
		this.script = script;
		cmdSize = script.getNumOfCommands();
	}
	
	/**
	 * Liefert das nchste Kommando.
	 * @return Das nchste Kommando oder <code>null</code>, wenn
	 * das Ende des Skripts erreicht wurde.
	 */
	private HTSCommand getNextCommand() {
		++index;
		if(index < cmdSize)
			return script.getCommand(index);
		else
			return null;
	}
	
	/**
	 * Startet die strukturelle Prfung des Skripts.
	 */
	public void parse() 
	{
		numCleanup = 0;
		
		// alle Fehler zurcksetzen
		for(int i = 0; i < cmdSize; ++i)
			script.getCommand(i).setError(null);

		// einzelne Kommandos parsen
		while( (currentCmd = getNextCommand()) != null )
			parseCommand();
	}
	
	/** berprft das aktuelle Kommando.  */
	private void parseCommand() 
	{
		if(currentCmd instanceof HTSLoop)
		{
			parseLoopStructure( (HTSLoop)currentCmd );
		}
		else if(currentCmd instanceof HTSWhile)
		{
			parseWhileStructure( (HTSWhile)currentCmd );
		}
		else if(currentCmd instanceof HTSEndLoop)
		{
			((HTSEndLoop)currentCmd).setStart(null);
		}
		else if(currentCmd instanceof HTSEndWhile)
		{
			((HTSEndWhile)currentCmd).setStart(null);
		}
		else if(currentCmd instanceof HTSCleanup)
		{
			++numCleanup;
			if(numCleanup > 1)
				currentCmd.setError(Messages.getString("HTSScriptStructureParser.0")); //$NON-NLS-1$
		}
	}
	
	/**
	 * berprft ein LOOP Kommando, ob es auch ein ENDLOOP hat und ob
	 * nicht etwa ein CLEANUP enthalten ist.
	 * @param loop Das zu prfende LOOP.
	 */
	private void parseLoopStructure(HTSLoop loop) 
	{
		HTSEndLoop end = null;
		while( (currentCmd = getNextCommand()) != null )
		{
			if(currentCmd instanceof HTSEndLoop)
			{
				end = (HTSEndLoop) currentCmd;
				end.setStart(loop);
				break;
			}
			else if(currentCmd instanceof HTSCleanup) 
			{
				++numCleanup;
				currentCmd.setError(Messages.getString("HTSScriptStructureParser.1")); //$NON-NLS-1$
			}
			else
			{
				parseCommand();
			}
		}
		
		loop.setEnd(end);
	}
	
	/**
	 * berprft ein WHILE Kommando, ob es auch ein ENDWHILE hat und ob
	 * nicht etwa ein CLEANUP enthalten ist.
	 * @param whileCmd Das zu prfende LOOP.
	 */
	private void parseWhileStructure(HTSWhile whileCmd) 
	{
		HTSEndWhile end = null;
		while( (currentCmd = getNextCommand()) != null )
		{
			if(currentCmd instanceof HTSEndWhile)
			{
				end = (HTSEndWhile) currentCmd;
				end.setStart(whileCmd);
				break;
			}
			else if(currentCmd instanceof HTSCleanup) 
			{
				++numCleanup;
				currentCmd.setError(Messages.getString("HTSScriptStructureParser.2")); //$NON-NLS-1$
			}
			else
			{
				parseCommand();
			}
		}
		
		whileCmd.setEnd(end);
	}
}
