package atosj.model.urf;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.component.ComponentId;
import atosj.editors.BaseChangeable;
import atosj.exception.AtosJException;
import atosj.listener.ContentChangeListener;
import atosj.model.hts.parser.HTSSyntax;
import atosj.util.CSVLine;
import atosj.util.CSVParser;

/**
 * Speichert eindeutige Bezeichner zu den Components des Testobjekts.
 * Diese werden fr die Prfung der referentiellen Ingegritt der
 * HTS Kommandos genutzt.
 * @author Nicos Tegos
 */
public class URFFile extends BaseChangeable
{
	private HashSet ids = new HashSet(); // Vector<ComponentId>
	private String filePath;
	
	/**
	 * Ldt die URF-Datei.
	 * @param filePath Der absolute Pfad der URF-Datei.
	 * @throws AtosJException Syntaktischer Fehler in der URF-Datei, 
	 * URF-Datei nicht gefunden.
	 */
	public URFFile(String filePath) throws AtosJException 
	{
		this.filePath = filePath;
		load();
	}

	public Vector getIds() {
		return new Vector(ids);
	}
	
	/**
	 * Entfernt eine ID.
	 * @param id Die zu lschende id.
	 * @return <code>true</code> wenn <code>id</code> enthalten war, <code>false</code> sonst.
	 */
	public boolean removeId(ComponentId id) 
	{
		boolean ret = ids.remove(id);
		if(ret)
		{
			fireContentChange();
			setModified(true);
		}
		
		return ret;
	}
	
	/**
	 * Entfernt eine Menge von IDs.
	 * @param toRemove Collection(ComponentId) die zu lschenden IDs.
	 */
	public void removeIds(Collection toRemove) 
	{
		boolean removed = false;
		for (Iterator iter = toRemove.iterator(); iter.hasNext();) {
			if( ids.remove(iter.next()) )
				removed = true;
		}
		
		if(removed)
		{
			fireContentChange();
			setModified(true);
		}
	}
	
	/**
	 * Fgt eine ID hinzu.
	 * @param id Die hinzuzufgende ID.
	 * @return <code>true</code> falls noch nicht enthalten, <code>false</code> sonst.
	 */
	public boolean addId(ComponentId id) 
	{
		boolean ret =  ids.add(id);
		if(ret)
		{
			fireContentChange();
			setModified(true);
		}
		
		return ret;
	}
	
	/**
	 * Fgt eine Menge von IDs hinz.
	 * @param toAdd Collection(ComponentId) die zu hinzuzufgenden IDs.
	 */
	public void addIds(Collection toAdd) 
	{
		boolean added = false;
		for (Iterator iter = toAdd.iterator(); iter.hasNext();) 
		{
			ComponentId id = (ComponentId) iter.next();
			
			// wir wandeln eine id mit eines Components im Hauptfenster um,
			// so dass statt des Hauptfenstertitels das Schlsselwort MAIN steht
			String mainWindowTitle = AtosJApp.getInstance().getInput().getTestObject().getMainWindow();
			if( id.getWindowName().equals(mainWindowTitle) )
			{
				// beim Hauptfenster sind Fenstername und Name immer gleich
				if( id.getClassName().equals(HTSSyntax.TYPE_WINDOW) )
					id = new ComponentId(HTSSyntax.DEF_MAIN, id.getClassName(), HTSSyntax.DEF_MAIN);
				else
					id = new ComponentId(id.getName(), id.getClassName(), HTSSyntax.DEF_MAIN);
			}
			
			if( ids.add(id) )
				added = true;
		}
		
		if(added)
		{
			fireContentChange();
			setModified(true);
		}
	}
	
	/**
	 * Gibt an ob <code>id</code> enthalten ist.
	 * @param id Die ID.
	 * @return <code>true</code>, wenn enthalten.
	 */
	public boolean containsId(ComponentId id) {
		return ids.contains(id);
	}
	
	/*public HashSet getWindowNames() 
	{
		HashSet windowNames = new HashSet();
		
		Iterator it = ids.iterator();
		while( it.hasNext() )
		{
			windowNames.add( ((ComponentId)it.next()).getWindowName() ); 
		}
		
		return windowNames;
	}
	
	public HashSet getClassNames() 
	{
		HashSet classNames = new HashSet();
		
		Iterator it = ids.iterator();
		while( it.hasNext() )
		{
			classNames.add( ((ComponentId)it.next()).getClassName() ); 
		}
		
		return classNames;
	}
	
	public HashSet getComponentNames() 
	{
		HashSet componentNames = new HashSet();
		
		Iterator it = ids.iterator();
		while( it.hasNext() )
		{
			componentNames.add( ((ComponentId)it.next()).getName() ); 
		}
		
		return componentNames;
	}*/
	
	/**
	 * Ldt die Datei und initialisiert die Liste der IDs.
	 * @throws AtosJException Syntaktischer Fehler in der URF-Datei, 
	 * URF-Datei nicht gefunden
	 */
	private void load() throws AtosJException
	{
		ids = new HashSet();
		
		CSVParser parser = null;
		try {
			parser = new CSVParser(filePath);
		}
		catch(FileNotFoundException e) // datei nicht gefunden
		{
			try // neu anlegen
			{
				File f = new File(filePath);
				f.createNewFile();
				parser = new CSVParser(filePath);
			}catch(IOException e2) // neu anlegen fehlgeschlagen also endgltig schluss
			{
				throw new AtosJException(Messages.getString("URFFile.0")); //$NON-NLS-1$
			}
		}
		catch(IOException e) // anderer IOFehler
		{
			throw new AtosJException(Messages.getString("URFFile.1")); //$NON-NLS-1$
		}
		
		
		CSVLine line; 
		while( (line = parser.getNextLine()) != null )
		{
			ids.add( new ComponentId(
					line.getElementUnquoted(0), // Name
					line.getElement(1), // Klassenname
					line.getElementUnquoted(2)) // Fenstertitel
			);
		}
	}
	
	public void discard() throws AtosJException 
	{
		if( !isModified() )
			return;
		load();
		fireContentChange();
		setModified(false);
	}
	
	public void save() throws AtosJException
	{
		if( !isModified() )
			return;
		
		try
		{
			FileWriter writer = new FileWriter(filePath);
			Iterator it = ids.iterator();
			while( it.hasNext() ) // ComponentIds in CSV Zeilen umwandeln und rausschreiben
			{
				ComponentId id = (ComponentId) it.next();
				Vector elements = new Vector(3);
				elements.add( "\"" + id.getName() + "\"" ); //$NON-NLS-1$ //$NON-NLS-2$
				elements.add( id.getClassName() );
				elements.add( "\"" + id.getWindowName() + "\"" ); //$NON-NLS-1$ //$NON-NLS-2$
				CSVLine line = new CSVLine(elements);
				
				writer.write( line.toString() + "\n" ); //$NON-NLS-1$
			}
			writer.close();
		}
		catch(IOException ex) {
			throw new AtosJException(Messages.getString("URFFile.7")); //$NON-NLS-1$
		}
		
		setModified(false);
	}
	
	/* (non-Javadoc)
	 * @see atosj.editors.Changeable#getName()
	 */
	public String getName() {
		return Messages.getString("URFFile.8"); //$NON-NLS-1$
	}
	
	/** Registriert einen Listener, der ber Vernderungen des URF-Inhalts informiert wird. */ 
	public synchronized void addContentChangeListener(ContentChangeListener listener) {
		addListener(listener);
	}
	
	/** Entfernt einen Listener, der ber Vernderungen des URF-Inhalts informiert wird. */ 
	public synchronized void removeContentChangeListener(ContentChangeListener listener) {
		removeListener(listener);
	}
	
	/** 
	 * Informiert alle <code>ContentChangeListener</code> ber Vernderungen des
	 * Inhalts der URF-Datei. Beispielsweise, wenn IDs hinzugefgt oder gelscht wurden.
	 */
	protected synchronized void fireContentChange() {
		Iterator it = getListeners().iterator();
		while( it.hasNext() )
		{
			Object listener = it.next();
			if(listener instanceof ContentChangeListener)
				((ContentChangeListener)listener).contentChanged(this);
		}
	}
}
