/*
 * Created on 25.09.2005
 */
package atosj.model.urf;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.exception.AtosJException;
import atosj.exception.MainWindowNotFoundException;
import atosj.listener.AtosJTopLevelWindowListener;
import atosj.listener.FinishedListener;
import atosj.util.WindowMonitor;

/**
 * Diese Klasse wird durch einen <code>URFReceiver</code> gestartet.
 * Sie startet das Testobjekt und initialisiert die Aufzeichnung von
 * <code>ComponentIds</code>. Neue IDs werden bei jedem ffnen eines
 * Fenster im Testobjekt generiert und an den Kommunikationspartner
 * im ATOSj Prozess (<code>URFReceiver</code>) weitergeleitet.
 * This class is not intended to be run solely.
 * @see atosj.model.urf.URFReceiver
 * @author Nicos Tegos
 */
public class URFGenerator extends UnicastRemoteObject implements AtosJTopLevelWindowListener, IURFGenerator, FinishedListener
{
	private IURFReceiver remoteObject;
	
	/**
	 * Erstellt einen neuen Generator fr <code>ComponentIds</code>.
	 * @param remoteObj Der Empfnger der generierten IDs.
	 */
	public URFGenerator(IURFReceiver remoteObj) throws RemoteException	
	{
		super();
		this.remoteObject = remoteObj;
		// dem receiver melden wenn der Prozess beendet wird.
		Runtime.getRuntime().addShutdownHook(new Thread(){
			public void run() {
				try {
					if(remoteObject != null)
						remoteObject.clientShutdown();
				} catch (RemoteException e) {System.err.println(e.getMessage());}
			}
		});
	}

	/**
	 * ffnet sich ein Fenster im Testobjekt, werden IDs zu allen enthaltenen Elementen generiert.
	 * Diese werden dann an den Kommunikationspartner im ATOSj Prozess weitergeleitet.
	 * @see IURFReceiver#publishIds(java.util.Vector)
	 */
	public void topLevelWindowCreated(Object w) 
	{
		try {
			remoteObject.publishIds( WindowMonitor.getInstance().getComponentFinder().getComponentIds(w) );
		} catch (RemoteException e) { /* egal */ }
	}

	/** Leere Implementation. */
	public void topLevelWindowDestroyed(Object w) 
	{
		// do nothing
	}
		
	/**
	 * Startet das Testobjekt und registriert sich als Listener fr Fensterereignisse.
	 * @throws MainWindowNotFoundException wenn das Hauptfenster des Testobjekts
	 * nicht startet.
	 */
	public void start(TestObject testee) throws AtosJException
	{
		testee.start(10000, null, this, this, null);
	}
	
	/** Entfernt dieses Objekt aus der RMI-Registry damit der Prozess beendet wird. */
	public void testObjectFinished() 
	{
		try {
			// wir mssen uns abmelden, sonst wird der Prozess nicht beendet
			UnicastRemoteObject.unexportObject(this, true);
		} 
		catch (Exception e) {/*egal */}
	}
	
	/**
	 * Startet das Testobjekt und den ID Generator. Der Generator registriert
	 * sich unter dem angegebenen Namen in der RMI-Registry.
	 * @param args Programmargumente
	 * <ul>
	 * <li>args[0] - Der Name des Kommunikationspartners
	 * (<code>IURFReceiver</code> in der RMI-Registry.</li>
	 * <li>args[1] - Der Name unter dem sich der Generator in der
	 * RMI-Registry registrieren soll.</li>
	 * </ul>
	 * @see IURFReceiver
	 */
	public static void main(String[] args) 
	{
		if (args.length != 2)	{
			System.err.println(Messages.getString("URFGenerator.0")); //$NON-NLS-1$
			System.exit(1);	
		}
		
		try 
		{
			IURFReceiver remoteObject = (IURFReceiver) Naming.lookup(args[0]);
			URFGenerator generator = new URFGenerator(remoteObject);
			// wir fgen den generator in die RMI Registry ein.
			Naming.rebind(args[1], generator);
		}  
		catch (Exception e) 
		{
			System.err.println(e.getMessage());
			System.exit(1);
		}
	}

}
