/*
 * Created on 12.06.2006
 */
package atosj.rmi;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Vector;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.exception.AtosJException;
import atosj.listener.StreamListener;
import atosj.util.Snooze;

/**
 * Ein <code>Receiver</code> mit bidirektionaler Kommunikationsmglichkeit. 
 * Bildet einen Kommunikationsendpunkt im ATOSj Prozess der andere ist im Testobjekt Prozess.
 * Startet den entfernten Prozess und darin ein Objekt des Interfaces Remote 
 * mit dem dann kommuniziert werden kann.
 * 
 * @author Nicos Tegos
 */
public abstract class BidirectionalReceiver extends Receiver 
{
	private Remote remoteObject; // Der Kommunikationspartner im Remote Prozess.
	private String remoteNaming; // RMI Name des Kommunikationspartners im Remote Prozess.
	private Vector remoteNamings = new Vector(); // // Vector<String> die Namings aller bisher gestartetet RemoteObjekte
	
	/**
	 * Erstellt den Kommunikationsendpunkt im ATOSj Prozess.
	 * @param testee Das Testobjekt.
	 */
	public BidirectionalReceiver(TestObject testee) throws RemoteException {
		super(testee);
	}
		
	/**
	 * bergibt zustzlich zu den normalen <code>procArgs</code>, als erstes Argument noch das RMI-Naming
	 * fr ein Objekt des Typs <code>Remote</code>, welches dann im entfernten Prozess in die RMI-Registry
	 * eingefgt wird. Nach dem starten wird auf die Bereitstellung des Kommunikationspartners in der
	 * RMI-Registry gewartet. Danach kann das dieser mit der Funktion <code>getRemoteObject()</code>
	 * ermittelt werden.
	 * @param procArgs Die Argumente fr den Remote Prozess oder <code>null</code>, falls keine Argumente.
	 */
	protected void startProcess(boolean doCover, Class procClass, String[] procArgs, StreamListener stdoutListener, StreamListener stderrListener) throws AtosJException 
	{
		remoteNaming = RMIConnectionManager.getInstance().getNextServerName();
		remoteNamings.add(remoteNaming);
		String[] newArgs;
		if(procArgs == null)
		{
			newArgs = new String[] {remoteNaming};
		}
		else
		{
			newArgs = new String[procArgs.length+1];
			newArgs[0] = remoteNaming;
			System.arraycopy(procArgs, 0, newArgs, 1, procArgs.length);
		}
		
		super.startProcess(doCover, procClass, newArgs, stdoutListener, stderrListener);
		waitForRemoteObject();
	}
	
	/**
	 * Ruft <code>startProcess(false, procClass, procArgs, stdoutListener, stderrListener)</code>.
	 */
	protected void startProcess(Class procClass, String[] procArgs, StreamListener stdoutListener, StreamListener stderrListener) throws AtosJException 
	{
		startProcess(false, procClass, procArgs, stdoutListener, stderrListener);
	}

	/**
	 * Wartet bis der Kommunikationspartner im Remote Prozess bereit ist.
	 * @throws AtosJException Falls der Prozess beendet wird bevor der Kommunikationspartner
	 * gefunden wurde.
	 */
	private void waitForRemoteObject() throws AtosJException
	{
		// warten auf die verfgbarkeit des Remoteobjekts HTSPlayer
		boolean started = false; 
		while(started == false)	
		{
			new Snooze(20);
			
			try	
			{
				getProcess().exitValue();
				//wenn wir hier sind, ist der Prozess definitiv beendet :-(
				throw new AtosJException(Messages.getString("BidirectionalReceiver.0")); //$NON-NLS-1$
			}
			catch(IllegalThreadStateException ex) // der Prozess ist noch nicht beendet 
			{
				try
				{
					remoteObject = Naming.lookup(remoteNaming);
					started = true;
				}
				catch (MalformedURLException e) {
					//kann eigentlich nie passieren
					throw new AtosJException(Messages.getString("BidirectionalReceiver.1") + e.getMessage()); //$NON-NLS-1$
				}
				catch (RemoteException e) {
					// die Registry ist nicht da
					throw new AtosJException(Messages.getString("BidirectionalReceiver.2") + e.getMessage()); //$NON-NLS-1$
				}
				catch (NotBoundException e) {
					// solange das remoteobject nicht gestartet wurde warten wir weiter
				}	
			}
		}
	}
	
	/**
	 * Liefert den aktuellen Kommunikationspartner im Remote Prozess oder <code>null</code>, falls nicht verfgbar.
	 */
	protected Remote getRemoteObject() {
		return remoteObject;
	}
	
	/**
	 * Liefert den RMI-Namen des aktuellen Kommunikationspartners im Remote Prozess
	 * oder <code>null</code>, falls nicht verfgbar.
	 */
	protected String getRemoteNaming() {
		return remoteNaming;
	}
	
	/**
	 * Entfernt zustzlich zur super Implementation die erstellten Kommunikationspartner
	 * aus der RMI-Registry.
	 */
	public void stop() 
	{
		if( !isRunning() )
			return;
		
		for(int i = 0; i < remoteNamings.size(); ++i)
		{
			try {
				Naming.unbind( (String) remoteNamings.get(i) );
			} catch (Exception e) { /*egal*/ }
		}
		remoteNamings.clear();
		remoteNaming = null;
		remoteObject = null;
		
		super.stop();
	}
}
