/*
 * Created on 18.02.2006
 */
package atosj.rmi;

import java.io.File;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.UnmarshalException;
import java.util.HashMap;

import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import atosj.Messages;
import atosj.app.TestObject;
import atosj.dialog.CommandStatisticsTableElement;
import atosj.dialog.PackageTableFilter;
import atosj.dialog.ScriptStatisticsTableElement;
import atosj.exception.AtosJException;
import atosj.exception.ExecutionException;
import atosj.listener.PackagePlayerReceiverListener;
import atosj.listener.PlayerReceiverListener;
import atosj.listener.StreamListener;
import atosj.model.hts.command.HTSAction;
import atosj.model.hts.command.HTSCleanup;
import atosj.model.hts.command.HTSCommand;
import atosj.model.hts.command.HTSComment;
import atosj.model.hts.command.HTSDisable;
import atosj.model.hts.command.HTSEndLoop;
import atosj.model.hts.command.HTSEndWhile;
import atosj.model.hts.command.HTSLoop;
import atosj.model.hts.command.HTSRead;
import atosj.model.hts.command.HTSStart;
import atosj.model.hts.command.HTSTest;
import atosj.model.hts.command.HTSWhile;
import atosj.model.hts.command.HTSWindowExists;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;
import atosj.util.Snooze;
import atosj.util.Statistics;

/**
 * Fhrt ein Skript oder ein Testpaket aus. Erstellt bei Bedarf einen Remote-Prozess
 * fr das Testobjekt. Einige Kommandos laufen im ATOSj-Prozess andere werden direkt
 * im Prozess des Testobjektes durch den <code>HTSCommandExecutor</code> ausgefhrt. 
 * Die Ergebnisse der Ausfhrung werden den einzelnen Listenern zur Laufzeit bekanntgegeben.
 * 
 * @author Nicos Tegos
 * @see atosj.model.script.HTSScript
 * @see atosj.model.pack.HTSPackage
 */
public class HTSPlayer extends BidirectionalReceiver implements IHTSPlayer
{
	private class UserBreak extends Exception {}
	
	private Shell runDlgShell; // Fenster des ausfhrenden Dialogs
	
	private HTSScript[] scripts; // alle Scripte
	private Object[] commandStatistics; // beinhaltet zu jedem Script ein CommandStatistics[]
	private ScriptStatisticsTableElement[] scriptStatistics;
	private boolean hasError = false;
	private boolean stoppedByUser = false;
	private int currentScriptIndex = 0;
	private int currentCmdIndex = 0;
	
	private HashMap nameToVarMap = new HashMap(); // <String, Object> mapped VariablenNamen auf LaufzeitVariablen
	private int delay = 0;
	private boolean haltOnError = false;
	private boolean haltPackage = false;
	private StreamListener[] stdoutListeners;
	private StreamListener[] stderrListeners;
	private PackageTableFilter scriptFilter;
	
	/**
	 * Erstellt einen neuen Ausfhrer fr ein einzelnes Skript.
	 * @param testee Das Testobjekt.
	 * @param script Das Skript, das ausgefhrt werden soll.
	 */
	public HTSPlayer(TestObject testee, HTSScript script) throws RemoteException 
	{
		super(testee);

		scripts = new HTSScript[1];
		scriptStatistics = new ScriptStatisticsTableElement[1];
		scripts[0] = script;
		scriptStatistics[0] = new ScriptStatisticsTableElement(script, 0);
		
		commandStatistics = new Object[1];
		CommandStatisticsTableElement[] realStatistics = new CommandStatisticsTableElement[script.getNumOfCommands()];
		commandStatistics[0] = realStatistics;
		for(int i = 0; i < script.getNumOfCommands(); ++i) 
			realStatistics[i] =  new CommandStatisticsTableElement(script.getCommand(i));
	}
	
	/**
	 * Erstellt einen neuen Ausfhrer fr ein Testpaket.
	 * @param testee Das Testobjekt.
	 * @param pack Das Testpaket, das ausgefhrt werden soll.
	 */
	public HTSPlayer(TestObject testee, HTSPackage pack) throws RemoteException 
	{
		super(testee);
		
		scripts = new HTSScript[pack.getNumberOfScripts()];
		scriptStatistics = new ScriptStatisticsTableElement[pack.getNumberOfScripts()];
		commandStatistics = new Object[pack.getNumberOfScripts()];
		for(int i = 0; i < pack.getNumberOfScripts(); ++i)
		{
			HTSScript script = pack.getElement(i).getScript();
			scripts[i] = script;
			scriptStatistics[i] = new ScriptStatisticsTableElement(script, i);
			
			CommandStatisticsTableElement[] realStatistics = new CommandStatisticsTableElement[script.getNumOfCommands()];
			commandStatistics[i] = realStatistics;
			for(int j = 0; j < script.getNumOfCommands(); ++j)
				realStatistics[j] =  new CommandStatisticsTableElement(script.getCommand(j));
		}
	}
	
	/**
	 * Startet die Ausfhrung einer einzelnen Testsequenz.
	 * @param delay Pause zwischen den Kommandos in ms.
	 * @param haltOnError wenn <code>true</code> wird die Ausfhrung nach einem Fehler beendet, 
	 * wenn <code>false</code> wird mit der Ausfhrung trotz Fehler fortgefahren.
	 * @param stdoutListener Wird ber neue Zeichen in stdout des Remote-Prozesses informiert. 
	 * @param stderrListener Wird ber neue Zeichen in stderr des Remote-Prozesses informiert.
	 */
	public void start(Shell runDlgShell, int delay, boolean haltOnError, StreamListener stdoutListener, StreamListener stderrListener) 
	{
		stdoutListeners = new StreamListener[1];
		stdoutListeners[0] = stdoutListener;
		stderrListeners = new StreamListener[1];
		stderrListeners[0] = stderrListener;
		start(runDlgShell, null, delay, haltOnError, true, stdoutListeners, stderrListeners);
	}
	
	/**
	 * Startet die Ausfhrung des Testpaketetes mit Filter fr Skripte.
	 * @param scriptFilter Dieser Filter gibt an welche Skripte des Testpaketes ausgefhrt werden sollen
	 * und welche nicht, optional auch <code>null</code> fr keine Filterung.
	 * @param delay Pause zwischen den Kommandos in ms.
	 * @param haltOnError wenn <code>true</code> wird die Ausfhrung des aktuellen Skriptes nach einem Fehler 
	 * bei der Ausfhrung eines Kommandos beendet, wenn <code>false</code> wird mit der Ausfhrung trotz 
	 * Fehler fortgefahren.
	 * @param haltPackage wenn <code>true</code> wird die Ausfhrung des Paketes nach einem 
	 * fehlerhaften Skript beendet, sonst werden alle Skripte, ungeachtet von Fehlern, abgespielt.
	 * @param stdoutListeners Wird ber neue Zeichen in stdout des Remote-Prozesses informiert. 
	 * Der Listener wird entsprechend des Array Index seinem Skript zugeordnet.
	 * @param stderrListeners Wird ber neue Zeichen in stderr des Remote-Prozesses informiert. 
	 * Der Listener wird entsprechend des Array Index seinem Skript zugeordnet.
	 */
	public void start(Shell runDlgShell, PackageTableFilter scriptFilter, int delay, boolean haltOnError, boolean haltPackage, StreamListener[] stdoutListeners, StreamListener[] stderrListeners) 
	{
		hasError = false;
		stoppedByUser = false;
		resetStatistics();
		
		this.runDlgShell = runDlgShell;
		this.scriptFilter = scriptFilter;
		this.delay = delay;
		this.haltOnError = haltOnError;
		this.haltPackage = haltPackage;
		this.stdoutListeners = stdoutListeners;
		this.stderrListeners = stderrListeners;
		
		currentCmdIndex = 0;
		currentScriptIndex = 0;
		
		nameToVarMap.clear();
		setRunning(true);
		play();
	}
	
	/**
	 * Startet den Prozess fr den <code>HTSCommandExecutor</code>.
	 * @see HTSCommandExecutor
	 */
	protected void startProcess() throws AtosJException 
	{
		super.startProcess(
				true,
				HTSCommandExecutor.class, 
				null, 
				stdoutListeners[currentScriptIndex], 
				stderrListeners[currentScriptIndex]
		);
	}
//////////////////////////////////////////////////////////////////////	
	public Object getVar(String varName) {
		return nameToVarMap.get(varName);
	}
	
	public void setVar(String varName, Object value) {
		nameToVarMap.put(varName, value);
	}
	
	private ICommandExecutor getExecutor() {
		return (ICommandExecutor) getRemoteObject();
	}
	
	/**
	 * bernimmt die Ausfhrung.
	 */
	private void play() 
	{
		try {
			// Scripte abspielen
			for(currentScriptIndex = 0; currentScriptIndex < scripts.length; ++currentScriptIndex)
			{
				HTSScript script = scripts[currentScriptIndex];
				
				// script nur abspielen wenn es nicht gefiltert wurde
				boolean runScript = true;
				if(scriptFilter != null)
					runScript = scriptFilter.select(script);
				
				if( runScript )
				{
					boolean success = playScript();
					if(!success && haltPackage) // gesamtes Package bei einem Fehler innerhalb eines Scriptes abbrechen
						break;
				}
			}
			
			stop();
		} 
		catch(UserBreak e) {}
	}
	
	/**
	 * Spielt das aktuelle Skript ab.
	 * @return <code>true</code> wenn alle Kommandos erfolgreich ausgefhrt wurden,
	 * <code>false</code> sonst.
	 */
	private boolean playScript() throws UserBreak
	{
		scriptStarted();
		setNextIndex(0);
				
		long startTime = System.currentTimeMillis();
		
		while( hasNext() )
		{
			boolean success = playNextCommand();
			
			// Verzgerung abwarten
			if( hasNext() )
			{
				if(success) {
					new Snooze(delay);
				}
				else {
					if(!haltOnError)
						new Snooze(delay);
				}
			}
			
			if(!success && haltOnError) // bei einem Fehler Cleanup ausfhren und Script stoppen
			{
				playCleanup();
				break;
			}
		}
		
		// Statistik fr das Script erstellen
		int stopTime = (int) (System.currentTimeMillis() - startTime);
		String error = getErrors(currentScriptIndex);
		Statistics stat;
		if( error.length() != 0 )
			stat = new Statistics(currentScriptIndex, stopTime, error);
		else
			stat = new Statistics(currentScriptIndex, stopTime);
		
		scriptExecuted(stat);
		return !stat.hasError();
	}
	
	/**
	 * Spielt das Kommando mit dem Index <code>currentCmdIndex</code>
	 * im aktuellen Skript ab.
	 * @return <code>true</code> wenn das Kommando erfolgreich ausgefhrt wurde,
	 * <code>false</code> sonst.
	 */
	private boolean playNextCommand() throws UserBreak
	{
		HTSCommand next = getCurrentScript().getCommand(currentCmdIndex);
		if( !isExecuteable(next) ) // nicht abspielbare Kommandos werden ignoriert
		{
			setNextIndex( next.getIndex() + 1 );
			return true;
		}
		
		if(next instanceof HTSLoop)
		{
			return playLoop( (HTSLoop)next );
		}
		else if(next instanceof HTSEndLoop)
		{
			return playEndLoop( (HTSEndLoop)next );
		}
		else if(next instanceof HTSWhile)
		{
			return playWhile( (HTSWhile)next );
		}
		else if(next instanceof HTSEndWhile)
		{
			return playEndWhile( (HTSEndWhile)next );
		}
		else 
			return playNormalCommand(next);
	}
	
	/** 
	 * Fhrt ein LOOP Kommando aus und setzt den Index des nchsten Kommandos. 
	 * Die Kommandos innerhalb des LOOP-Blockes werden  nicht ausgefhrt.
	 * @param loop Das auszufhrende LOOP Kommando.
	 * @return <code>true</code> wenn das Kommando erfolgreich ausgefhrt wurde,
	 * <code>false</code> sonst.
	 * @see #setNextIndex(int)
	 * @see HTSLoop#execute(IHTSPlayer)
	 */
	private boolean playLoop(HTSLoop loop) throws UserBreak
	{
		boolean success = executeCommand(loop); // weiterzhlen
		if(success)
		{
			setNextIndex(loop.getIndex()+1); // das beenden der schleife erfolgt in playEndLoop
		}
		else // ExecutionException -> Schleife beenden
		{
			loop.reset();
			setNextIndex(loop.getEnd().getIndex()+1);
		}
		
		return success;
	}
	
	/** 
	 * Setzt den Index des nchsten Kommandos anhand der Schleifendurchlufe
	 * im zugehrigen LOOP Kommando. 
	 * @param endloop Das ENDLOOP Kommando.
	 * @return <code>true</code> wenn das Kommando erfolgreich ausgefhrt wurde,
	 * <code>false</code> sonst.
	 * @see #setNextIndex(int)
	 * @see HTSEndLoop#execute(IHTSPlayer)
	 */
	private boolean playEndLoop(HTSEndLoop endloop) throws UserBreak
	{
		boolean success = executeCommand(endloop);
		if(success)
		{
			HTSLoop loop =  (HTSLoop) endloop.getStart();
			if( loop.isFinished() ) // schleife beenden
			{
				loop.reset();
				setNextIndex( endloop.getIndex()+1 );
			}
			else
				setNextIndex( endloop.getStart().getIndex() );  /*Rcksprung zum Loop Kommando*/
		}
		else
			setNextIndex(endloop.getIndex()+1);
		
		return success;
	}
	
	/** 
	 * Fhrt ein WHILE Kommando aus und setzt den Index des nchsten Kommandos. 
	 * Die Kommandos innerhalb des WHILE-Blockes werden nicht ausgefhrt.
	 * @param whileCmd Das auszufhrende WHILE Kommando.
	 * @return <code>true</code> wenn das Kommando erfolgreich ausgefhrt wurde,
	 * <code>false</code> sonst.
	 * @see #setNextIndex(int)
	 * @see HTSWhile#execute(IHTSPlayer)
	 */
	private boolean playWhile(HTSWhile whileCmd) throws UserBreak
	{
		boolean success = executeCommand(whileCmd);
		if(success)
		{
			if( whileCmd.isFinished() ) // schleife beenden
				setNextIndex( whileCmd.getEnd().getIndex()+1 );
			else
				setNextIndex( whileCmd.getIndex() + 1 );
		}
		else // ExecutionException -> Schleife beenden
		{
			setNextIndex( whileCmd.getEnd().getIndex()+1 );
		}
		
		return success;
	}
	
	/** 
	 * Setzt den Index des nchsten Kommandos anhand des letzten Ergebnisses
	 * der Ausfhrung des zugehrigen WHIlE Kommandos. 
	 * @param endwhile Das ENDWHILE Kommando.
	 * @return <code>true</code> wenn das Kommando erfolgreich ausgefhrt wurde,
	 * <code>false</code> sonst.
	 * @see #setNextIndex(int)
	 * @see HTSEndWhile#execute(IHTSPlayer)
	 */
	private boolean playEndWhile(HTSEndWhile endwhile) throws UserBreak
	{
		boolean success = executeCommand(endwhile);
		if(success)
			setNextIndex( endwhile.getStart().getIndex() ); /*Rcksprung zum WHILE Kommando*/
		else
			setNextIndex(endwhile.getIndex()+1);
		
		return success;
	}
	
	/**
	 * Fhrt ein Kommando aus das keiner Spezialbehandlung bedarf.
	 * Setzt den Kommandoindex auf das folgende Kommando.
	 * @param cmd Das auszufhrende Kommando.
	 * @return <code>true</code> wenn das Kommando erfolgreich ausgefhrt wurde,
	 * <code>false</code> sonst.
	 */
	private boolean playNormalCommand(HTSCommand cmd) throws UserBreak
	{
		boolean success = executeCommand(cmd);
		setNextIndex( cmd.getIndex()+1 );
		return success;
	}
	
	/**
	 * Fhrt das Kommando entsprechend seiner Spezifika aus. Vor der
	 * Ausfhrung von START wird erst ein entsprechender Remote-Prozess
	 * erstellt. Remote-Kommandos werden auch im Remote-Prozess ausgefhrt.
	 * Erstellt eine Statistik ber das Ergebnis der Ausfhrung und
	 * leitet sie weiter.
	 * @param cmd Das auszufhrende Kommando.
	 * @return <code>true</code> wenn die Ausfhrung erfolgreich war, false
	 * bei einem Fehler, z.B. durch ExecutionException oder RemoteException.
	 * @see HTSPlayer#isRemoteCommand(HTSCommand)
	 */
	private boolean executeCommand(HTSCommand cmd) throws UserBreak
	{
		commandStarted(cmd);
		long startTime = System.currentTimeMillis();
		
		try 
		{
			// wir starten den remoteProzess
			if( cmd instanceof HTSStart )
			{
				try {
					startProcess();
				}
				catch(AtosJException e) {
					throw new ExecutionException(cmd, e.getMessage());
				}
			}
			
			
			if( isRemoteCommand(cmd) )
			{
				ICommandExecutor executor = getExecutor();
				if(executor != null)
				{
					try {
						getExecutor().setPrefPath(System.getProperty("user.dir")+ File.separator + "user.prf");
						getExecutor().executeCommand(cmd);
					}
					catch (RemoteException e)
					{
						// UnmarshallException heisst, dass das Testobjekt durch System.exit beendet wurde
						// Alle anderen Exceptions sind echte Fehler
						if( !(e instanceof UnmarshalException) )
							throw new ExecutionException(cmd, Messages.getString("HTSPlayer.0")); //$NON-NLS-1$
					}
				}
				else
					throw new ExecutionException(cmd, Messages.getString("HTSPlayer.1")); //$NON-NLS-1$
			}
			else
			{
				cmd.execute(this);
			}
			
			if(stoppedByUser)
				throw new UserBreak();
			
			Statistics statistics = new Statistics(cmd.getIndex(), 
					(int) (System.currentTimeMillis() - startTime));
			commandExecuted(statistics);
			
			return true;
		}
		catch(ExecutionException e) 
		{
			if(stoppedByUser)
				throw new UserBreak();
			
			Statistics statistics = new Statistics(cmd.getIndex(), 
					(int) (System.currentTimeMillis() - startTime),
					e.getMessage());
			commandExecuted(statistics);
			
			return false;
		}
	}
	
	/**
	 * Springt zum nchstgelegenen CLEANUP Kommando und fhrt dieses sowie
	 * alle folgenden Kommandos, ungeachtet eventueller Fehler, aus, es sei denn die Ausfhrung wurde vom
	 * Nutzer abgebrochen.
	 */
	private void playCleanup() throws UserBreak
	{
		// nchstes Cleanup suchen
		int numCmds = getCurrentScript().getNumOfCommands();
		int i = currentCmdIndex;
		for(; i < numCmds; ++i)
		{
			if( getCurrentScript().getCommand(i) instanceof HTSCleanup )
				break;
		}
		setNextIndex(i);
		
		while( hasNext() ) // die restlichen Kommandos, inklusive Cleanup abspielen ohne Rcksicht auf etwaige Fehler
		{
			playNextCommand();
			if( hasNext() )
				new Snooze(delay);
		}
	}
	
	/**
	 * Setzt den Index des Kommandos, welches als nchstes ausgefhrt werden soll.
	 * Dieser kann auch ausserhalb der gltigen Werte liegen, um anzuzeigen,
	 * dass die Ausfhrung beendet ist.
	 * @param index Der Index des als nchstes auszufhrenden Kommandos im aktuellen Script.
	 */
	private void setNextIndex(int index) {
		currentCmdIndex = index;
	}
	
	/**
	 * Gibt an ob noch ein weiteres Kommando ausgefhrt werden kann.
	 * @return <code>true</code> wenn noch nicht alle Kommandos des aktuellen Skripts abgespielt wurden, 
	 * <code>false</code> wenn alle Kommandos abgespielt wurden.
	 */
	private boolean hasNext() {
		return currentCmdIndex < getCurrentScript().getNumOfCommands();
	}
	
	/**
	 * Liefert das akutell ausgefhrte Script.
	 */
	private HTSScript getCurrentScript() {
		return scripts[currentScriptIndex];
	}
	
	/**
	 * Gibt an ob das Kommando im Prozess des Testobjektes ausgefhrt
	 * werden muss.
	 * @return <code>true</code>, wenn das Kommando Zugriff auf das
	 * Testobjekt bentigt und deshalb im RemoteProzess ausgefhrt werden muss.
	 */
	private boolean isRemoteCommand(HTSCommand cmd) {
		if(cmd instanceof HTSRead 
			|| cmd instanceof HTSTest
			|| cmd instanceof HTSStart
			|| cmd instanceof HTSAction
			|| cmd instanceof HTSWindowExists)
			return true;
		
		return false;
	}
	
	/**
	 * Gibt an ob dieser Kommandotyp ausfhrbar ist. Ein nicht ausfhrbares
	 * Kommando ist zum Beispiel <code>HTSComment</code> oder <code>HTSDisable</code>.
	 * @param cmd Ein Kommando.
	 * @return <code>true</code> wenn ausfhrbar, <code>false</code> sonst.
	 */
	private boolean isExecuteable(HTSCommand cmd) {
		if(cmd instanceof HTSComment || cmd instanceof HTSDisable)
			return false;
		return true;
	}
	
	public void cleanup() 
	{
		nameToVarMap.clear();
		Process proc = getProcess();
		if( proc!= null )
			proc.destroy();
		
		try {
			String executorNaming = getRemoteNaming();
			if(executorNaming != null)
				Naming.unbind(executorNaming);
		} catch (Exception e) {
		}
	}
/////////////////////////////////////////////////////////////////////////////
	
	/**
	 * Liefert die Statistiken zu allen Kommandos eines Skripts.
	 * @param index Der Index des Skriptes im Paket oder 0, wenn nur ein einzelnes Script
	 * abgespielt wird.
	 * @return Die Statistiken zu den Kommandos im Script.
	 */
	public CommandStatisticsTableElement[] getCommandStatistics(int index) {
		return (CommandStatisticsTableElement[]) commandStatistics[index];
	}
	
	/**
	 * Liefert die Kommandostatistiken zu allen Skripten. Jedes Element
	 * des Arrays entspricht einem <code>CommandStatistics[]</code>. 
	 */
	public Object[] getCommandStatistics() {
		return commandStatistics;
	}
	
	
	/**
	 * Liefert die Statistiken zu den Skripten.
	 */
	public ScriptStatisticsTableElement[] getScriptStatistics() {
		return scriptStatistics;
	}
	
	/**
	 * Setzt alle Statisken auf die Ausgangswerte zurck.
	 */
	private void resetStatistics() 
	{
		for (int i = 0; i < scriptStatistics.length; i++) {
			scriptStatistics[i].reset();
		}
		
		for(int i = 0; i < commandStatistics.length; ++i)
		{
			CommandStatisticsTableElement[] realStatistics = getCommandStatistics(i);
			for(int j = 0; j < realStatistics.length; ++j)
				realStatistics[j].reset();
		}
	}
	
	/**
	 * Gibt an, ob bei der Ausfhrung der Skripte Fehler aufgetreten sind.
	 * @return <code>true</code>, wenn bei der Ausfhrung Fehler aufgetreten sind.
	 */
	public boolean hasError() {
		return hasError;
	}
	
	/**
	 * Liefert die Fehler zu einem bestimmten Skript.
	 * @param index Der Index des Scriptes im Paket oder 0, wenn nur ein einzelnes Skript
	 * abgespielt wird.
	 * @return Alle Fehler, die whrend der Ausfhrung des Skriptes aufgetreten sind, oder
	 * leerer String wenn keine.
	 */
	public String getErrors(int index) 
	{
		String error = ""; //$NON-NLS-1$
		
		CommandStatisticsTableElement[] realStatistics = getCommandStatistics(index);
		for(int i = 0; i < realStatistics.length; ++i) 
		{
			if( realStatistics[i].hasError() )
				error += realStatistics[i].getError() + "\n\n"; //$NON-NLS-1$
		}
		
		return error;
	}
	
	/**
	 * Liefert alle Fehler in allen Skripten, die ausgefhrt wurden.
	 */
	public String getErrors() 
	{
		String error = ""; //$NON-NLS-1$
		
		for(int i = 0; i < scriptStatistics.length; ++i) 
		{
			if( scriptStatistics[i].hasError() )
			{
				error += Messages.getString("HTSPlayer.5") + (i+1) + " \"" + scripts[i].getName() + Messages.getString("HTSPlayer.7"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				error += scriptStatistics[i].getError();
				error += "*************************************\n\n"; //$NON-NLS-1$
			}
		}
		
		return error;
	}
	
	/**
	 * Gibt an, ob die Ausfhrung durch den Nutzer abgebrochen wurde.
	 * @return <code>true</code> wenn bei Nutzerabbruch, <code>false</code> sonst.
	 */
	public boolean isStoppedByUser() {
		return stoppedByUser;
	}
	
	/**
	 * Beendet die Ausfhrung. Wird durch den Nutzer ausgefhrt.
	 */
	public void userStop() 
	{
		if( !isRunning() )
			return;
		
		stoppedByUser = true;
		
		// Statistik auf abgebrochen setzen
		if(currentScriptIndex < scripts.length)
		{
			if( currentCmdIndex < scripts[currentScriptIndex].getNumOfCommands() )
			{
				getCommandStatistics(currentScriptIndex)[currentCmdIndex].reset();
				Statistics stat = new Statistics(currentCmdIndex, 0, Messages.getString("HTSPlayer.9")); //$NON-NLS-1$
				commandExecuted(stat);
			}
			
			scriptStatistics[currentScriptIndex].reset();
			Statistics stat = new Statistics(currentScriptIndex, 0, Messages.getString("HTSPlayer.10")); //$NON-NLS-1$
			scriptExecuted(stat);
		}
				
		stop();
	}
	
	public void stop() 
	{
		if( !isRunning() )
			return;
		
		// alle Kommandos resetten
		for(int i = 0; i < scripts.length; ++i)
		{
			int numCommands = scripts[i].getNumOfCommands();
			for(int j = 0; j < numCommands; ++j)
				scripts[i].getCommand(j).reset();
		}
		
		super.stop();
	}
	
	/**
	 * Informiert alle <code>PlayerReceiverListener</code> ber den Start eines Kommandos.
	 * @param cmd Das gestartete Kommando.
	 */
	private void commandStarted(final HTSCommand cmd)
	{
		getCommandStatistics(currentScriptIndex)[cmd.getIndex()].setRunning();
		
		Runnable run = new Runnable()
		{
			public void run() {
				for(int i = 0; i < listeners.size(); ++i)
					((PlayerReceiverListener)listeners.get(i)).commandStarted( cmd.getIndex() );
			}
		};
		
		Display.getDefault().asyncExec(run);
	}
	
	/**
	 * Informiert alle <code>PlayerReceiverListener</code> ber das Ende der Ausfhrung 
	 * des aktuellen Kommandos und bermittelt die entsprechende Statistik.
	 */
	private void commandExecuted(final Statistics statistics)
	{
		getCommandStatistics(currentScriptIndex)[statistics.getLine()].update(statistics);
		if(statistics.hasError())
			hasError = true;
		
		Runnable run = new Runnable()
		{
			public void run() {
				for(int i = 0; i < listeners.size(); ++i)
					((PlayerReceiverListener)listeners.get(i)).commandStatisticsReceived(statistics);
			}
		};
		
		Display.getDefault().asyncExec(run);
	}
	
	/**
	 * Informiert alle <code>PackagePlayerReceiverListener</code> ber das Ende der Ausfhrung 
	 * des aktuellen Scriptes und setzt die entsprechende Statistik.
	 */
	private void scriptExecuted(final Statistics statistics) 
	{
		scriptStatistics[currentScriptIndex].update(statistics);
		Runnable run = new Runnable()
		{
			public void run() 
			{
				for(int i = 0; i < listeners.size(); ++i)
				{
					if(listeners.get(i) instanceof PackagePlayerReceiverListener)
						((PackagePlayerReceiverListener)listeners.get(i)).scriptStatisticsReceived(statistics);
				}
			}
		};
		
		Display.getDefault().asyncExec(run);
	}
	
	/**
	 * Informiert alle <code>PackagePlayerReceiverListener</code> ber den Start
	 * des Scriptes bei currentScriptIndex.
	 */
	private void scriptStarted() {
		scriptStatistics[currentScriptIndex].setRunning();
		
		Runnable run = new Runnable()
		{
			public void run() {
				for(int i = 0; i < listeners.size(); ++i)
				{
					if(listeners.get(i) instanceof PackagePlayerReceiverListener)
						((PackagePlayerReceiverListener)listeners.get(i)).scriptStarted(currentScriptIndex);
				}
			}
		};
		
		Display.getDefault().asyncExec(run);
	}

	/**
	 * Wir brauchen fr die Kommandos QUESTION und MESSAGE eine parent shell
	 * fr den angezeigten Dialog. Diese parent shell muss der aktuelle Dialog
	 * zur Script-/Paketausfhrung sein, sonst gibt es Probleme.
	 * @return Die shell des gerade aktiven Run Dialogs.
	 */
	public Shell getRunDlgShell() {
		return runDlgShell;
	}
}
