package atosj.rmi;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.rmi.Naming;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.Vector;

import atosj.Messages;
import atosj.exception.AtosJException;
import atosj.listener.StreamListener;
import atosj.util.Snooze;

/**
 * Diese Klasse liest kontinuierlich Zeichen aus einem der beiden
 * InputStreams eines Prozesses und teilt diese seinem <code>StreamListener</code>
 * mit.
 * 
 * @author Nicos Tegos
 * @see atosj.listener.StreamListener
 */
class StreamReader extends Thread 
{
	private Process p;
	private boolean stdErr;
	private StreamListener listener;
	
	/**
	 * Erstellt einen neuen berwacher fr einen Ausgabepuffer
	 * eines Prozesses.
	 * @param p Der zu berwachende Prozess.
	 * @param stdErr <code>true</code>, wenn der Standardfehlerpuffer (stderr)
	 * ausgelesen werden soll, <code>false</code> fr stdout.
	 */
	public StreamReader(Process p, boolean stdErr) {
		this.p = p;
		this.stdErr = stdErr;
	}
	
	/** 
	 * Registriert einen Listener, der ber neue Zeichen 
	 * im berwachten Ausgabepuffer informiert wird. */
	public void addStreamListener(StreamListener listener) {
		this.listener = listener;
	}
	
	/**
	 * Liest alle im berwachten Ausgabepuffer vorhanden Zeichen
	 * und leitet sie an die <code>StreamListener</code> weiter.
	 * @throws IOException Fehler beim auslesen des Puffers.
	 */
	private void receiveChars() throws IOException 
	{
		InputStream stream = stdErr ? p.getErrorStream() : p.getInputStream();
		int available = stream.available();
		
		if(available > 0)
		{
			char[] chars = new char[available];
			for(int i = 0; i < available; ++i)
				chars[i] = (char) stream.read();
			
			if(listener != null)
				listener.charsReceived(chars);
		}
	}
	
	/**
	 * Startet die berwachung des Ausgabepuffers und wartet bis
	 * der Prozess beendet wird. Diese Funktion sollte nicht direkt
	 * gerufen werden. Stattdessen rufe man <code>Thread.start()</code>.
	 */
	public void run() 
	{
		boolean running = true;
		while(running)
		{
			new Snooze(100);
			
			try	{
				p.exitValue();
				running = false; // der prozess ist beendet
			}
			catch(IllegalThreadStateException e) {
				// der Prozess luft weiter running = true;
			}
			
			try	{
				receiveChars();
			}
			catch(IOException e) {
				running = false;
			}
		}
		
		listener.streamFinished();
		/*try { // die letzten Zeichen nach Beendigung des Threads abholen
			receiveChars();
		}
		catch(IOException e) {
		}*/
	}
}

/**
 * Diese Klasse registriert Objekte in der RMI-Registry nach einem
 * internen Schema. Dazu werden automatisch eindeutige Namen fr
 * die Registrierung der Objekte generiert. Zur Bequemlichkeit werden 
 * ebenso neue Prozesse erstellt.
 */
public class RMIConnectionManager 
{
	private static RMIConnectionManager instance;
	
	private Registry registry = null;
	private int numServersStarted = 0;
	
	private Vector processes = new Vector(); // Vector<Process> alle gestarteten Prozesse
	private Vector remotes = new Vector(); // Vector<String> alle mit Namen registrierten Remote Objekte
	
	//nicht den RMI-Standardport, weil wir sonst Probleme bekommen knnten,
	//wenn wir RMI-Applikationen testen 
	private static final int ATOSJ_PORT = 28675; //entspricht "ATOSj" auf einer Telefontastatur :-)
	
	private RMIConnectionManager() {
	}
	
	/**
	 * Initialisiert die RMI-Registry. Testet nacheinander 10 Ports
	 * und nimmt den ersten freien, beginnend bei <code>ATOSJ_PORT</code>.
	 * @throws AtosJException wenn die RMI-Registry auf keinem der untersuchten
	 * Ports initialisiert werden konnte.
	 */
	private void setUpRegistry() throws AtosJException	
	{
		if (registry == null)
		{	
			int runs = 0;
			while(registry == null && runs < 10)
			{
				try {
					registry = LocateRegistry.createRegistry(ATOSJ_PORT + runs);
					break;
				} catch (RemoteException e) {}
				
				++runs;
			}
			
			if(registry == null)
			{	
				throw new AtosJException(Messages.getString("RMIConnectionManager.0")  //$NON-NLS-1$
						+ ATOSJ_PORT + ", "+ runs + Messages.getString("RMIConnectionManager.2")); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
	}
	
	/**
	 * Liefert den nchsten eindeutigen Namen fr die Registrierung
	 * eines Remote Objekts in der RMI-Registry.
	 * @return Der RMI-Namen.
	 */
	public String getNextServerName() {
		++numServersStarted;
		return "//localhost:" + ATOSJ_PORT+ "/" + Messages.getString("RMIConnectionManager.5") + numServersStarted; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	public static RMIConnectionManager getInstance() {
		if(instance == null)
			instance = new RMIConnectionManager();
		
		return instance;
	}
	
	/**
	 * Alle mit dem <code>RMIConnectionManager</code> erstellten Prozesse oder
	 * Eintrge in der RMI-Registry werden hier gestoppt bzw. deregistriert. 
	 * Diese Funktion ist fr ein Aufrumen bei Programmende gedacht.
	 */
	public void dispose()	
	{
		for(int i = 0; i < processes.size(); ++i)
		{
			try { 
				((Process) processes.get(i)).destroy();	
			} 
			catch (Exception e) { }
		}
		processes.clear();

		for(int i = 0; i < remotes.size(); ++i)
		{
			try { 
				Naming.unbind( (String) remotes.get(i) );	
			} 
			catch (Exception e) { }
		}
		remotes.clear();
	}
	
	/**
	 * Macht ein Remote Objekt in der RMI Registry bekannt. Die RMI-Registry
	 * wird intern verwaltet.
	 * @param remote Das zu registrierende Remote Objekt.
	 * @return Der fr das Remote Objekt erzeugte Name.
	 * @throws AtosJException wenn die Registrierung nicht vorgenommen werden konnte.
	 * @see Naming#unbind(java.lang.String)
	 */
	public String registerRemoteObject(Remote remote) throws AtosJException
	{
		setUpRegistry();
		String connection = getNextServerName();
		try {
			Naming.rebind(connection, remote);
		}
		catch(Exception e) {
			throw new AtosJException(Messages.getString("RMIConnectionManager.6") + connection + Messages.getString("RMIConnectionManager.7")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		remotes.add(connection);
		return connection;
	}
	
	/**
	 * Erstellt einen Prozess. Achtung die Verwendung von <code>StreamListener</code>
	 * bewirkt das Abholen der Zeichen vom Stream somit sind nachfolgende Aufrufe
	 * von <code>getStdout</code> oder <code>getStderr</code> nutzlos.
	 * @param commandLine Die Kommandozeile, die den Prozess startet.
	 * @param env Environmentvariablen (in der form name=value) oder null.
	 * @param dir Das Ausfhrungsverzeichnis.
	 * @param stdoutListener Dieser listener wird ber neue Zeichen in stdout des erstellten Prozesses informiert.
	 * @param stderrListener Dieser listener wird ber neue Zeichen in stderr des erstellten Prozesses informiert.
	 * @return Der neu erstellte Prozess.
	 * @throws AtosJException wenn der Prozess nicht gestartet werden konnte.
	 */
	public Process createProcess(String commandLine, String[] env, String dir, 
			StreamListener stdoutListener, StreamListener stderrListener) throws AtosJException 
	{
		Process process = null;
		try 
		{
			process = Runtime.getRuntime().exec(commandLine, env, new File(dir) );
			
			if(stdoutListener != null)
			{
				StreamReader reader = new StreamReader(process, false);
				reader.addStreamListener(stdoutListener);
				reader.start();
			}
			
			if(stderrListener != null)
			{
				StreamReader reader = new StreamReader(process, true);
				reader.addStreamListener(stderrListener);
				reader.start();
			}
		} catch (IOException e) {
			throw new AtosJException(Messages.getString("RMIConnectionManager.8") //$NON-NLS-1$
					+ Messages.getString("RMIConnectionManager.9") + commandLine //$NON-NLS-1$
					+ Messages.getString("RMIConnectionManager.10") + dir); //$NON-NLS-1$
		}
		
		processes.add(process);
		return process;
	}
	
	public static String streamToString(InputStream stream)
	{
		String ret = ""; //$NON-NLS-1$
		
		try{
			while( stream.available() > 0 )
				ret += (char) stream.read();
		}
		catch(IOException ex) {
		}
		
		return ret;
	}

	/**
	 * Wandelt alle Zeichen im angegebenen Ausgabepuffer (stderr) in einen String um.
	 * Funktioniert nur, wenn vorher keine Zeichen aus dem Puffer gelesen wurden. 
	 * @param process Der Prozess dessen Ausgabepuffer gelesen werden soll.
	 * @return Alle Zeichen im Ausgabepuffer.
	 */
	public static String getStdOut(Process process) {
		return RMIConnectionManager.streamToString(process.getInputStream() );
	}
	
	/**
	 * Wandelt alle Zeichen im angegebenen Ausgabepuffer (stderr) in einen String um.
	 * Funktioniert nur, wenn vorher keine Zeichen aus dem Puffer gelesen wurden. 
	 * @param process Der Prozess dessen Ausgabepuffer gelesen werden soll.
	 * @return Alle Zeichen im Ausgabepuffer.
	 */
	public static String getStdErr(Process process) {
		return RMIConnectionManager.streamToString(process.getInputStream() );
	}

}
