/*
 * Created on 20.02.2006
 */
package atosj.rmi;

import java.io.File;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Vector;

import org.eclipse.swt.widgets.Display;

import atosj.app.TestObject;
import atosj.exception.AtosJException;
import atosj.listener.ReceiverListener;
import atosj.listener.StreamListener;

/**
 * Basisklasse fr alle Klassen die einen Remote-Prozess starten und sich
 * selbst in der RMI-Registry fr weitere Kommunikation mit dem Remote-Prozess
 * registrieren.
 * @author Nicos Tegos
 */
public abstract class Receiver extends UnicastRemoteObject 
{
	private TestObject testee;
	private String myNaming; // Name fr die RMI-Registry
	private Process proc; // der aktuelle Prozess
	private boolean running = false;
	private Vector procs = new Vector(); // Vector<Process> alle bisher gestarteten Prozesse
	protected Vector listeners = new Vector(); // Vector<ReceiverListener>
	
	/**
	 * Erstellt einen neuen Receiver.
	 * @param testee Das Testobjekt, das gestartet werden soll.
	 */
	public Receiver(TestObject testee) throws RemoteException {
		super();
		this.testee = testee;
	}

	/** Liefert das Testobjekt. */
	public TestObject getTestObject() {
		return testee;
	}
	
	/** -Setzt das zu startende Testobjekt */
	protected void setTestObject(TestObject testee) {
		// diese Funktion existiert leider aus implementationstechnischen
		// grnden im Zusammenhang mit der Klasse TestStarterReceiver
		this.testee = testee;
	}
	
	/**
	 * Liefert den zuletzt gestarteten Prozess
	 * oder <code>null</code>, wenn noch keiner gestartet wurde.
	 */
	protected Process getProcess() {
		return proc;
	}
	
	/**
	 * Startet einen Remote-Prozess und fgt ihn der Liste der gestarteten Prozesse hinzu.
	 * @param procClass main-Klasse des Remote-Prozesses.
	 * @param procArgs Programmargumente.
	 * @param stdoutListener Listener fr die Ausschriften des Remote-Prozesses auf stdout.
	 * @param stderrListener Listener fr die Ausschriften des Remote-Prozesses auf stderr.
	 * @throws AtosJException wenn der Remote-Prozess nicht gestartet werden konnte.
	 */
	protected void start(Class procClass, String[] procArgs, 
			StreamListener stdoutListener, StreamListener stderrListener) throws AtosJException
	{
		try
		{
			setRunning(true);
			startProcess(procClass, procArgs, stdoutListener, stderrListener);
		}
		catch(AtosJException e) {
			stop();
			throw e;
		}
	}
	
	/**
	 * Setzt den Status und informiert die entsprechenden Listener
	 * darber. 
	 */
	protected void setRunning(boolean running)
	{
		if(running == this.running)
			return;
		
		this.running = running;
		fireRunningStateChanged();
	}
	
	/**
	 * Leitet weiter an die Methode <code>startProcess(boolean, Class, String[], StreamListener, StreamListener)</code>
	 * mit <code>false</code> als erstem Argument.
	 * @see #startProcess(boolean, Class, String[], StreamListener, StreamListener)
	 */
	protected void startProcess(Class procClass, String[] procArgs, StreamListener stdoutListener, StreamListener stderrListener) throws AtosJException {
		startProcess(false, procClass, procArgs, stdoutListener, stderrListener);
	}
	
	/**
	 * Startet einen Remote-Prozess und fgt ihn der Liste der gestarteten Prozesse hinzu.
	 * @param doCover wenn, <code>true</code> wird das gestartete Testobjekt einer 
	 * berdeckungsanalyse unterzogen.
	 * @param procClass main-Klasse des Remote-Prozesses.
	 * @param procArgs Programmargumente.
	 * @param stdoutListener Listener fr die Ausschriften des Remote-Prozesses auf stdout.
	 * @param stderrListener Listener fr die Ausschriften des Remote-Prozesses auf stderr.
	 * @throws AtosJException wenn der Remote-Prozess nicht gestartet werden konnte.
	 */
	protected void startProcess(boolean doCover, Class procClass, String[] procArgs, StreamListener stdoutListener, StreamListener stderrListener) throws AtosJException
	{
		myNaming = RMIConnectionManager.getInstance().registerRemoteObject(this);
		
		String commandLine = "java";
		
		// Klassenpfad = Testobjektklassenpfad + EXT\ext.jar
		String cp = testee.getClassPath(); //System.getProperty("java.class.path");
		// wir suchen aus dem aktuellen Classpath das atosj.jar und das javassist.jar
		// und fgen sie als absolute pfade dem klassenpfad fr das testobjekt hinzu
		String[] atosjCp = System.getProperty("java.class.path").split(File.pathSeparator);
		for (int i = 0; i < atosjCp.length; i++) 
		{
			if( testee.isAWTProject() )
			{
				if( atosjCp[i].contains("atosj") || atosjCp[i].contains("javassist")
						|| atosjCp[i].contains("jaccess") )
					cp += File.pathSeparator + new File(atosjCp[i]).getAbsolutePath();
			}
			else
			{
				if( atosjCp[i].contains("atosj") || atosjCp[i].contains("javassist") )
					cp += File.pathSeparator + new File(atosjCp[i]).getAbsolutePath();
			}
		}
		
		// plugin-pfad hinzufgen
		String ext = testee.getProjectDir();
		if( !ext.endsWith(File.separator) )
			ext += File.separator;
		ext += "EXT" + File.separator + "ext.jar";
		cp += File.pathSeparator + ext;
		
		commandLine += " -cp \"" + cp + "\""; 
		commandLine += " " + ProcessRunner.class.getName();
		
		if( doCover && testee.isCoverage() )
			commandLine += " -cover \"" + testee.getCoverageDatabasePath() + "\"";
		
		if( testee.isAWTProject() )
			commandLine += " awt";
		else
			commandLine += " swt";
		
		commandLine += " " + procClass.getName();
		
		// das erste Argument ist immer das Naming dieses RemoteObjects
		commandLine += " " + myNaming;
		if(procArgs != null)
		{
			for(int i = 0; i < procArgs.length; ++i)
				commandLine += " " + procArgs[i];
		}
		
		proc = RMIConnectionManager.getInstance().createProcess(commandLine, null, testee.getMainClassDir(), stdoutListener, stderrListener);
		procs.add(proc);
	}
	
	/*
	/**
	 * Gibt an, ob der RemoteProzess gerade luft.
	protected boolean isProcRunning() 
	{
		if( proc != null )
		{
			try {
				proc.exitValue();
				return true;
			}
			catch(IllegalThreadStateException e) {
				return false;
			}
		}
		
		return false;
	}*/
	
	/**
	 * Entfernt diesen Receiver aus der RMI-Registry und
	 * killt alle aktuell ausgefhrten Prozesse.
	 */
	public void dispose()	
	{
		stop();
		
		try {
			UnicastRemoteObject.unexportObject(this, true);
		} catch (Exception e) {/*egal*/}
		
		listeners.clear();
	}

	/**
	 * Beendet einen Durchlauf. Killt alle aktuell ausgefhrten Prozesse.
	 */
	public void stop() 
	{
		if( !isRunning() )
			return;
		
		for(int i = 0; i < procs.size(); ++i)
			((Process)procs.get(i)).destroy();
		procs.clear();
		proc = null;
		
		try {
			if(myNaming != null)
				Naming.unbind(myNaming);
		}
		catch(Exception e) {}
		
		setRunning(false);
	}

	/**
	 * Gibt an, ob der mit <code>start</code> gestartete Durchlauf noch luft.
	 * @return <code>true</code>, wenn der Durchlauf noch nicht beendet ist.
	 */
	public boolean isRunning() {
		return running;
	}
	
	protected void fireRunningStateChanged() {
		Runnable run = new Runnable() {
			public void run() {
				for(int i = 0; i < listeners.size(); ++i)
					((ReceiverListener)listeners.get(i)).runningStateChanged(running);
			}
		};
		
		Display.getDefault().asyncExec(run);
	}
	
	/** Registriert einen Listener, der ber die Ausfhrung des Testobjekts informiert wird. */
	public void addListener(ReceiverListener listener) {
		if( !listeners.contains(listener) )
			listeners.add(listener);
	}
	
	/** Entfernt einen Listener, der ber die Ausfhrung des Testobjekts informiert wird. */
	public void removeListener(ReceiverListener listener) {
		listeners.remove(listener);
	}
}
