/*
 * Created on 19.11.2005
 */
package atosj.util;

import java.io.Serializable;
import java.util.Vector;

import atosj.Messages;
import atosj.editors.AtosJCloneable;
import atosj.exception.ParseException;

/**
 * Reprsentiert eine einzeilige kommaseparierte Liste.
 * @author Nicos Tegos
 */
public class CSVLine implements Serializable, AtosJCloneable
{
	private Vector elements;
	private int lineNumber;
	private String filePath;
	
	/**
	 * Erstellt eine Zeile aus einer kommaseparierten Datei.
	 * @param filePath Der Dateipfad
	 * @param lineNumber Die Zeilennummer, 0-basiert.
	 * @param elements Die einzelnen Elemente zwischen den Kommata einer Zeile,
	 * Elemente die Leerzeichen enthalten mssen in Anfhrungszeichen geschrieben
	 * werden. Zeichenketten in Anfhrungszeichen werden in der unmaskierten Form
	 * erwatet.
	 */
	public CSVLine(String filePath, int lineNumber, Vector elements) {
		this.filePath = filePath;
		this.lineNumber = lineNumber;
		this.elements = elements;
	}
	
	/**
	 * Eine kommaseparierte Zeile die keiner Datei zugeordnet ist und keine Zeilennummer hat.
	 * @param elements Die einzelnen Elemente zwischen den Kommata einer Zeile,
	 * Elemente die Leerzeichen enthalten mssen in Anfhrungszeichen geschrieben
	 * werden. Zeichenketten in Anfhrungszeichen werden in der unmaskierten Form
	 * erwatet.
	 */
	public CSVLine(Vector elements) {
		this.filePath = null;
		this.lineNumber = -1;
		this.elements = elements;
	}
	
	/** 
	 * Liefert ein Element aus der kommaseparierten Zeile.
	 * @param index Der Index des Elements.
	 */
	public String getElement(int index) throws ParseException
	{
		try	{
			return (String) elements.get(index);
		}
		catch(Throwable e)	{
			throw new ParseException(this, Messages.getString("CSVLine.0") + (index+1) + Messages.getString("CSVLine.1") ); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	/** 
	 * Liefert ein Element aus der kommaseparierten Zeile, vorher werden
	 * Anfhrungszeichen entfernt.
	 * @param index Der Index des Elements.
	 */
	public String getElementUnquoted(int index) throws ParseException {
		return CSVParser.stripQuotes( getElement(index) );
	}
	
	/** Liefert die Anzahl der Elemente in der Liste. */
	public int getNumOfElements() {
		return elements.size();
	}
	
	/**
	 * Liefert den vollstndigen Pfad der Datei aus dem die Zeile stammt
	 * oder <code>null</code>, falls die Zeile nicht aus einer Datei 
	 * geparset wurde.
	 */
	public String getFilePath() {
		return filePath;
	}
	
	/**
	 * Liefert die Nummer der Zeile in der Datei oder
	 * <code>null</code>, falls die Zeile nicht aus einer Datei 
	 * geparset wurde.
	 */
	public int getLineNumber() {
		return lineNumber;
	}
	
	/**
	 * Transformiert die Zeile in einen String, wie er zur Speicherung in
	 * einer Datei verwendet wird. Elemente in Anfhrunszeichen werden mit
	 * dem Backslash maskiert.
	 * @return Die Zeile in der maskierten Form, mit einem Komma als Trennzeichen
	 * zwischen den einzelnen Elementen.
	 */
	public String toString() 
	{
		try {
			String escaped = ""; //$NON-NLS-1$
			for(int i = 0; i < elements.size(); ++i)
			{
				String elem = getElement(i);
				if( CSVParser.isQuoted(elem) )
				{
					elem = CSVParser.stripQuotes(elem);
					elem = elem.replaceAll("\\\\", "\\\\\\\\").replaceAll("\"", "\\\\\"").replaceAll("\\n", "\\\\n").replaceAll("\\r", "\\\\r").replaceAll("\\t", "\\\\t"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-10$
					elem = '"' + elem + '"';
				}
				
				escaped += elem;
				if(i != elements.size() - 1)
					escaped += ", "; //$NON-NLS-1$
			}
			return escaped;
		}
		catch(ParseException e) {
			// will never happen
			return ""; //$NON-NLS-1$
		}
	}
	
	/**
	 * Kopiert dieses Objekt jedoch ohne die Zeilennummer und den
	 * Scriptpfad.
	 */
	public Object clone() {
		return new CSVLine( new Vector(elements) );
	}
}
