package atosj.util;

import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import atosj.exception.ParseException;

/**
 * Liest Dateien ein deren Inhalt aus kommaseparierten Listen
 * besteht.
 * @author Volker Janetschek
 * @author Nicos Tegos
 */
public class CSVParser {
	
	private static final char EOL = '\0'; // rckgabewert von getChar(), wenn das ende einer Zeile erreicht wurde
	private String filePath;
	private String[] rawLines;
	private int row = -1; // die aktuell zu parsende Zeile
	private int col = 0;
	
	/**
	 * Erstellt einen neuen Parser fr eine Datei
	 * mit kommaseparierten Listen.
	 * @param filePath Der vollstndige Pfad der Datei.
	 * @throws IOException Wenn die Datei nicht gelesen werden
	 * konnte.
	 */
	public CSVParser(String filePath) throws IOException 
	{
		this.filePath = filePath;
		
		rawLines = fileToString().split("\n");
	}
	
	/**
	 * Erstellt einen neuen Parser fr einen String,
	 * der kommaseparierte Listen enthlt.
	 * @param content Der zu parsende String.
	 * @param hack Zustzlicher Parameter zur Unterscheidung
	 * vom anderen Konstruktor, wird ignoriert.
	 */
	public CSVParser(String content, boolean hack) {
		this.filePath = "";
		rawLines = content.split("\n");
	}
	
	/** Liest den Inhalt einer Datei ein und gibt ihn als String zurck. */
	private String fileToString() throws IOException 
	{
		FileReader reader = new FileReader(filePath);
		StringBuffer buff = new StringBuffer(512);
		char[] chars = new char[256];
		int numRead = 0;
		while( (numRead = reader.read(chars)) != -1 )
		{
			for(int i = 0; i < numRead; ++i)
				buff.append(chars[i]);
		}
		
		reader.close();
		return buff.toString();
	}
	
	/**
	 * Liefert die nchste geparsete Zeile.
	 * @return CSVLine Eine komplette Zeile mit allen Elementen.
	 * oder <code>null</code> bei Dateiende.
	 * @throws ParseException bei syntaktischem Fehler.
	 */
	public CSVLine getNextLine() throws ParseException 
	{
		CSVLine ret = null;
		while(ret == null && row < rawLines.length -1)
		{
			ret = parseLine();
		}
		
		return ret;
	}
	
	/**
	 * Parset die nchste Zeile.
	 * @return CSVLine Die geparsete Zeile
	 * oder <code>null</code> bei einer leeren Zeile.
	 * @throws ParseException bei syntaktischem Fehler
	 */
	private CSVLine parseLine() throws ParseException
	{
		Vector items = new Vector();
		boolean startsWithComma = false; // beim ersten durchlauf
		col = 0;
		++row;
		
		while( hasNextItem(startsWithComma) )
		{
			char c = getChar();
			if(c == '"')
			{
				ungetChar();
				items.add( getQuotedItem() );
			}
			else
			{
				ungetChar();
				items.add( getUnquotedItem() );
			}
			
			startsWithComma = true;
		}
		
		if( items.isEmpty() ) 
			return null;
		else
			return new CSVLine(filePath, row, items);
	}
	
	/**
	 * Der Zeiger (col) auf das aktuelle Zeichen steht ein vor dem ersten
	 * buchstaben dieses items. Danach steht er auf dem letzten buchstaben
	 * dieses items.
	 * @return Ein Element der Kommasepariertenliste welches nicht in Hochkommata eingeschlossen ist.
	 */
	private String getUnquotedItem() 
	{
		char c = getChar();
		StringBuffer word = new StringBuffer();
		word.append(c);
		
		c = getChar();
		while( !isWhiteSpace(c) && c != ',' && c != EOL)
		{
			word.append(c); // c ans ende von text anhaengen
			c = getChar();
		}
		
		ungetChar();
		return word.toString();
	}
	
	/** 
	 * Der Zeiger (col) auf das aktuelle Zeichen steht ein vor dem ersten
	 * buchstaben dieses items, d.h. ". Danach steht er auf dem letzten buchstaben
	 * dieses items d.h. ".
	 * @return Dieses item erstreckt sich vom ersten Hochkomma 
	 * bis zum ersten gefunden nicht maskierten Hochkomma.
	 * */
	private String getQuotedItem() throws ParseException 
	{
		char c = getChar();
		StringBuffer word = new StringBuffer();
		word.append(c);
		
		c = getChar();
		while(c != '"')
		{
			if(c == EOL)
			{
				throwParseException("Der String wurde nicht mit \" abgeschlossen.");
			}
			else
			if(c == '\r')
			{
				throwParseException("Keine Carriage-Return innerhalb eines Strings erlaubt. Bitte mit \\r maskieren.");
			}
			else
			if(c == '\t')
			{
				throwParseException("Keine Tab innerhalb eines Strings erlaubt. Bitte mit \\t maskieren.");
			}
			else
			if(c == '\n')
			{
				throwParseException("Keine Zeilenumbrche innerhalb eines Strings erlaubt. Bitte mit \\n maskieren.");
			}
			else
			if( c == '\\' ) // maskierung in richtige zeichen umwandeln
			{
				c = getChar();
				
				if(c == '\\')
					word.append('\\');
				else
				if(c == 't')
					word.append('\t');
				else
				if(c == 'n')
					word.append('\n');
				else
				if(c == 'r')
					word.append('\r');
				else
				if(c == '"')
					word.append('"');
				else
					throwParseException("Ungltige Maskierung in einem String: " + '\\' + c);
			}
			else // alles okay und auch keine Maskierung
			{
				word.append(c);
			}
			
			c = getChar();
		}
		word.append(c); // das letzte " anhngen
		
		return word.toString();
	}
	
	private boolean isWhiteSpace(char c) {
		return " \r\t\n".indexOf(c) != -1;
	}
	
	/**
	 * Erhht den Zeiger fr das aktuelle Zeichen. 
	 * @return Das nchste Zeichen in der aktuellen Zeile oder EOL, wenn
	 * die Zeile zuende ist.
	 */
	private char getChar() {
		if( col < rawLines[row].length() ) 
		{
			char c = rawLines[row].charAt(col);
			++col;
			return c;
		}
		else
		{
			++col; // unbedingt so, wegen ungetChar
			return EOL;
		}
	}
	
	/**
	 * Setzt den Zeiger fr das aktuelle Zeichen auf das vorhergehende
	 * Zeichen, sofern mglich.
	 */
	private void ungetChar() {
		if(col > 0)
			--col;
	}
	
	/**
	 * Gibt an ob es ein weiteres item in der Kommaseparierten Liste gibt.
	 * Der Zeiger auf das aktuelle Zeichen steht danach vor dem ersten
	 * Zeichen des nchsten items.
	 * @param startsWithComma dem nchsten item muss ein Komma vorausgehen.
	 * @return wenn es ein weiteres item in der Kommaseparierten Liste gibt.
	 * @throws ParseException
	 */
	private boolean hasNextItem(boolean startsWithComma) throws ParseException 
	{
		char c = getChar();
		while( isWhiteSpace(c) ) // eat up whitespace
			c = getChar();

		if( c == EOL ) // zeilenende erreicht
			return false;
		
		if(startsWithComma)
		{
			if(c != ',')
				throwParseException("Komma erwartet.");
			
			c = getChar();
			while( isWhiteSpace(c) ) // eat up whitespace
				c = getChar();

			if( c == EOL )
				throwParseException("Kein Komma erwartet.");
		
			if( c == ',')
				throwParseException("Leere Werte zwischen zwei Kommata sind nicht erlaubt.");
			
			ungetChar();
		}
		else
		{
			if(c == ',')
				throwParseException("Kein Komma erwartet.");
			
			ungetChar();
		}
		
		return true;
	}
	
	/**
	 * Gibt an, ob eine Zeichenkette in Anfhrungszeichen steht.
	 * @param str Die Zeichenkette.
	 * @return <code>true</code>, wenn in Anfhrungszeichen.
	 */
	public static boolean isQuoted(String str) {
		int len = str.length();
		if(len >= 2)
		{
			if( str.charAt(0) == '"' && str.charAt(len-1) == '"')
				return true;
			else
				return false;
				
		}
		else
			return false;
	}
	
	/**
	 * Entfernt ffnenden und schlieenden Anfhrungszeichen einer
	 * Zeichenkette, sofern vorhanden.
	 * @param str Die Zeichenkette.
	 * @return Die Zeichenkette ohne Anfhrungszeichen.
	 */
	public static String stripQuotes(String str)
	{
		if( isQuoted(str) )
			return str.substring(1, str.length()-1);
		else
			return str;
	}
		
	private void throwParseException(String message) throws ParseException {
		throw new ParseException(row, filePath, message, rawLines[row]);
	}
}
