/*
 * Created on 11.01.2006
 */
package atosj.util;

import java.util.Collection;
import java.util.Vector;

import atosj.listener.ClipboardListener;

/**
 * Eine programminterne Zwischenablage, die als zentrale Instanz
 * zum Aufbewahren von kopierten/ausgeschnittenen/gedraggten Elementen dient.
 * @author Nicos Tegos
 */
public class Clipboard 
{
	/** Hinzufgen durch kopierter Elemente */
	public static final int MODE_COPY = 1;
	/** Hinzufgen durch ausgeschnittener Elemente */
	public static final int MODE_CUT = 2;
	/** Hinzufgen durch gedragter Elemente */
	public static final int MODE_DRAG = 3;
	/** Enthaltene Elemente sind ungltig */
	public static final int MODE_INVALID = 4;
	
	private static Clipboard instance = new Clipboard();
	private Vector elements = new Vector();
	private int mode = MODE_INVALID;

	private Vector listeners = new Vector();
	
	private Clipboard() {
	}

	/** Liefert die Singleton Instanz der Zwischenablage. */
	public static Clipboard getInstance() {
		return instance;
	}
	
	/** 
	 * Gibt an zu welchem Zweck, die Elemente in die Zwischenablage
	 * eingefgt wurden.
	 * @return Eine Konstante die den Einfgemodus bezeichnet, siehe
	 * <code>MODE_*</code> Konstantendefinition.
	 */
	public int getMode() {
		return mode;
	}
	
	/**
	 * Gibt an, ob die Zwischenablage Elemente enthlt.
	 * @return <code>true</code>, wenn Elemente enthalten.
	 */
	public boolean hasElements() {
		return !elements.isEmpty();
	}
	
	/** Liefert die Anzahl der enthalenen Elemente */
	public int getNumOfElements() {
		return elements.size();
	}
	
	/**
	 * Setzt die Elemente in der Zwischenablage. Die vorhergehenden
	 * Elemente sind dann nicht mehr verfgbar. Ist als Modus <code>MODE_COPY</code> 
	 * ausgewhlt, mssen die Elemente <code>AtosJCloneable</code> implementieren.
	 * @param add Die neuen Elemente.
	 * @param mode Eine Konstante vom Typ MODE_*, wobei MODE_INVALID ungltig ist.
	 * @see atosj.editors.AtosJCloneable
	 */
	public void setElements(Collection add, int mode) {
		this.mode = mode;
		elements.clear();
		
		elements.addAll(add);
		fireChange();
	}
	
	/**
	 * Setzt ein einzelnes Element in die Zwischenablage. Die vorhergehenden
	 * Elemente sind dann nicht mehr verfgbar. Ist als Modus <code>MODE_COPY</code> 
	 * ausgewhlt, muss das Element <code>AtosJCloneable</code> implementieren.
	 * @param element Das neue Element.
	 * @param mode Eine Konstante vom Typ MODE_*, wobei MODE_INVALID ungltig ist.
	 * @see atosj.editors.AtosJCloneable
	 */
	public void setElement(Object element, int mode) {
		Vector elements = new Vector(1);
		elements.add(element);
		setElements(elements, mode);
	}
	
	/**
	 * Liefert die enthaltenen Elemente. Bei MODE_CUT oder MODE_DRAG wird die Zwischenablage
	 * geleert. Bei MODE_COPY bleiben die Elemente erhalten.
	 * @return Bei MODE_COPY, MODE_CUT oder
	 * MODE_DRAG die enthaltenen Elemente. Bei MODE_INVALID <code>null</code>.
	 * Bei MODE_COPY ist zu beachten, dass die Kopie der Elemente selbst
	 * erstellt werden muss. 
	 */
	public Vector getElements() {
		Vector ret = null;
		
		if(mode == MODE_CUT || mode == MODE_DRAG)
		{
			ret = new Vector(elements);
			elements.clear();
			mode = MODE_INVALID;
			
			fireChange();
		}
		else
		if(mode == MODE_COPY)
		{
			ret = new Vector(elements);
		}
		
		return ret;
	}
	
	/**
	 * Entfernt die angegebenen Elemente aus dem Clipboard. 
	 * @param toRemove Die zu entfernenden Elemente.
	 * TODO Dies sollte insbesondere auch nach dem Lschen von Elementen
	 * geschehen.
	 */
	public void removeElements(Vector toRemove) 
	{
		boolean changed = false;
		for(int i = 0; i < toRemove.size(); ++i)
		{
			if( elements.remove( toRemove.get(i) ) )
				changed = true;
		}
		
		if( elements.isEmpty() )
			mode = MODE_INVALID;
		
		if(changed)
			fireChange();
	}
	
	/**
	 * Gibt an, ob sich nur Elemente der angegebenen Klasse in der 
	 * Zwischenablage befinden.
	 * @return <code>true</code> wenn nicht leer und alle enthaltenen Elemente Instanzen
	 * der Klasse sind, die durch <code>clazz</code> angegeben wird.
	 */
	public boolean containsOnlyClass(Class clazz) 
	{
		if( hasElements() )
		{
			for(int i = 0; i < elements.size(); ++i)
			{
				if( !clazz.isInstance(elements.get(0)) )
					return false;
			}
			
			return true;
		}
		
		return false;
	}
	
	/** Informiert <code>ClipboardListener</code> ber nderung der Zwischenablage */
	private synchronized void fireChange() {
		for(int i = 0; i < listeners.size(); ++i)
			((ClipboardListener)listeners.get(i)).clipboardChanged(this);
	}
	
	/** Registriert einen Listener, der ber Vernderungen des Clipboards informiert wird. */
	public synchronized void addClipboardListener(ClipboardListener listener) {
		if( !listeners.contains(listener) )
			listeners.add(listener);
	}
	
	/** Entfernt einen Listener, der ber Vernderungen des Clipboards informiert wird. */
	public synchronized void removeClipboardListener(ClipboardListener listener) {
		listeners.remove(listener);
	}
}
