package atosj.util;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import atosj.Messages;
import atosj.exception.AtosJException;
import atosj.listener.AtosJTopLevelWindowListener;
import atosj.listener.FinishedListener;

/**
 * Startet das Testobjekt mittels reflection.
 * @author Nicos Tegos
 */
public class GUIRunner	{
	protected String mainClassName;
	protected String mainWindowTitle;
	protected MainRunner appThread; // startet die Main Funktion des Testobjekts
	
	/**
	 * Hilfsklasse: Startet das Testobjekt in einem eigenen Thread.
	 */
	protected class MainRunner extends Thread	{
		
		private Method main;
		private String[] args;
		private AtosJException runError = null;
		
		/**
		 * Initialisiert den Starter. Ermittelt die main-Methode der zu startenden Klasse.
		 * @param group <code>ThreadGroup</code> der dieser Thread zugeordnet werden soll.
		 * @param mainClassName Name der main-Klasse, des zu startenden Programms.
		 * @param args Die Programmargumente.
		 * @throws AtosJException wenn main-Klasse/main-Methode nicht gefunden oder
		 * wenn kein Zugriff auf die Klasse/Methode.
		 */
		MainRunner(ThreadGroup group, String mainClassName, String args[])	throws AtosJException
		{
			super(group, "ATOSj Testobject: " + mainClassName);
			this.args = args;
		
			try	{
				Class mainClass = getClass().getClassLoader().loadClass(mainClassName);
				main = mainClass.getMethod("main", new Class[]{String[].class});
			}
			catch (Exception e)	
			{
				String messageDetail = "";
				if(e instanceof ClassNotFoundException)
				{
					messageDetail += Messages.getString("GUIRunner.3");
				}
				else if(e instanceof NoSuchMethodException || e instanceof SecurityException)
				{
					messageDetail += Messages.getString("GUIRunner.4");
				}
								
				throw new AtosJException( messageDetail +
						Messages.getString("GUIRunner.5") + mainClassName +
						Messages.getString("GUIRunner.6") + System.getProperty("java.class.path")
					);		
			}
		}
	
		/**
		 * Startet die main-Methode und speichert eventuelle Laufzeitfehler.
		 */
		public void run() {
			try	{
				main.invoke( null, new Object[] {args} );
			}
			catch(Throwable e)	
			{
				String messageDetail = "";
				if(e instanceof IllegalAccessException)
				{
					messageDetail += Messages.getString("GUIRunner.9");
				}
				else if(e instanceof InvocationTargetException)
				{
					messageDetail += Messages.getString("GUIRunner.10");
					if(e.getMessage() == null)
					{
						Throwable cause = e.getCause();
						while(cause != null)
						{
							if( cause.getMessage() != null )
							{
								messageDetail += ": \"" + cause.getMessage() + "\"";
								break;
							}
							cause = cause.getCause();
						}
					}
					else
					{
						messageDetail += ": \"" + e.getMessage() + "\"";
					}
				}
				else
					messageDetail += Messages.getString("GUIRunner.15");
				
				String argsString = "";
				for (int i = 0; i < args.length; i++)
					argsString+=" \""+args[i]+"\"";
				
				runError = new AtosJException( messageDetail +
					Messages.getString("GUIRunner.19") + mainClassName +
					Messages.getString("GUIRunner.20") + argsString);
			}
		}
		
		/** 
		 * berprft, ob beim Aufruf der main-Methode Fehler aufgetreten sind.
		 * @throws AtosJException wenn beim Aufruf der main-Methode ein Fehler aufgetreten ist.
		 */
		public void assertError() throws AtosJException {
			if(runError != null)
				throw runError;
		}
	}
	
	/**
	 * Hilfsklasse: Wartet auf die Beendigung des Testobjektes und benachrichtigt alle Listener.
	 */
	protected class FinishWaiter extends Thread
	{
		private FinishedListener listener;
		private ThreadGroup appThreadGroup;
		
		/**
		 * Erstellt einen neuen <code>FinischWaiter</code>, der auf die Beendigung aller
		 * Threads der angegebenen Gruppe wartet.
		 * @param appThreadGroup Die ThreadGroup des Threads, der die main-Methode des 
		 * Testobjekts ausfhrt.
		 */
		public FinishWaiter(ThreadGroup appThreadGroup, FinishedListener listener)
		{
			this.appThreadGroup = appThreadGroup;
			this.listener = listener;
		}
		
		/** berprft in Abstnden, ob noch ein Thread aus dem Testobjekt luft. */
		public void run() 
		{
			while( isAlive(appThreadGroup) )
				new Snooze(400);
			
			listener.testObjectFinished();
			listener = null;
		}
		
		/**
		 * berprft rekursiv fr die angegebenen Gruppe
		 * und alle Untergruppen, ob sie noch laufende Threads
		 * enthalten.
		 * @param group Die zu berprfende Gruppe.
		 * @return <code>true</code>, wenn <code>group</code> oder
		 * eine ihrer Untergruppen noch laufende Threads enthalten.
		 */
		private boolean isAlive(ThreadGroup group) 
		{
			Thread[] threads = new Thread[group.activeCount()];
			group.enumerate(threads);
			for(int i = 0; i < threads.length; ++i)
			{
				if(threads[i].isAlive())
					return true;
			}
			
			ThreadGroup[] subGroups = new ThreadGroup[group.activeGroupCount()];
			group.enumerate(subGroups);
			for(int i = 0; i < subGroups.length; ++i)
			{
				if(isAlive(subGroups[i]))
					return true;
			}
			
			return false;
		}
	}
	
	/** 
	 * Erstellt einen neuen Starter fr die Ausfhrung des Testobjekts.
	 * @param mainClassName Name der main-Klasse des Testobjekts.
	 * @param mainWindowTitle Titel des Hauptfensters des Testobjekts.
	 */
	public GUIRunner(String mainClassName, String mainWindowTitle)	{
		super();
		this.mainClassName = mainClassName;
		this.mainWindowTitle = mainWindowTitle;
		this.appThread = null;
	}

	/**
	 * Startet den Thread, der das Testobjekt startet.
	 * Wartet maximal <code>timeout</code> Millisekunden auf das ffnen des Hauptfensters.
	 * <em>ACHTUNG:</em> Die <code>WindowMonitor</code> Instanz muss vorher erstellt worden sein.
	 * @param timeout Die Wartezeit.
	 * @param args Die Programmargumente.
	 * @param windowListener Ein Listener der ber das ffnen und Schlieen von Fenstern
	 * im Testobjekt benachrichtigt wird oder <code>null</code>, wenn nicht bentigt.
	 * @param finishedListener Ein Listener der benachrichtigt wird, wenn alle Threads des Testobjektes beendet sind
	 * oder <code>null</code>, wenn nicht bentigt.
	 * @return <code>true</code> wenn sich das Hauptfenster in der Frist <code>timeout</code> 
	 * geffnent hat, <code>false</code> sonst.
	 * @throws AtosJException wenn main-Klasse/main-Methode nicht gefunden oder
	 * wenn kein Zugriff auf die Klasse/Methode.
	 */
	public boolean openMainWindow(long timeout, String[] args, AtosJTopLevelWindowListener windowListener, FinishedListener finishedListener) throws AtosJException
	{
		if(windowListener != null)
			WindowMonitor.getInstance().addTopLevelWindowListener(windowListener);
		
		ThreadGroup appThreadGroup = new ThreadGroup("ATOSj Testobject ThreadGroup"); // beinhaltet alle Threads des Testobjektes
		appThread = new MainRunner(appThreadGroup, mainClassName, args);
		appThread.start();
		
		boolean ret = waitForMainWindowToOpen(timeout);
		appThread.assertError();
		
		if(finishedListener != null) // wird benachrichtigt bei Testobjekt ende
			new FinishWaiter(appThreadGroup, finishedListener).start();
		
		return ret;
	}
	
	/**
	 * Testet in Intervallen, ob sich das Hauptfenster des Testobjekts
	 * bereits geffnet hat.
	 * @param timeout Maximale Wartezeit.
	 * @return <code>true</code> wenn sich das Hauptfenster innerhalb
	 * von <code>timeout</code> geffnet hat.
	 * @throws AtosJException bei einem Fehler im Testobjetk Thread.
	 */
	protected boolean waitForMainWindowToOpen(long timeout) throws AtosJException
	{
		long start = System.currentTimeMillis();
		
		while( System.currentTimeMillis() - start <= timeout )	
		{
			// das berprfen wir bei jedem durchlauf, damit wir nicht bis auf das ende der Funktion warten muessen
			// was bei einer langen maximalen wartezeit inakzeptabel ist
			appThread.assertError();
			
			if( isMainWindowOpen() )
				return true;
			//for Win XP Problems
			new Snooze(100);
		}
		
		return false;
	}
	
	/** Gibt an, ob das Hauptfenster geffnet ist */
	protected boolean isMainWindowOpen() {
		return WindowMonitor.getInstance().getWindow(mainWindowTitle) != null;
	}
}
