/*
 * Created on 25.10.2005
 */
package atosj.util;

import java.awt.Color;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.eclipse.swt.graphics.RGB;

import atosj.Messages;
import atosj.app.AtosJApp;
import atosj.dialog.CommandStatisticsTableElement;
import atosj.dialog.ScriptStatisticsTableElement;
import atosj.model.pack.HTSPackage;
import atosj.model.script.HTSScript;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Cell;
import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Font;
import com.lowagie.text.PageSize;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Table;
import com.lowagie.text.pdf.PdfWriter;

/**
 * Klasse zum Erstellen von PDF Reports zur Ausfhrung von
 * Testdurchlufen.
 * @author Nicos Tegos
 */
public class ReportPrinter {
    
    private static ReportPrinter instance = null;
    private Font textFont;
    private Font h1Font, h2Font, h3Font;
    private Font gotoFont;
    
    
    private ReportPrinter() 
    {
        h1Font = new Font(Font.HELVETICA, 14, Font.BOLD);
        h2Font = new Font(Font.HELVETICA, 12, Font.BOLD);
        h3Font = new Font(Font.HELVETICA, 8, Font.BOLD);
        textFont = new Font(Font.HELVETICA, 8, Font.NORMAL);
        gotoFont = new Font(textFont);
    	gotoFont.setColor( new Color(0, 0, 255) );
    }
    
    /** Liefert die Singleton-Instanz des ReportPrinters. */
    public static ReportPrinter getInstance() 
    {
        if (instance == null) {
            instance = new ReportPrinter();
        }
        return instance;
    }
    
    /**
     * Generiert einen pdf Report zu einem Paketdurchlauf.
     * @param pack Das ausgefhrte Testpaket.
     * @param scriptStatistics Statistiken zu den einzelnen Scripten im Paket.
     * @param commandStatistics Statistiken der Kommandos der einzelnen Skripte.
     * Ein Array bestehend aus CommandStatisticsTableElement[].
     * @param stdout Konsoleausgabe pro Skript.
     * @param stderr Konsolefehlerausgabe pro Skript.
     * @return den Pfad des erstellten Reports oder <code>null</code> bei Fehler.
     */
    public String createPackagePdf(HTSPackage pack, Object[] scriptStatistics, Object[]commandStatistics, String[] stdout, String[] stderr)
    {
    	Document document = new Document(PageSize.A4);
    	// Margins: float left, right, top, bottom
    	float dist15 = getPoints(15);
    	document.setMargins(dist15, dist15, dist15, dist15);
    	Date currentDate = Calendar.getInstance().getTime();
    	
    	SimpleDateFormat fileDf = new SimpleDateFormat("dd.MM.yyyy HH.mm.ss"); //$NON-NLS-1$
    	String path = AtosJApp.getInstance().getInput().getTestObject().getProjectDir() + File.separator + "LOG" + File.separator + pack.getName() + " " + fileDf.format(currentDate) + ".pdf";         //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    	PdfWriter writer = null;
    	
    	try {            
    		
    		writer = PdfWriter.getInstance( document, new FileOutputStream(path) );
    		document.open();
    		
    		// Dokumentberschrift
    		Paragraph p = new Paragraph();
    		p.setAlignment(Paragraph.ALIGN_CENTER);
    		p.add( new Chunk(Messages.getString("ReportPrinter.4"), h1Font) ); //$NON-NLS-1$
    		SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss"); //$NON-NLS-1$
    		p.add( new Chunk(
    				Messages.getString("ReportPrinter.6") + df.format(currentDate) //$NON-NLS-1$
    				, textFont) 
    		);
    		
    		document.add(p);
    		
    		p = new Paragraph();
    		p.add( createHeading(Messages.getString("ReportPrinter.7") + pack.getName()) ); //$NON-NLS-1$
    		document.add(p);
    		
    		// Tabelle der Scripte mit Statistiken
    		p = new Paragraph();
    		Table table = createDefaultTable(6, true); 
    		table.setWidths( new int[] {8, 56, 8, 8, 12, 8} );
    		
    		addHeaderCell(table, Messages.getString("ReportPrinter.8")); //$NON-NLS-1$
    		addHeaderCell(table, Messages.getString("ReportPrinter.9")); //$NON-NLS-1$
    		addHeaderCell(table, Messages.getString("ReportPrinter.10")); //$NON-NLS-1$
    		addHeaderCell(table, Messages.getString("ReportPrinter.11")); //$NON-NLS-1$
    		addHeaderCell(table, Messages.getString("ReportPrinter.12")); //$NON-NLS-1$
    		addHeaderCell(table, Messages.getString("ReportPrinter.13")); //$NON-NLS-1$
    		
    		for(int i = 0; i < scriptStatistics.length; ++i)
    		{
    			ScriptStatisticsTableElement stat = (ScriptStatisticsTableElement) scriptStatistics[i];
    			
    			addNormalCell(table, stat.getLineNumber());
    			// eine Zelle mit link zur entsprechenden Testsequenz
    			addGotoCell(table, stat.getScriptName(), Messages.getString("ReportPrinter.14") + stat.getScriptName()); //$NON-NLS-1$
    			addNormalCell(table, stat.getInteractive());
    			addNormalCell(table, stat.getCleanup());
    			RGB rgb = stat.getStatusColor();
    			addNormalCell(table, stat.getStatus(), new Color(rgb.red, rgb.green, rgb.blue));
    			addNormalCell(table, stat.getAvgDuration());
    		}
    		p.add(table);
    		
    		document.add(p);
    		
    		// die einzelnen Skripte protokolieren
    		for(int i = 0; i < scriptStatistics.length; ++i)
    		{
    			ScriptStatisticsTableElement stat = (ScriptStatisticsTableElement) scriptStatistics[i];
    			createScriptSection(document, stat.getScript(), (Object[])commandStatistics[i], stdout[i], stderr[i]);
    		}
    	
    	} catch (FileNotFoundException e) {
    		e.printStackTrace();
    		return null;
    	} catch (DocumentException e) {
    		e.printStackTrace();
    		return null;
    	}
    	catch(Throwable e) {
    		e.printStackTrace();
    		return null;
    	}
    	finally
    	{
    		if (document != null && document.isOpen())
    			document.close();
    		
    		if(writer != null)
    			writer.close();
    	}
    	
		return path;
    }
    
    /**
     * Generiert einen pdf Report zu einem Skriptdurchlauf.
     * @param statistics Vector(StatisticsTableElement)
     * @param stdout Der Konsoleausgabe des Testobjekts.
     * @param stderr Der Konsolefehlerausgabe des Testobjekts.
     * @return den Pfad des erstellten Reports oder <code>null</code> bei Fehler.
     */
    public String createScriptPdf(HTSScript script, Object[] statistics, String stdout, String stderr)
    {
    	Document document = new Document(PageSize.A4);
    	// Margins: float left, right, top, bottom
    	float dist15 = getPoints(15);
    	document.setMargins(dist15, dist15, dist15, dist15);
    	Date currentDate = Calendar.getInstance().getTime();
    	
    	SimpleDateFormat fileDf = new SimpleDateFormat("dd.MM.yyyy HH.mm.ss"); //$NON-NLS-1$
    	String path = AtosJApp.getInstance().getInput().getTestObject().getProjectDir() + File.separator + "LOG" + File.separator + script.getName() + " " + fileDf.format(currentDate) + ".pdf";         //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    	PdfWriter writer = null;
    	
    	try {            
    		
    		writer = PdfWriter.getInstance( document, new FileOutputStream(path) );
    		document.open();
    		
    		// Dokumentberschrift
    		Paragraph p = new Paragraph();
    		p.setAlignment(Paragraph.ALIGN_CENTER);
    		p.add( new Chunk(Messages.getString("ReportPrinter.0"), h1Font) ); //$NON-NLS-1$
    		SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss"); //$NON-NLS-1$
    		p.add( new Chunk(
    				Messages.getString("ReportPrinter.21") + df.format(currentDate) //$NON-NLS-1$
					, textFont) 
    		);
    		
    		document.add(p);
    		createScriptSection(document, script, statistics, stdout, stderr);
    	} catch (FileNotFoundException e) {
    		e.printStackTrace();
    		return null;
    	} catch (DocumentException e) {
    		e.printStackTrace();
    		return null;
    	}
    	catch(Throwable e) {
    		e.printStackTrace();
    		return null;
    	}
    	finally
		{
    		if (document != null && document.isOpen())
    			document.close();
    		
    		if(writer != null)
    			writer.close();
		}
    	
    	return path;
    }
    
    private boolean createScriptSection(Document document, HTSScript script, Object[] statistics, String stdout, String stderr)
    {
    	try
		{
    		Paragraph p = new Paragraph();
    		p.add( createHeading(Messages.getString("ReportPrinter.22") + script.getName()) ); //$NON-NLS-1$
    		document.add(p);
    		
    		// Tabelle der Kommandos mit Statistiken
    		p = new Paragraph();
    		Table table = createDefaultTable(5, true); 
    		table.setWidths( new int[] {8, 60, 12, 12, 8} );
    		
    		addHeaderCell(table, Messages.getString("ReportPrinter.23")); //$NON-NLS-1$
    		addHeaderCell(table, Messages.getString("ReportPrinter.24")); //$NON-NLS-1$
    		addHeaderCell(table, Messages.getString("ReportPrinter.25")); //$NON-NLS-1$
    		addHeaderCell(table, Messages.getString("ReportPrinter.26")); //$NON-NLS-1$
    		addHeaderCell(table, Messages.getString("ReportPrinter.27")); //$NON-NLS-1$
    		
    		String error = ""; //$NON-NLS-1$
    		for(int i = 0; i < statistics.length; ++i)
    		{
    			CommandStatisticsTableElement stat = (CommandStatisticsTableElement) statistics[i];
    			
    			addNormalCell(table, stat.getLineNumber());
    			addNormalCell(table, stat.getCommandLine());
    			RGB rgb = stat.getStatusColor();
    			addNormalCell(table, stat.getStatus(), new Color(rgb.red, rgb.green, rgb.blue));
    			addNormalCell(table, stat.getRuns());
    			addNormalCell(table, stat.getAvgDuration());
    			if(stat.hasError())
    				error += stat.getError() + "\n\n"; //$NON-NLS-1$
    		}
    		
    		p.add(table);
    		
    		// bei Fehler die Fehlernachricht darstellen
    		if( error.length() > 0 )
    		{
    			table = createDefaultTable(1, false);
    			addHeaderCell(table, Messages.getString("ReportPrinter.30")); //$NON-NLS-1$
    			addNormalCell(table, error);
    			p.add(table);
    		}
    		else
    		{
    			table = createDefaultTable(1, false);
    			addHeaderCell(table, Messages.getString("ReportPrinter.31")); //$NON-NLS-1$
    			p.add(table);
    		}
    		
    		// Konsole: stdout
    		table = createDefaultTable(1, false); 
    		addHeaderCell(table, Messages.getString("ReportPrinter.32")); //$NON-NLS-1$
    		if( stdout.equals("") ) //$NON-NLS-1$
    			addNormalCell(table, Messages.getString("ReportPrinter.34")); //$NON-NLS-1$
    		else
    			addNormalCell(table, stdout);
    		
    		p.add(table);
    		
    		// Konsole: stderr
    		table = createDefaultTable(1, false);
    		addHeaderCell(table, Messages.getString("ReportPrinter.35")); //$NON-NLS-1$
    		if( stderr.equals("") ) //$NON-NLS-1$
    			addNormalCell(table, Messages.getString("ReportPrinter.37")); //$NON-NLS-1$
    		else
    			addNormalCell(table, stderr);
    		
    		p.add(table);
    		document.add(p);
    	} catch (DocumentException e) {
    		e.printStackTrace();
    		return false;
    	}
    	catch(Throwable e) {
    		e.printStackTrace();
    		return false;
    	}
    	
    	return true;
    }
    
    /**
     * Soll eine berschrift fr einen Absatz erstellen, die mit einer Farbe unterlegt
     * ist. Damit die Unterlegung ber die gesamte Breite sichtbar ist, wird eine
     * Tabelle verwendet. Die berschrift ist gleichzeitig auch das Ziel einer Sprungmarke.
     * Der Name der Sprungmarke ist gleich der berschrift.
     * @param heading Text der berschrift und Name der Sprungmarke.
     * @return Die Tabelle, die die berschrift enthlt, diese solte sinnvollerweise
     * nicht weiterverwendet werden sondern gleich zu einem anderen Element
     * hinzugefgt werden. 
     * @throws BadElementException
     */
    private Table createHeading(String heading) throws BadElementException
    {
    	Table table  = createDefaultTable(1, false);
    	Cell cell = new Cell( new Chunk(heading, h2Font).setLocalDestination(heading) );
		cell.setBackgroundColor( new Color(164, 204, 249) );
		table.addCell(cell);
		
		return table;
    }
    
    private Table createDefaultTable(int cols, boolean hasBorders) throws BadElementException
    {
    	Table table = new Table(cols); 
		table.setPadding(2.0f);
		table.setAlignment(Table.ALIGN_LEFT);
		table.setDefaultHorizontalAlignment(Table.ALIGN_LEFT);
		table.setDefaultVerticalAlignment(Table.ALIGN_CENTER);
		
		if(!hasBorders)
		{
			table.setBorder(Table.NO_BORDER);
            table.setDefaultCellBorder(Table.NO_BORDER);
		}
		
		return table;
    }
    
    private void addCell(Table table, String str, Font font, Color bgColor) throws BadElementException
    {
    	Cell c = new Cell( new Chunk(str, font));
    	c.setBackgroundColor(bgColor);
    	table.addCell(c);
    }
    
    private void addHeaderCell(Table table, String str) throws BadElementException
    {
    	addCell(table, str, h3Font, new Color(255, 255, 255));
    }
    
    private void addNormalCell(Table table, String str) throws BadElementException
    {
    	addNormalCell(table, str, new Color(255, 255, 255));
    }
    
    private void addNormalCell(Table table, String str, Color bgColor) throws BadElementException
    {
    	addCell(table, str, textFont, bgColor);
    }
    
    private void addGotoCell(Table table, String content, String gotoDestination) throws BadElementException
    {
    	Chunk chunk = new Chunk(content, gotoFont).setLocalGoto(gotoDestination);
    	chunk.setUnderline(0.8f, -1.1f);
    	table.addCell( new Cell(chunk) );
    }
    
    private float getPoints(float mm) {
	    return mm*2.8346f;
	}
}
