/*
 * Created on 14.07.2005
 */
package atosj.util;

import java.util.Comparator;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

/**
 * Sortiert Eintrge in einem <code>Viewer</code>. Aufsteigende und
 * Absteigende Sortierung ist mglich.
 * @author Nicos Tegos
 */
public class UniversalViewerSorter extends ViewerSorter 
{
	private Comparator comp = null;
	
	private Class[] categories;
	private int direction = 1;
	
	/**
	 * Sortierer mit einem default Comparator, der Comparable aufruft, wenn implementiert
	 * oder toString() vergleicht. Die Standardsortierreihenfolge ist aufsteigend.
	 */
	public UniversalViewerSorter() 
	{
		this( new Comparator() {
			public int compare(Object arg0, Object arg1) {
				if(arg0 instanceof Comparable)
					return ((Comparable)arg0).compareTo(arg1);
				else
					return arg0.toString().compareTo(arg1.toString());
			}
		}, new Class[0]);
	}
	
	/**
	 * Sortierer der Eintrge mit dem angegebenen Comparator vergleicht.
	 * Die Standardsortierreihenfolge ist aufsteigend.
	 */
	public UniversalViewerSorter(Comparator comp)
	{
		this(comp, new Class[0]);
	}
	
	/**
	 * Sortierer der Eintrge mit dem angegebenen Comparator vergleicht. Eintrge bestimmter
	 * Klassen werden getrennt sortiert.
	 * @param comp Der Comparator.
	 * @param categories Ermglicht eine Kategorisierung nach Klassen. Reihenfolge der Kategorien
	 * entspricht der Reifenfolge im Array.
	 */
	public UniversalViewerSorter(Comparator comp, Class[] categories)
	{
		this.comp = comp;
		this.categories = categories;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerSorter#compare(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	final public int compare(Viewer viewer, Object e1, Object e2) {
		int cat1 = category(e1);
		int cat2 = category(e2);

		if (cat1 != cat2) // Kategorie bleibt Kategorie, wird nicht beeinflusst durch reverse()
			return cat1 - cat2;
		
		return direction * comp.compare(e1, e2);
	}
	
	public int category(Object element) 
	{
		for(int i = 0; i < categories.length; ++i)
		{
			if( element.getClass().equals(categories[i]) )
				return i;
		}
		
		return categories.length;		
	}
	
	/**
	 * Setzt die Sortierrichtung.
	 * @param asc wenn <code>true</code> wird aufsteigend sortiert, sonst absteigend.
	 */
	final public void setDirection(boolean asc) {
		direction = asc ? 1 : -1;
	}
	
	/**
	 * Gibt die Sortierrichtung an.
	 * @return wenn <code>true</code> wird aufsteigend sortiert, sonst absteigend.
	 */
	final public boolean isAscending() {
		return direction == 1;
	}
	
	/** Kehrt die Sortierreihenfolge um. */
	final public void reverse() {
		direction *= -1;
	}
	
	/**
	 * Setzt fr alle angegegebenen Spalten der Tabelle einen Sortierer.
	 * Bei Druck auf den Spaltenkopf wird die Sortierreihenfolge umgekehrt.
	 * Darf UNBEDINGT erst gerufen werden nachdem die Spalten der Tabelle
	 * erzeugt wurden.
	 * @param tv Der <code>TableViewer</code> dessen Spalten sortiert sein sollen.
	 * @param colSorters Index der Sortierer entspricht Spalten der Tabelle,
	 * ist eine Sortierer <code>null</code> wird diese Spalte nicht sortiert.
	 * @param defIndex Index des Sortierers in <code>colSorters</code>, der standardmig
	 * gesetzt werden soll, -1 falls kein Sortierer festgelgt werden soll.
	 */
	public static void setColumnSorters(final TableViewer tv, UniversalViewerSorter[] colSorters, int defIndex)
	{
		final Table table = tv.getTable();
		final TableColumn[] cols = table.getColumns();
		final Cursor waitCursor = new Cursor(table.getDisplay(), SWT.CURSOR_WAIT);
		
		for(int i = 0; i < colSorters.length; ++i)
		{
			final UniversalViewerSorter colSorter = colSorters[i];
			final TableColumn col = cols[i];
			if(colSorter == null) // die Spalte wird nicht sortiert
				continue;
			
			// evtl. den DefaultSorter setzen
			if(i == defIndex)
			{
				tv.setSorter(colSorter);
				table.setSortColumn(col);
				table.setSortDirection(colSorter.isAscending() ? SWT.UP : SWT.DOWN);
			}	
			
			col.addSelectionListener( new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e)	{
					TableColumn sortColumn = table.getSortColumn();
					TableColumn currentColumn = (TableColumn) e.widget;
					int dir;
					
					if (sortColumn == currentColumn)
						dir = ((table.getSortDirection() == SWT.UP) ? SWT.DOWN : SWT.UP);
					else	{
						dir = SWT.UP;
						tv.setSorter(colSorter);
						table.setSortColumn(currentColumn);
					}

					colSorter.setDirection(dir == SWT.UP);
					table.setSortDirection(dir);
					
					table.setCursor(waitCursor);
					tv.refresh();
					table.setCursor(null);
				}

			});
		}
	}
	
	/**
	 * Erstellt aus allen Comparatoren entsprechende <code>UniversalViewerSorter</code> und
	 * leitet den Aufruf an die Funktion 
	 * <code>setColumnSorters(final TableViewer tv, UniversalViewerSorter[] colSorters, int defIndex)</code> weiter.
	 * @see #setColumnSorters(TableViewer, UniversalViewerSorter[], int)
	 */
	public static void setColumnSorters(final TableViewer tv, Comparator[] comparators, int defIndex)
	{
		UniversalViewerSorter[] colSorters = new UniversalViewerSorter[comparators.length];
		for(int i = 0; i < comparators.length; ++i)
		{
			if(comparators[i] != null)
				colSorters[i] = new UniversalViewerSorter(comparators[i]);
			else
				colSorters[i] = null;
		}
		
		setColumnSorters(tv, colSorters, defIndex);
	}
}
