/*
 * Created on 08.09.2005
 */
package atosj.util;


import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;

import atosj.component.ComponentFinder;
import atosj.component.ComponentId;
import atosj.component.EventMonitorWrapper;
import atosj.component.ICommandCreator;
import atosj.component.MComponent;
import atosj.component.NamingStrategy;
import atosj.exception.ComponentNotFoundException;
import atosj.listener.AtosJTopLevelWindowListener;

/**
 * Singleton-Klasse zur berwachung des ffnens und Schlieens von Fenstern im Testobjekt.
 * Speichert alle geffneten Fenster und ermglicht Zugriff auf sie. 
 * @author Nicos Tegos
 */
public abstract class WindowMonitor 
{
	private static WindowMonitor _instance = null;
	
	private List _windows = new ArrayList(); // List<Object> Liste der aktuell geffneten TopLevelWindows
	private List _windowTitles = new ArrayList(); // List<String> Liste der Fenstertitel, korrespondiert mit der Fensterliste
	private List _topLevelWindowListeners = new ArrayList();
	private ComponentFinder finder;
	private EventMonitorWrapper eventMonitor;

	/**
	 * Erstellt den Monitor.
	 * @param finder bernimmt die Suche nach Components mittels einer <code>ComponentId</code>.
	 * @param eventMonitor Globale berwachungsinstanz aller Ereignisse, die auf den Components
	 * des Testobjekts auftreten. Ist nur im Capture Modus erforderlich, sonst <code>null</code>.
	 * 
	 */
	protected WindowMonitor(ComponentFinder finder, EventMonitorWrapper eventMonitor) {
		this.finder = finder;
		this.eventMonitor = eventMonitor;
	}
	
	/**
	 * Setzt die Singleton-Instanz des WindowMonitors entsprechend
	 * des aktuellen Testobjekts als AWT- oder SWTWindowMonitor.
	 * Sollte nur einmal vor dem Start des Testobjekts gerufen werden.
	 * @param instance Die neue Instanz.
	 */
	protected static void setInstance(WindowMonitor instance) {
		if(_instance == null)
			_instance = instance;
	}
	
	/**
	 * Liefert die Singleton-Instanz.
	 * @return Die aktuelle Instanz oder <code>null</code>, wenn die Instanz mit
	 * noch nicht initialisiert wurde.
	 * @see #setInstance(WindowMonitor)
	 */
	public synchronized static WindowMonitor getInstance() {
		return _instance;
	}
	
	/**
	 * Liefert alle geffneten Fenster mit dem angegebenen Titel.
	 * @param title Der Titel des Fensters, den es im Moment des ffnens hatte.
	 * @return Alle gefundenen Fenster oder <code>null</code>, wenn
	 * kein Fenster gefunden werden konnte.
	 */
	public synchronized Object[] getWindow(String title) 
	{
		Vector win = new Vector();
		for(int i = 0; i < _windowTitles.size(); ++i) 
		{
			if( title.equals(_windowTitles.get(i)) ) 
				win.add( _windows.get(i) );
		}
		
		if( win.isEmpty() )
			return null;
		
		return win.toArray();
	}
	
	/**
	 * Liefert den Titel/Namen des Fensters, den es im Moment des ffnens hatte,
	 * ermittelt anhand der aktuellen <code>NamingStrategy</code>
	 * @param window Das Fenster dessen Titel/Name ermittelt werden soll.
	 * @return Der Fenstertitel/-name
	 * @see NamingStrategy
	 */
	public String getTitle(Object window) 
	{
		for(int i = 0; i < _windows.size(); ++i) 
		{
			if( window == _windows.get(i) ) 
				return (String) _windowTitles.get(i);
		}
		
		return null;
	}
	
	/**
	 * Fgt das Fenster der Liste der geffneten Fenster hinzu. Informiert alle Listener.
	 * Ermittelt den Titel des Fensters, unter dem es spter referenziert werden kann.
	 * Es knnen mehrere Fenster mit demselben Titel enthalten sein.
	 * @param window Das neu geffnete Fenster.
	 */
	protected synchronized void addWindow(Object window) 
	{
		// die AtosJ eigenen Fenster ignorieren
		String title = getNamingStrategy().getName(window);
		if(title.equals(ICommandCreator.TEST_ACTION_DIALOG_TITLE) )
			return;
		
		if( !_windows.contains(window) )
		{
			_windows.add(window);
			_windowTitles.add(title);
			
			fireTopLevelWindowEvent(window, true);
		}
	}
	
	/**
	 * Entfernt das Fenster aus der Liste der geffneten Fenster hinzu. 
	 * Informiert alle Listener.
	 * @param window Das geschlossene Fenster.
	 */
	protected synchronized void removeWindow(Object window) 
	{
		if( _windows.contains(window) )
		{
			finder.removeAll(window);
			
			for(int i = 0; i < _windows.size(); ++i)
			{
				if( _windows.get(i) == window )
				{
					_windows.remove(i);
					_windowTitles.remove(i);
					break;
				}
			}
			
			fireTopLevelWindowEvent(window, false);
		}
	}
	
	/** Registriert einen Listener, der auf das Hinzufgen und entfernen von Fenstern reagiert. */
	public void addTopLevelWindowListener(AtosJTopLevelWindowListener listener)
	{
		if( !_topLevelWindowListeners.contains(listener) )
			_topLevelWindowListeners.add(listener);
	}
	
	/** Entfernt einen Listener, der auf das Hinzufgen und entfernen von Fenstern reagiert. */
	public void removeTopLevelWindowListener(AtosJTopLevelWindowListener listener)
	{
		_topLevelWindowListeners.remove(listener);
	}
	
	/**
	 * Wenn das Testobjekt im Capture Modus gestartet wurde, knnen
	 * hier EventListener zu den einzelnen Components registiert werden.
	 * Der Listener sorgt fr die automatische Generierung von Kommandos.
	 * Tatschlich wird der Aufruf einfach an den enthaltenen <code>EventMonitorWrapper</code>
	 * weitergeleitet.
	 * @param component Das Component auf dessen Events gehrt werden soll.
	 * @param listener Der entsprechende Eventlistener.
	 * @see EventMonitorWrapper#addListener(Object, Object)
	 */
	public void addListener(Object component, Object listener) {
		if(eventMonitor != null)
			eventMonitor.addListener(component, listener);		
	}

	/**
	 * Entfernt den hinzugefgten Listener wieder. 
	 * @see WindowMonitor#addListener(Object, Object)
	 */
	public void removeListener(Object component, Object listener) {
		if(eventMonitor != null)
			eventMonitor.removeListener(component, listener);
	}
	
	/**
	 * Benachrichtigt alle <code>AtosJTopLevelWindowListener</code> ber
	 * das fnnen und Schlieen von Fenstern.
	 * @param window Das Fenster.
	 * @param created <code>true</code>, wenn ber das ffnen informiert werden
	 * soll, <code>false</code> fr Schlieen.
	 */
	protected void fireTopLevelWindowEvent(Object window, boolean created)
	{
		for(ListIterator it = _topLevelWindowListeners.listIterator(); it.hasNext(); )
		{
			if(created)
				((AtosJTopLevelWindowListener)it.next()).topLevelWindowCreated(window);
			else
				((AtosJTopLevelWindowListener)it.next()).topLevelWindowDestroyed(window);
		}
	}

	/** 
	 * Leitet weiter an den assoziierten <code>ComponentFinder</code>.
	 * @see ComponentFinder#getMComponent(ComponentId)
	 */
	public MComponent getMComponent(ComponentId id) throws ComponentNotFoundException {
		return finder.getMComponent(id);
	}
	
	/** 
	 * Leitet weiter an den assoziierten <code>ComponentFinder</code>.
	 * @see ComponentFinder#getNamingStrategy()
	 */
	public NamingStrategy getNamingStrategy() {
		return finder.getNamingStrategy();
	}
	
	/** 
	 * Liefert den assoziierten <code>ComponentFinder</code>.
	 * @see #getMComponent(ComponentId)
	 * @see #getNamingStrategy()
	 * */
	public ComponentFinder getComponentFinder() {
		return finder;
	}
}
